#!/usr/bin/python
# coding: latin-1

has_readline = False
# mit dem folgenden hat man eine persistente command history                                                                                                                                                       
# und autocompletion mit der Tab-Taste                                                                                                                                                                             
import os
try:
  import rlcompleter
  import readline
  import atexit
  readline.parse_and_bind('tab: complete')
  historyPath = os.path.expanduser("~/.python_history")
  #print historyPath                                                                                                                                                                                               
  readline.set_history_length(1000)
  def save_history(historyPath=historyPath):
     import readline
     # why can't the next line see the global readline?                                                                                                                                                            
     readline.write_history_file(historyPath)
     #print 'saving history'                                                                                                                                                                                       
  if os.path.exists(historyPath):
     readline.read_history_file(historyPath)
  atexit.register(save_history)
  # clean up the namespace                                                                                                                                                                                         
  del atexit, readline, save_history, historyPath, rlcompleter
  has_readline = True
except:
  print 'Kein Readline usw.'
  #raise

"""
Alle Daten fr Bundesstatistik zusammensuchen und in eine einheitliche dict packen.

Quellen sind:
- _fb_data in jghstatistik.py
- JGHDatensatz_2022.datensatz_info in jghexport.py
- Datenbanktabelle Feld



Die dict sollte diese Struktur haben:

Bundesstatistik
  Abschnitt
    Item
      Optionen (nur fr kat_codes, nicht fr andere Datentypen)



Aus dieser dict dann die Vorlage fr jghdef_2023.py erzeugen.
Da mssen nicht alle gesammelten Daten drin sein.

"""

import sys
import json
from itertools import groupby
from operator import itemgetter
import init
from ebkus.db.sql import getDBHandle, opendb
from ebkus.app.ebapi import TabelleList, FeldList, CodeList
from ebkus.app.ebapih import get_codes
from ebkus.app import ebapi
from ebkus.html.jghstatistik import _fb_data
from ebkus.app.jghexport import JGHDatensatz_2022
from ebkus.app.jghdef import pp_dict, read_bundesstatistik_def, write_bundesstatistik_def


file_jghdef_2022 = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/jghdef_2022.py'
file_jghdef_v2_2022 = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/jghdef_v2_2022.py'
file_xml_json = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/xml_schema_01652200009999000002.json'
file_namen_idev = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/namen_idev.py'
file_namen_core = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/namen_core.py'
file_namen_zuordnen_generiert = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/file_namen_zuordnen_generiert.py'
file_namen_zuordnen_manuell = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/file_namen_zuordnen_manuell.py'
file_namen_zuordnen_fertig = '/home/ebkus/ebkus_test/ebkus/bundesstatistik/file_namen_zuordnen_fertig.py'



opendb()




class JGHDefGen(object):
    """
    info_fb_keys = ('gruende_bsp', 'name', 'title', 'kat_code', 'extra', 'frage', 'checkbox_value', 'maximum', 'default', 'minimum', 'abschnitt', 'seq', 'no_empty_option', 'typ')
    info_ds_keys = ('name', 'typ', 'wireb_name', 'wireb_nr', 'von', 'bis', 'field', 'format', 'substitution', 'value',)
    info_db_keys = ('name', 'langname', 'kat_code', 'verwtyp__code', 'typ')
    """
    def __init__(self):
        self.fb_list = info_fb()
        self.fb_dict = dict((x['name'], x) for x in self.fb_list)
        self.db_list = info_db()
        self.db_dict = dict((x['name'], x) for x in self.db_list)
        self.ds_list = info_ds()
        self.ds_dict = dict((x['name'], x) for x in self.ds_list)
        self.core_dict_raw = self.get_core_dict_raw(file_xml_json) # json repr der xml schema def
        self.idev_namen_pos = self.get_namen_position(file_namen_idev) # name -> pos
        self.core_namen_pos = self.get_namen_position(file_namen_core) # name -> pos
        self.wireb_namen_pos = dict((x['wireb_name'], x['wireb_nr']) for x in self.ds_list) # name -> pos (oder None)
        self.all = set(k for k in self.fb_dict) | set(k for k in self.db_dict) | set(k for k in self.ds_dict)
        self.common = set(k for k in self.fb_dict) & set(k for k in self.db_dict) & set(k for k in self.ds_dict)
        self.in_db_und_ds = set(k for k in self.db_dict if k in self.ds_dict and k not in self.common)
        self.nur_in_db = set(k for k in self.db_dict if k not in self.ds_dict and k not in self.common)
        self.nur_in_ds = set(k for k in self.ds_dict if k not in self.db_dict and k not in self.common)
        self.ebkus_namenszuordnung = self.read_namenszuordnung(file_namen_zuordnen_fertig) # dict: ebkus_name --> dict: {wireb,idev,core} --> (name, position,...)

        assert len(self.all) == len(self.common) + len(self.in_db_und_ds) + len(self.nur_in_db) + len(self.nur_in_ds)
        # common: aip, aort_nac, aort_vor, ausl_her, bgm, bgy, ees, em, ey, gef8a, gem, gey, gr1, gr2, gr3, grende, gs, hda, hilf_art, hilf_ort, inobh42_22, lbk6m, nbkakt, nbkges, rgu, sit_fam, traeger, unh, va52, vor_dt, wirt_sit, zustw

        # in_db_und_ds(4): einrnr, kr, land, lnr
        # nur_in_db(9): fall_fn, fall_id, gfall, id, inobh42, jahr, mit_id, stz, zeit
        # nur_in_ds(14): ags, aktuellbesuchteausbildungsstaette, ba, datensatzende, e2e3leer, extra_hilfen_lnr, extra_kennnummer, extra_mehrere_hilfen, familienorientiertehilfe, hilfeausserhalbelternhaus, j2leer, m2leer, ort, plz


        self.abschnitt_keys = ('aip', 'aort_nac', 'aort_vor', 'ausl_her', 'bgm', 'bgy', 'ees', 'em', 'ey', 'gef8a', 'gem', 'gey', 'gr1', 'gr2', 'gr3', 'grende', 'gs', 'hda', 'hilf_art', 'hilf_ort', 'inobh42_22', 'lbk6m', 'nbkakt', 'nbkges', 'rgu', 'sit_fam', 'traeger', 'unh', 'va52', 'vor_dt', 'wirt_sit', 'zustw')
        self.versteckt_keys = ('aktuellbesuchteausbildungsstaette', 'ba', 'datensatzende', 'e2e3leer', 'extra_hilfen_lnr', 'extra_kennnummer', 'extra_mehrere_hilfen', 'familienorientiertehilfe', 'hilfeausserhalbelternhaus', 'j2leer', 'm2leer', 'plz', 'ort', 'ags', 'gemeinde')
        self.falldaten_keys = ('jahr', 'fall_fn', 'land', 'bgm', 'bgy', 'kr', 'mit_id__name', 'einrnr', 'gfall', 'lnr', 'wohnt_ausserhalb', 'beginn')
        self.bundesstatistik_v1 = self.generate_html_driver()
        self.v1_items_by_ebkus_name = self.get_v1_items_by_ebkus_name()
        self.v2_items_by_ebkus_name = {}
        self.bundesstatistik_v2 = self.generate_v2()
    def generate_v2(self):
        teile = []
        items = self.generate_v2_items()
        for x in items:
            self.v2_items_by_ebkus_name[x['ebkus_name']] = x
        bundesstatistik = dict(
            art='Bundesstatistik',
            berichtsjahr=2022,
            statistik_items=items,
            datensatz_laenge=237,
            core_csv_positionen=76, # TODO ??
            idev_csv_positionen=75, # TODO ??
            fragebogen_elemente=self.generate_v2_fragebogen()
            )
        for item in bundesstatistik['statistik_items']:
            #print item.get('ebkus_name')
            #print item['ebkus_name'], item.get('typ', 'Kein Typ'), item.get('vorgabe_wert', 'Keine Vorgabe'), repr(item.get('fester_wert', 'Kein fester Wert'))
            #print
            # print pp_dict(item)
            # if 'typ' not in item:
            #     print item.get('ebkus_name')
            pass
        self.check_driver_dicts_v2(bundesstatistik)
        return bundesstatistik
    def generate_v2_items(self):
        """
        Die Items decken genau die abzugebenden Daten ab. Einige pauschal (leer),
        weil die EBKuS-Anwender die nicht abgeben mssen.
        """
        # items_v1 = self.v1_items_by_ebkus_name
        # print len(items_v1)
        # print set(items_v1) - set(self.ebkus_namenszuordnung)
        # print set(self.ebkus_namenszuordnung) - set(items_v1)
        items = []
        # Fehlende Items fr Falldaten:
        # TODO das funkt
        for ebkus_name, name in \
            (('jahr', 'Jahr'), ('fall_fn', 'Fallnummer'),
             ('mit_id__name', 'Mitarbeiter'), ('gfall', 'Geschwisterfall'),
             ('wohnt_ausserhalb', 'Wohntausserhalb'), ('beginn', 'Beginn')):
            
            items.append(self.generate_v2_item(ebkus_name, name=name))
        for key in self.ebkus_namenszuordnung:
            items.append(self.generate_v2_item(key))
        self.v2_items_add_core_langname(items)
        return items
    def v2_items_add_core_langname(self, items):
        core_name_to_desc = dict(self.get_core_name_description_list())
        for x in items:
            langname = core_name_to_desc.get(x.get('name'))
            if langname is not None:
                x['core_langname'] = langname
        #print pp_dict(core_name_to_desc)
    def v2_item_add_gruende_berichtseinheitid(self, item):
        """
        typ Option
        optionsgruppe: bei Option, muss nur bei einem Item stehen bei wiederholter Verwendung
        optionen: 
        code: {text: fett}
        feldlnge 2
        gr3: status optional
        """
        ebkus_name = item['ebkus_name']
        # print ebkus_name
        # if ebkus_name in ('gr1', 'gr2', 'gr3', 'berichtseinheitid'):
        #     print ebkus_name, item
        if ebkus_name in ('gr1', 'gr2', 'gr3'):
            item.update(optionsgruppe='gruende',
                        feldlaenge=2,
                        typ='Option',
                        datensatz_position=
                        (self.ds_dict[ebkus_name]['von'],
                         self.ds_dict[ebkus_name]['bis']))
        if ebkus_name == 'gr1':
            codes = get_codes('gruende')
            optionen = []
            gruende_bsp = self.fb_dict['gr3']['gruende_bsp']
            for c in codes:
                code = c['code']
                text = c['name']
                zusatz = gruende_bsp[code]
                if zusatz:
                    text += "<br>{}".format(zusatz)
                option = dict(code=code,
                              text=text)
                optionen.append(option)
            item.update(optionen=optionen)
        if ebkus_name in ('gr2',  'gr3'):
            item.update(status='optional')
        if ebkus_name == 'berichtseinheitid':
            item.update(typ='Zeichenkette',
                        feldlaenge=14)
    def generate_v2_item(self, key, name=None):
        """
  name: core cvs + einige extra
  ebkus_name: aus historischen und praktischen Grnden, wre aber technisch nicht unbedingt ntig.
  idev_name:
  core_langname:
  typ: Ganzzahl, Zeichenkette, Option
  optionsgruppe: bei Option, muss nur bei einem Item stehen bei wiederholter Verwendung
  optionen: 
      code: ohne code ist es einfach eine zwischenberschrift. falls das vorkommt, werden radiobuttons statt selectboxen genommen
      text:
      fett:
  minimum: bei Zahlen
  maximum:
  feldlaenge: fr alle Typen
  doku:
  vorgabe_wert: ein Wert, der als Anfangswert genommen werden soll, aber im Fragebogen verndert werden kann.
                einige werte werden von ebkus initialisiert: Beginn, Anzahl der Kontakte.
  fester_wert: der wert von items, die bei EBKuS immer gleich beantwortet werden, oder "leer", 
               wenn das/die Item/s in EBKuS nicht beantwortet werden.
  datensatz_position: (2,5)
  idev_csv_position: 33 # bzw (50,66), wenn mehrere Leerspalten erzeugt werden sollen
  core_csv_position: 22 # wie idevcsv_position
  wireb_csv_position: 22 # wie idevcsv_position
  bedingung: nur bei andauernd, nur bei abgeschlossen
  status: optional (z.B. dritter Grund, Wohnort, etc, auch Checkboxen, berall wo bei core xml required=False steht) 
        """

        item = dict(ebkus_name=key, art='Item')
        if name:
            item.update(name=name)
        self.v2_item_add_namenszuordnung(key, item)
        self.v2_item_add_v1_info(key, item)
        self.v2_item_add_gruende_berichtseinheitid(item)
        self.v2_item_add_verschiedenes(item)
        if 'typ' not in item:
            item.update(typ='Zeichenkette')
        return item
    def v2_item_add_verschiedenes(self, item):
        if item.get('ebkus_name') == 'zustw':
            item.update(
                typ='Checkbox',
                feldlaenge=1,
                minimum=0,
                maximum=1,
                vorgabe_wert=0,
                )
        if item.get('ebkus_name') == 'jahr':
            item.update(
              feldlaenge=4,
              minimum=2022,
              maximum=2070,
            )
        if item.get('ebkus_name') == 'fall_fn':
            item.update(
              readonly=True,
              feldlaenge=10,
            )
        if item.get('ebkus_name') == 'mit_id__name':
            item.update(
              readonly=True,
              feldlaenge=50,
            )
        if item.get('ebkus_name') == 'gfall':
            item.update(
              feldlaenge=1,
              vorgabe_wert='1',
              optionen=[
                dict(
                  text='Nein',
                  code='1',
                ),
                dict(
                  text='Ja',
                  code='2',
                ),
              ],
            )
        if item.get('ebkus_name') == 'wohnt_ausserhalb':
            item.update(
              readonly=True,
              feldlaenge=4,
            )
        if item.get('ebkus_name') == 'beginn':
            item.update(
              feldlaenge=7,
              readonly=True,
            )
            

    def v2_item_add_v1_info(self, key, item):
        """
        minimum, maximum
        datensatz_position
        feldlaenge berechnen
        typ
        optionsgruppe (kat_code)
        optionen
        vorgabe_wert (default)
        fester_wert
        bedingung
        status
        """
        # if key not in self.v1_items_by_ebkus_name:
        #     print '************NICHT IN V1:', key
        v1_item = self.v1_items_by_ebkus_name.get(key)
        if v1_item is None:
            self.v2_item_add_gruende_berichtseinheitid(item)
            return
        if v1_item.get('teile'):
            v1_eingabe = [x for x in v1_item['teile'] if x['art'].endswith('eingabe')][0]
        else:
            v1_eingabe = None
        minimum, maximum = v1_item.get('minimum'), v1_item.get('maximum')
        if minimum:
            item.update(minimum=minimum, maximum=maximum,
                        typ='Ganzzahl', feldlaenge=len(str(maximum)))
        von, bis = v1_item.get('datensatz_position_von'), v1_item.get('datensatz_position_bis')
        if von:
            assert bis
            item.update(datensatz_position=(von, bis),
                        feldlaenge=bis-von+1)
        if 'kat_code' in v1_item:
            assert v1_eingabe
            optionen = v1_eingabe['optionen']
            # damit das poppen keine Auswirkung auf self.bundesstatistik_v1 hat
            optionen = [x.copy() for x in optionen]
            # print key, optionen
            _ = [o.pop('art') for o in optionen if 'art' in o]
            code_laengen = [len(o['code']) for o in optionen]
            feldlaenge = code_laengen[0]
            assert not [laenge for laenge in code_laengen if laenge != feldlaenge]
            item.update(typ='Option', optionsgruppe=v1_item['kat_code'], optionen=optionen,
                        feldlaenge=feldlaenge)
        default = v1_item.get('default')
        # will nur wirkliche Vorgabe nehmen. Das Vorhandensein eines leeren Elements
        # bei den Optionen wird anders geregelt (immer da, wenn keine Vorgabe)
        if default and default.strip():
            item.update(vorgabe_wert=default)
        konstanter_wert = v1_item.get('konstanter_wert')
        if konstanter_wert is not None:
            item.update(fester_wert=konstanter_wert)
        # typ updaten
        known_typ = item.get('typ')
        typ = None
        fester_wert = item.get('fester_wert')
        if v1_eingabe:
            eingabetyp = v1_eingabe['art'][:-7]
            if eingabetyp == 'Text':
                typ = 'Zeichenkette'
            elif eingabetyp == 'Zahl':
                typ = 'Ganzzahl'
            elif eingabetyp == 'Select':
                typ = 'Option'
            elif eingabetyp == 'Checkbox':
                typ = 'Option'
        if fester_wert:
            typ = 'Zeichenkette'
        assert known_typ is None or known_typ == typ
        if typ:
            item['typ'] = typ
        #print v1_item
        #print
    def v2_item_add_namenszuordnung(self, key, item):
        namenszuordnung = self.ebkus_namenszuordnung.get(key)
        if namenszuordnung is None:
            # extra item fr falldaten nicht relevant fr bundesstatistik
            return
        core = namenszuordnung['core']
        wireb = namenszuordnung['wireb']
        idev = namenszuordnung['idev']
        # print 'KEY:', key
        # print 'NAMENSZUORDNUNG:', namenszuordnung
        # print 'core:', core
        # print 'wireb:', wireb
        # print 'idev:', idev
        if core:
            if len(core) == 1:
                item.update(name=core[0][0],
                            core_csv_position=core[0][1])
            else:
                item.update(name=wireb[0][0])
                item.update(core_namen=tuple(t[0] for t in core))
                item.update(core_csv_position=tuple(sorted([t[1] for t in core])))
        else:
            item.update(name=wireb[0][0])
        if idev:
            if len(idev) == 1:
                item.update(idev_name=idev[0][0],
                            idev_csv_position=idev[0][1])
            else:
                item.update(idev_namen=tuple(t[0] for t in idev))
                item.update(idev_csv_position=tuple(sorted([t[1] for t in idev])))
        if wireb:
            if len(wireb) == 1:
                item.update(wireb_name=wireb[0][0],
                            wireb_csv_position=wireb[0][1])
            else:
                assert False, "Multiple Wireb sollte es nicht geben: {}".format(wireb)
    def generate_v2_fragebogen(self):
        """
        """
        elems = []
        # versteckt und falldaten weglassen
        # abschnitt -> dict
        v1_abschnitte = dict((x['abschnitt'], x) for x in self.bundesstatistik_v1['teile'] if len(x['abschnitt']) < 100)
        for x in ('Falldaten', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P'):
        # for x in ('A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P'):
            v1_abschnitt = v1_abschnitte.get(x)
            v2_abschnitt = self.generate_v2_abschnitt(x, v1_abschnitt)
            self.add_v2_abschnitt_verschiedenes(v2_abschnitt)
            elems.append(v2_abschnitt)
            if v2_abschnitt.get('gliederung') == 'K':
                # hier Zwischentext einfgen
                elems.append(dict(
                    art='ZwischenAbschnittText',
                    gliederung='Zwischen',
                    text="Nachfolgende Angaben bitte zustzlich beim Ende "
                    "der Hilfe/Beratung ausfllen."))
        return elems
    def add_v2_abschnitt_verschiedenes(self, abschnitt):
        if abschnitt.get('gliederung') == 'F':
            for x in abschnitt.get('teile', []):
                if x.get('item') == 'SozialSicherung':
                    x.update(text='Die Familie bzw. die/der junge Volljhrige lebt teilweise oder ganz von <ul><li>Arbeitslosengeld (nach dem Zweiten Buch Sozialgesetzbuch - SGB II), auch in Verbindung mit Sozialgeld,</li><li>Sozialhilfe oder Grundsicherungsleistungen im Alter und bei Erwerbsminderung (nach dem Zwlften Buch Sozialgesetzbuch - SGB XII),</li><li>einem Kinderzuschlag</li></ul>',
                             )
        if abschnitt.get('gliederung') == 'J':
            abschnitt['teile'].insert(0, dict(
                art='Zwischentext',
                text='(Nur ausfllen, falls die Hilfe/Beratung am Jahresende andauert)',
                fett=True,
                ))
                    
    def generate_v2_abschnitt(self, gliederung, v1_abschnitt):
        if gliederung == 'K':
            return self.generate_v2_abschnitt_k()
        teile = []
        #print v1_abschnitt
        abschnitt = dict(art='Abschnitt',
                         gliederung=gliederung,
                         titel=v1_abschnitt['titel'],
                         teile=teile)
        #print gliederung
        for x in v1_abschnitt.get('teile', []):
            art = x['art']
            # print '  ', art
            if art.startswith('Item'):
                ebkus_name = x['name']
                v2_item = self.v2_items_by_ebkus_name[ebkus_name]
                if ebkus_name in ('bgm', 'bgy') and gliederung == 'Falldaten':
                  continue
            else:
                print 'ABSCHNITTTEIL KEIN ITEM:', art
            for y in x.get('teile', []):
                art = y['art']
                if art.endswith('eingabe'):
                    #print '    ', art
                    eingabe = dict(art='Eingabe',
                                   item=v2_item['name'],
                                   text=y['text'])
                    teile.append(eingabe)
        if gliederung == 'Falldaten' and 'layout' in v1_abschnitt:
            layout_v1 = v1_abschnitt['layout']
            layout = []
            for x in layout_v1:
                elem = []
                for ebkus_name in x:
                    if ebkus_name == 'dummy':
                        name = 'Dummy'
                    else:
                        name = self.v2_items_by_ebkus_name[ebkus_name]['name']
                    elem.append(name)
                layout.append(elem)
            abschnitt.update(layout=layout,
                             art='AbschnittFalldaten')
        return abschnitt
    def generate_v2_abschnitt_k(self):
        return dict(gliederung='K',
                    titel='Grnde fr die Hilfegewhrung',
                    art='Abschnitt',
                    teile=[
                        dict(
                            art='EingabeK',
                            text='Grnde (bitte 1 - 3 Grnde ankreuzen)',
                            items=('HauptgrundHilfegewaehrung',
                                   'ZweiterGrund', 'DritterGrund'),
                            headers=('Hauptgrund', '2. Grund', '3. Grund',),
                        )])
    def show(self):
        def pr(attr):
            s = getattr(self, attr, None)
            if s is None: return
            l = list(s)
            l.sort()
            print "{}({}): {}".format(a, len(l), ', '.join(l))
            print
        for a in 'common', 'in_db_und_ds', 'nur_in_db', 'nur_in_ds':
            pr(a)
        for a in 'abschnitt_keys', 'versteckt_keys', 'falldaten_keys':
            pr(a)
        set1 = self.common | self.in_db_und_ds | self.nur_in_db | self.nur_in_ds
        print 'In common, etc:', len(set1)
        set2 = set(self.versteckt_keys) | set(self.falldaten_keys) | set(self.abschnitt_keys)
        print 'In versteckt, falldaten, abschnitt etc:', len(set2)
        print set1 - set2
    
    def write_bundesstatistik(self, fname=file_jghdef_2022):
        # TODO
        # merkwrdiger weise verdoppelt die dict, wenn diese
        # doppelt aufgerufen wird innerhalb einer Inkarnation
        # bleibt unkla
        self.bundesstatistik_v1 = self.generate_html_driver()
        dict_ = self.bundesstatistik_v1
        self.check_driver_dicts(dict_)
        write_bundesstatistik_def(dict_, fname)
        print 'Written:', fname
    def write_bundesstatistik_v2(self, fname=file_jghdef_v2_2022):
        # TODO
        # merkwrdiger weise verdoppelt die dict, wenn diese
        # doppelt aufgerufen wird innerhalb einer Inkarnation
        # bleibt unkla
        dict_ = self.bundesstatistik_v2
        write_bundesstatistik_def(dict_, fname)
        print 'Written:', fname
    def get_core_dict_raw(self, filename):
        with open(filename) as f:
            d = json.load(f, encoding='utf8')
        def dict2latin1(d):
            res = {}
            for k, v in d.items():
                assert isinstance(k, unicode)
                if isinstance(v, unicode):
                    value = v.encode('latin1')
                elif isinstance(v, list):
                    value = [dict2latin1(x) for x in v]
                elif isinstance(v, dict):
                    value = dict2latin1(v)
                else:
                    value = v
                res[k.encode('latin1')] = value
            return res
        return dict2latin1(d)
    def get_v1_items_by_ebkus_name(self):
        """Items aus v1 direkt holen durch ebkus_name"""
        items_v1 = {} # ebkus_dbname --> item_dict
        for abschnitt in self.bundesstatistik_v1['teile']:
            for i in abschnitt['teile']:
                # items liegen alle auf dieser Ebene! (hoffentlich?)
                if i['art'].startswith('Item'):
                    items_v1[i['name']] = i
                # print i['name'] 
                #print i['art'], i['name'], '-->'
                #print
        return items_v1
    def get_core_name_description_list(self):
        variables = self.core_dict_raw["DatML-SDF"]["inputDataModel"]["variables"]["variable"]
        # for v in variables:
        #     print v['_name'], v['description']['__text']
        return [(v['_name'], v['description']['__text']) for v in variables]
    def get_namen_position(self, filename):
        res = {}
        for i, x in enumerate(open(filename).readlines(), start=1):
            x = x.strip()
            name = x.decode('utf8').encode('latin1')
            # print name, i
            res[name] = i
        return res
    def compare_core_names_with_wireb_names(self):
        core_name_to_desc = dict(self.get_core_name_description_list())
        core_names = set(core_name_to_desc.keys())
        wireb_names = set(x['wireb_name'] for x in self.ds_dict.values())
        print core_names
        print '----------_'
        print wireb_names
        count = 0
        for x in core_names & wireb_names:
            count += 1
            print  'Gemeinsam:   {:<35} {}'.format(x, core_name_to_desc[x])
        print count
        for x in core_names:
            if x not in wireb_names:
                print 'Nur in core: {:<35} {}'.format(x, core_name_to_desc[x])
        for x in wireb_names:
            if x not in core_names:
                print 'Nur in wireb:', x
        
    def compare_core_xml_names_with_core_csv_names(self):
        core_name_to_desc = dict(self.get_core_name_description_list())
        core_xml_names = set(core_name_to_desc.keys())
        core_csv_names = set(k for k in self.core_namen_pos)
        for x in core_xml_names & core_csv_names:
            print  'Beide xml csv:   {:<35} {}'.format(x, core_name_to_desc[x])
        for x in core_xml_names:
            if x not in core_csv_names:
                print 'Nur in xml: {:<35} {}'.format(x, core_name_to_desc[x])
        for x in core_csv_names:
            if x not in core_xml_names:
                print 'Nur in csv:', x
    def read_namenszuordnung(self, filename):
        """
        dict: ebkus_name --> dict: {wireb,idev,core} --> (name, position,...)
        """
        content = open(file_namen_zuordnen_fertig).read()
        gruppen = [x.strip() for x in content.split('\n\n')]
        listen = [x.split('\n') for x in gruppen]
        listen = [[y.strip() for y in x] for x in listen]
        def liste_str_to_liste_tuple(liste_str):
            assert liste_str[1:3] == ': ', liste_str
            ind = liste_str[0]
            assert ind in ('e', 'w', 'c', 'i')
            rest = liste_str[3:]
            position = None
            if ', ' in rest:
                #print rest
                rest_list = rest.split(', ')
                name = ''.join(rest_list[:-1])
                if rest_list[-1].isdigit():
                    position = rest_list[-1]
            else:
                name = rest
            if position in (None, 'None'):
                position = None
            else:
                position = int(position)
            liste_tuple = (ind, name, position)
            return liste_tuple
        liste_von_tupeln = [[liste_str_to_liste_tuple(x) for x in y] for y in listen]
        ebkus_name_dict = {}
        for gruppe in liste_von_tupeln:
            idev = []
            core = []
            wireb = []
            for t in gruppe:
                ind, name, pos = t
                if ind == 'e':
                    key = name
                elif ind == 'c':
                    core.append((name, pos))
                elif ind == 'w':
                    wireb.append((name, pos))
                elif ind == 'i':
                    idev.append((name, pos))
            # if not idev or not core or not wireb:
            #     print gruppe
            ebkus_name_dict[key] = dict(idev=idev, core=core, wireb=wireb)
        return ebkus_name_dict
    def create_datei_fuer_namenszuordnung(self):
        """
        Als Basis nehme ich core_csv-Namen. Die berschneiden sich mit den wireb_name 
        (ca. 33) gemeinsam.
        Die wireb und die ebkus Namen knnen da gleich korrekt zugeordnet werden. 

        Danach mssen alle verbliebenen Namen unverbunden in die Datei geschrieben werden,
        damit sie spter manuell per cutnpaste zugeordent werden knnen.

        Das sind alle wireb-eintrge, die nicht in common sind.
        Das sind alle core-eintrge, die nicht in commen sind.

        Das sind berhaupt smtliche idev-eintrge, die spter alle manuell zugeordnet werden mssen.
        Eeine mgliche teilautomatisierung lohnt sich nicht.

        Datei fr die manuelle Zuordnung:
        latin1, # als Kommentarzeichen, e: datenbank-name, i: idev-name, c: core-name, w: wireb-name

        Segmente in der Datei:
        # common: w: e: c: 

        # wireb rest: w: e:

        # core-rest: c:

        # idev-alle: i:
        """
        content = ['# common', '']
        for x in self.ds_list:
            name, wireb_name, wireb_nr = x['name'], x['wireb_name'], x['wireb_nr']
            if wireb_name in self.core_namen_pos:
                core_pos = self.core_namen_pos[wireb_name]
                content.append('w: {wireb_name}, {wireb_nr}'.format(**locals()))
                content.append('e: {name}'.format(**locals()))
                content.append('c: {wireb_name}, {core_pos}'.format(**locals()))
                content.append('')
        content.append('# wireb rest')
        content.append('')
        for x in self.ds_list:
            name, wireb_name, wireb_nr = x['name'], x['wireb_name'], x['wireb_nr']
            if wireb_name not in self.core_namen_pos:
                content.append('w: {wireb_name}, {wireb_nr}'.format(**locals()))
                content.append('e: {name}'.format(**locals()))
                content.append('')
        content.append('# core rest')
        content.append('')
        wireb_namen = set()
        for core_name, core_pos in self.core_namen_pos.items():
            if core_name not in self.wireb_namen_pos:
                content.append('c: {core_name}, {core_pos}'.format(**locals()))
                content.append('')
        content.append('# idev alle')
        content.append('')
        for idev_name, idev_pos in self.idev_namen_pos.items():
            content.append('i: {idev_name}, {idev_pos}'.format(**locals()))
            content.append('')
        content.append('')
        with open(file_namen_zuordnen_generiert, 'wb') as f:
            f.write('\n'.join(content))
        
        
    def generate_html_driver(self):
        """
        Genau die dict erzeugen, mit deren Hilfe ebkus.app.jghdef.JGHBundesstatistik 
        die jgh07-HTML-Seite erzeugen kann, die man in jghstatistik in der Methode _formular(..)
        einsetzen kann.

        Mglichst schematisch aus info_fb, info_db, info_ds, aber wenn extra Informationen ntig
        sind, genau kennzeichnen.

        Vermutlich braucht man die fr den Kopfabschnitt und evt. fr andere Texte im JGH-Formular.

        Dies ist sicher nur ein Zwischenschritt auf dem Weg zur kompletten deklarativen Bundesstatistik,
        die unabhngig von EBKuS wartbar sein soll.
        """
        # Im ersten Anlauf lasse ich den Kopf weg.
        # Struktur: (('A', ('ort','trager', ...)), ('B', ('asd','asd')),...
        teile = []
        bundesstatistik = dict(art='Bundesstatistik', teile=teile)
        teile.append(self.gen_versteckt())
        teile.append(self.gen_falldaten())
        for abschnitt, keys in self.jgh_structure():
            if abschnitt == 'K':
                # TODO
                continue
            teile.append(self.gen_abschnitt(abschnitt, keys))
        return bundesstatistik

    def check_driver_dicts_v2(self, dict_, parent=None):
        items = dict_['statistik_items']
        elemente = dict_['fragebogen_elemente']
        self.check_items_v2(items, dict_)
        # self.check_elemente_v2(elemente, dict_)
    def check_elemente_v2(self, elemente, dict_):
        item_dict = dict((x['name'], x) for x in dict_['statistik_items'])
        for x in elemente:
            art = x.get('art')
            assert (art and ((x.get('gliederung') and x.get('titel'))
                             or art == 'ZwischenAbschnittText'))
            # for y in x.get('teile', []):
            #     print '  ', y['art']
    def check_items_v2(self, items, dict_):
        # items
        # datensatz, idev_csv, core_csv Abdeckung
        # datensatz
        positionen = [x['datensatz_position'] for x in items if 'datensatz_position' in x]
        positionen.sort()
        # print positionen
        assert positionen[0][0] == 1
        assert positionen[-1][1] in (dict_['datensatz_laenge'],239)
        for i, (begin, end) in enumerate(positionen[1:], start=1):
            # print positionen[i]
            assert begin == positionen[i-1][1] + 1
        for x in items:
            ebkus_name = x['ebkus_name']
            if 'datensatz_position' in x:
                von, bis = x['datensatz_position']
                feldlaenge = x.get('feldlaenge')
                if feldlaenge is None:
                    print 'Keine Feldlaenge: {}'.format(ebkus_name)
                assert feldlaenge == (bis - von + 1), "Feldlaenge: {} von: {} bis: {}". format(feldlaenge, von, bis)
        # idev
        positionen_raw = [x['idev_csv_position'] for x in items if 'idev_csv_position' in x]
        positionen = []
        for x in positionen_raw:
            if isinstance(x, int):
                positionen.append(x)
            else:
                positionen += x
        positionen.sort()
        for i, x in enumerate(positionen, start=1):
            assert i == x
        assert len(positionen) == dict_['idev_csv_positionen']
        # print 'idev:', positionen
        # core
        core_items = [(x['ebkus_name'],x['core_csv_position']) for x in items if 'core_csv_position' in x]
        # print core_items
        positionen_raw = [x['core_csv_position'] for x in items if 'core_csv_position' in x]
        positionen_raw = [x for x in positionen_raw if x is not None]
        # print positionen_raw
        positionen = []
        for x in positionen_raw:
            if isinstance(x, int):
                positionen.append(x)
            else:
                positionen += x
        positionen.sort()
        for i, x in enumerate(positionen, start=1):
            assert i == x
        assert len(positionen) == dict_['core_csv_positionen']
        # print 'core:', positionen
        # typ, name, ebkus_name haben alle
        for x in items:
            for y in ('typ', 'name', 'ebkus_name'):
                if y not in x:
                    print "Kein {} in {}".format(y, x)
        
    def check_driver_dicts(self, dict_, parent=None):
        """
        Hier alles prfen, was fr die Vollstndigkeit und Konsistenz 
        ntig ist.
        """
        if parent is None:
            msg = 'Root'
        else:
            abschnitt = parent.get('abschnitt')
            item = parent.get('name')
            if abschnitt:
                msg = "Abschnitt: {}".format(abschnitt)
            elif item:
                msg = "Item: {}".format(item)
            else:
                msg = "Weder Item noch Abschnitt: {}".format(parent)
            
        # art muss immer dabei sein
        art = dict_.get('art')
        name = dict_.get('name')
        layout = dict_.get('layout')
        # print art
        if art is None:
            if name:
                print '--->Item:', name
            print 'Kein art: Objekt ist Teil von {}'.format(msg)
        # alle in layout genannten Namen mssen als Items mit dem Namen
        # vorhanden sein.
        if layout:
            itemnamen = set(x.get('name') for x in dict_.get('teile', []))
            for row in layout:
                for item in row:
                    if item != 'dummy':
                        assert item in itemnamen, "{} nicht in {}".format(item, itemnamen)
        for x in dict_.get('teile', []):
            self.check_driver_dicts(x, parent=dict_)

    def add_ds_infos(self, key, item):
        ds_item = self.ds_dict[key]
        langname = ds_item['wireb_name']
        datensatz_position_von = ds_item.get('von')
        datensatz_position_bis = ds_item.get('bis')
        if datensatz_position_von is not None:
            assert datensatz_position_bis is not None
            item.update(datensatz_position_von=datensatz_position_von,
                        datensatz_position_bis=datensatz_position_bis)
        konstanter_wert = ds_item.get('value')
        if konstanter_wert is not None:
            assert len(konstanter_wert) == datensatz_position_bis - datensatz_position_von + 1
            item.update(konstanter_wert=konstanter_wert)
        substitution = ds_item.get('substitution')
        if substitution is not None:
            item.update(substitution=substitution)
            
        
    def gen_falldaten(self):
        teile = []
        abschnitt = dict(art='Abschnitt',
                         abschnitt='Falldaten',
                         titel='Falldaten',
                         layout=[['jahr', 'dummy'], ['fall_fn', 'land'], ['beginn', 'kr'], ['mit_id__name', 'einrnr'],
                                 ['gfall', 'lnr'], ['wohnt_ausserhalb']],
                         teile=teile)
        for key in self.falldaten_keys:
            teile.append(self.gen_item_falldaten(key))
                     
        return abschnitt
    def gen_item_falldaten(self, key):
        if key == 'beginn':
            return dict(name='beginn',
                        art='ItemText',
                        typ='str',
                        readonly=True,
                        teile=[dict(text='Beginn',
                                    art='Texteingabe')])
    
        if key == 'wohnt_ausserhalb':
            return dict(name='wohnt_ausserhalb',
                        art='ItemText',
                        typ='str',
                        readonly=True,
                        teile=[dict(text='Wohnt auerhalb',
                                    art='Texteingabe')])
        if key == 'mit_id__name':
            return dict(name='mit_id__name',
                        art='ItemText',
                        typ='str',
                        readonly=True,
                        teile=[dict(text='Mitarbeiter',
                                    art='Texteingabe')])
        teile = []
        ds_item = self.ds_dict.get(key)
        db_item = self.db_dict.get(key)
        item = dict(name=key, teile=teile)
        if ds_item:
            self.add_ds_infos(key, item)
        if db_item:
            verwtyp = db_item.get('verwtyp__code')
            typ = db_item.get('typ')
            kat_code = db_item.get('kat_code')
            if kat_code:
                item.update(kat_code=kat_code, typ='kat',
                            art='ItemEinfachauswahlSelect')
                optionen = []
                teile.append(dict(art='Selecteingabe',
                                  text=db_item['langname'],
                                  optionen=optionen))
                self.gen_optionen(kat_code, optionen)
            elif verwtyp == 'p' and typ == 'INT':
                item.update(typ='int',
                            art='ItemZahl')
                teile.append(dict(art='Zahleingabe',
                                  text=db_item['langname']))
            elif verwtyp == 'p' and typ.startswith('VARCHAR'):
                item.update(typ='str',
                            art='ItemText')
                teile.append(dict(art='Texteingabe',
                                  text=db_item['langname']))
                item.update(typ='Kein bekannter Typ',
                            art='ItemText')
        if key in ('fall_fn', 'lnr'):
            item.update(readonly=True)
        if key in ('fall_fn',):
            item.update(typ='str', art='ItemText')
        return item
    def gen_versteckt(self):
        teile = []
        abschnitt = dict(art='Abschnitt',
                         abschnitt='Versteckt',
                         teile=teile)
        for key in self.versteckt_keys:
            assert key not in self.db_dict
            teile.append(self.gen_item_versteckt(key))
        return abschnitt
    def gen_item_versteckt(self, key):
        ds_item = self.ds_dict[key]
        item = dict(name=key, typ='str', teile=[], art='ItemText')
        self.add_ds_infos(key, item)
        if key=='datensatzende':
            item.update(konstanter_wert=r'\r\n')
        return item
    def gen_abschnitt(self, abschnitt_name, keys):
        teile = []
        abschnitt = dict(art='Abschnitt',
                         abschnitt=abschnitt_name,
                         titel=self.fb_dict[keys[0]]['title'],
                         teile=teile)
        for key in keys:
            teile.append(self.gen_item(key))
        return abschnitt
    def gen_item(self, key):
        # ItemZahl=JGHItemZahl,
        # ItemText=JGHItemText,
        # ItemCheckbox=JGHItemCheckbox,
        # ItemEinfachauswahl=JGHItemEinfachauswahl,
        # ItemEinfachauswahlMehrfach=JGHItemEinfachauswahlMehrfach,
        fb_item = self.fb_dict[key]
        ds_item = self.ds_dict[key]
        typ = fb_item['typ']
        teile = []
        item = dict(name=key,
                    teile=teile)
        self.add_ds_infos(key, item)
        assert typ in ('kat', 'int', 'checkbox')
        if typ == 'kat':
            kat_code = fb_item['kat_code']
            item.update(art='ItemEinfachauswahlSelect',
                        kat_code=kat_code,
                        teile=teile)
            default = fb_item.get('default')
            no_empty_option = fb_item.get('no_empty_option')
            if default:
                item.update(default=default)
            if no_empty_option:
                item.update(no_empty_option=no_empty_option)
            optionen = []
            teile.append(dict(art='Selecteingabe',
                              text=fb_item['frage'],
                              optionen=optionen))
            self.gen_optionen(kat_code, optionen)
            return item
        if typ == 'int':
            item.update(art='ItemZahl',
                        minimum=fb_item['minimum'],
                        maximum=fb_item['maximum'])
            teile.append(dict(art='Zahleingabe',
                              text=fb_item['frage']))
            return item
        if typ == 'checkbox':
            item.update(art='ItemCheckbox',
                        checkbox_wert=fb_item['checkbox_value'])
            teile.append(dict(art='Checkboxeingabe',
                              text=fb_item['frage']))
            return item
    def gen_optionen(self, kat_code, teile):
        codes = get_codes(kat_code)
        for code in codes:
            teile.append(dict(art='Option',
                              text=code['name'],
                              code=code['code']))
    def jgh_structure(self):
        struct_dict = {}
        for i in self.fb_list:
            abschnitt = i.get('abschnitt')
            if abschnitt:
                #print abschnitt
                items = struct_dict.setdefault(abschnitt, [])
                items.append(i)
        #print struct_dict
        struct_list = []
        for abschnitt, items in struct_dict.items():
            items.sort(key=lambda x: x['seq'])
            struct_list.append((abschnitt, [x['name'] for x in items]))
            struct_list.sort(key=lambda x: x[0])
        return struct_list
        
def info_fb():
    items = []
    # keys = set()
    for a in _fb_data:
        if isinstance(a, basestring):
            continue
        abschnitt = a['abschnitt']
        if len(abschnitt) != 1:
            continue
        title = a['title']
        #print abschnitt, title
        if abschnitt == 'K':
            # print a['title']
            gr1 = dict(name='gr1', kat_code='gruende',
                       abschnitt=abschnitt, title=title,
                       typ='kat', gruende_bsp=a['gruende_bsp'],
                       seq=1)
            gr2 = gr1.copy()
            gr2.update(name='gr2', seq=2)
            gr3 = gr1.copy()
            gr3.update(name='gr3', seq=3)
            items += [gr1, gr2, gr3]
            continue
        seq = 1
        for i in a['items_data']:
            if not isinstance(i, basestring):
                assert isinstance(i, dict)
                typ_tuple = i.get('typ')
                try:
                    typ_ = typ_tuple[0]
                except:
                    typ_ = 'Kein Typ'
                i.update(abschnitt=abschnitt, title=title, typ=typ_, seq=seq)
                if typ_ == 'kat':
                    i.update(kat_code=typ_tuple[1])
                elif typ_ == 'int':
                    i.update(minimum=typ_tuple[1])
                    i.update(maximum=typ_tuple[2])
                elif typ_ == 'checkbox':
                    i.update(checkbox_value=typ_tuple[1])
                #items.append((i['name'], i['typ'][0], i['typ'][1], abschnitt))
                items.append(i)
                seq += 1
            else:
                # print 'STRING', i
                pass
    # for x in items:
    #     keys.update(x.keys())
    items.sort(key=lambda x: x.get('abschnitt', 'Kein Name'))
    # for x in items:
    #     print x
    # return dict((t[0], t) for t in items)
    # print keys
    return items

def info_ds():
    """
    """
    items = []
    keys = ('wireb_name', 'wireb_nr', 'von', 'bis', 'field', 'format', 'substitution', 'value',)
    for value_tuple in JGHDatensatz_2022.datensatz_info:
        value_dict = dict(zip(keys, value_tuple))
        items.append(value_dict)
    for vd in items:
        # name, typ eintragen
        field = vd.get('field')
        if field:
            if '__code' in field:
                vd.update(name=field.split('__')[0], typ='kat')
            else:
                vd.update(name=field)
        else:
            vd.update(name=vd['wireb_name'].lower())
    items.sort(key=lambda x: x['name'])
    return items

def info_db():
    keys = ['name', 'kat_code', 'verwtyp__code', 'feld', 'typ']
    verw2typ = dict()
    items = []
    tab_id = TabelleList(where="tabelle='jghstat07'")
    fl = FeldList(where='tab_id=24')
    for f in fl:
        value_dict = dict((key, f[key]) for key in keys)
        value_dict.update(langname=value_dict['name'])
        value_dict.update(name=value_dict['feld'])
        del value_dict['feld']
        if False:
            print keys
            value_list = [value_dict[key] for key in keys]
            print value_list
        items.append(value_dict)
    
    items.sort(key=lambda x: x['name'])
    return items

def auswerten_ueberschneidungen():
    # info_fb
    # muessen nicht alle in jeder value_dict praesent sein
    info_fb_keys = ('gruende_bsp', 'name', 'title', 'kat_code', 'extra', 'frage', 'maximum', 'default', 'minimum', 'abschnitt', 'no_empty_option', 'typ')
    info_ds_keys = ('name', 'typ', 'wireb_name', 'wireb_nr', 'von', 'bis', 'field', 'format', 'substitution', 'value',)
    info_db_keys = ('name', 'langname', 'kat_code', 'verwtyp__code', 'typ')
    fb = info_fb()
    ds = info_ds()
    db = info_db()
    # def check_keys(items, keys):
    #     global count
    #     for i in items:
    #         for k in i.keys():
    #             assert k in keys
    # check_keys(fb, info_fb_keys)
    # check_keys(ds, info_ds_keys)
    # check_keys(db, info_db_keys)
    names_fb = set(x['name'] for x in fb)
    names_ds = set(x['name'] for x in ds)
    names_db = set(x['name'] for x in db)
    common = names_fb & names_ds & names_db
    assert names_fb.issubset(names_ds)
    assert names_fb.issubset(names_db)
    print common
    print len(common)
    print '************fb nicht common'
    for n in names_fb:
        if n not in common:
            print n
    print '************db nicht common'
    for n in names_db:
        if n not in common:
            print n
    print '************ds nicht common'
    for n in names_ds:
        if n not in common:
            print n
    print '************ in ds und db (nicht common)'
    for n in names_ds:
        if n in names_db and n not in common:
            print n
    print '************ nur in ds'
    for n in names_ds:
        if n not in names_db and n not in names_fb:
            print n
    print '************ nur in db'
    for n in names_db:
        if n not in names_ds and n not in names_fb:
            print n
    
def xauswerten():
    okset = set()
    fb = info_fb_data()
    db = info_db()
    ds = info_datensatz()
    for x in ds.keys():
        if x in fb and x in db:
            okset.add(x)
    def only_in_one(in_, out1, out2):
        res = set()
        for x in in_:
            if x not in out1 and x not in out2:
                res.add(x)
        return res
            
    print 'Nur in datensatz:'
    for x in only_in_one(ds, db, fb):
        print '  ',x
    print 'Nur in datenbank:'
    for x in only_in_one(db, ds, fb):
        print '  ',x
    print 'Nur in fb_data:'
    for x in only_in_one(fb, ds, db):
        print '  ',x


    def only_in_two(in1, in2, out):
        res = set()
        for x in in1:
            if x in in2 and x not in out:
                res.add(x)
        return res
        
    print 'Nur in datensatz und datenbank:'
    for x in only_in_two(ds, db, fb):
        print '  ',x
    print 'Nur in datensatz und fb_data:'
    for x in only_in_two(ds, fb, db):
        print '  ',x
    print 'Nur in datenbank und fb_data:'
    for x in only_in_two(db, fb, ds):
        print '  ',x

def tabelle():
    fb = info_fb_data()
    db = info_db()
    ds = info_datensatz()
    allset = set()
    allset.update(fb.keys())
    allset.update(ds.keys())
    allset.update(db.keys())
    #print allset
    alllist = list(allset)
    alllist.sort()
    def pr(x):
        inds = 'ds' if x in ds else '-'
        infb = 'fb' if x in fb else '-'
        indb = 'db' if x in db else '-'
        von = ds[x][3] if x in ds else ''
        bis = ds[x][4] if x in ds else ''
        wireb_name = ds[x][0] if x in ds else x
        return wireb_name, x, inds, indb, infb, von, bis
    alltuples = [pr(x) for x in alllist]
    alltuples.sort(key=lambda x: x[1:])
    for t in alltuples:
        print "{:>35} {:>35} {:>4} {:>4} {:>4} {:>4} {:>4}".format(*t)
    

jg = JGHDefGen()
# jg.show()
# jg.write_bundesstatistik()
# jg.write_bundesstatistik(fname='tt.py')
# jg.get_core_name_description_list()
# jg.compare_core_names_with_wireb_names()
# print jg.core_dict_raw
# print jg.idev_namen_pos
# print jg.core_namen_pos
# jg.compare_core_xml_names_with_core_csv_names()
# jg.create_datei_fuer_namenszuordnung()
# for x in jg.namenszuordnung:
#    print x
# print jg.namenszuordnung
# for x, v in jg.ds_dict.items(): 
#     field, value = v['field'], v['value']
#     assert (field is None and value) or (field is not None)
#     print repr(field), repr(value)
# kein c
# def keinc(tuple):
#     c_is = any(x[0]=='c' for x in tuple)
#     return not c_is
# def both_i_and_c(tuple):
#     c_is = any(x[0]=='c' for x in tuple)
#     i_is = any(x[0]=='i' for x in tuple)
#     return (c_is and i_is)
    
# for x,v in jg.ebkus_namenszuordnung.items():
#     print x, '--->', v
#     print

#d = dict(a=2, b='c', teile=[dict(a=4,b='d',optionen=[dict(a=5,b='e',teile=[dict(a=89)])])])
#print pp_dict(d)

def v2():
    res = jg.generate_v2()
def v2pp():
    # res = jg.generate_v2()
    print pp_dict(jg.bundesstatistik_v2)
def v2wr():
    jg.write_bundesstatistik_v2()


def v1pp():
    print pp_dict(jg.bundesstatistik_v1)
    
def test():
    print 'test'

def cmpp():
    palt = """fallid=8&fall_fn=5-2019A&stz=259&jghid=8&file=updjgh07&mitid=2&jahr=2019&land=440&kr=551&einrnr=553&gfall=421&bgm=4&bgy=2019&zustw=1&gef8a=460&inobh42_22=559&hilf_art=465&hilf_ort=468&traeger=491&gs=7&gem=9&gey=2012&aort_vor=482&sit_fam=456&ausl_her=464&vor_dt=463&wirt_sit=463&aip=506&ees=461&va52=461&rgu=461&hda=461&nbkakt=+&gr1=513&gr2=515&gr3=516&em=11&ey=2019&nbkges=37&lbk6m=461&grende=521&aort_nac=485&unh=531"""
    pneu = """fallid=8&fall_fn=5-2019A&stz=259&jghid=8&file=updjgh07&mitid=2&jahr=2019&fall_fn=5-2019A&land=440&beginn=4.2019&kr=551&mit_id__name=test_vn2+test_na2&einrnr=553&gfall=421&lnr=8&wohnt_ausserhalb=++&bgm=4&bgy=2019&zustw=1&gef8a=460&inobh42_22=559&hilf_art=465&hilf_ort=468&traeger=491&gs=7&gem=9&gey=2012&aort_vor=482&sit_fam=456&ausl_her=464&vor_dt=463&wirt_sit=463&aip=506&ees=461&va52=461&rgu=461&hda=461&nbkakt=&gr1=513&gr2=515&gr3=516&em=11&ey=2019&nbkges=37&lbk6m=461&grende=521&aort_nac=485&unh=531"""
    def todict(s):
        l = s.split('&')
        pairs = (x.split('=') for x in l)
        # for x in pairs:
        #     print x
        print 'PAIRS:', pairs
        d = dict((k,v) for k,v in pairs)
        return d
    dalt = todict(palt)
    dneu = todict(pneu)
    print 'ALT:', dalt
    print
    print 'NEU:', dneu

    print 'DIFF:'
    for k in dalt:
        if k in dneu:
            if dalt[k] != dneu[k]:
                print k, dalt[k], dneu[k]
        else:
            print k, 'ist nicht in dneu'
    print 'NICHT in dalt:'
    for k in dneu:
        if k not in dalt:
            print k, 'ist nicht in dalt'
    
if __name__ == '__main__':
    if len(sys.argv) > 1:
        locals()[sys.argv[1]]()

