#!/usr/bin/python2
# coding: utf8

"""Bringt die EBKuS Dokumentation auf den neuesten Stand.

Setzt voraus, dass das Paket ``docutils`` installiert ist,
(rst2html, rst2latex).
Quelle: http://docutils.sourceforge.net/

Dieses Paket definiert eine Syntax für Textdateien, die dann
automatische nach HTML und Latex/PDF übersetzt werden können.

Quellen:
  make.py (dieses Skript)
  docutils.conf                  -- 
  manual.css
  manual.sty
  manual.txt
  einleitung.txt
  anwendungsbeschreibung.txt
  bedienungsanleitung.txt
  administration.txt
  installation.txt

Targets:
  EBKuS_Handbuch.html
  manual.tex --> EBKuS_Handbuch.pdf


In erster Näherung:

Wenn es eine Quelle gibt, die neuer als eine Target ist, wird alles
neu gemacht.

Kann man später vielleicht verfeinern.
"""

import os, sys
from os.path import join, split, dirname, exists, isdir, isfile, normpath, normcase, abspath
from urllib import urlretrieve
assert sys.version_info >= (2,3), "Es wird Python 2.3 oder höher benötigt"

#sources = ['manual.txt', 'docutils.conf', 'manual.css', 'manual.sty', 'make.py']
kapitel = ['einleitung.txt', 'anwendungsbeschreibung.txt', 'bedienungsanleitung.txt',
           'administration.txt', 'installation.txt']
latex_sources = ['manual.txt', 'docutils.conf', 'manual.sty', 'make.py'] + kapitel
html_sources = ['manual.txt', 'docutils.conf', 'manual.css', 'make.py'] + kapitel

# Navigationstipp und Kapitelnumerierung nur bei HTML
html_spezifisch = {'NAVIG': """
[Navigationstipp: Inhaltsverzeichnis und Kapitelüberschriften
sind wechselseitig verlinkt. Ein Klick auf eine Kapitelüberschrift führt
zurück zum Inhaltsverzeichnis!] 
""",

                   'SECNUM': ".. sectnum::"
                   }
latex_spezifisch = {'NAVIG': '', 'SECNUM': ''
                   }

def pre_process(filein, fileout, dict):
    """In filename Ersetzungen gemäß dict vornehmen"""
    f = open(filein, "rU")  # U ist Universal Newline
    t = f.read()
    f.close()
    t = t % dict
    open(fileout, 'w').write(t)
    
def mtime(path):
    try:
        return os.path.getmtime(path)
    except:
        # gibs nich, gaanz alt
        return 0
        
def is_newer(file1, file2):
    """Ist file1 nach file2 modifiziert worden? (mtime)"""
    if not exists(file1):
        return False
    if not exists(file2):
        return True
    return mtime(file1) > mtime(file2)

def newest(*paths):
    return max([mtime(p) for p in paths])

def post_process_latex(filename):
    """Figure Optionen ändern"""
    f = open(filename, "rU")  # U ist Universal Newline
    t = f.read()
    f.close()
    t = t.replace(r"\begin{figure}[htbp]\begin{center}",
                  r"\begin{figure}[H]\begin{center}")
    open(filename, 'w').write(t)

def run_cmd(cmd):
    print cmd
    res = os.system(cmd)
    if res != 0:
        msg = '  Fehlgeschlagen: %s' % cmd
        #print msg
        raise Exception(msg)

def usage():
    msg ="""\
make.py -r --readme [-f --force] [-h --help]

   Wenn -r oder --readme angegeben wird, wird bei Bedarf
   NEU_IN_DIESER_VERSION, VERSIONS_GESCHICHTE, ebkusvm_readme,
   ebkusvm_readme2 und Bundesstatistikdefinition
   neu generiert. Die übrige Dokumentation bleibt unangetastet.

   Wenn -f oder --force angegeben wird, wird immer neu generiert.

   Setzt voraus, dass das Pythonpaket ``docutils`` installiert ist
   (rst2html5.py).
"""
    print msg
    
#    Quelle: http://docutils.sourceforge.net/
# make.py [-h, --help, -r, --readme] [html|latex|pdf|clean]

#    WICHTIG: Zur Zeit funktioniert nur 
#    EBKuS-Dokumentation im HTML-Format oder als PDF erstellen.
#    oder mit Option -r, --readme: nur HTML-Versionen der READMEs
#    im Distributionsverzeichnis erstellen, nicht das Handbuch)
   
#    Ohne Argumente wird bei Bedarf die gesamte Dokumentation neu
#    erstellt. (Bedarf besteht, wenn in einer der Quelldateien sich
#    etwas geändert hat.)

#    Wenn html, latex oder pdf angegeben wird, wird nur der betreffende
#    Teil bei Bedarf generiert.


#    Setzt ferner voraus, dass das Kommando pdflatex installiert ist.
#    In Suse 10.0 erreicht man das, indem man die Pakete
#    tetex und te_latex installiert.
   
#    Falls ``docutils`` nicht installiert ist, wird das Handbuch im HTML- und im
#    PDF-Format von http://ebkus.berlios.de/ebkus-3.3/ heruntergeladen.
# """

def latex_uptodate():
    if (isfile('manual.tex') and
        newest(*latex_sources) <= mtime('manual.tex')):
        return True
    return False

def pdf_uptodate():
    if (isfile('EBKuS_Handbuch.pdf') and
        newest(*latex_sources) <= mtime('EBKuS_Handbuch.pdf')):
        return True
    return False
    
def html_uptodate():
    if (isfile('EBKuS_Handbuch.html') and
        newest(*html_sources) <= mtime('EBKuS_Handbuch.html')):
        return True
    return False

def make_latex():
    if not latex_uptodate():
        pre_process('manual.txt', 'manual_for_latex.txt', latex_spezifisch)
        run_cmd('rst2latex manual_for_latex.txt manual.tex')
        print "postprocessing 'manual.tex'"
        post_process_latex('manual.tex')
    

def make_readme(force=False):
    "HTML-Versionen der Dokumentation im Distributionsverzeichnis erstellen."
    docs = ('NEU_IN_DIESER_VERSION', 'VERSIONS_GESCHICHTE', 'ebkusvm_readme', 'ebkusvm_readme2',
            'Bundesstatistikdefinition', )
    needs_update = set([doc for doc in docs if is_newer(doc + '.txt', doc + '.html')])
    if 'NEU_IN_DIESER_VERSION' in needs_update:
        needs_update.add('VERSIONS_GESCHICHTE')
    for doc in docs:
        if force or doc in needs_update:
            print doc + '.txt', '-->', doc + '.html'
            os.system("rst2html5.py --input-encoding=utf8 --output-encoding=utf8 "
                      "--stylesheet-path=myminimalplain.css "
                      "--no-source-link "
                      " {}.txt {}.html".format(doc, doc))
        else:
            print "Uptodate: " + doc + '.html'


if __name__ == '__main__':
    os.chdir(dirname(abspath(sys.argv[0])))
    # pdf = html = latex = clean = False
    force = False
    args = sys.argv
    if '-h' in args or '--help' in args:
        usage()
        sys.exit(0)
    if len(args) > 1:
        # if 'pdf' in args:
        #     pdf = True
        # if 'html' in args:
        #     html = True
        # if 'latex' in args:
        #     latex = True
        # if 'clean' in args:
        #     clean = True
        if '-f' in args or '--force' in args:
            force = True
        if '-r' in args or '--readme' in args:
            make_readme(force=force)
            sys.exit(0)
        else:
            usage()
            sys.exit(1)
        # if not(pdf or html or latex or clean):
        #     usage()
        #     sys.exit(1)
    else:
        usage()
        sys.exit(1)
        # pdf = html = latex = True
        # if pdf_uptodate():
        #     # im default-Fall kümmern wir uns um Latex nicht, wenn pdf ok ist.
        #     latex = False

    # if clean:
    #     pats = ('*~', '*.pyc',
    #             'manual_for_html.txt',
    #             'manual_for_latex.txt',
    #             '*toc', '*log', '*out', '*tex', '*aux')
    #     from glob import glob
    #     for p in pats:
    #         m = glob(p)
    #         for f in m:
    #             os.remove(f)
    #     sys.exit(0)
    
    # if os.system('rst2html --version') != 0:
    #     # keine docutils, downloaden
    #     print "docutils zum Generieren des Handbuchs sind nicht installiert"
    #     url_dir = 'http://ebkus.berlios.de/ebkus-3.3/'
    #     target = 'EBKuS_Handbuch.html'
    #     if html:
    #         if isfile(target):
    #             print "  %s bereits vorhanden" % target
    #         else:
    #             url = "%s/%s" % (url_dir, target)
    #             print '  herunterladen: %s  ... ' % url
    #             urlretrieve(url, target)
    #             print '    erfolgreich heruntergeladen'
    #     target = 'EBKuS_Handbuch.pdf'
    #     if pdf:
    #         if isfile(target):
    #             print "  %s bereits vorhanden" % target
    #         else:
    #             url = "%s/%s" % (url_dir, target)
    #             print '  herunterladen: %s  ... ' % url
    #             urlretrieve(url, target)
    #             print '    erfolgreich heruntergeladen'
    #     sys.exit(0)

    # try:
    #     if html:
    #         if html_uptodate():
    #             print "html uptodate"
    #         else:
    #             pre_process('manual.txt', 'manual_for_html.txt', html_spezifisch)
    #             run_cmd('rst2html manual_for_html.txt EBKuS_Handbuch.html')
    #     if latex:
    #         if latex_uptodate():
    #             print 'latex uptodate'
    #         else:
    #             make_latex()
    #     if pdf:
    #         if pdf_uptodate():
    #             print 'pdf uptodate'
    #         else:
    #             make_latex()
    #             run_cmd('pdflatex manual.tex')
    #             # zweimal wg. Inhaltsverzeichnis
    #             run_cmd('pdflatex manual.tex')
    #             os.rename('manual.pdf', 'EBKuS_Handbuch.pdf')
    # except Exception, e:
    #     print e
    #     sys.exit(1)
    # sys.exit(0)

