# coding: latin-1
"""Anwendungsschnittstelle fr die EB Klientenverwaltung.

Die eigentlichen Objektklassen liegen im automatisch generierten Modul
ebapigen.py und werden hier nur importiert. Weiter manuelle
Ergnzungen, Subklassendefinitionen, Methodendefinitionen, etc. knnen
in diesem Modul abgelegt werden.

"""

import sys, string, math, os, re
import datetime
import urllib
from ebkus.app.ebapigen import *
from ebkus.db import dbapp, sql
from ebkus.config import config
import traceback

####################
# utility Funktionen
####################

def nfc(code_nr):
    try:
        name=Code(code_nr)['name']
    except:
        return ''
        pass
    return name
    
def is_binary(arg):
    code = None
    if isinstance(arg, (Dokument, Gruppendokument)):
        code = arg['mtyp__code']
    elif isinstance(arg, Code):
        code = arg['code']
    elif isinstance(arg, (basestring, int, long)):
        # ist id
        code = Code(arg)['code']
    assert code
    # Nun muss es Objekt sein
    if code in ('txt', 'asc', 'html', 'htm', 'ps', 'rtf', 'rtx',):
        return False
    return True
        
def ist_gruppen_mitarbeiter(gruppen_id,mit_id):
    gruppex = Gruppe(int(gruppen_id))
    mitarbeitergruppe = gruppex['mitarbeiter']
    is_gr_mit = 0
    for mg in mitarbeitergruppe:
        if int(mit_id) == int(mg['mit_id']):
            is_gr_mit = 1
    return is_gr_mit
    
def getDBSite():
    return config.SITE
    
def cc(kat_code, code):
    """Liefert die code id fr eine kat_code/code Kombination.
    Bsp:  cc('stzei', 'B') liefert die Zahl 230"""
    try: id = Code(kat_code = kat_code, code =  code)['id']
    except:
        raise dbapp.DBAppError("Code '%s' fr Kategorie '%s' existiert nicht" % (code, kat_code))
    return id
def cn(kat_code, name):
    """Liefert die code id fr eine kat_code/codename Kombination.
    Bsp:  cn('stzei', 'EFB-Wienerstr.') liefert die Zahl 230"""
    try: id = Code(kat_code = kat_code, name =  name)['id']
    except:
        raise dbapp.DBAppError("Code '%s' fr Kategorie '%s' existiert nicht" % (name, kat_code))
    return id

def bcode(kat_code, value,
          default=None # default fr value!
          ):
    "liefert das Code-Objekt der Bereichskategorie, in deren "
    "Bereich value fllt."
    from ebkus.app.ebapih import get_codes
    if value or value==0:
        value = int(value)
    else:
        if default != None:
            value = default
        else:
            raise EE('Kein Wert fr Bereichskategorie')
    code_list = get_codes(kat_code)
    for c in code_list:
        try:
            if value >= int(c['mini']) and value <= int(c['maxi']):
                return c
        except:
            pass
    raise EE('Keine Bereich fr Wert gefunden')
    
    


def get_feld(feldname, tabelle=None, klasse=None):
    "Liefert das entsprechende Feldobjekt."
    if tabelle:
        return Feld(feld=feldname,
                    tab_id=Tabelle(tabelle=tabelle)['id'])
    elif klasse:
        return Feld(feld=feldname,
                    tab_id=Tabelle(klasse=klasse)['id'])

    
class Date(object):
    def __init__(self, year = None, month = None, day = None):
        """No error checking. Use the check method.
        2-stellige Jahresangaben werden mit 2000 addiert.
        """
        # if year is month is day is None: # assume today
        #     import time
        #     year, month, day = time.localtime(time.time())[:3]
        if year is month is day is None: # assume today
            d = today()
            year, month, day = d.year, d.month, d.day
        if month is None: month = 1
        if day is None: day = 1
        if 0 < year < 100:
            year += 2000
        self.year, self.month, self.day = year, month, day
        self.years = self.months = self.days = ''
        for a in ('month', 'day'):
            v = getattr(self, a)
            if isinstance(v, (int, long)):
                setattr(self, a+'s', "%02d" % v)
        if isinstance(self.year, (int, long)):
                self.years = "%04d" % self.year
        
    def to_py_date(self):
        """Throws an exception if its not a correct date."""
        return datetime.date(self.year, self.month, self.day)

    def is_zero(self):
        return self.year == self.month == self.day == 0

    def totuple(self):
        return self.year, self.month, self.day
        
    def quartal(self):
        """Liefert das Quartal: 1,2,3,4. 
        Falls Datum(0,0,0), 0.
        """
        if self.year == 0:
            return 0
        else:
            return (self.month + 2) / 3
    def __getitem__(self, key):
        if key == 'day': return self.day
        elif key == 'month': return self.month
        elif key == 'year': return self.year
        else:
            raise KeyError, key
            
    def diff(self, other):
        """Unterschied in Monaten, positiv und negativ:
        other - self
        Tage werden nicht bercksichtigt.
        """
        diff = other.year*12 + other.month - (self.year*12 + self.month)
        return diff

    def diff_weekdays(self, other):
        """Unterschied in Werktagen.
        Betrag ist positiv, wenn other grer ist als self.
        """
        me = datetime.date(self.year, self.month, self.day)
        other = datetime.date(other.year, other.month, other.day)
        if other < me:
            x = other
            other = me
            me = x
            negative = True
        else:
            negative = False
        daygenerator = (me + datetime.timedelta(x + 1) for x in xrange((other - me).days))
        res = sum(1 for day in daygenerator if day.weekday() < 5)
        if negative:
            res = -res
        return res
    # def add_weekdays(self, days):
    #     """Werktage addieren, so diff_weekdays erfllt ist.
    #     Neues Datum zurckgeben.
    #     """
    #     me = datetime.date(self.year, self.month, self.day)
    #     daygenerator = (me + datetime.timedelta(x + 1) for x in xrange((other - me).days))
    #     other = datetime.date(other.year, other.month, other.day)
    #     if other < me:
    #         x = other
    #         other = me
    #         me = x
    #         negative = True
    #     else:
    #         negative = False
    #     res = sum(1 for day in daygenerator if day.weekday() < 5)
    #     if negative:
    #         res = -res
    #     return res
    def add_month(self, months):
        """Liefert ein neues Datum, indem months Monate
        addiert werden. Tage werden nicht bercksichtigt,
        aber wenn ntig korrigiert, d.h. Date(2000, 7, 31).add_month(2)
        ergibte Date(2000, 9, 30).
        """
        d_years = months / 12
        d_months = months % 12
        year = self.year + d_years
        month = self.month + d_months
        day = self.day
        if month > 12:
            month -= 12
            year += 1
        elif month < 1:
            month += 12
            year -= 1
        new_date = Date(year, month, day)
        if not new_date.check():
            # liegts am zu grossen Tag fr den neuen Monat?
            if month == 2 and day > 28:
                new_date = Date(year, 2, 28)
            elif day > 30:
                new_date = Date(year, month, 30)
            #print 'ADD_MONTH', new_date
            assert new_date.check(), "Addition von Monaten misslungen"
        return new_date

    def check(self, alle_jahrgaenge_akzeptieren=False, can_be_zero=True):
        """Liefert true, falls Datum (0,0,0) ist, oder ein korrektes Datum.
        Folgende Jahreszahlen sind gltig (etwas unlogisch :-)
        alle_jahrgaenge_akzeptieren = True
          - Wird fr Geburtsdatum von Erwachsenen verwendet
        1890 <= jahr <= today().year
        alle_jahrgaenge_akzeptieren = False:
          - Wird sonst zu Plausibilittsprfung von Datumsangaben verwendet
        1970 <= jahr <= 2050
        """

        y = self.year
        m = self.month
        d = self.day
        if not can_be_zero:
            # darf nicht alles 0 sein
            if y==0 or m==0 or d==0:
                return False
        else:
            # wenn alles 0, ok
            if y == m == d == 0:
                return True
        if alle_jahrgaenge_akzeptieren:
            if y < 1890 or y > today().year:
                return False
        else:
            if y < 1970 or y > 2050:
                return False
        try:
            self.to_py_date()
            return True
        except:
            return False

    def __cmp__(self, other):
        """Works only if dates have been checked true. If a date is all zero,
        it compares greater than any other date."""
        if not isinstance(other, Date):
            try:
                other = str2date(other)
            except:
                return 1
        try:
            return cmp(self.to_py_date(), other.to_py_date())
        except:
            # mindestens ein Argument ist 0,0,0
            if self.is_zero():
                if other.is_zero():
                    return 0
                return 1
            else:
                return -1
            
    def __str__(self):
        return "%02d.%02d.%04d" % (self.day, self.month, self.year)
    __repr__ = __str__
    
def today():
    from ebkus.config import config
    date_str = register_get('today')
    if date_str:
        return str2date(date_str)
    else:
        now = datetime.datetime.now()
        return Date(now.year, now.month, now.day)

def str2date(str):
    "3.4.1999 -> Date"
    if isinstance(str, Date):
        return str
    try:
        if str:
            nums = None
            if '.' in str:
                nums = [int(i) for i in str.split('.')]
                nums.reverse()
            elif '-' in str:
                nums = [int(i) for i in str.split('-')]
        else:
            nums = [0,0,0]
        assert len(nums) == 3
    except:
        raise EE('Fehler in Datumsstring: %s' %str)
    return Date(*nums)

def calc_age(gb_datum_als_string, aktuelles_datum_als_date):
    d,m,y = [int(x) for x in gb_datum_als_string.split('.')]
    assert y > 1900
    gb = Date(y, m, d)
    diff = gb.diff(aktuelles_datum_als_date)
    return diff/12

#print today()


#########################################################
# getDate, setDate
#
# Achtung!
# Beruht auf der Konvention, da Datum immer mit Feldnamen
# angegeben ist, die wie folgt aufgebaut sind:
#   <key>y   (bgy)
#   <key>m   (bgm)
#   <key>d   (bgd)
##########################################################

def getDate(self, key):
    # Ganz doof, dass das nicht None liefert, wenn kein Datum gesetzt
    y,m,d = key + 'y', key + 'm', key + 'd'
    return Date(self[y], self[m], self.get(d))
    
def setDate(self, key, date):
    y,m,d = key + 'y', key + 'm', key + 'd'
    self[y], self[m], self[d] = date.totuple()
    
dbapp.DBObjekt.getDate = getDate
dbapp.DBObjekt.setDate = setDate


class Time(object):
    def __init__(self, hour=None, minute=None):
        "Ohne hour und minute --> leeres Zeitobjekt"
        "Falls hour einen Wert hat und minute nicht, wird minute auf 0"
        "gesetzt"
        if hour or hour == 0: # muss Ziffer sein
            if not minute:    # wenn leer --> 0
                minute = 0
            self.hour = int(hour)
            self.hours = "%02d" % self.hour
            self.minute = int(minute)
            self.minutes = "%02d" % self.minute
        else:
            self.hours = self.minutes = ''
            self.hour = self.minute = None
        self.empty = (self.hour == None)
    def __str__(self):
        if self.empty:
            return ''
        else:
            return "%s:%s" % (self.hours, self.minutes)
    def check(self):
        if self.empty or ((0 <= self.hour <= 23) and (0 <= self.minute <= 59)):
            return True
        return False
        
            
def getTime(self, key):
    h,min = key + 'h', key + 'min'
    return Time(self.get(h), self.get(min))
    
def setTime(self, key, time):
    h,min = key + 'h', key + 'min'
    self[h], self[min] = time.hour, time.minute
    
dbapp.DBObjekt.getTime = getTime
dbapp.DBObjekt.setTime = setTime


def getQuartal(monat):
    """Gibt das Quartal fr einen bestimmten Monat aus."""
    if int(monat) > 0 and int(monat) < 13:
        q = ((int(monat) - 1) / 3) + 1
        return int(q)
    else:
        raise EE("Keine Monatszahl zwischen 1 und 12")

def getNewId(self):
    """Standardmethode, um neue Werte fr Schlsselfelder zu erzeugen.
    
    Verwendet maxist-Feld von Tabelle, um eine neue id zu
    generieren.
    
    Es gengt nicht, das Maximum der Spalte zu nehmen, da mehrere Benutzer
    das gleiche tun knnen und hinterher alle versuchen, ein Objekt mit derselben
    ID einzufgen.

    Mit der Zeile
    
         DBObjekt.getNewId = getNewId
    
    wird diese Funktion als Methode der Klasse DBObjekt installiert und
    damit auf alle Unterklassen von DBObjekt, die in 'ebapigen'
    definiert werden, vererbt.
    
    Da die 'new' Methode von DBObjekt getNewId verwendet, kann z.B. eine
    neuer Mitarbeiter wie folgt erzeugt werden:
    
           m = Mitarbeiter()
           m.new()
           m['vn'] = 'Tom'
           m['na'] = 'Friedrich'
           m.insert()
    
    """
    
    if not self.primarykey:
        raise dbapp.DBAppError("Cannot getNewId without primarykey")
    maxid = sql.SQL("select max(%s) from %s" % (self.primarykey, self.table)).execute()[0][0]
    if not maxid:
        maxid = 0
    tab = Tabelle(tabelle=self.table)
    maxist = tab['maxist']
    newid = max(maxid, maxist) + 1
    tab.update({'maxist': newid})
    return newid
    
dbapp.DBObjekt.getNewId = getNewId


def getNewFallnummer(stz_code, jahr):
    """Neue Fallnummer erzeugen."""
    return _getNewNummer(stz_code, jahr, FallList, 'fn')
def getNewGruppennummer(stz_code, jahr):
    """Neue Gruppennummer erzeugen."""
    return _getNewNummer(stz_code, jahr, GruppeList, 'gn')
def _getNewNummer(stz_code, jahr, klass, feld):
    """Neue Fall- oder Gruppennummer erzeugen."""
    # Wenn nachtrglich der Fallbeginn in ein frheres Jahr verlegt wurde,
    # taucht bei dieser Abfrage die Fallnummer nicht mehr auf, da auch nach
    # dem Beginnjahr gefragt wird. Dadurch kann es zu Dubletten bei den
    # Fallnummer kommen.
    # Ab 4.12 wird die nachtrgliche Verlegung des Fallbeginns in ein frheres Jahr
    # beim Fallupdate ausgeschlossen.
    jahresfallliste = klass(
      where="bgy = %s and %s like '%%%s%s'" % (jahr, feld, jahr, stz_code))
    if jahresfallliste:
        groesste_fallnummer = max([ int(f[feld].split('-')[0]) for f in jahresfallliste])
    else:
        groesste_fallnummer = 0
    return "%s-%s%s" % (groesste_fallnummer + 1, jahr, stz_code)
    
    
    
    ##############################
    # Berechnete Felder fr Akte
    ##############################
    
    ##*************************************************************************
    ##
    ##def _wiederaufnehmbar(self, key):
    ##  letzter_fall = self['letzter_fall']
    ##  if letzter_fall and letzter_fall['zday'] != 0:
    ##    zdazeit_in_monaten = letzter_fall['zday']*12 + letzter_fall['zdam']
    ##    heute_in_monaten =   today().year*12 + today().month
    ##    wiederaufnehmbar =  (heute_in_monaten - zdazeit_in_monaten) > 0
    ##  else: wiederaufnehmbar = 0
    ##  self._cache_field('wiederaufnehmbar', wiederaufnehmbar)
    ##  return wiederaufnehmbar
    ##
    ## Bisher logisch falsch.
    ## Korrektur
    ## Datum: 22.11.2001 msg systems ag mastaleckT
    ## Schaltjahre und unterschiedliche Monatslngen mssen nicht bercksichtigt
    ## werden, da eine derartige Genauigkeit hier nicht notwendig ist.
    ##*************************************************************************
    
def _akte_name(self, key):
    return "%(vn)s %(na)s" % self

def _akte_telefon(self, key):
    tl1 = self['tl1']
    tl2 = self['tl2']
    res = ''
    if tl1:
        res += tl1
        if tl2:
            res += ',  %s' % tl2
    elif tl2:
        res = tl2
    return res

def _wiederaufnehmbar(self, key):
    """Ein Fall ist wiederaufnehmbar, wenn der Abschluss *lnger* zurckliegt,
    als die konfigurierte Wiederaufnahmefrist.
    Ansonsten (nmlich innerhalb der Wiederaufnahmefrist) wird
    der einfach aktiviert, dh 'Zda Rckgngig'
    """
    letzter_fall = self['letzter_fall']
    if letzter_fall and letzter_fall['zday'] != 0:
        zda_date = letzter_fall.getDate('zda')
        #print 'ZDA_DATE:', zda_date
        zda_rueck_bis = zda_date.add_month(config.WIEDERAUFNAHMEFRIST)
        return today() > zda_rueck_bis
    return False
    
def _erster_fall(self, key):
    faelle = self['faelle']
    if faelle:
        faelle.sort('bgy', 'bgm', 'bgd')
        erster_fall = faelle[0]
    else:
        letzter_fall = None
        #self.data['letzter_fall'] = letzter_fall # data cache
    self._cache_field('erster_fall', erster_fall)
    return erster_fall

def _letzter_fall(self, key):
    faelle = self['faelle']
    if faelle:
        faelle.sort('bgy', 'bgm', 'bgd')
        letzter_fall = faelle[-1]
    else:
        letzter_fall = None
        #self.data['letzter_fall'] = letzter_fall # data cache
    self._cache_field('letzter_fall', letzter_fall)
    return letzter_fall
    
    
def _aktueller_fall(self, key):
    letzter_fall = self['letzter_fall']
    aktueller_fall = None
    if letzter_fall:
        if letzter_fall['zday'] == 0:
            aktueller_fall = letzter_fall
            #self.data['aktueller_fall'] = aktueller_fall # data cache
    self._cache_field('aktueller_fall', aktueller_fall)
    return aktueller_fall
    
def _aktuell_akte(self, key):
    res = not self['aktueller_fall'] is None
    #self.data['aktuell'] = res # data cache
    self._cache_field('aktuell', res)
    return res
    
def _str_inner(self, key):
    res = ''
    try:
        if int(self['lage']) == Code(cc('lage', '0'))['id']:
          # in Berlin
            res = self['str']
    except:
        pass
    return res
        
def _str_ausser(self, key):
    res = ''
    try:
        if int(self['lage']) == Code(cc('lage', '1'))['id']:
          # ausserhalb Berlins
            res = self['str']
    except:
        pass
    return res

def _get_strkat_ortsangabe(self, key):
    strasse = self.get('strasse')
    if strasse == None:
        from ebkus.html.strkat import get_strasse
        strasse = self['strasse'] = get_strasse(self)
    return strasse.get(key)

def _get_mail_link(self, key):
    mail = self['mail']
    if not mail:
        return ''
    # Einrichtungskontakt hat keinen vn
    try: vn = self['vn']
    except: vn = ''
    na = self['na']
    if vn or na:
        anr = "%s %s" % (vn, na)
        href = "%s <%s>" % (anr.strip(), mail)
    else:
        href = "%s" % mail
    href = urllib.quote(href)
    #link = '<a href="mailto:%s">%s</a>' % (href, 'E-Mail')
    link = '<a href="mailto:%s">%s</a>' % (href, mail)
    return link

def _akte_gesperrt(self, key):
    if self['sperre'] == cc('ja_nein', '1'):
        return True
    return False
def _bezugsperson_gesperrt(self, key):
    if self['akte__gesperrt']:
        return True
    return False
def _vorname_oder_xxx(self, key):
    if self['gesperrt']:
        return 'xxx'
    else:
        return self['vn']
def _nachname_oder_xxx(self, key):
    if self['gesperrt']:
        return 'xxx'
    else:
        return self['na']

def _loeschfrist(self, key):
    abwlf = self['abwlf']
    if abwlf is not None:
        return abwlf
    aufbewahrungskategorie = self['aufbew__name']
    loeschfrist = getattr(
        config, 
        'LOESCHFRIST' if aufbewahrungskategorie == 'Beratung'
        else 'LOESCHFRIST_' + aufbewahrungskategorie.upper())
    return loeschfrist
    
Akte.attributemethods['name'] = _akte_name
Akte.attributemethods['telefon'] = _akte_telefon
Akte.attributemethods['wiederaufnehmbar'] = _wiederaufnehmbar
Akte.attributemethods['letzter_fall'] = _letzter_fall
Akte.attributemethods['erster_fall'] = _erster_fall
Akte.attributemethods['aktueller_fall'] = _aktueller_fall
Akte.attributemethods['aktuell'] = _aktuell_akte
Akte.attributemethods['str_inner'] = _str_inner
Akte.attributemethods['str_ausser'] = _str_ausser
Akte.attributemethods['bezirk'] = _get_strkat_ortsangabe
Akte.attributemethods['ortsteil'] = _get_strkat_ortsangabe
Akte.attributemethods['samtgemeinde'] = _get_strkat_ortsangabe
# Achtung ganz doof: Akte hat eigenes Feld planungsr
Akte.attributemethods['plraum'] = _get_strkat_ortsangabe
Akte.attributemethods['mail_link'] = _get_mail_link
Akte.attributemethods['gesperrt'] = _akte_gesperrt
Akte.attributemethods['vn_xxx'] = _vorname_oder_xxx
Akte.attributemethods['na_xxx'] = _nachname_oder_xxx
Akte.attributemethods['loeschfrist'] = _loeschfrist

Bezugsperson.attributemethods['name'] = _akte_name
Bezugsperson.attributemethods['telefon'] = _akte_telefon
Bezugsperson.attributemethods['str_inner'] = _str_inner
Bezugsperson.attributemethods['str_ausser'] = _str_ausser
Bezugsperson.attributemethods['bezirk'] = _get_strkat_ortsangabe
Bezugsperson.attributemethods['ortsteil'] = _get_strkat_ortsangabe
Bezugsperson.attributemethods['samtgemeinde'] = _get_strkat_ortsangabe
Bezugsperson.attributemethods['mail_link'] = _get_mail_link
Bezugsperson.attributemethods['gesperrt'] = _bezugsperson_gesperrt
Bezugsperson.attributemethods['vn_xxx'] = _vorname_oder_xxx
Bezugsperson.attributemethods['na_xxx'] = _nachname_oder_xxx

Einrichtungskontakt.attributemethods['mail_link'] = _get_mail_link

############################
# Berechnete Felder fr Fall
############################

def _fall_name(self, key):
    return self['akte__name']

def _aktuell_fall(self, key):
    return self['zday'] == 0
    
def _zustaendig_fall(self, key):
    "die aktuelle Zustndigkeit (*nicht* der Mitarbeiter)"
    found = 0
    for z in self['zustaendigkeiten']:
        if z['ed'] == 0:
            found = 1
            break
    if found: res =  z
    else: res =  None
    self._cache_field(key, res)
    return res
    
def _zuletzt_zustaendig_fall(self, key):
    # id als Sortierkriterium aufgenommen, da es mehrere Zustndikeiten
    # mit demselben Beginn-/Enddatum geben kann.
    zs = self['zustaendigkeiten'].sorted('bgy', 'bgm', 'bgd', 'id')
    # Gibt None zurck, wenn es keine Zustndikeiten gibt,
    # was leider manchmal vorkommt
    res = zs[-1] if zs else None
    self._cache_field(key, res)
    return res
    
# Datenbankaufruf teuer, kann aus den inversen berechnet werden
# def _get_jgh(self, key):
#     where_cl = "fall_id=%s" % self['id']
#     jghs = []
#     jghs += JugendhilfestatistikList(where=where_cl)
#     jghs += Jugendhilfestatistik2007List(where=where_cl)
#     assert len(jghs) <= 1, "Mehr als eine Bundesstatistik fr Fall %s" % fall['fn']
#     if jghs:
#         self._cache_field(key, jghs[0])
#         return jghs[0]
#     else:
#         self._cache_field(key, None)
#         return None
def _get_jgh(self, key):
    jghs = []
    jghs += self['jgh07_statistiken']
    jghs += self['jgh_statistiken']
    assert len(jghs) <= 1, "Mehr als eine Bundesstatistik fr Fall %s" % fall['fn']
    if jghs:
        self._cache_field(key, jghs[0])
        return jghs[0]
    else:
        self._cache_field(key, None)
        return None

def _fn_count(self, key):
    fn = self['fn']
    return int(fn.split('-')[0])

## def _has_fachstatistik(self, key):
##     if self['aktuell']:
##         if self['fachstatistiken']:
##             return str(self['fachstatistiken'][0]['jahr'])[2:]
##         else:
##             return 'nv'
##     else:
##         return '-'
## def _has_jghstatistik(self, key):
##     if self['aktuell']:
##         if self['jgh07_statistiken']:
##             return str(self['jgh07_statistiken'][0]['jahr'])[2:]
##         else:
##             return 'nv'
##     else:
##         return '-'
def _has_statistik(self, key):
    if key == 'has_fachstatistik':
        k = 'fachstatistiken'
    elif key == 'has_jghstatistik':
        k = 'jgh07_statistiken'
    if self['aktuell']:
        if self[k]:
            return str(self[k][0]['jahr'])[2:]
        else:
            return 'nv'
    else:
        return '-'

# def _fall_beratungskontakte(self, key=None):
#     # das ist hchst ineffizient !!!!
#     bkont_list = BeratungskontaktList(
#         where='fallberatungskontakt.fall_id=%s' % self['id'],
#         join=[('fallberatungskontakt',
#                'fallberatungskontakt.bkont_id=beratungskontakt.id')],
#         order = 'ky, km, kd, kh, kmin'
#         )
#     #bkont_list.sort('ky', 'km', 'kd')
#     if key:
#         self._cache_field(key, bkont_list)
#     return bkont_list

def _fall_beratungskontakte(self, key=None):
    from ebkus.db.dbapp import cache_beratungskontakte
    if 'fallberatungskontakte_ids' not in self:
        cache_beratungskontakte([self['akte']])
    bkont_ids = [fb['bkont_id'] for fb in self['fallberatungskontakte']]
    bkont_list = BeratungskontaktList(bkont_ids)
    bkont_list.sort('ky', 'km', 'kd', 'kh', 'kmin')
    if key:
        self._cache_field(key, bkont_list)
    # Testen, ob die neue Impl mit der alten bereinstimmt:
    # bkont_list_old = BeratungskontaktList(
    #     where='fallberatungskontakt.fall_id=%s' % self['id'],
    #     join=[('fallberatungskontakt',
    #            'fallberatungskontakt.bkont_id=beratungskontakt.id')],
    #     order = 'ky, km, kd, kh, kmin'
    #     )
    # assert bkont_list_old == bkont_list, 'Fehler in fall[beratungskontakte]'
    return bkont_list

def _fall_letzter_beratungskontakt_datum(self, key):
    """Datum des letzten Beratungskontakt als String dd.mm.yyyy, falls vorhanden.
    Sonst None.
    """
    bkont_list = self['beratungskontakte']
    if bkont_list:
        datum = str(bkont_list[-1].getDate('k'))
    else:
        datum = None
    self._cache_field(key, datum)
    return datum





# def _fall_do_bkont_dok(self, key):
#     """Return True, wenn fr diesen Fall der Inhalt der Beratungskontakte
#     dokumentiert werden soll.
#     Dazu muss zum einen config.BERATUNGSKONTAKT_DOKUMENTATION auf True stehen,
#     zum anderen muss dies auch schon fr alle frheren Beratungskontakte dieses Falles
#     der Fall gewesen sein. Es ist nicht sinnvoll, mittendrin damit zu beginnen.
#     Zu erkennen ist das daran, dass fr die frheren Beratungskontakte 
#     das Feld erstgespraech nicht auf None steht.
#     """
#     if config.BERATUNGSKONTAKT_DOKUMENTATION:
#         beratungskontakte = _fall_beratungskontakte(self)
#         if len(beratungskontakte) == 0 or beratungskontakte[0]['erstgespraech'] is not None:
#             return True
#     return False
        
def _fall_leistungsbeginn(self, key):
    """Datum der ersten Leistung fr den Fall.

    Datum ist ein ebapi.Date Objekt.
    """
    leistungen = self['leistungen']
    leistungen.sort('bgy', 'bgm', 'bgd')
    return leistungen[0].getDate('bg')

def _fall_wartezeit(self, key):
    # Anzahl der Tage zwischen Anmeldung und erster Leistung
    return (self['leistungsbeginn'].to_py_date() - 
            self['bg'].to_py_date()).days

    
Fall.attributemethods['name'] = _fall_name
Fall.attributemethods['aktuell'] = _aktuell_fall
Fall.attributemethods['zustaendig'] = _zustaendig_fall
Fall.attributemethods['zuletzt_zustaendig'] = _zuletzt_zustaendig_fall
Fall.attributemethods['jgh'] = _get_jgh
Fall.attributemethods['fn_count'] = _fn_count
Fall.attributemethods['has_fachstatistik'] = _has_statistik
Fall.attributemethods['has_jghstatistik'] = _has_statistik
Fall.attributemethods['bg'] = getDate
Fall.attributemethods['zda'] = getDate
# inversefields ist fallberatungskontakte, wird also nicht berschrieben
Fall.attributemethods['beratungskontakte'] = _fall_beratungskontakte
Fall.attributemethods['letzter_beratungskontakt_datum'] = _fall_letzter_beratungskontakt_datum
Fall.attributemethods['leistungsbeginn'] = _fall_leistungsbeginn
Fall.attributemethods['wartezeit'] = _fall_wartezeit
#Fall.attributemethods['do_bkont_dok'] = _fall_do_bkont_dok


def _prev_zust(self, key):
    zustaendigkeiten = self['fall__zustaendigkeiten'].sorted('bgy', 'bgm', 'bgd', 'id')
    i = zustaendigkeiten.index(self)
    if i > 0:
        return zustaendigkeiten[i-1]
    else:
        return None
def _next_zust(self, key):
    zustaendigkeiten = self['fall__zustaendigkeiten'].sorted('bgy', 'bgm', 'bgd', 'id')
    i = zustaendigkeiten.index(self)
    if i+1 < len(zustaendigkeiten):
        return zustaendigkeiten[i+1]
    else:
        return None
    

Zustaendigkeit.attributemethods['prev'] = _prev_zust
Zustaendigkeit.attributemethods['next'] = _next_zust
Zustaendigkeit.attributemethods['bg'] = getDate
Zustaendigkeit.attributemethods['e'] = getDate



def _alter(self, key):
    # Alter berechnen:
    # mindestens 0, hchstens 27
    # berechnet aus der Differenz zwischen Hilfebeginnjahr und -monat und
    # Geburtsjahr und -monat
    alter = min(27,
                max(0, ((self['bgm'] + 12*self['bgy']) -
                        (self['gem'] + 12*self['gey'])) / 12))
    return alter
        
Jugendhilfestatistik2007.attributemethods['alter'] = _alter

# Felder vn, na wie bei Akte
Mitarbeiter.attributemethods['name'] = _akte_name

def _gruppe_mitarbeiternamen(self, key):
    """Ein String mit komma-getrennten Nachnamen der Mitarbeiter einer Gruppe."""
    mitarbeiter = self['mitarbeiter'].sorted('mit__na')
    return ', '.join([m['mit__na'] for m in mitarbeiter])
Gruppe.attributemethods['mitarbeiternamen'] = _gruppe_mitarbeiternamen
Gruppe.attributemethods['bg'] = getDate
Gruppe.attributemethods['e'] = getDate


# def _bkont_mitarbeiter(self, key):
#     mitarbeiter_list = MitarbeiterList(
#         where='mitarbeiterberatungskontakt.bkont_id=%s' % self['id'],
#         join=[('mitarbeiterberatungskontakt',
#                'mitarbeiterberatungskontakt.mit_id=mitarbeiter.id')])
#     mitarbeiter_list.sort('na')
#     return mitarbeiter_list
def _bkont_mitarbeiter(self, key):
    mitarbeiter = [mb['mit'] for mb in self['mitarbeiterberatungskontakte']]
    return mitarbeiter
# def _bkont_faelle(self, key):
#     fall_list = FallList(
#         where='fallberatungskontakt.bkont_id=%s' % self['id'],
#         join=[('fallberatungskontakt',
#                'fallberatungskontakt.fall_id=fall.id')])
#     fall_list.sort('akte__na')
#     return fall_list
def _bkont_faelle(self, key):
    faelle = [fb['fall'] for fb in self['fallberatungskontakte']]
    return faelle
    # fall_list = FallList(
    #     where='fallberatungskontakt.bkont_id=%s' % self['id'],
    #     join=[('fallberatungskontakt',
    #            'fallberatungskontakt.fall_id=fall.id')])
    # fall_list.sort('akte__na')
    # return fall_list
## def _brutto_dauer_bkont_bs(self, key):
##     """Eval'ed den dok-String der Kategorie art_bs aus,
##     so dass man dort eintragen kann, wie aus der Netto-Dauer
##     die Brutto-Dauer berechnet wird.
##     """
##     if config.BERATUNGSKONTAKTE_BS:
##         dauer = self.get('dauer', 0)
##         term = self['art_bs__dok']
##         try:
##             brutto = int(round(eval(term)))
##         except:
##             brutto = 0
##         return brutto
def _brutto_dauer_bkont_bs(self, key):
    """Brutto-Dauer fr die Kontaktanzahl der Bundesstatistik.
    
    Eval'ed den dok-String der Kategorie art_bs aus,
    so dass man dort eintragen kann, wie aus der Netto-Dauer
    die Brutto-Dauer berechnet wird.
    Speziell fr die Brutto-Dauer im Sinne der Bundesstatistik
    kann man nach dem Trenner || eine zweite Formel eingeben, die nur
    fr die Berechnung der Kontaktzeit fr die Bundesstatistik
    genommen wird:

    dauer * 1.8 || dauer*1.2 # bs:ja
    """
    if config.BERATUNGSKONTAKTE_BS:
        dauer = self.get('dauer', 0)
        term = self['art_bs__dok']
        # Kommentar absplitten
        try:
            term, comment = term.split('#')
        except:
            pass
        # zweite Formal absplitten
        try:
            brutto_term, brutto_bs_term = term.split('||')
        except:
            # falls nicht vorhanden, die erste nehmen
            brutto_term = brutto_bs_term = term
        if key == 'brutto':
            term = brutto_term
        elif key == 'brutto_bs':
            term = brutto_bs_term
        try:
            brutto = int(round(eval(term)))
        except:
            brutto = 0
        return brutto
        
def _jghkontakte(self, key):
    """Berechnet die aus einem realen Beratungskontakt die Anzahl der fr
    die Bundesstatistik anzurechnenden Kontakte.

    Eine lngere Dauer des Kontakts ergibt mehr Kontakte im Sinne der Bundesstatistik.
    Es gibt fr Braunschweig eine spezielle Berechnungsmethode, die Daten im 
    Dokumentationsstring fr die Art des Kontaktes verwendet.
    """
    try:
        if config.BERATUNGSKONTAKTE_BS:
            if 'bs:ja' in self['art_bs__dok'].lower():
                # als Grundlage fr die Berechnung der Anzahl der Kontakte
                # wird das oben berechnete brutto_bs genommen
                return int(bcode('kdbs', self['brutto_bs'])['code'])
            else:
                return 0
        elif config.BERATUNGSKONTAKTE:
            return int(bcode('fskd', self['dauer'])['code'])
    except:
        pass

# wird von dokumentenkarte.py importiert
def dokumentation_bearbeitungs_frist_abgelaufen(date):
    """
    Ohne config.BERATUNGSKONTAKT_DOKUMENTATION:
    - immer False, da es keine Bearbeitungsfrist gibt.
    Mit config.BERATUNGSKONTAKT_DOKUMENTATION:
    - Falls das bergebene Datum nicht mehr in der konfigurierten Frist
      liegt True
      sonst False, auch wenn das bergebene Datum kein Datum ist, sondern zB None.
    """
    _today = today()
    if config.BERATUNGSKONTAKT_DOKUMENTATION:
        if config.BERATUNGSKONTAKT_DOKUMENTATION_BEARBEITUNGS_FRIST < 0:
            return False
        if (isinstance(date, Date) and date.year > 0 and
            date.diff_weekdays(_today) >
            config.BERATUNGSKONTAKT_DOKUMENTATION_BEARBEITUNGS_FRIST):
            return True
    return False
    
def _noch_modifizierbar(self, key):
    """
    Wenn der Beratungskontakt nicht zu einem aktuellen Fall gehrt,
      immer False
    Wenn aktueller Fall:
      Ohne BERATUNGSKONTAKT_DOKUMENTATION immer True
      Mit BERATUNGSKONTAKT_DOKUMENTATION 
        True, wenn innerhalb der Bearbeitungsfrist
           bezogen auf Doku-Datum, falls vorhanden
           sonst bezogen auf Kontaktdatum
    """
    aktuell = [True for f in self['faelle'] if f['aktuell']]
    if not aktuell:
        return False
    if self['dy'] is None:
        # wenn der Beratungskontakt vor Einfhrung der Kontaktdokumentation
        # angelegt wurde, ist dd, dm und dy NULL
        # Frist wird auf Datum des Kontakts selbst bezogen
        date = self.getDate('k')
    else:
        date = self.getDate('d')
    return not dokumentation_bearbeitungs_frist_abgelaufen(date)
    
Beratungskontakt.attributemethods['mitarbeiter'] = _bkont_mitarbeiter
Beratungskontakt.attributemethods['faelle'] = _bkont_faelle
Beratungskontakt.attributemethods['brutto'] = _brutto_dauer_bkont_bs
# brutto fr die Bundesstatistik
Beratungskontakt.attributemethods['brutto_bs'] = _brutto_dauer_bkont_bs
Beratungskontakt.attributemethods['jghkontakte'] = _jghkontakte
Beratungskontakt.attributemethods['noch_modifizierbar'] = _noch_modifizierbar
for x in ('k', 'd', 'dlc'):
    Beratungskontakt.attributemethods[x] = getDate


def _brutto_dauer_fua_bs(self, key):
    dauer = self.get('dauer', 0)
    term = self['art__dok']
    try:
        brutto = int(round(eval(term)))
    except:
        brutto = 0
    return brutto

# def _fua_bs_mitarbeiter(self, key):
#     if self['mit_id']:
#         mitarbeiter_list = MitarbeiterList([Mitarbeiter(self['mit_id'])])
#     else:
#         mitarbeiter_list = MitarbeiterList(
#             where='mitarbeiterfua_bs.fua_bs_id=%s' % self['id'],
#             join=[('mitarbeiterfua_bs',
#                    'mitarbeiterfua_bs.mit_id=mitarbeiter.id')])
#         mitarbeiter_list.sort('na')
#     return mitarbeiter_list

def _fua_bs_mitarbeiter(self, key):
    if self['mit_id']:
        mitarbeiter_list = MitarbeiterList([Mitarbeiter(self['mit_id'])])
    else:
        mit_ids = [mf['mit_id'] for mf in self['mitarbeiterfuaktivitaeten']]
        mitarbeiter_list = MitarbeiterList(mit_ids)
        # mitarbeiter_list_old = MitarbeiterList(
        #     where='mitarbeiterfua_bs.fua_bs_id=%s' % self['id'],
        #     join=[('mitarbeiterfua_bs',
        #            'mitarbeiterfua_bs.mit_id=mitarbeiter.id')])
        # assert mitarbeiter_list == mitarbeiter_list_old
        mitarbeiter_list.sort('na')
    return mitarbeiter_list
    
def _fua_bs_mitarbeiternamen(self, key):
    """Ein String mit komma-getrennten Nachnamen der Mitarbeiter einer Fua."""
    return ', '.join([m['na'] for m in self['mitarbeiter']])

Fua_BS.attributemethods['mitarbeiter'] = _fua_bs_mitarbeiter
Fua_BS.attributemethods['mitarbeiternamen'] = _fua_bs_mitarbeiternamen
Fua_BS.attributemethods['brutto'] = _brutto_dauer_fua_bs

FallGruppe.attributemethods['bg'] = getDate
FallGruppe.attributemethods['e'] = getDate
BezugspersonGruppe.attributemethods['bg'] = getDate
BezugspersonGruppe.attributemethods['e'] = getDate

Leistung.attributemethods['bg'] = getDate
Leistung.attributemethods['e'] = getDate

Dokument.attributemethods['v'] = getDate
Gruppendokument.attributemethods['v'] = getDate


############################
# Pfaddefinitionen
############################


Akte.pathdefinitions = {
  'akte': 'self'
}
Anmeldung.pathdefinitions = {
  'akte': 'fall_id__akte'
}

Leistung.pathdefinitions = {
  'akte': 'fall_id__akte'
}
Beratungskontakt.pathdefinitions = {
  'akte': 'fall_id__akte'
}
Zustaendigkeit.pathdefinitions = {
  'akte': 'fall_id__akte'
}
Fachstatistik.pathdefinitions = {
  'akte': 'fall_id__akte'
}
Jugendhilfestatistik.pathdefinitions = {
  'akte': 'fall_id__akte'
}
Jugendhilfestatistik2007.pathdefinitions = {
  'akte': 'fall_id__akte'
}

Dokument.pathdefinitions = {
  'akte': 'fall_id__akte'
}

Wireb.pathdefinitions = {
  'akte': 'fall_id__akte'
}

Gruppe.pathdefinitions = {
  'gruppe': 'self'
}

MitarbeiterGruppe.pathdefinitions = {
  'gruppe': 'gruppe_id__gruppe'
}

Gruppendokument.pathdefinitions = {
  "gruppe": "gruppe_id__gruppe"
}

FallGruppe.pathdefinitions = {
  'gruppe': 'gruppe_id__gruppe', 'akte': 'fall_id__akte'
}

BezugspersonGruppe.pathdefinitions = {
  'gruppe': 'gruppe_id__gruppe', 'akte': 'bezugsp_id__akte'
}

def akte_undo_cached_fields(self):
    """Zieht alle gecachten Feldwerte zurck, aus allen Objekten,
    die mit einer Akte zusammenhngen. Nach jedem insert oder update
    aufzurufen, damit keine falschen gecachten Werte auftreten."""
    
    for f in self['faelle']:
        for l in f['leistungen']: l.undo_cached_fields()
        for b in f['beratungskontakte']: b.undo_cached_fields()
        for b in f['fallberatungskontakte']: b.undo_cached_fields()
        for z in f['zustaendigkeiten']: z.undo_cached_fields()
        for a in f['anmeldung']: a.undo_cached_fields()
        for fs in f['fachstatistiken']: fs.undo_cached_fields()
        for jgh in f['jgh_statistiken']: jgh.undo_cached_fields()
        for jgh07 in f['jgh07_statistiken']: jgh07.undo_cached_fields()
        for d in f['dokumente']: d.undo_cached_fields()
        for g in f['gruppen']: g.undo_cached_fields()
        f.undo_cached_fields()
    for b in self['bezugspersonen']:
        for gr in b['gruppen']: gr.undo_cached_fields()
        b.undo_cached_fields()
    for e in self['einrichtungen']: e.undo_cached_fields()
    self.undo_cached_fields()
    
Akte.akte_undo_cached_fields = akte_undo_cached_fields

def gruppe_undo_cached_fields(self):
    """Zieht alle gecachten Feldwerte zurck, aus allen Objekten,
    die mit einer Gruppe zusammenhngen. Nach jedem insert oder update
    aufzurufen, damit keine falschen gecachten Werte auftreten."""
    
    for m in self['mitarbeiter']: m.undo_cached_fields()
    for b in self['bezugspersonen']: b.undo_cached_fields()
    for f in self['faelle']: f.undo_cached_fields()
    for d in self['gruppendokumente']: d.undo_cached_fields()
    self.undo_cached_fields()
    
Gruppe.gruppe_undo_cached_fields = gruppe_undo_cached_fields

## Allgemein Stringberrfung
## Datumsberprfung
## Codeberprfung
## Foreignkeyberprfung


#############################################################################
##
## Allgemeines fr die folgenen Prffunktionen (check_*)
## =====================================================

## 1. und 2. Argument ist eine dictionary und ein key, wodurch der
## zu prfende Wert definiert ist (in unserem Beispiel die form).

## Der Wert kann immer (mu aber nicht) ein String sein, der je nach
## Prffunktion umgewandelt wird. Falls die Umwandlung milingt, wird
## eine Exception geworfen.

## Falls ein Fehler auftritt, wird eine EBUpdateError Exception geworfen mit
## dem errorstring als Argument.

## Falls das default Argument belegt ist (d.h. nicht None ist), wird es
## zurckgegeben falls
## - die dictionary keinen Eintrag unter key hat
## - ein Eintrag existiert, dieser aber None oder '' ist

## Die default Werte werden *nicht* weiter berprft.

## Falls kein default Wert angegeben ist (default = None), fhrt das Fehlen
## eines gltigen Wertes zu einem Fehler.
##
## Gltige Werte:
##  check_str_not_empty: ein nicht-leerer String
##  check_int_not_empty: ein Integer, kann auch 0 sein
##  check_fk: eine id, fr die ein Objekt der entsprechenden Klasse existiert
##  check_code: ein code, fr das ein Code Objekt der entsprechenden Kategorie
##              existiert
##  check_date: ein gltiges Datum zwischen 1980 und today()
##              falls maybezero true ist, ist auch (0,0,0) zulssig
##              falls maybefuture true ist, kann das Datum in der Zukunft liegen
##                (bis 2030)
##              falls nodayallowed true ist, kann der Tag weggelassen werden (d=None)
##                Er wird dann auf 1 gesetzt.
##
#############################################################################

# Hilfsfunktion, damit das default Argument auch ein Objekt sein kann,
# aus dem der Wert geholt wird
def k_or_val(key, default):
    """
    key: string
    Falls default eine dict ist mit einem Schlssel key,
    gebe dessen Wert zurck, sonst default selbst.
    """
    try: val = default[key]
    except: val = default
    return val
    
    # Garantiert einen nicht-leeren String
def check_str_not_empty(dict, key, errorstring, default = None):
    """
    Garantiert einen nicht-leeren String, wenn default None ist.
    Es gengt aber schon ein String mit einem Leerzeichen.
    """
    val = dict.get(key)
    if val is None or val == '':
        if not default is None: return k_or_val(key, default)
    if not val or type(val) != type(''): raise EE(errorstring)
    return val
    
def check_list(dict, key, errorstring, default=None):
    "garantiert eine Liste mit nicht-leeren Elementen"
    "Falls default == [], auch eine leere Liste"
    val = dict.get(key)
    if val:
        if isinstance(val, basestring):
            val = [val]
        else:
            # TODO: das muss man nicht immer prfen
            assert isinstance(val, list)
        # keine leeren strings
        val = [v for v in val if v]
        #print 'VAL', val
        return val
    else:
        #print 'VAL EMPTY', val
        if default == None:
            raise EE(errorstring)
        else:
            return default
    
            
def check_int_not_empty(dict, key, errorstring, default = None):
    val = dict.get(key)
    if isinstance(val, (int, long)):
        return val
    if val is None or val.strip() == '':
        if not default is None:
            return k_or_val(key, default)
    try:
        val = int(val)
    except:
        raise EE(errorstring)
    return val
    
    
def check_fk(dict, key, klass, errorstring, default = None):
    fk = dict.get(key)
    if fk in (None, '', ' '):
        if not default is None: return k_or_val(key, default)
    try: fk = int(fk)
    except: raise EE(errorstring)
    try:
      #print 'FK', type(fk), fk
        obj = klass(fk)
    except Exception, e:
        raise EE("%s: %s" % (errorstring, e))
    return fk
    
    

def check_time(dict, key, errorstring, default=None):
    if type(key) == type(()):
        h, min = key[0], key[1]
    else:
        h, min = key + 'h', key + 'min'
    h, min = dict.get(h), dict.get(min)
    try:
        t = Time(h, min)
        assert t.check()
    except:
        traceback.print_exc()
        raise EE(errorstring)
    if t.empty:
        if default != None:
            return default
        else:
            traceback.print_exc()
            raise EE(errorstring)
    return t
    
def check_date(dict, key, errorstring, default = None,
               maybezero = None,    # darf 0 sein
               maybefuture = None,  # darf in der Zukunft liegen
               nodayallowed = None, # Tag darf fehlen
               alle_jahrgaenge_akzeptieren=False,
               # falls True, nur Jahre zwischen 1890 und heute akzeptieren
               # falls False, nur Jahre zwischen 1970 und 2050
               # (siehe Date.check())
               ):
    """Liefert korrektes Datum aus der Form oder Date(0,0,0),
    unter Bercksichtigung der Keyword-Parameter.
    """
    if type(key) == type(()):
        d, m, y = key[0], key[1], key[2]
    else:
        d, m, y = key + 'd', key + 'm', key + 'y'
    d, m, y = dict.get(d), dict.get(m), dict.get(y)
    empty = (None, '', ' ')
    if d in empty and m in empty and y in empty:
        if not default is None:
            if isinstance(default, Date): return default
            if isinstance(default, DBObjekt): return default.getDate(key)
            if type(default) == type(()): return apply(Date, default)
    if nodayallowed and d is None or d == '':
        d = '1'
    try:
        d,m,y = int(d), int(m), int(y)
    except:
        raise EE(errorstring)
    datum = Date(y,m,d)
    test = (not datum.check(alle_jahrgaenge_akzeptieren) or
            (not maybefuture and datum.year != 0 and today() < datum) or
            (not maybezero and datum.year == 0))
    if test:
        raise EE("%s: %s" % (errorstring, str(datum)))
    return datum
    
def check_code(dict, key, kat_code, errorstring, default = None):
    code = dict.get(key)
    #print key, kat_code, code
    if code is None or code == '':
        if not default is None:
            if type(default) == type(''): return cc(kat_code, default)
            else: return k_or_val(key, default)
    try: code = int(code)
    except: raise EE(errorstring)
    try:
        codeobj = Code(code)
        assert codeobj['kat_code'] == kat_code
        # print 'Code:' ,code
    except Exception, e:
        raise EE("%s: %s" % (errorstring, e))
    return code

def check_multi_code(dict, key, kat_code, errorstring, default = None):
    """mehrere code ids als string in einem Feld
    default kann sein:
    - Liste von Code-Objekten 
    - String mit Ids
    - ein Objekt, wo mit key der Wert nachgeschlagen wird
    """
    code = dict.get(key)
    #print key, kat_code, code
    if code is None or code == '':
        if not default is None:
            if type(default) == type(''):
                return default
            elif isinstance(default, (list,tuple)):
                return ' '.join([str(o['id']) for o in default])
            else:
                return k_or_val(key, default)
    if not isinstance(code, list):
        code = [code]
    try:
        code = [int(c) for c in code]
    except:
        raise EE(errorstring)
    try:
        codes = CodeList(code)
    except Exception, e:
        raise EE("%s: %s" % (errorstring, e))
    return ' '.join([str(i) for i in codes.getIds()])
    
    
def get_string_fields(object, form, formnames, default = None, objectnames = None):
    if not objectnames:
        objectnames = formnames
    pairs = map(None, formnames, objectnames)
    for fname, objname in pairs:
        val = form.get(fname)
        if val is None:
            if not default is None:
                val = k_or_val(objname, default)
        elif type(val) != type(''): raise TypeError(fname)
        object[objname] = val
    # Feldlngen prfen fr alle belegten Felder
    # TODO prfen ob alle Flle abgedeckt werden
    # Wenn Felder ohne get_string_fields() in ebupd aus der Form geholt werden,
    # findet diese Prfung nicht statt.
    object.check_string_field_lengths()

        # Falls der Wert eines Feldes in der form None oder '' ist und kein Default-
        # wert angegeben ist,  wird der Wert auf None gesetzt.
        # Ansonsten wird berprft, ob es sich um ein Integer handelt,
        # sonst Fehler.
        
#def get_string_fields(object, form, formnames, default = None, objectnames = None):
#def get_string_fields_new(result_obj, form, names, default_obj=None, default=None,

def get_string_fields_new(result_obj, form, names, default=None, check_empty=False):
    """
    result_obj muss eine Instanz von dbapp.DBObjekt sein. Falls das nicht gebraucht wird,
           einfach leeres Objekt instanziieren. Das wird bentigt, um die Feldlngen
           zu prfen. Da werden die Werte eingetragen.
    form ist dict aus der die Werte entnommen werden
    names: Liste von Elementen, die entweder einfache strings sind oder ein Paar von strings.
           Falls Paar, ist erstes Element der Key fr form und zweites Element Key fr object,
           sonst ist Key fr beides.
    default: Typischerweise das alte DBObjekt, was updatet werden soll. Falls vorhanden,
             wird ein default-Wert daraus genommen.
         oder
    default: Wenn kein Wert in default gefunden wurde, ist es  der Default selbst.
    
    Die gefunden String-Werte werden lngenberprft mit dbapp.DBOjbekt.check_string_field_lengths

    Als Werte werden nur Strings geliefert oder gegebenfalls None, wenn keine Defaults da sind.

    Wenn check_empty True ist darf kein Wert leer sein (None oder ''). Sonst EE('Kein ...')

    Die Werte werden sowohl in result_obj geschrieben, als auch als Liste zurckgegeben.
    """
    pairs = []
    assert isinstance(result_obj, DBObjekt)
    for x in names:
        if isinstance(x, basestring):
            pair = (x,x)
        else:
            pair = x
        pairs.append(pair)
    res = []
    for fname, objname in pairs:
        val = form.get(fname)
        if val is None:
            try:
                val = default[objname]
            except:
                val = default
        assert val is None or isinstance(val, basestring)
        if check_empty:
            if val is None or not val.strip():
                try:
                    name = get_feld(objname, result_obj.table)['name']
                except:
                    name = objname
                raise EE("Kein Wert fr '{}'".format(name))
        result_obj[objname] = val
        res.append(val)
    result_obj.check_string_field_lengths()
    return res
        
def get_int_fields(object, form, formnames, default = None, objectnames = None):
    if not objectnames:
        objectnames = formnames
    pairs = map(None, formnames, objectnames)
    for fname, objname in pairs:
        val = form.get(fname)
        if val is None or val == '':
            if not default is None:
                val = k_or_val(objname, default)
            else: val = None
        else:
            try: val = int(val)
            except: raise TypeError(fname)
        object[objname] = val
        
        
        # Gibt das existierende Objekt zurck.
        # Hier gibt es keine defaults, da das Objekt existieren mu.
def check_exists(dict, key, klass, errorstring):
    """Raise an error if an instance of klass with id id doesn't exists."""
    try:
        id = int(dict[key])
        obj = klass(id)
    except:
        raise EE(errorstring)
    return obj
    
def get_object(id, klass):
    """Return instance of klass with id if it exists else None."""
    try:
        obj = klass(int(id))
        return obj
    except:
        pass
    return None

    #######################
    # Weiter Prffunktionen
    #######################
    
    # geaenderte check_unique. huber
    
def check_unique(value, klassList, field, errorstring):
    """Raise an error if value already exists for field for any instance of klass."""
    query = "%s = '%s'" % (field,value)
    liste = klassList(where = query)
    if len(liste) > 0:
        raise EE(errorstring)
    else:
        return value
        
        
        # Alte check_unique
        
        #def check_unique(value, klass, field, errorstring):
        #  """Raise an error if value already exists for field for any instance of klass."""
        #  try:
        #    obj = klass(field, val)
        #    raise EE(errorstring % obj)
        #  except: pass
        #  return value
        
def check_not_exists(id, klass, errorstring):
    """Raise an error if an instance of klass with id id already exists."""
    try:
        obj = klass(id)
    except: pass
    else:
        raise EE(errorstring % obj)
        
        
        ########################
        # Exceptions
        ########################
        
class EBUpdateError(Exception):
    """Ein Fehler, der normalerweise nicht passieren sollte und daher
    als ERROR geloggt werden sollte"""
    pass
    
ER = EBUpdateError


class EBUpdateDataError(Exception):
    """Ein Fehler, der in der Regel durch falsche Eingaben des Benutzers
    entsteht und daher auch von diesem selber zu korrigieren ist.
    Sollte nicht geloggt werden."""
    pass
    
EE = EBUpdateDataError

# Es knnte auch Warnungen geben, die aber nicht einen Eintrag
# verhindern, z.B.zu alte Datumsangaben (mehr als zwei Jahre zurck)

class EBUpdateWarning(Exception):
    pass
    
EW = EBUpdateWarning    
    
def mk_daten_dirs(n=None):
    """Erstellt die Datenverzeichnispfade bei der Installation fuer die Akten
       und das Exportverzeichnis."""
    
    daten_path = os.path.join(config.INSTANCE_HOME, 'daten')
    akten_path = os.path.join(daten_path, 'akten')
    gruppen_path = os.path.join(daten_path, 'gruppen')
    export_path = os.path.join(daten_path, 'export')
    try:
        os.mkdir(daten_path, 0700)
    except: pass
    try:
        os.mkdir(akten_path, 0700)
    except: pass
    try:
        os.mkdir(gruppen_path, 0700)
    except: pass
    try:
        os.mkdir(export_path, 0700)
    except: pass

## NICHT MEHR NOETIG! Wird von mk_akte_dir mit erledigt.
##     if not n:
##         n = int(AK_DIRS_MAX)
##     print 'Lege Aktenverzeichnisse an (insg. %s )' % n
##     for a in range(n):
##         try:
##             os.mkdir(os.path.join(akten_path, str(a)), 0700)
##         except Exception, e:
##             raise ('Keine Aktenverzeichnisse angelegt: %s' %(e))
##     print 'Aktenverzeichnisse angelegt'
    
def get_akte_path(akid):
    """Ermittelt den Pfad fr die Dokumente 1 Akte anhand der Aktenid."""
    
    n = int(config.AK_DIRS_MAX) # max. je n Akten in n Verzeichnissen
    akdirs = akid/n
    try:
        akte_path = os.path.join(config.INSTANCE_HOME, 'daten', 'akten', str(akdirs), str(akid))
    except Exception, e:
        raise ('Kein Aktenverzeichnis. %s' %(e))
    return akte_path
    
    
def mk_akte_dir(akid):
    """Erstellt 1 Verzeichnis fuer 1 Akte anhand der Aktenid;
       gibt Pfad der Akte zurueck."""
    
    n = int(config.AK_DIRS_MAX) # max. je n Akten in n Verzeichnissen
    akdir = akid/n

    akdir_path = os.path.join(config.INSTANCE_HOME, 'daten', 'akten', str(akdir))
    try:
        os.mkdir(akdir_path, 0700)
    except: pass
    
    akte_path = os.path.join(config.INSTANCE_HOME, 'daten', 'akten', str(akdir), str(akid))
    try:
        os.mkdir(akte_path, 0700)
    except: pass
    return akte_path
    
def mk_gruppe_dir(gruppeid):
    """Erstellt 1 Verzeichnis fuer 1 Gruppe anhand der Gruppenid;
       gibt Pfad der Gruppe zurueck. """
    
    gruppe_path = os.path.join(config.INSTANCE_HOME, 'daten', 'gruppen', str(gruppeid))
    try:
        os.mkdir(gruppe_path, 0700)
    except: pass
    return gruppe_path
    
def get_gruppe_path(gruppeid):
    """Ermittelt den Pfad fr die Dokumente 1 Gruppe anhand der Gruppenid. """
    
    try:
        gruppe_path = os.path.join(config.INSTANCE_HOME, 'daten', 'gruppen', str(gruppeid))
    except Exception, e:
        raise ('Kein Gruppenverzeichnis. %s' %(e))
    return gruppe_path
    
# TODO beseitigen, ist verzockt. Besser Date.add_month benutzen. Funktioniert
# auch mit negativen Werten.
# Mon Feb 19 15:03:22 2024 Obsolet, wird nicht gebraucht.
def get_rm_datum(frist=None):
    """Ermittelt anhand der Loeschfrist (Aufbewahrungsfrist) das Loeschdatum. """
    
    if frist is None or frist == '':
        frist = config.LOESCHFRIST
    jahr = today().year
    monat = today().month
    heute = int(jahr)*12 + int(monat)
    loeschzeitm = int(heute)-int(frist)
    loeschjahr = int(loeschzeitm) / int(12)
    loeschmonat = int(loeschzeitm) - (int(loeschjahr) * int(12))
    loeschdatum = {'loeschjahr' : loeschjahr, 'loeschmonat' : loeschmonat}
    return loeschdatum

def convert_pstoascii():
    """Konvertiert die PDF-Dokumente in schlechtes Ascii. Unixspezifisch,
       verwendet Systemaufruf fuer das Programm: pstoascii. """
    
    import ebkus.db.sql
    sql.opendb()
    
    dokl = DokumentList(where = 'mtyp = %s'
                        % cc('mimetyp', 'pdf'), order = 'fall_id,id')
    grdokl = GruppendokumentList(where = 'mtyp = %s'
                                 % cc('mimetyp', 'pdf'), order = 'gruppe_id,id')
    
    for d in grdokl:
        gruppe_path = get_gruppe_path(d['gruppe_id'])
        dateil = os.listdir('%s' % gruppe_path)
        if '%s.txt' % d['id'] not in dateil and '%s.pdf' % d['id'] in dateil:
            os.system('ps2ascii %s/%s.pdf %s/%s.txt'
                      % (gruppe_path,d['id'],gruppe_path,d['id']))
            
    for d in dokl:
        fall = Fall(d['fall_id'])
        akte_path = get_akte_path(fall['akte_id'])
        dateil = os.listdir('%s' % akte_path)
        if '%s.txt' % d['id'] not in dateil and '%s.pdf' % d['id'] in dateil:
            os.system('ps2ascii %s/%s.pdf %s/%s.txt'
                      % (akte_path,d['id'],akte_path,d['id']))
            
    sql.closedb()
    


# Ein Register in der Datenbank, wo man beliebige Python-Objekte
# mit einem Schlssel persistent ablegen kann
def register_set(key, value=None):
    if isinstance(value, basestring):
        if len(value) > 65535:
            raise EE("Wert fr register_set zu lang: %s" % value)
    else:
        assert value == None
    #print 'REGISTER_SET', key, value
    r = None
    try:
        r = Register(regkey=key)
    except dbapp.DBAppError:
        pass
    if value == None:
        if r:
            r.delete()
        return
    if r:
        # key vorhanden -> update
        r.update({'valuestr': value})
    else:
        # neu einfhren
        r = Register()
        r.init(regkey=key,
               valuestr=value,)
        r.new()
        r.insert()
def register_get(key, default=None):
    try:
        r = Register(regkey=key)
    except dbapp.DBAppError:
        return default
    value = r['valuestr']
    return value


def get_berichtseinheitid(stz_code=None, config_string=None):
    """
    Falls stz_code None:
    dict mit stz --> berichtseinheitid
    sonst berichtseinheitid fr stz

    Falls config_string None:
    config aus config.BERICHTSEINHEITID nehmen
    sonst in config_string bergeben, z.B. in update, 
    wo die config noch nicht richtig initialisiert ist.
    """
    if config_string is not None:
        s = config_string
    else:
        s = config.BERICHTSEINHEITID
    res = parse_berichtseinheitid(s)
    if stz_code:
        return res.get(stz_code)
    else:
        return res
def parse_berichtseinheitid(s):
    """
    Korrekt: Stelle A: 12345678123456 ; Stelle B : 12345   234567;Stelle C:12345678345678;
    Korrekt: 12345678345678
    Liefert dict: str_code --> bereichtseinheitid
    oder Exception
    """
    from ebkus.app.ebapih import get_codes
    match = re.match(list(x.valid_pattern for x in config.iter() if x.name == 'berichtseinheitid')[0], s)
    if match is None:
        raise EE("BerichtseinheitID ist nicht korrekt konfiguriert: {}".format(s))
    stz_codes = [x['code'] for x in get_codes('stzei')]
    parts = s.split(';')
    if len(parts) == 1:
        return dict((c, parts[0]) for c in stz_codes)
    parts = [p.strip() for p in parts]
    parts = [p for p in parts if p]
    # print 'PARTS:', parts
    res = {}
    for p in parts:
        # print 'PART:', p
        stelle, id = p.split(':')
        stelle = stelle.strip()
        id = id.strip()
        _, stz_code = stelle.split()
        res[stz_code] = id
    for c in stz_codes:
        if c not in res:
            raise EE("Keine BerichtseinheitID fr Stelle {}".format(c))
    return res

# einmal gebraucht
# def pcode(code):
#     print "                dict("
#     print "                     code='{}',".format(code['code'])
#     print "                     text='{}',".format(code['name'])
#     print "                     mini={},".format(code['mini'])
#     print "                     maxi={},".format(code['maxi'])
#     print "                ),",

# def pcodedict():
#     from ebkus.app.ebapih import get_all_codes
#     codes = get_all_codes('fskat')
#     for c in codes:
#         pcode(c)
