# coding: latin-1

"""Infos ber die Instanz.

Aufruf: info

TODO:
- Aufschlsseln ber Stellen: Akten, Flle, Mitarbeiter
- Gruppen
- Dokumente, wieviele wie gro
- Beratungskontakte, an denen mehrere Flle beteiligt sind?
  mehrere Mitarbeiter? Als Gruppensitzungen?
- fua_bs muss mit rein, wieviel pro jahr
- mysql-Version, mysql-Konfiguration? (show variables)
  (e.sql.getDBHandle().query('show variables', dictionary=True))
- Apacheversion (letzte Zeile der Fehlermeldung, funktioniert nicht berall)
- Python-Version
- Prozessgre, cache-Gre sollte mit drin sein

- info solle nur mit Anmeldung oder mit einem speziellem Schlssel funktionieren,
  nicht bedingungslos.

"""
from operator import itemgetter
from ebkus.config import config
from ebkus.app.ebapi import AkteList, FallList, today, DokumentList, MitarbeiterList, CodeList
from ebkus.app import Request
from ebkus.db.sql import execute
from ebkus.app.ebapi import register_get



def get_info(sections, q):
    import sys
    import platform

    """sections: list of config sections, e.g. ['instance', 'apache', 'ebkus']"""
    info = 'SYSTEMUMGEBUNG:\n\n'
    info += 'Betriebssystem: {}\n'.format(platform.platform())
    info += '              : {}\n'.format(' '.join(platform.uname()))
    info += 'Python: {}\n\n'.format(sys.version)
    info += '\nDATENBANK:\n\n'
    info += str(datenbank_info())
    info += '\nINSTANZ:\n\n'
    info += 'Stellen: {}\n'.format(', '.join(
            ["{} ({})".format(st['name'], st['code']) 
             for st in CodeList(where="kat_code = 'stzei'")]))
    info += 'Akten: {}\n'.format(execute("select count(*) from akte")[0][0])
    info += 'Flle:   offen: {} geschlossen: {}\n'.format(
        execute("select count(*) from fall where zday = 0")[0][0],
        execute("select count(*) from fall where zday > 0")[0][0],)
    info += '        offene: Beginn von {} bis {}\n'.format(
        execute("select min(bgy) from fall where zday = 0")[0][0],
        execute("select max(bgy) from fall where zday = 0")[0][0],)
    info += '  geschlossene:    zdA von {} bis {}\n'.format(
        execute("select min(zday) from fall where zday > 0")[0][0],
        execute("select max(zday) from fall where zday > 0")[0][0],)
    info += 'Mitarbeiter: {}\n'.format(execute("select count(*) from mitarbeiter")[0][0])
    info += 'Beratungskontakte: {}\n'.format(execute("select count(*) from beratungskontakt")[0][0])
    info += 'Dokumente: {}\n'.format(execute("select count(*) from dokument")[0][0])
    info += 'Gruppen: {}\n'.format(execute("select count(*) from gruppe")[0][0])
    info += 'Straenkatalog: {}\n'.format(execute("select count(*) from strkatalog")[0][0])
    info += 'Gemeindeschlssel (AGS): {}\n'.format(execute("select count(*) from ags")[0][0])
    info += 'Teilmengendefinitionen: {}\n'.format(execute("select count(*) from abfrage")[0][0])
    info += '\n\nKONFIGURATION:\n\n'
    last_section = None
    for section, name, value, db_value in get_config():
        if section != last_section:
            last_section = section
            info += "[{section}]\n".format(**locals())
        if section[0] in sections and (q is None or q.lower() in name.lower()):
            if db_value is not None and value != db_value:
                db = " !! {db_value}\n"
            else:
                db = "\n"
            info += ("{name:>50}: {value}" + db).format(**locals())
    return info

def get_refint():
    from ebkus.test.check_referential_integrity import CheckForeignKeys
    rows = CheckForeignKeys().get_invalid_references()
    invalid_references = '\nUngueltige Fremdschluessel:\n'
    if rows:
        invalid_references += '\n'.join(rows)
    else:
        invalid_references += 'Keine ungueltigen Fremdschluessel gefunden'
    return invalid_references

def datenbank_info():
    res = execute("SHOW VARIABLES LIKE '%%version%%';")
    s = ''
    for k,v in res:
        s += "  {:>22}: {:<}\n".format(k,v)
    return s

def get_config():
    res = []
    for kv in config.iter():
        section = kv.section
        if section == 'instance' and kv.fachlich:
            section = 'fachlich'
        name = kv.name
        value = kv.value
        db_value = None
        v_db = register_get(kv.name)
        #print 'CONFIG', v_db, kv.value
        if v_db != None:
            if kv.is_boolean():
                db_value = (v_db == 'true')
            elif kv.is_int():
                db_value = int(v_db)
            else:
                db_value = v_db
        res.append((section, name, value, db_value))
    res.sort()
    return res

class info(Request.Request):
    # Es drfen alle Angemeldeten ...
    permissions = Request.ALL

    def processForm(self, REQUEST, RESPONSE):
        q = self.form.get('q')
        config = self.form.get('config')
        refint = self.form.get('refint')
        if config == 'all':
            sections = ('a', 'd', 'e', 'f', 'i', 'o')
        else:
            sections = config.split(',') if config else []
            sections = [s[0] for s in sections]
            for s in sections:
                assert s in ('a', 'd', 'e', 'f', 'i', 'o'), 'Unknown config section'
        res = get_info(sections, q)
        if refint:
            res += get_refint()
        self.RESPONSE.setHeader('content-type', 'text/plain')
        self.RESPONSE.setBody(res)

def main():
    print get_info()


if __name__ == '__main__':
    main()

