# coding: latin-1

# dieser Text diente der Selbstverstndigung. Muss umgearbeitet werden,
# damit er der heutigen real implementierten Realitt entspricht.
# siehe die neue bundesstatistikdefinition Doku
"""
Deklarative Bundesstatistikdefinition

Die deklarative Bundesstatistik besteht aus einer Python-Datenstruktur, einer dict 
(Schlssel-Werte-Paare).

Bundesstatistik
  Berichtsjahr:
  statistik_items: [
  ],
  fragebogen_elemente: [
  ]

Bei den statistik_items gibt es einen Eintrag fr jedes Element 
einer abzugebenden Statistik, wobei einzelne Items nicht bei allen Abgabeformen
vorkommen mssen. Aber das betrifft nur ganz wenige, die eigentlichen inhaltlichen
Items kommen natrlich bei allen Abgabeformen vor.

Die Abgabeformen sind IDEV (Datensatz), IDEV (CSV) und Core (CSV).

Die kanonischen Namen sind die Namen aus Core plus die Namen fr die Elemente, die 
nicht in Core vorkommen:
- BerichtseinheitID (fr Core, aber Sonderstellung)
- Einrichtungsnummer (fr Datensatz)
- Land, Kreis, Gemeinde (fr Datensatz)
- LaufendeNummer (fr Datensatz)
- BA (fr Datensatz)

Andere Variablen, die EBKuS im Zusammenhang mit der Bundesstatistik verwendet, 
kommen nicht in die Tabelle, da das die Anwender verwirren knnte (Fallnummer, 
Geschwisterfall, etc.). Das wird alles von EBKuS im Hintergrund gehandelt, der
Kopfabsatz mit den Falldaten wird ebenfalls von EBKuS direkt erzeugt und ausgewertet
und wird nicht von dieser Tabelle beeinflusst.

Namen: Sat Jan 27 09:41:18 2024 neue berlegungen
- ebkus_name: traditionelles oft kryptisches Krzel in EBKuS. Wird sowohl als Feldname
              in der DB verwendet, als auch als fr HTML-name Attribut in Forms.
              Funktioniert auch fr das Lookup in JGH-Objekten in EBKuS.
              Obligatorisch 
- name: ein kurzer aber verstndlicher Name ohne Leerzeichen. In der Regel core_name, 
       auer wenn item nicht in core vorkommt
- langname: Mglichst kurzer Klartext fr das Item. Grammatisch korrekt mit Leerzeichen.
             Unabhngig von der Umgebung verstndlich. Kann z.B. fr die Bezeichnung
             der Items in der Statistik verwendendet werden. Steht in der Datenbank
             bei feld als name. Freigewhlt, ein Mix aus core_langname, idev_langname,
             und dem Text aus dem Fragebogen. Oft auch identisch mit einem dieser.
- core_csv_name: Name, der fr erste Zeile von CSV-Dateien genommen wird. Meist identisch
                 mit _name aus der xml-Beschreibung von Core
                 Muss nur da sein, wenn core_csv_header_schreiben auf True steht.
- core_langname: description aus der xml-Beschreibung von Core
- idev_csv_name: Name, der fr erste Zeile von CSV-Dateien genommen wird. 
                 Muss nur da sein, wenn idev_csv_header_schreiben auf True steht.
                 Lasse ich erstmal weg, weil idev wohl ohne header attribute gemacht wird.
- idev_langname: Was im idev-dokument als Feldbezeichnung verwendet wird. Mit Leerzeichen
                 und manchmal komisch.
- wireb_csv_name: Name, der fr erste Zeile von CSV-Dateien genommen wird. 
                  Muss nur da sein, wenn wireb_csv_header_schreiben auf True steht. 
                  Muss zur Zeit nicht funktionieren, da keiner (so viel ich wei) wireb benutzt.
Items:
  name: core cvs + einige extra
  ebkus_name: aus historischen und praktischen Grnden, wre aber technisch nicht unbedingt ntig.
  name_idev:
  langname_core:
  typ: Ganzzahl, Zeichenkette, Option
  optionsgruppe: bei Option, muss nur bei einem Item stehen bei wiederholter Verwendung
  optionen: 
      code: ohne code ist es einfach eine zwischenberschrift. falls das vorkommt, werden radiobuttons statt selectboxen genommen
      text:
      fett:
    # status: ab <jahr>, bis <jahr> ich glaube, das braucht man nicht
  minimum: bei Zahlen
  maximum:
  laenge: bei Zeichenkette
  doku:
  vorgabe_wert: ein Wert, der als Anfangswert genommen werden soll, aber im Fragebogen verndert werden kann.
                einige werte werden von ebkus initialisiert: Beginn, Anzahl der Kontakte.
                Heuristik: ein vorhandener Vorgabewert bewirkt, dass der Nutzer bei diesem Item nichts
                eintragen muss und trotzdem ohne Fehler speichern knnte.
                Fehlt ein vorgabe_wert, gibt es beim Speichern eine Fehlermeldung, wenn der Nutzer
                bei diesem Item keine Entscheidung getroffen hat.
  fester_wert: der wert von items, die bei EBKuS immer gleich beantwortet werden, oder "leer", 
               wenn das/die Item/s in EBKuS nicht beantwortet werden.
               Items mit festem Wert brauch nicht im Fragebogen zu erscheinen, knnen es aber,
               sind dann readonly, also nicht modifizierbar.


  alternativ: ein attribute wert_quelle: {fragebogen, vorgabe, fest, system} ??
  oder ein attribut status, wofr eine reihe von keywords definiert werden, die beim einlesen 
  in boolsche attribute verwandelt werden:
  nur_bei_andauernd, nur_bei_abgeschlossen,
  optional  -- Attribut muss keinen Wert haben, auch nicht fr die Bundesstatistik
  berechnet -- Attribut erhlt Wert von EBKuS
  eingabe -- Wert wird im Fragebogen ermittelt  

  datei_satz_position: (2,5)
  idev_csv_position: 33 # bzw (50,66), wenn mehrere Leerspalten erzeugt werden sollen
  core_csv_position: 22 # wie idevcsv_position
  wireb_csv_position: 22 # wie idevcsv_position
  bedingung: nur bei andauernd, nur bei abgeschlossen
  status: optional (z.B. dritter Grund, Wohnort, etc, auch Checkboxen, berall wo bei core xml required=False steht) 
  status: ab <jahr>, bis <jahr>, 

Die fragebogen_elemente beschreiben die Struktur des Fragebogens, wobei sie bei
den Eingabelementen auf Eintrge in den Items verweisen, unter Verwendung 
des Core-Namens.

Fragebogen
  
Abschnitte:
  art: Abschnitt
  gliederung: 'A'
  titel: 
Die Optionen bei Eingabe mit Auswahl stehen beim Item, nicht hier im Abschnitt. Auch Zwischentext bei Optionen stehen beim Item.
Die Frage oder der Text bei Eingaben steht aber hier als Text bei einem Element der art eingabe.
  teile: [
       art: {info, text, eingabe}
       item: Nur bei art=eingabe. Wenn item gegeben wird, ist es ein Eingabeelement, sonst nicht, art des Elements ergibt sich aus typ des items und weiteren Merkmalen
       text: Die Frage beim Item oder einfach Text oder Text in info box
       gliederung: Nur bei art={eingabe, text} Wenn Gliederung fehlt, alles ganz links, z.B bei Grnde, wenn blank, einrcken
       fett: true, false bei true pauschal fett, sonst kann man aber auch html formatieren
  ]

Zwischentext:
  text: kann auch html enthalten
  fett:

Die ebkus-Datenbanknamen werden gar nicht erwhnt. Sie werden unter der Hand verwendet
aufgrund der bekannten Verbindung zwischen Core-Namen und EBKuS-Namen. Wenn neue
Namen spezifiert werden sollten, werden die (Lower-cased) Core-Name genommen.

Interne Abbildung: ebkus_dbname hinzufgen. Im Programm kann man dann diesen verwenden, weil in der Regel krzer.


Was fr Werte gibt es?

- ohne Vorgabe
- mit Vorgabe
  - Vorgabe ist fester Wert initialer wert
  - Vorgabe wird durch eine bereitgestellte Funktion ermittelt initiale funktion
  - Vorgabe kann vom Benutzer im Fragebogen verndert werden nicht readonly
  - Vorgabe kann nicht verndert werden readonly, unveraenderlich




Durch den Fragebogen 
- modifizierbar
- nicht modifizierbar

Werte, die immer da sein mssen, auch fr den neuen Fragebogen:
- kreis, land, gemeinde, einrichtungsnummer
- berichtseinheit_id
- mitarbeiter_name
- laufende Nummer
- bgm, bgy
- gs, gm, gy
- fall_fn
- jahr
- nbkakt, nbkges (Kontaktzahl, nur bei Beratungskontakten)



Felder fr items
================

datei_satz_position 
- werte: python tuple (2, 5) inklusive
- nur vorhanden, wenn das Item fr den klassischen Exportdatensatz verwendet wird

idev_csv_position
- werte: integer oder tuple fr wiederholte Leerstellen
- nur vorhanden, wenn der wert in die idev csv-datei eingeht

core_csv_position
- werte: integer oder tuple fr wiederholte Leerstellen
- nur vorhanden, wenn der wert in die core csv-datei eingeht

idev_name

core_name
core_langname

wireb_name

ebkus_name 
ebkus_langname (aus der Datenbank)

anfangs_wert

anfangs_wert_funktion: get_bgm 

unveraenderbar 

doku







Die berprfung der Attribute lasse ich erstmal weg.
Ich habe noch kein einfach verstndliches Verfahren gefunden.

Vielleicht fr jede instantiierbare Klasse folgendes:
- Eine Liste der Attribute, die bei der Instantiierung mit einem Wert da sein mssen
- Eine Dict der Attribute, die bei der Instantiierung da sein knnen, aber nicht mssen.
  Wenn sie nicht da sind, bekommen sie ihren Wert aus dieser Dict.

Vielleicht alles gar nicht notwendig?


Prinzip: Einmal zum Programmstart wird unter Verwendung der in EBKuS_HOME/bundesstatistik/jghdef_<jahr>.py
ein Objekt der Klasse JGHBundesstatistik erzeugt und in einer globalen Variable im Modul ebkus.html.jghstatistik
gespeichert (ebkus.html.jghstatistik.jghdef_<jahr>). 
Wenn eine jghstatistik-Seite angefordert wird, wird die Methode jghdef_<jahr>.generate_html(jgh) aufgerufen, 
die aus Objekten aus htmlgen besteht, wobei alle bentigten Variablenwerte dem jgh-Objekt entnommen werden 
(jgh[<name>]). Es werden genau die Werte verwendet, die im jgh-Obekt sind. 

Bei der Initialisierung eines neuen jgh-Objekts (jgh07neu) kann das jghdef_<jahr>-Objekt mit anderen Methoden
befragt werden, zB zur Ermittlung von default-Werten oder speziellen Funktionen, die initiale Werte liefern
knnen.


"""


import logging
import os
import re
from ebkus.config import config
import ebkus.html.htmlgen as h
from ebkus.app.ebapi import Code, cc, Feld, FeldList, Tabelle, EE, today


# ffentliche Funktionen

# Bei Programmstart wird im Modul update ber get_jghdef(newest=True) von
# ebkus.update.UpdateBundesstatistik.__init__()
# aufgerufen. jghdef_<jahr>.py muss also valide sein, bevor die Datenbank updatet werden kann.

def get_jghdef(jahr=None, strict=True, allow_future=False, raise_exception=True):
    """
    Effizienter Zugriff auf die Bundesstatistikdefinition.
    
    An folgenden Stellen wird das gebraucht:
    - jghstatistik.py: Zur Auswahl des JGH-Formulars.
      - jghneu: nur fr das aktuelle Jahr und das Vorjahr, sonst irrelevant
                wenn Bundesstatistikdefinition noch nicht da, die vom Vorjahr
                nehmen.
                jghdef(jahr, strict=False)
      - jghupd: wird auch zur Anzeige verwendet. Bei Berichtsjahr < 2022
                2022 verwenden. Ohne Speicherbutton, update nur ab 2022.
                Bei Berichtsjahr >= 2022 wie jghneu
                jghdef(jahr, strict=False)
      - im JGH Formular: welche Jahre sollten angeboten werden zur Vernderung
                des Berichtsjahrs? Alle Jahre >= 2022 >= Fallbeginn <= today
    - ebupd.py: Berichtsjahr < 2022 wird abgelehnt. Ansonsten wie bei jghneu,
                das exakt Jahr bzw das Vorjahr, wenn nicht vorhanden
                jghdef(jahr, strict=False)
    - jghexport: Der Export sollte immer nur exakt mit der Bundesstatistikdefinition
                des Berichtsjahr gemacht werden. Sonst ist es ein Fehler.
                Wenn die Bundesstatistikdefinition noch nicht da ist und man meint, die
                vom Vorjahr geht auch, kann man die einfach umbenennen und das neue Berichtsjahr
                eintragen.
                get_jghdef(jahr, strict=True, raise_exception=True)
     - statistikauswertung: mglichst ohne jghdef auskommen, nur die Datenbankfelder
                verwenden. Das sollte gehen, da ja die Modifikationen ja alte Sachen nicht
                falsch machen sollten. Kleine Abweichungen bei Benennungen sind in Kauf zu nehmen.
      - update.py: Es muss immer die vom aktuellen Jahr genommen werden. Wenn es keine
                vom aktuellen Jahr gibt, die vom Vorjahr. Wenn es die auch nicht gibt, ist
                es ein Fehler.
                jghdef(jahr, strict=True)
    
    
    Alle Zugriffe mit allow_future==False schlieen jghdefs fr knftige Jahr aus.

    jahr=None --> die neueste BS-Definition
    jahr=jahr, strict==True --> nur das jahr, sonst fehler
    jahr=jahr, strict=False --> jahr, falls vorhanden, sonst: kleiner 2022 --> 2022 sonst neueste

    List_vorhanden liefert immer alle vorhandenden jghdefs auf, auch solche die in der 
    Zukunft liegen.

    Wenn keine Bundesstatistikdefinitionen vorhanden sind, wird immer ein Exception geworfen.

    Wenn keine passende Bundesstatistikdefinition gefunden wird:
    raise_exception==True, --> Exception
    sonst None
    """
    bm = get_bundesstatistikmanager()
    if allow_future:
        vorhanden_list = bm.vorhanden_list
    else:
        vorhanden_list = bm.vorhanden_list_sans_futur
    if not vorhanden_list:
        raise EE("Keine Bundesstatistikdefinition vorhanden")
    jghdef = None
    if jahr:
        # falls vorhanden, alles gut
        jghdef = bm.get_jghdef(jahr)
        if jghdef is None and not strict:
            if jahr < 2022:
                jghdef = bm.get_jghdef(2022)
            else:
                jghdef = bm.get_jghdef(vorhanden_list[-1])
    elif not strict:
        # nicht strict, kein Jahr: das neueste
        jghdef = bm.get_jghdef(vorhanden_list[-1])
    else:
        # strict, aber kein Jahr: geht nicht
        jghdef = None
    if jghdef is None:
        if raise_exception:
            if jahr:
                raise EE("Keine Bundesstatistikdefinition fr das Jahr {}".format(jahr))
            else:
                raise EE("Keine geeignete Bundesstatistikdefinition vorhanden")
    return jghdef

def list_jghdef_vorhanden():
    """
    Liefert eine Liste der Jahr, fr die Definition vorhanden sind.
    """
    return get_bundesstatistikmanager().list_jghdef_vorhanden()

# Singleton _BundesstatistikManager

_bundesstatistikmanager = None
def get_bundesstatistikmanager():
    global _bundesstatistikmanager
    if _bundesstatistikmanager is None:
        _bundesstatistikmanager = _BundesstatistikManager()
    return _bundesstatistikmanager

class _BundesstatistikManager(object):
    _regex_jghdef = re.compile(r"^jghdef_(?P<jahr>\d{4}).py$")
    def __init__(self):
        self.vorhanden = {} # jahr --> filename
        self.initialisiert = {} # jahr --> jghdef objekt
        self.bundesstatistik_dir = os.path.join(config.EBKUS_HOME,'bundesstatistik')
        for name in os.listdir(self.bundesstatistik_dir):
            match = self._regex_jghdef.match(name)
            if match:
                self.vorhanden[int(match.groupdict()['jahr'])] = name
        ty = today().year
        self.vorhanden_list = self.list_jghdef_vorhanden()
        self.vorhanden_list_sans_futur = [j for j in self.vorhanden_list if j <= ty]
        # assert contiguous? Drfen Lcken sein?
        # dieses Jahr und zwei frhere (falls vorhanden) initialisieren
        for jahr in self.vorhanden_list_sans_futur:
            if (ty - 2) <= jahr <= ty:
                self.init_jahr(jahr)
    def get_path_for_year(self, jahr):
        path = os.path.join(self.bundesstatistik_dir, "jghdef_{}.py".format(jahr))
        if os.path.isfile(path):
            return path
    def init_jahr(self, jahr):
        # print '_BundesstatistikManager.init_jahr()', jahr
        path = os.path.join(self.bundesstatistik_dir, self.vorhanden[jahr])
        dict_ = read_bundesstatistik_def(path)
        assert dict_['berichtsjahr'] == jahr, "Berichtsjahr stimmt nicht mit dem Jahr im Dateinamen berein."
        jghdef = JGHBundesstatistik(dict_)
        jghdef.validate() # muss funktionieren bevor die Datenbank auf dem neuesten Stand ist.
        self.initialisiert[jahr] = jghdef
        logging.info("Bundesstatistik fuer Berichtsjahr {} initialisiert".format(jahr))
        logging.info("Bundesstatistik fuer Berichtsjahr {} validiert".format(jahr))
        # write_bundesstatistik_def(
        #     jghdef._data,
        #     os.path.join(config.EBKUS_HOME, 'bundesstatistik', 'jghtemp.py'))
    
    def list_jghdef_vorhanden(self):
        vorhanden_list = self.vorhanden.keys()
        vorhanden_list.sort()
        return vorhanden_list
    def get_jghdef(self, jahr):
        """
        Liefert das initialisierte jghdef-Objekt fr das Jahr,
        oder None, fall nicht vorhanden.
        """
        if jahr not in self.vorhanden:
            return None
        if jahr not in self.initialisiert:
            self.init_jahr(jahr)
        return self.initialisiert[jahr]
        
    # def get_jghdef(self, jahr, newest, allow_future, raise_exception):
    #     assert bool(jahr) != bool(newest)
    #     if newest:
    #         vorhanden_list = self.list_jghdef_vorhanden()
    #         if not vorhanden_list:
    #             raise EE("Keine Bundesstatistikdefinition vorhanden")
    #         if allow_future:
    #             # print 'JAHR:', jahr
    #             jahr = vorhanden_list[-1]
    #         else:
    #             vorhanden_sans_futur = [j for j in vorhanden_list if j <= today().year]
    #             if not vorhanden_sans_futur:
    #                 raise EE("Keine gltige Bundesstatistikdefinition fr {} oder frher vorhanden".format(today().year))
    #             jahr = vorhanden_sans_futur[-1]
    #     if jahr not in self.vorhanden:
    #         if raise_exception:
    #             raise EE("Keine Bundesstatistikdefinition fr das Jahr {}".format(jahr))
    #         else:
    #             return None
    #     if jahr not in self.initialisiert:
    #         self.init_jahr(jahr)
    #     return self.initialisiert[jahr]
        
def write_bundesstatistik_def(dict_, path):
    """
    Schreibt dict mit deklarativer Beschreibung der Bundesstatistik
    in eine Datei.

    Die dict sollte alles String latin1-kodiert haben.
    Geschrieben wird eine utf8-Datei.
    """
    s = pp_dict(dict_)
    open(path, 'wb').write(s)
    print 'Written:', path
def read_bundesstatistik_def(filename):
    """
    Liest dict mit deklarativer Beschreibung der Bundesstatistik
    aus einer Datei.

    Die Datei sollte utf8-kodiert sein.
    Zurckgegeben wird ein dict mit allen strings in latin1.
    """
    with open(filename, 'rb') as f:
        contents = f.read()
    dict_ = eval(contents.decode('utf8').encode('latin1'))
    attributes = set()
    pp_dict(dict_, attributes=attributes)
    # for a in sorted(list(attributes)):
    #     print a
    return dict_

def pp_dict(dict_, **kw):
  """
  Wieder einlesbare dict als String

  Geht davon aus, dass alle Strings in dict_ latin1-kodiert sind.
  Ergebnis ist ein utf8-Kodierter String.
  """
  return '\n'.join(_pp_dict_mit_teilen(dict_, **kw))


_attribute_order = (
    'art',
    'gliederung',
    'titel',
    'name',
    'langname',
    'ebkus_name',
    'typ',
    'optionsgruppe',
    'bereichsgruppe',
    'feldlaenge',
    'minimum',
    'maximum',
    'vorgabe_wert',
    'fester_wert',
    'datei_satz_position',
    'core_csv_position',
    'core_csv_positionen',
    'core_csv_positionen_laenge',
    'core_csv_header_schreiben',
    'core_csv_berichtseinheitid_voranstellen',
    'core_csv_name',
    'core_csv_namen',
    'core_langname',
    'core_langnamen',
    'idev_csv_position',
    'idev_csv_positionen',
    'idev_csv_positionen_laenge',
    'idev_csv_header_schreiben',
    'idev_csv_name',
    'idev_csv_namen',
    'idev_langname',
    'idev_langnamen',
    'wireb_csv_position',
    'wireb_csv_name',
    'berichtsjahr',
    'datei_satz_laenge',
    'item',
    'code',
    'mini',
    'maxi',
    'text',
    'fett',
    'readonly',
    'nur_bei_beendet',
    'nur_bei_andauernd',
    'berechnet',
    'optional',
    'optionen',
    'bereiche',
)
# Damit die Namen besser zusammen sind zum bearbeiten
x_attribute_order = (
    'art',
    'ebkus_name',
    'name',
    'langname',
    'db_langname',
    'core_csv_name',
    'core_csv_namen',
    'core_langname',
    'core_langnamen',
    'idev_csv_name',
    'idev_csv_namen',
    'idev_langname',
    'idev_langnamen',
    'wireb_csv_name',
    'gliederung',
    'titel',
    'typ',
    'optionsgruppe',
    'feldlaenge',
    'minimum',
    'maximum',
    'vorgabe_wert',
    'fester_wert',
    'datei_satz_position',
    'datei_satz_positionen',
    'datei_satz_laenge',
    'core_csv_position',
    'core_csv_positionen',
    'core_csv_positionen_laenge',
    'core_csv_header_schreiben',
    'core_csv_berichtseinheitid_voranstellen',
    'idev_csv_position',
    'idev_csv_positionen',
    'idev_csv_positionen_laenge',
    'idev_csv_header_schreiben',
    'wireb_csv_position',
    'berichtsjahr',
    'item',
    'text',
    'fett',
    'readonly',
    'nur_bei_beendet',
    'nur_bei_andauernd',
    'berechnet',
    'optional',
)

def _pp_dict_mit_teilen(
        dict_, indent=0, lines=[], end_with_comma=False,
        part_attributes=('teile', 'optionen', 'statistik_items', 'fragebogen_elemente'),
        attribute_order=_attribute_order, attributes=set()):
                       
    """
    Pretty print dict.
    Geht davon aus, dass alle Strings in dict_ latin1-kodiert sind.
    Ergebnis sind Zeilen, die utf8-Kodiert sind.
    """
    def format_value(lines, indent, key, dict_):
        value = dict_[key]
        if isinstance(value, str):
            value = value.decode('latin1').encode('utf8')
            lines.append("{}{}='{}',".format('    '*(indent+1), key, value))
        else:
            lines.append("{}{}={},".format('    '*(indent+1), key, value))
    lines.append('    '*indent + 'dict(')
    keys = set(dict_.keys()) - set(part_attributes)
    attributes.update(keys)
    for key in attribute_order:
        if key in keys:
            format_value(lines, indent, key, dict_)
            keys.discard(key)
    for key in keys:
        format_value(lines, indent, key, dict_)

    def process_parts(part_name):
        parts = dict_.get(part_name, [])
        if parts:
            lines.append('    '*(indent+1) + part_name + '=[')
            for x in parts:
                _pp_dict_mit_teilen(x, indent=indent+2, lines=lines, end_with_comma=True, attributes=attributes)
            lines.append('    '*(indent+1) + '],')
    for part in part_attributes:
        process_parts(part)
    lines.append('    '*indent + ')' + (',' if end_with_comma else ''))
    return lines

def diff(d1, d2, verbose=0):
    """
    Vergleich zweier Bundesstatistikdefinitions-Dateien.
    
    :param d1: 
    Aufruf ber:
    ebkusdev.py bundesstatistik -j <jahr> -d <andere Datei im bundesstatistik-Verzeichnis>
    """
    jghdiff = JGHDiff(d1, d2, verbose=verbose)
    jghdiff.run()
class JGHDiff(object):
    """
    Diff gilt nur fr die aktuelle Version JGHDEF-Spec.,
    d.h. nur fr valide defs, die fehlerfrei eingelesen werden.

    Die Funktionen (teile_func etc.) liefern einen String mit
    dem Elemente von l1 und l2 identifizieren kann, z.B. 
    dem ebkus_name fr Items oder Gliederung fr Abschnitte.
    Das Diff kann dann sagen, ein Element ist nur in der einen Liste enthalten
    und nicht in der anderen.
    """
    part_attributes = frozenset(('teile', 'optionen', 'statistik_items', 'fragebogen_elemente'))
    def teile_func(eingabe):
        art = eingabe.get('art')
        if art == 'Eingabe':
            return "Eingabe: {}".format(eingabe['item'])
        else:
            return "{}: {}".format(art, eingabe['text'][:25])
    def statistik_items_func(item):
        "Alle Items haben einen identifizierenden Namen."
        return "Item: {}".format(item['name'])
    def fragebogen_elemente_func(abschnitt):
        "Hier gibt es nur Abschnitt mit Gliederungsbezeichnung und Zwischentexte."
        art = abschnitt.get('art')
        if art == 'Abschnitt':
            return "Abschnitt: {}".format(abschnitt['gliederung'])
        else:
            return "{}: {}".format(art, abschnitt['text'][:25])
    def optionen_func(option):
        "Optionen haben einen Code. Zwischenberschriften haben einen Text."
        code = option.get('code')
        if code is not None:
            return "Option: {}".format(code)
        else:
            return  "Optionzwischentext: {}".format(option['text'][:25])
    ident_funcs = dict(
        teile=teile_func,
        statistik_items=statistik_items_func,
        fragebogen_elemente=fragebogen_elemente_func,
        optionen=optionen_func,
        )
    def __init__(self, dict1, dict2, verbose=0):
        self.d1 = dict1
        self.d2 = dict2
        self.verbose = verbose
    def run(self):
        if self.d1 == self.d2:
            if self.verbose:
                print 'identisch'
        else:
            print 'Bundesstatistik'
            self.diff(self.d1, self.d2, '')
    def diff(self, d1, d2, indent):
        if d1 == d2:
            if self.verbose:
                print 'identisch'
        else:
            # print "{}{}".format(indent, title)
            self.diff_skalar(d1, d2, indent)
            for attr in self.part_attributes:
                if attr in d1 or attr in d2:
                    self.diff_parts(d1.get(attr, []),
                                    d2.get(attr, []),
                                    attr,
                                    indent + '  ')
    def diff_skalar(self, d1, d2, indent):
        # print 'DIFF_SKALAR'
        keys1 = set(d1.keys()) - self.part_attributes
        keys2 = set(d2.keys()) - self.part_attributes
        only1 = keys1 - keys2
        only2 = keys2 - keys1
        both = keys1 & keys2
        diff = set()
        for k in both:
            if d1[k] != d2[k]:
                diff.add(k)
        # print 'ONLY1', only1
        # print 'ONLY2', only2
        # print 'BOTH', both
        # print 'DIFF', diff
        # if only1 or only2 or diff:
        #     print "{}{}:".format(indent, title)
        if only1:
            print "{}  Nur in 1: {}".format(indent, ', '.join([repr(x) for x in list(only1)]))
        if only2:
            print "{}  Nur in 2: {}".format(indent, ', '.join([repr(x) for x in list(only2)]))
        if diff:
            for k in diff:
                print "{}  Wert von '{}': \"{}\" <> \"{}\"".format(indent, k, d1[k],d2[k])
        # return only1, only2, diff

    def diff_parts(self, l1, l2, attr, indent=''):
        """
        """
        #print 'DIFF_PARTS', attr
        if l1 == l2:
            return
        func = self.ident_funcs[attr]
        l1dict = dict((func(x), x) for x in l1)
        # print l1dict.keys()
        l2dict = dict((func(x), x) for x in l2)
        # print l2dict.keys()
        only1 = set()
        only2 = set()
        diff = set()
        for k, v in l1dict.items():
            if k not in l2dict:
                only1.add(k)
            else:
                if l1dict[k] != l2dict[k]:
                    diff.add(k)
        for k, v in l2dict.items():
            if k not in l1dict:
                only2.add(k)
        #print only1, only2, diff
        if only1 or only2 or diff:
            print "{}{}".format(indent, attr)
        if only1:
            print "{}  Nur in 1: {}".format(indent, ', '.join([repr(x) for x in list(only1)]))
        if only2:
            print "{}  Nur in 2: {}".format(indent, ', '.join([repr(x) for x in list(only2)]))
        if diff:
            for k in diff:
                print "{}  Verschieden: '{}'".format(indent, k)
                # Nur hier die Rekursion
                self.diff(l1dict[k], l2dict[k], indent + '  ')

class _JGHBase(object):
    """
    Basisklasse fr alle Elemente der Bundesstatistikdefinition.

    
    Fr jedes Element knnte obligatorische und optionale Attribute
    definiert werden. validate() misslingt, wenn obligatorische Attribute fehlen
    bzw. nicht aufgefhrte Attribute prsent sind.
    """
    mandatory_attributes = []
    optional_attributes = []
    def __init__(self, root, parent, data, art=None):
        self._data = data
        # root ist das Basisobjekt vom Typ JGHBundesstatistik
        self.root = root
        # das unmittelbar bergeordnete Objekt
        self.parent = parent
        self.__class__ = self.get_class(art)
        self._init()
    def validate(self):
        # print self.art, self.mandatory_attributes
        for attr in self.mandatory_attributes:
            if isinstance(attr, tuple):
                # eins muss einen Wert haben
                assert any([(getattr(self, x) is not None) for x in attr]), 'Kein Wert fuer eines von {}'.format(attr)
            else:
                assert getattr(self, attr) is not None, "Kein Wert fuer Attribut '{}' fuer Objekt {}".format(attr, self)
        for key in self._data.keys():
            assert key == 'art' or key in self.mandatory_attributes or key in self.optional_attributes, "Unbekanntes Attribut: {} in {}".format(key, self._data)
        self._validate()
    def _validate(self):
        # In Unterklassen definieren
        pass
    def get_class(self, art):
        art_to_class_v2 = dict(
            Bundesstatistik=JGHBundesstatistik,
            Abschnitt=JGHAbschnitt,
            AbschnittK=JGHAbschnittK,
            AbschnittFalldaten=JGHAbschnittFalldaten,
            Item=JGHItem,
            Eingabe=JGHEingabe,
            EingabeK=JGHEingabeK,
            EingabeFalldaten=JGHEingabeFalldaten,
            Option=JGHOption,
            Bereich=JGHBereich,
            ZwischenAbschnittText=JGHZwischenAbschnittText,
            Zwischentext=JGHZwischentext,
        )
        if art is None:
            art = self._data['art']
        if art == 'Item':
            item_to_class_v2 = dict(
                Zeichenkette=JGHItemZeichenkette,
                Ganzzahl=JGHItemGanzzahl,
                Option=JGHItemOption,
                Checkbox=JGHItemCheckbox,
                )
            class_ = item_to_class_v2[self.typ]
        else:
            class_ = art_to_class_v2[art]
        return class_

    def _init(self):
        """Kann in einer Unterklasse implementiert werden."""
        pass
    def __getattr__(self, key):
        """
        Zugriff auf die Werte aus der Definitionsdict.
        Nicht vorhandene Keys fhren nicht zu einer Exception,
        sondern geben None zurck.
        """
        return self._data.get(key)
    @property
    def text_class(self):
        if self.fett:
            return 'legendtext'
        return 'labeltext'
    def get_item(self, name=None, ebkus_name=None):
        """
        Item Objekt nachschlagen im root-Objekt
        """
        if name:
            return self.root.item_dict.get(name)
        if ebkus_name:
            return self.root.item_dict_by_ebkus_name(ebkus_name)
        return None
    @property
    def abschnitt(self):
        """
        Abschnitt-Objekt nachschlagen in bundesstatistik.items entlang
        der parent-Hierarchie.
        """
        if self.art in ('Abschnitt', 'AbschnittFalldaten'):
            return self
        if self.art == 'Item':
            return self._abschnitt
        return self.parent.abschnitt
    def __str__(self):
        return "<{self.__class__.__name__} _data={self._data}".format(self=self)

class JGHBundesstatistik(_JGHBase):
    """
    Eine Instanz dieser Klasse verkrpert die Bundesstatistikdefinition fr ein Berichtsjahr.
    Wird vom _BundesstatistikManager() aufgerufen mit einer einer dict (jghdef_<jahr>.py),
    die als Datei im Ordner <ebkus_home>/bundesstatistik zur Verfgung steht.
    """
    separator = h.Tr(cells=[h.Dummy(height=12)])
    mandatory_attributes = (
        'berichtsjahr',
        'statistik_items',
        'fragebogen_elemente',
    )
    optional_attributes = (
        'datei_satz_laenge',
        'core_csv_positionen_laenge',
        'idev_csv_positionen_laenge',
        'core_csv_header_schreiben',
        'core_csv_berichtseinheitid_voranstellen',
        'idev_csv_header_schreiben',
    )
    def __init__(self, data_dict):
        super(JGHBundesstatistik, self).__init__(
            self, None, data_dict, art='Bundesstatistik')
    def _init(self):
        self.root = self
        # interne items und Abschnitt Falldaten hinzufgen, sind nicht in der ffentlichen dict
        statistik_items = _interne_items + self.statistik_items
        fragebogen_elemente = [_abschnitt_falldaten] + self.fragebogen_elemente
        for x in statistik_items:
            assert x['art'] == 'Item', \
                "Falsche Art an dieser Stelle: {} fr {x}".format(x.get('art'), x)
        for x in fragebogen_elemente:
            assert x['art'].startswith('Abschnitt') or x['art'] == 'ZwischenAbschnittText', \
                "Falsche Art an dieser Stelle: {} fr {x}".format(x.get('art'), x)
        self.option_katalog = {} # wird von JGHItem._init() gefllt
        self.item_dict = dict((x['name'], _JGHBase(self, self, x)) for x in statistik_items)
        self.item_dict_by_ebkus_name = dict((x.ebkus_name, x)
                                            for x in self.item_dict.values())
        self.teile = [_JGHBase(self, self, x) for x in fragebogen_elemente]
        self.fb_items = self.get_fb_items()
        # self.item_info()
        # self.export_item_info = self.get_export_item_info(pp=True)
        self.export_item_info = self.get_export_item_info(pp=False)
        kopf_items = set([x['item'] for x in _abschnitt_falldaten['teile']])
        # print self.fb_items
        # print kopf_items
        fb2_items = set([x.name for x in self.item_dict.values()
                         if not(x.berechnet or x.fester_wert or x.name in kopf_items)
                         ])
        # kein unterschied:
        # print 'SYMM DIFF', fb2_items.symmetric_difference(self.fb_items)
        # for x in self.item_dict.values():
        #     if x.berechnet or x.fester_wert is not None or x.name in kopf_items:
        #         continue
        #     print x.name
    def _validate(self):
        # Nur um die Felder zu validieren, danach gleich wieder freigeben
        self.jgh_db_felder_dict = dict((f['feld'], f) for f in FeldList(
            where="tab_id = {}".format(Tabelle(tabelle='jghstat07')['id'])))
        for item in self.item_dict.values():
            # Bundesstatistikdefinition darf keine neuen Datenbankfelder einfhren.
            # Dafr sind die Joker.
            if item.name in self.fb_items:
                assert item.ebkus_name in self.jgh_db_felder_dict, \
                    "Fragebogenitem (ebkus_name: '{}') nicht in der Datenbank".format(
                        item.ebkus_name)
            item.validate()
        self.jgh_db_felder_dict = None
        for teil in self.teile:
            teil.validate()
        # die get_*_export_*()-Methoden machen die ntigen Prfungen
        if self.datei_satz_laenge:
            items = self.get_datei_export_items()
        if self.idev_csv_positionen_laenge:
            item_data = self.get_idev_export_item_data()
            assert len(item_data) == self.idev_csv_positionen_laenge 
        if self.core_csv_positionen_laenge:
            item_data = self.get_core_export_item_data()
            assert len(item_data) == self.core_csv_positionen_laenge 
    def get_item(self, name=None, ebkus_name=None):
        if name:
            return self.item_dict.get(name)
        elif ebkus_name:
            for item in self.fb_item_iter():
                if item.ebkus_name == ebkus_name:
                    return item
    def fb_item_iter(self):
        for name in self.fb_items:
            item = self.item_dict[name]
            yield item
    def get_fb_items(self):
        """
        Liefert Menge von item.name.

        Alle Items aus dem EBKuS-Bundesstatistikformular aus den Abschnitten A bis P.
        Keine Kopfitems.
        Keine Items, die in der offiziellen Bundesstatistik drin sind, aber in EBKuS
        nicht auftauchen, weil EBKuS auf Erziehungsberatung beschrnkt ist.
        Siehe self.item_info()

        Daraus ergibt sich auch, dass diese Items alle in der Datenbank gespeichert
        werden. Wenn sie berechnet werden knnten, bruchten sie nicht im Fragebogen
        zu stehen.
        """
        res = set()
        for a in self.teile[1:]: # Ab Abschnitt A, ohne Falldaten
            teile = a.teile if a.teile else []
            for x in teile:
                if x.item:
                    res.add(x.item)
                elif x.items:
                    for y in x.items:
                        res.add(y)
        return res

    @property
    def available_export_formats(self):
        if self._available_export_formats is None:
            formats = ()
            if self.core_csv_positionen_laenge:
                formats += ('core',)
            if self.idev_csv_positionen_laenge:
                formats += ('idev',)
            if self.datei_satz_laenge:
                formats += ('datei',)
            self._available_export_formats = formats
        return self._available_export_formats
    def get_export_item_info(self, pp=False):
        """
        Liefert eine List von Tupeln:
        (item, datei_satz_position, idev_csv_position, core_csv_position,
        idev_name, core_name)
        Das item-Objekt kann mehrfach auftauchen, nmlich fr solche Items wie
        e2e3leer, die eine Menge von items zusammenfassen, die fr EBKuS niemals
        einen Wert haben, aber in den Exportdatensatz als Leerstelle eingehen 
        mssen. Insbesondere fr den Export ber CSV-Dateien wichtig.
        """
                
        def item_iter():
            for item in self.item_dict.values():
                dpos, dposen, ipos, iposen, cpos, cposen = (item.datei_satz_position,
                                                            item.datei_satz_positionen,
                                                            item.idev_csv_position,
                                                            item.idev_csv_positionen,
                                                            item.core_csv_position,
                                                            item.core_csv_positionen,
                                                            )
                if not(dpos or dposen or ipos or iposen or cpos or cposen):
                    # item geht in keine zu exportierende Statistik ein
                    continue
                if dposen or iposen or cposen:
                    # Leeres Item das fr mehrere Positionen steht
                    lengths = [len(x) for x in (dposen, iposen, cposen) if x is not None]
                    # alle Tuple mssen gleichlang sein
                    length = lengths[0]
                    for x in lengths[1:]:
                        assert x == length
                    for dpos, ipos, cpos, iname, cname in zip(
                            item.datei_satz_positionen if item.datei_satz_positionen else (None,)*length,
                            item.idev_csv_positionen if item.idev_csv_positionen else (None,)*length,
                            item.core_csv_positionen if item.core_csv_positionen else (None,)*length,
                            item.idev_csv_namen if item.idev_csv_namen else (None,)*length,
                            item.core_csv_namen if item.core_csv_namen else (None,)*length,
                    ):
                        yield (item, dpos, ipos, cpos, iname, cname)
                else:
                    # Normales Item, das einen Wert haben kann
                    yield (item, dpos, ipos, cpos, item.idev_csv_name, item.core_csv_name)
        items = list(item_iter())
        if pp:
            for x in sorted(items, key=lambda x: x[2]): print x
        return items
    def get_datei_export_items(self):
        """
        Die Items, fr die Daten in den Export eingehen, in der richtigen
        Reihenfolge in self.items ablegen.

        Die Fllitems wie e2e3leer mssen hier nicht aufgesplittet werden,
        da im Datensatz einfach eine entsprechend lange Leerstelle gesetzt wird,
        die als fester Wert schon definiert ist.

        Vollstndigkeit und Konsistenz der Items fr den Export prfen.
        Hier: Intervalle
        """
        items = [i for i in self.item_dict.values()
                 if i.datei_satz_position or i.datei_satz_positionen]
        items.sort(key=lambda x: x.datei_satz_position
                   if x.datei_satz_position else x.datei_satz_positionen[0])
        # check
        last_bis = 0
        for item in items:
            if item.datei_satz_position:
                von, bis = item.datei_satz_position
            else:
                von, bis = item.datei_satz_positionen[0][0], item.datei_satz_positionen[-1][1]
            assert von == last_bis + 1, "Fehler in den datei-Positionen entdeckt bei {}".format(item.ebkus_name)
            last_bis = bis
        assert last_bis == self.datei_satz_laenge
        return items
    def get_idev_export_item_data(self):
        """
        Die Items, fr die Daten in den Export eingehen, in der richtigen
        Reihenfolge in item_data ablegen.
        Die Position in der Liste entspricht der idev_csv_position.
        Items, die mehrere leere Positionen abdecken (wie zB e2e3leer), 
        kommen mehrfach vor.

        Vollstndigkeit und Konsistenz der Items fr den Export prfen.
        Hier: Positionen in der CSV-Datei
        """
        item_data = []
        for item, _,ipos,_,idev_csv_name,_ in self.export_item_info:
            if ipos is not None:
                item_data.append((item, ipos, idev_csv_name))
        item_data.sort(key=lambda x: x[1])
        # check positions
        last = None
        for pos, (item, ipos, name) in enumerate(item_data, start=1):
            # print 'idev-check:', pos, ipos, item.idev_csv_position
            assert pos == ipos, "Fehler in idev-csv-Positionsdaten entdecke bei Item: {}{}".format(
                item.ebkus_name,
                '' if last is None else " (Davor war Item {})".format(last.ebkus_name))
            if isinstance(item.idev_csv_positionen, tuple):
                assert ipos in item.idev_csv_positionen
            else:
                assert ipos == item.idev_csv_position
            if self.idev_csv_header_schreiben:
                assert name, "Kein Name fr idev-csv-Header-Attribut: {}".format(item.ebkus_name)
            last = item
        # print 'idev-items ok: ', len(self.item_data)
        return item_data
    def get_core_export_item_data(self):
        """
        Die Items, fr die Daten in den Export eingehen, in der richtigen
        Reihenfolge in self.items ablegen.
        Die Position in der Liste entspricht der idev_csv_position.
        Items, die mehrere leere Positionen abdecken (wie zB e2e3leer), 
        kommen mehrfach vor.

        Vollstndigkeit und Konsistenz der Items fr den Export prfen.
        Hier: Positionen in der CSV-Datei
        """
        item_data = []
        for item, _,_,cpos,_,name in self.export_item_info:
            if cpos is not None:
                item_data.append((item, cpos, name))
        item_data.sort(key=lambda x: x[1])
        # for pos, (item, cpos, name) in enumerate(item_data, start=1):
        #     print 'GET_CORE_EXPORT_ITEM_DATA', pos, (item, cpos, name)
        # check
        last = None
        for pos, (item, cpos, name) in enumerate(item_data, start=1):
            #print 'GET_CORE_EXPORT_ITEM_DATA', pos, (item, cpos, name)
            assert pos == cpos, "Fehler in core-csv-Positionsdaten entdecke bei Item: {}{}".format(
                item.ebkus_name,
                '' if last is None else " (Davor war Item {})".format(last.ebkus_name))
            if isinstance(item.core_csv_positionen, tuple):
                assert cpos in item.core_csv_positionen
            else:
                assert cpos == item.core_csv_position
            last = item
            if self.core_csv_header_schreiben:
                assert name, "Kein Name fr core-csv-Header-Attribut: {}".format(item.ebkus_name)
        return item_data
    def item_info(self):
        """
        Items in Gruppen einteilen. Eine Gruppe ist eine Menge von Namen, 
        wie in item.name (nicht ebkus_name!).
        Gruppen:
          - in_fd: item wird in Falldaten (Kopfdaten im JGH-Formular) verwendet
          - in_fb: item wird in den Abschnitten des JGH-Formulars verwendet und kann
                   von dort einen Wert erhalten.
          - in_bs: item wird in Bundesstatistik verwendet (Datensatz, IDEV oder Core)
          - in_db: item steht mit ebkus_name als Feldname in der Datenbank
        """
        alle_items = set(self.item_dict.keys())
        fd_items = set(eingabe.item for eingabe in self.teile[0].teile)
        fb_items = self.fb_items
        
        bs_items = set(item.name for item in self.item_dict.values()
                       if item.datei_satz_position or item.idev_csv_position or item.core_csv_position or
                       item.idev_csv_positionen or item.core_csv_positionen)
        db_felder = dict((f['feld'], f) for f in FeldList(where="tab_id = {}".format(Tabelle(tabelle='jghstat07')['id'])))
        db_items = set(item.name for item in self.item_dict.values()
                       if item.ebkus_name in db_felder)
        def pp(title, set, typ=True, in_db=True):
            print
            print title
            print
            for x in set:
                item = self.item_dict.get(x)
                in_db_ind = typ_ind = ''
                if in_db:
                    in_db_ind = " ***** NOT IN DB" if x not in db_items else ""
                if typ:
                    typ_ind = "{}({})".format(item.typ, item.feldlaenge)
                print item.name, item.ebkus_name, typ_ind, in_db_ind
            print

        pp('KOPFITEMS', fd_items)
        pp('FRAGEBOGENITEMS', fb_items)
        pp('SONSTIGE BS ITEMS', bs_items - fb_items - fd_items)
        pp('REST ITEMS', alle_items - fb_items - fd_items - bs_items)
        pp('DB ITEMS', db_items - fb_items)
        # db-Felder, die nicht in db_items sind
        rest_db_felder = set(db_felder.keys())
        for x in self.item_dict.values():
            rest_db_felder.discard(x.ebkus_name)
        print 'REST DB FELDER:', rest_db_felder
        print
        for i in self.fb_item_iter():
            print i.name
        print
    def generate_teile(self, jgh):
        # liefert ersteinmal rows, d.h. Liste von <tr>-Elementen mit je einem <td>-Element,
        # darin in diesem Fall fieldsets
        rows = []
        for x in self.teile:
            rows.append(x.generate_html(jgh))
            rows.append(self.separator)
        rows =  rows[:-1]
        # print rows
        return rows
    def text_zeile(self, parent, text, fett=False,
                   class_=None,
                   gliederung=None, # text der 1. Spalte
                   dummy_one=None, # wert ist width des ersten dummies
                   dummies_pre=None, # weitere dummies ohne width
                   dummies_post=None, # dummies danach ohne width
                   n_col=1, # Spaltenanzahl fr Stringelement
                   tip=None,
                   ):

        klass = class_ if class_ else ('legendtext' if fett else 'labeltext')
        res = []
        if gliederung:
            res.append(h.String(string=gliederung,
                                class_=klass,
                                valign='top'))
        if dummy_one:
            res.append(h.Dummy(width="{}%".format(dummy_one)))
        if dummies_pre:
            for x in range(dummies_pre):
                res.append(h.Dummy())
        res.append(h.String(string=text,
                            class_=klass,
                            n_col=n_col,
                            tip=tip))
        if dummies_post:
            for x in range(dummies_post):
                res.append(h.Dummy())
        return res

class JGHZwischenAbschnittText(_JGHBase):
    mandatory_attributes = ('text',)
    optional_attributes = ('fett',)
    def generate_html(self, jgh):
        res = h.Tr(cells=[h.String(string=self.text,
                                   class_='titeltext',
                                   height=12)])
        return res
class JGHZwischentext(_JGHBase):
    """
    Erscheint in den Teilen eines Abschnitts anstelle eines Eingabeelements.
    Kann auch ein Gliederungspunkt haben.

    Wird auch fr Zwischenberschriften in Optionen verwendet.
    """
    mandatory_attributes = ('text',)
    optional_attributes = ('fett', 'gliederung')
    def generate_html(self, jgh):
        # print 'ZWISCHENTEXT: ', self.text
        res = self.root.text_zeile(self.parent, self.text, fett=self.fett,
                                   gliederung=self.gliederung,
                                   n_col=2 if self.gliederung else 3)
        return [res]
class JGHAbschnitt(_JGHBase):
    """Standardabschnitt mit einspaltigem Layout."""
    mandatory_attributes = ('gliederung', 'titel', 'teile')
    optional_attributes = ('layout',)
    def _validate(self):
        for teil in self.teile:
            teil.validate()
    def _init(self):
        self.teile = [_JGHBase(self.root, self, x) for x in self.teile]
        # if self.gliederung == 'A':
        #     for x in self.teile:
        #         print x
    def generate_html(self, jgh):
        daten = []
        for x in self.teile:
            # print 'ART:', x.art
            res = x.generate_html(jgh)
            # xprint res
            daten += res
        # if self.gliederung == 'A':
        #     print 'ABSCHNITT A', daten
        abschnitt_html = h.FieldsetInputTable(
            legend="{} {}".format(self.gliederung, self.titel),
            daten = daten
        )
        return abschnitt_html
class JGHAbschnittK(JGHAbschnitt):
    """Spezialabschnitt K fr die Grnde."""
    def generate_html(self, jgh):
        daten = []
        for x in self.teile:
            # print 'ART:', x.art
            res = x.generate_html(jgh)
            # print res
            daten += res
        abschnitt_html = h.FieldsetInputTable(
            legend="{} {}".format(self.gliederung, self.titel),
            daten = daten
        )
        return abschnitt_html
        
class JGHAbschnittFalldaten(JGHAbschnitt):
    """Spezialabschnitt Falldaten mit zweispaltigem Layout."""
    def _init(self):
        self.teile = [_JGHBase(self.root, self, x, art='EingabeFalldaten') for x in self.teile]
    def generate_html(self, jgh):
        daten = self._get_daten_with_layout(jgh)
        abschnitt_html = h.FieldsetInputTable(
            legend="{}".format(self.titel),
            daten = daten
        )
        return abschnitt_html
    def _get_daten_with_layout(self, jgh):
        daten = []
        daten.append(self.root.text_zeile(
            self.parent,
            "Bundesstatistikversion {}".format(self.root.berichtsjahr),
            class_='smalltext',
            dummies_pre=3, dummies_post=0,
            tip="Die Version der von EBKuS verwendeten Definitionsdatei fr die Bundesstatistik"))
        eingabe_dict_by_item_name = dict((x.item, x) for x in self.teile if x.item)
        eingabe_dict_by_item_name.update(Dummy=JGHDummy())
        for row in self.layout:
            for name in row:
                    assert name in eingabe_dict_by_item_name, \
                            "{} nicht in {}".format(name, item_dict.keys())
            assert len(row) in (1,2), \
                    "layout in Abschnitt {} nicht korrekt".format(self.abschnitt)
            if len(row) == 1:
                # Ganze Breite einnehmen
                name = row[0]
                if name != 'Wohntausserhalb' or jgh.get('wohnt_ausserhalb').strip():
                    # print name, ">{}<".format(jgh.get('wohnt_ausserhalb'))
                    # wohnt_ausserhalb weglassen, wenn kein Wert vorhanden
                    eingabe = eingabe_dict_by_item_name[name]
                    eingabe.n_col = 4
                    zeile =  [eingabe.generate_html(jgh)]
                    # print 'ZEILE SINGLE:', name, '--->',  zeile, eingabe.n_col
                else:
                    zeile = []
            elif row[0] == 'Jahr': # Spezialbehandlung mit dropdown-menu
                zeile = self.get_berichtsjahr(jgh) + [eingabe_dict_by_item_name[row[1]].generate_html(jgh)]
            else:
                assert len(row) == 2
                name1, name2 = row[0], row[1]
                left_item = eingabe_dict_by_item_name[name1]
                right_item = eingabe_dict_by_item_name[name2]
                left_item.zweispaltig = True
                right_item.zweispaltig = True
                zeile = [left_item.generate_html(jgh), 
                          right_item.generate_html(jgh)]
                # print 'ZEILE PAIR:', name1, name2, '--->',  zeile
            daten.append(zeile)
        return daten
    def get_berichtsjahr(self, jgh):
        """
        Bei abgeschlossenen Fllen:
        - Berichtsjahr wird readonly angezeigt
        Bei offenden Fllen:
        - es werden Berichtsjahre in Form eines Dropdown-Mens angeboten, 
          zu denen man wechseln kann, und zwar:
          - das kleinste Berichtsjahr, zu dem gewechselt werden kann, ist 2022.
            Die Bearbeitung der Statistik fr frhere Berichtsjahre ist nicht mehr mglich
            (auch nicht ntig, da fr diese Jahre nichts mehr abgegeben werden wird).
        - ansonsten alle Jahr ab dem Jahr des Fallbeginns bis zu dem aktuellen Jahr.
        """
        berichtsjahr = int(jgh['jahr'])
        def get_url(jahr_change):
            if jgh['file'] == 'jgh07einf':
                url = "jgh07neu?jghid={jghid}&jahr_change={jahr_change}&fallid={fall_id}".format(jahr_change=jahr_change, **jgh)
            else:
                url = "updjgh07?jghid={jghid}&jahr_change={jahr_change}".format(jahr_change=jahr_change, **jgh)
            return url
                
        if jgh['fall__aktuell']:
            # Man kann grundstzlich zu keinem Berichtsjahr vor 2022 wechseln
            min_jahr = max(2022, jgh['fall__bgy'])
            max_jahr = today().year
            alt_jahre = [j for j in range(min_jahr, max_jahr+1) if j != berichtsjahr]
            url_name_pairs = [(get_url(jahr_change), jahr_change) for jahr_change in alt_jahre]
            res = [h.String(string='Berichtsjahr', class_='legendtext', align='right'),
                   h.DropdownMenu(name=berichtsjahr,
                                  url_name_pairs=url_name_pairs,
                                  button_class='button',
                                  ),
                   ]
        else:
            # Jahr darf nicht gendert werden fr abgeschlossenen Fall
            res = [h.TextItem(label='Berichtsjahr',
                              label_class='legendtext',
                              label_align='right',
                              name='', # keine bermittlung, jahr steht in hidden
                              value=berichtsjahr,
                              readonly=True,
                              class_='textbox100prz',
                              ),
                   ]

        return res

    
    
class JGHItem(_JGHBase):
    optional_attributes = ('fester_wert',
                           'vorgabe_wert',
                           'readonly',
                           'optionen',
                           'bereiche',
                           'bereichsgruppe',
                           'datei_satz_position',
                           'datei_satz_positionen',
                           'core_csv_position',
                           'core_csv_positionen',
                           'core_csv_name',
                           'core_csv_namen',
                           'core_langname',
                           'core_langnamen',
                           'idev_csv_position',
                           'idev_csv_positionen',
                           'idev_csv_name',
                           'idev_csv_namen',
                           'idev_langname',
                           'idev_langnamen',
                           'wireb_csv_position',
                           'wireb_csv_name',
                           'nur_bei_beendet',
                           'nur_bei_andauernd',
                           'berechnet',
                           'optional',
                           'radio',
                           )
    def item_message(self, message):
        item_kennzeichung = "%s / %s / %s" % (self.abschnitt.gliederung, self.abschnitt.titel, self.eingabe.text)
        return "{}:<br>{}".format(message, item_kennzeichung)
    def is_andauernd(self, form):
        return str(form.get('hda')) == str(Code(kat_code='ja_nein', code='1')['id'])
    def get_and_check_value(self, form):
        """
        Werte aus der CGI-Form holen und berprfen.
        Wird fr ebupd verwendet.
        Checks:
        - Typ-spezifisch
        - nur bei andauernd, nur bei abgeschlossen
        - optional
        """
        raw_value = form.get(self.ebkus_name)
        value = self.check(raw_value)
        # print 'GET_AND_CHECK_VALUE', self.ebkus_name, raw_value, value, self.optional, self.is_andauernd(form)

        if value is None:
            fehlt = True
            if self.nur_bei_beendet and self.is_andauernd(form):
                fehlt = False
            if self.nur_bei_andauernd and not self.is_andauernd(form):
                fehlt = False
            if self.optional:
                fehlt = False
            if fehlt:
                raise EE(self.item_message('Fehlt'))
        else:
            if self.nur_bei_beendet and self.is_andauernd(form):
                raise EE(self.item_message('Nur bei beendeter Hilfe ausfllen'))
            if self.nur_bei_andauernd and not self.is_andauernd(form):
                raise EE(self.item_message('Nur bei andauernder Hilfe ausfllen'))
        return value
    def get_datei_export_value(self, jgh):
        value = self.get_export_value(jgh)
        value_formated = self.datei_format(value)
        assert len(value_formated) == self.feldlaenge
        return value_formated
    def get_csv_export_value(self, jgh):
        value = self.get_export_value(jgh)
        # print 'GET_CSV_EXPORT_VALUE:', self.ebkus_name, jgh.get(self.ebkus_name), value
        return self.csv_format(value)
        
class JGHItemOption(JGHItem):
    mandatory_attributes = ('name', 'langname', 'ebkus_name', 'typ', 'feldlaenge',
                            'optionsgruppe')
    def _init(self):
        """
        Optionen in einer dict kat_code --> optionsliste bei 
        bundesstatistik-objektablegen (parent von item).
        Falls in den Optionen Zwischenberschriften vorkommen (kein code!)
        item mit dem Attribute optionen_mit_zwischenueberschriften=True
        versehen.
        """
        einrueck_space = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
        optionen = self._data.get('optionen', [])
        kat_code = self._data['optionsgruppe']
        # evt. schon von einem anderen Item vorhanden
        options_vorhanden = self.root.option_katalog.get(kat_code)
        if optionen:
            optionen_maxlen = 0
            options = []
            for o in optionen:
                art = 'Option'
                if 'code' not in o:
                    # print o
                    art = 'Zwischentext'
                text = o.get('text', '')
                optionen_maxlen = max(len(text), optionen_maxlen)
                if text.startswith('  '):
                    text =  "{}{}".format(einrueck_space, text.lstrip())
                    o['text'] = text 
                options.append(_JGHBase(self.root, self, o, art=art))
            self.optionen_maxlen = optionen_maxlen
            # Falls die Optionsgruppe (kat_code) mehrfach definiert wurde,
            # prfen, ob beide identisch sind.
            if options_vorhanden:
                assert len(options) == len(options_vorhanden), kat_code
                for i in range(len(options)):
                    o, ov = options[i], options_vorhanden[i]
                    assert o.text == ov.text
                    assert o.code == ov.code
                    assert o.art == ov.art
            else:
                self.root.option_katalog[kat_code] = options

    @property
    def optionen(self):
        return self.root.option_katalog[self.optionsgruppe]
    @property
    def radio(self):
        "Wenn True, Auswahl mit Radiobuttons statt Select"
        self._radio = self._data.get('radio')
        if self._radio is not None:
            return self._radio
        for option in self.root.option_katalog[self.optionsgruppe]:
            if option.code is None:
                self._radio = True
                return True
        return None
    @radio.setter
    def radio(self, value):
        self._radio = True if value else None
    def _validate(self):
        "Falls in der DB, muss der Typ passen"
        feld = self.root.jgh_db_felder_dict.get(self.ebkus_name)
        if feld:
            # print 'FELD:', feld
            # kat_code (optionsgruppe) darf nicht verndert werden!
            assert self.optionsgruppe == feld['kat_code'], "Optionsgruppe '{}' entspricht nicht " \
                "dem kat_code '{}' des Datenbankfeldes".format(self.optionsgruppe, feld['kat_code'])
        # print 'SELF.OPTIONS:', self.options
        options = self.root.option_katalog.get(self.optionsgruppe)
        if options:
            for option in options:
                # print option.__class__
                option.validate()
    def check(self, raw_value):
        """
        Checks:
        - gltige Option bei Option
        """
        value = None
        try:
            value = int(raw_value)
        except (ValueError, TypeError):
            pass
        # value ist jetzt ein Zahl oder None
        if value is not None:
            try:
                # muss in der Datenbank sein, off wird aber nicht berprft
                code = Code(value)
            except:
                # sollte nie passieren
                raise EE(self.item_message('Falscher Code bermittelt'))
        return value
    def get_export_value(self, jgh):
        """Liefert einen gltigen Code oder None"""
        if self.fester_wert is not None:
            value = self.fester_wert.strip()
            try:
                value = Code(kat_code=self.optionsgruppe, code=value)['code']
                return value
            except:
                return None
        try:
            value = Code(jgh[self.ebkus_name])['code']
            if value == '0' and self.optional and self.optionsgruppe == 'ja_ne_un':
                return None
            return value
        except:
            return None
    def csv_format(self, value):
        if value is None:
            return ''
        return value
    def datei_format(self, value):
        if value is None:
            return ' '*self.feldlaenge
        assert len(value) == self.feldlaenge
        return value

class JGHItemGanzzahl(JGHItem):
    mandatory_attributes = ('name', 'langname', 'ebkus_name', 'typ', 'feldlaenge', 'minimum',
                            'maximum')
    def _init(self):
        """
        Wenn Bereiche in jghdef mehrfach angegeben werden, mssen sie 
        absolut identisch sein.
        Die Bereichsobjekte sind mit dem Attribute bereiche auffindbar.

        Beim update() am Anfang des Programmstarts werden die Bereiche
        mit der Datenbank abgeglichen.
        """
        data_bereiche = self._data.get('bereiche', [])
        kat_code = self._data.get('bereichsgruppe')
        if kat_code and data_bereiche:
            bereiche = []
            bereiche_vorhanden = self.root.option_katalog.get(kat_code)
            for b in data_bereiche:
                bereiche.append(_JGHBase(self.root, self, b, art='Bereich'))
            if bereiche_vorhanden:
                # mehrfach in jghdef definiert, mssen identisch sein
                assert len(bereiche) == len(bereiche_vorhanden)
                for i in range(len(bereiche)):
                    o, ov = bereiche[i], bereiche_vorhanden[i]
                    assert o.text == ov.text
                    assert o.code == ov.code
                    assert o.art == ov.art
                    assert o.mini == ov.mini
                    assert o.maxi == ov.maxi
            else:
                # erstmals definiert, im Katalog ablegen.
                self.root.option_katalog[kat_code] = bereiche
    @property
    def bereiche(self):
        if self.bereichsgruppe:
            return self.root.option_katalog[self.bereichsgruppe]
        else:
            return None
    def _validate(self):
        "Falls in der DB, muss der Typ passen"
        feld = self.root.jgh_db_felder_dict.get(self.ebkus_name)
        if feld:
            # print 'FELD:', feld
            verwtyp_code = feld['verwtyp__code']
            assert feld['typ'].startswith('INT'), "Ganzahl-Item kein INT in der Datenbank"
            assert verwtyp_code in ('p', 'b'), "Ganzahl-Item hat falschen Verwendungstyp"
        if verwtyp_code == 'b':
            assert self.bereichsgruppe, "Keine Bereichsgruppe fr {}".format(self.name)
            assert self.bereichsgruppe == feld['kat_code'], "Bereichssgruppe '{}' entspricht " \
            "nicht dem kat_code '{}' des Datenbankfeldes".format(
                    self.bereichssgruppe, feld['kat_code'])
            bereiche = self.root.option_katalog.get(self.bereichsgruppe)
            assert bereiche, "Bereichsgruppe, aber keine Bereiche fr {}".format(self.name)
            if bereiche:
                last_maxi = None
                for b in bereiche:
                    if last_maxi is None:
                        assert b.mini <= self.minimum
                    else:
                        assert b.mini == last_maxi + 1
                    assert b.maxi >= b.mini
                    last_maxi = b.maxi
                    # print b.__class__
                    b.validate()
                assert last_maxi >= self.maximum
    def check(self, raw_value):
        """
        Checks:
        - mininimal, maximal bei Ganzzahl
        """
        value = None
        if raw_value and isinstance(raw_value, basestring) and not raw_value.strip().isdigit():
            raise EE(self.item_message('Keine Zahl'))
        try:
            value = int(raw_value)
        except (ValueError, TypeError):
            pass
        if value is not None and (value < self.minimum or value > self.maximum):
            raise EE(self.item_message('Ungltiger Wert'))
        return value
    def get_export_value(self, jgh):
        """Liefert eine gltige Ganzzahl oder None"""
        if self.fester_wert is not None:
            value = self.fester_wert
        else:
            value = jgh[self.ebkus_name]
        try:
            return int(value)
        except:
            return None
    def csv_format(self, value):
        if value is None:
            return ''
        return str(value)
    def datei_format(self, value):
        if value is None:
            return ' '*self.feldlaenge
        return "{:>{feldlaenge}d}".format(value, feldlaenge=self.feldlaenge)
class JGHItemZeichenkette(JGHItem):
    mandatory_attributes = ('name',  'langname','ebkus_name', 'typ', 'feldlaenge')
    def _validate(self):
        "Falls in der DB, muss der Typ passen"
        feld = self.root.jgh_db_felder_dict.get(self.ebkus_name)
        if feld:
            typ = feld['typ']
            assert typ.startswith('VARCHAR') or typ.startswith('CHAR') or typ.startswith('TEXT'), "Zeichenketten-Item kein CHAR oder VARCHAR oder TEXT in der Datenbank"
            assert feld['verwtyp__code'] in ('p',), "Zeichenketten-Item hat falschen Verwendungstyp"
    def check(self, raw_value):
        """
        Checks:
        - nicht leer, feldlaenge
        """
        value = None
        if raw_value:
            # nicht None, nicht leer
            value = raw_value
            if len(value) > self.feldlaenge:
                raise EE(self.item_message('Zeichenkette zu lang'))
        return value
    def get_export_value(self, jgh):
        """Liefert einen gltigen nicht leeren String oder None"""
        if self.fester_wert is not None:
            value = self.fester_wert.strip()
        else:
            value = jgh[self.ebkus_name]
        if value and isinstance(value, basestring):
            return value
        else:
            return None
    def csv_format(self, value):
        if value is None:
            return ''
        return value
    def datei_format(self, value):
        if value is None:
            return ' '*self.feldlaenge
        return "{:<{feldlaenge}}".format(value, feldlaenge=self.feldlaenge)
class JGHItemCheckbox(JGHItem):
    mandatory_attributes = ('name', 'langname', 'ebkus_name', 'typ', 'feldlaenge', 'checked_wert')
    def _validate(self):
        "Falls in der DB, muss der Typ passen"
        feld = self.root.jgh_db_felder_dict.get(self.ebkus_name)
        if feld:
            typ = feld['typ']
            assert typ.startswith('VARCHAR'), "Checkbox-Item kein VARCHAR in der Datenbank"
            assert feld['verwtyp__code'] in ('p',), "Checkbox-Item hat falschen Verwendungstyp"
    def check(self, raw_value):
        """
        Checks:
        """
        if raw_value == self.checked_wert:
            return self.checked_wert
        if self.vorgabe_wert is not None:
            return self.vorgabe_wert
        return None
    def get_export_value(self, jgh):
        """
        Liefert einen String mit dem Wert '1',
        self.vorgabe_wert oder None
        """
        if self.fester_wert is not None:
            value = self.fester_wert
        else:
            value = jgh[self.ebkus_name]
        if value == self.checked_wert:
            return value
        if self.vorgabe_wert is not None:
            return self.vorgabe_wert
        return None
    def csv_format(self, value):
        if value is None:
            return ''
        return value
    def datei_format(self, value):
        if value is None:
            return ' '*self.feldlaenge
        return "{:<{feldlaenge}}".format(value, feldlaenge=self.feldlaenge)

class JGHOption(_JGHBase):
    mandatory_attributes = ('text',)
    optional_attributes =  ('code', 'fett',)
class JGHBereich(_JGHBase):
    mandatory_attributes = ('text', 'code', 'mini', 'maxi')
class JGHEingabe(_JGHBase):
    """
    Instanzen dieser Klasse liefern Teile von Abschnitten, in htmlgen
    wren das Listen von Listen. Die innere Liste bildet eine Zeile (row),
    es knnen mehrere Zeilen geliefert werden, also eine Liste von rows.
    # d.h. Objekte, die zwei td-Zellen liefern.
    # Instanzen dieser Klasse liefern grundstzlich InputItems aus htmlgen,
    # d.h. Objekte, die zwei td-Zellen liefern.
    """
    mandatory_attributes = ('item', 'text',)
    optional_attributes = ('fett', 'gliederung', 'radio')
    def _validate(self):
        assert self.item or self.items, 'Kein Item bzw. Items'
        if self.item:
            assert self.get_item(self.item), "Kein Item {}".format(self.item)
        if self.items:
            for item in self.items:
                assert self.get_item(item), "Kein Item {}".format(item)
    #item_separator = h.HR(n_col=3, style="heigth:1px;width:80%;color:#FFFFFF")
    item_separator = h.Dummy(height=12, n_col=3)
    def _init(self):
        if self.item:
            item_obj = self.get_item(self.item)
            if item_obj:
                item_obj._abschnitt = self.abschnitt
                item_obj.eingabe = self
                if self.radio:
                    item_obj._radio = True
        if self.items:
            for x in self.items:
                item_obj = self.get_item(x)
                if item_obj:
                    item_obj.eingabe = self
                    item_obj._abschnitt = self.abschnitt
                    if self.radio:
                        item_obj._radio = True
        
    def generate_html(self, jgh):
        # liefert ersteinmal rows, d.h. Liste von <tr>-Elementen mit je drei <td>-Elemente
        item = self.get_item(self.item)
        # print 'ITEM:', item.ebkus_name, item.art, item.name
        if self.gliederung:
            spalte1 = h.String(
                string=self.gliederung,
                class_='legendtext' if self.fett else 'labeltext',
                valign='top',
            )
        else:
            spalte1 = h.Dummy(width="10%")

        input_el = JGHInput.input_element(item, jgh, self.text,
                                          gliederung=self.gliederung,
                                          fett=self.fett, n_col=2,
                                          )
        # print 'GENERATE: OPTIONEN_MIT_ZWISCHENUEBERSCHRIFTEN:', item.langname, item.optionen_mit_zwischenueberschriften
        if item.radio:
            # in diesem Fall liefert input_element Liste von Zeilen
            res = input_el if input_el else []
        else:
            # sonst nur ein InputItem, n_col=2
            res = [[spalte1, input_el]]
        res.append([self.item_separator])
        return res
class JGHEingabeFalldaten(JGHEingabe):
    n_col = 2
    def generate_html(self, jgh):
        # liefert nur das input element
        item = self.get_item(self.item)
        # print 'ITEM:', item.ebkus_name, item.art, item.name
        input_el = JGHInput.input_element(item, jgh, self.text,
                                            gliederung=self.gliederung,
                                            fett=self.fett,
                                            n_col=self.n_col,
                                            zweispaltig=self.zweispaltig)
        return input_el

class JGHEingabeK(JGHEingabe):
    """
    Daten fr Fieldset von Abschnitt K liefern.
    Es gibt nur ein Eingabeelement.

    h.GruendeHeader() und h.GruendeItem() sind auf diesen
    speziellen Fall zugeschnitten.
    """
    mandatory_attributes = ('items', 'headers', 'text')
    optional_attributes = ('fett',)
    def generate_html(self, jgh):
        items = [self.get_item(x) for x in self.items]
        names = [item.ebkus_name for item in items]
        kat_code = items[0].optionsgruppe
        options = self.root.option_katalog[kat_code]
        daten = []
        # daten = [[h.Dummy(), h.String(n_col=3, string="K kann ich nicht")]]
        header = h.GruendeHeader(label=self.text, headers=self.headers)
        daten.append([header])
        for o in options[:-1]:
            daten.append([self.gruende_zeile(
                o, kat_code, names, jgh, fill=len(names))])
        o = options[-1]
        daten.append([self.gruende_zeile(o, kat_code, names[:1], jgh, fill=3)])
        return daten
    def gruende_zeile(self, option, kat_code, names, jgh, fill):
        value = Code(kat_code=kat_code, code=option.code)['id']
        checked = [(jgh[name] == value) for name in names]
        res = h.GruendeItem(label=option.text,
                            names=names,
                            value=value,
                            checked=checked,
                            fill=fill,
                            tip="2. und 3. Grund zurcksetzen")
        return res

class JGHDummy(object):
    def generate_html(self, jgh):
        return h.DummyItem()


class JGHInput(object):
    """Liefert nur die htmlgen InputItems."""
    zweispaltig = False
    @classmethod
    def input_element(cls, item, jgh, text, **kw):
        el = cls(item, jgh, text, **kw)
        return el.generate()
    def __init__(self, item, jgh, text, **kw):
        self.fett = self.gliederung = self.zweispaltig = None
        self.item = item
        self.jgh = jgh
        self.text = text
        for k,v in kw.items():
            setattr(self, k, v)
        self.name = item.ebkus_name
        self.labelclass = ('legendtext' if self.fett else 'labeltext')
        self.label_align = 'left' if '<ul' in self.text else 'right'
        self.label_width = ("30%" if not self.zweispaltig else None),
        self.textboxclass = 'textbox100prz'
    def generate(self):
        # print 'ITEM:', self.item
        # print 'TEXT:', self.text
        typ = self.item.typ
        if typ == 'Option':
            self.vorgabe_id = None
            for option in self.item.optionen:
                try:
                    # Sollte eigentlich nach dem UpdateBundesstatistik immer gehen.
                    option.id = Code(kat_code=self.item.optionsgruppe, code=option.code)['id']
                    if self.item.vorgabe_wert == option.code:
                        self.vorgabe_id = option.id
                except:
                    pass
            if self.item.radio:
                return self.select_mit_zwischenueberschriften()
            else:
                return self.select()
        elif typ == 'Ganzzahl':
            return self.zahl()
        elif typ == 'Zeichenkette':
            return self.string()
        elif typ == 'Checkbox':
            return self.checkbox()
        else:
            raise NotImplementedError, 'Keine Implentierung fr typ={}'.format(typ)
            
    def select_mit_zwischenueberschriften(self):
        """
        Liefert Liste von Listen im Unterschied zu den anderen Typen,
        d.h. List von Zeilen. Zeilen knnen sein:
        - das Label des Items (immer die erste Zeile), mit oder ohne Gliederung
        - ein Radiobutton mit Text fr jeweils eine der Optionen
        - Zwischenberschriften, die eine Menge von Radiobuttons gruppieren
        
        """
        kat_code = self.item.optionsgruppe
        optionen = self.item.optionen
        res = []
        if self.gliederung:
            dummy_one = None
        else:
            # ohne gliederung einrcken
            dummy_one = 10
        res.append(self.item.root.text_zeile(self.item, self.text, fett=self.fett,
                                             dummy_one=dummy_one,
                                             gliederung=self.gliederung, n_col=3))
        for option in optionen:
            if option.code:
                # radio
                res.append([h.Dummy(), self.radio_button(option)])
            else:
                # zwischenberschrift
                res.append(self.item.root.text_zeile(self.item, option.text,
                                                     fett=option.fett,
                                                     dummy_one=10,
                                                     dummies_pre=1,
                                                     n_col=1))
        return res

    def radio_button(self, option):
        """Liefert einen einzelnen Radiobutton."""
        name = self.item.ebkus_name
        jgh_value = self.jgh.get(name)
        if jgh_value is not None:
            checked = (jgh_value == option.id)
        elif self.vorgabe_id:
            checked = (self.vorgabe_id == option.id)
        else:
            checked = False
        # immer das richtige?
        self.label_align = 'right'
        # print 'RADIO label_align', name, self.label_align
        return h.RadioItem(label=option.text, # aus der jghdef
                           # label=code['name'], # aus der Datenbank
                           label_class=('legendtext' if option.fett else 'labeltext'),
                           name=name,
                           value=option.id,
                           checked=checked,
                           label_align=self.label_align,
                           label_right=True,
                           )
    def select(self):
        name = self.item.ebkus_name
        optionen = self.item.optionen
        readonly = self.item.readonly
        if readonly:
            value = self.jgh.get(name)
            candidates = [o for o in optionen if o.id == value]
            if candidates:
                option_text = candidates[0].text
            else:
                option_text = ''
            return h.TextItem(label=self.text,
                              label_class=self.labelclass,
                              label_align=self.label_align,
                              label_width=self.label_width,
                              name='',
                              value=option_text,
                              readonly=readonly,
                              maxlength=str(len(option_text)),
                              class_=self.textboxclass,
                              n_col=self.n_col)
        listboxclass = 'listbox310' if self.item.optionen_maxlen > 30 else 'listbox130'
        return h.SelectItem(label=self.text,
                            label_class=self.labelclass,
                            # label_width="50%",
                            label_width=self.label_width,
                            label_align=self.label_align,
                            name=name,
                            readonly=readonly,
                            class_=listboxclass,
                            options=self.make_option_list())
    def make_option_list(self):
        """
        Optionen fr SelectItem erstellen auf der Basis der jghdef-Optionen.
        Die ntiges ids mssen aus der Datenbank kommen.
        """
        # Nur die Optionen aus der jghdef in der Reihenfolge dort.
        # Mssen aber in der Datenbank stehen, was in update gewhrleistet wird.
        # Ganz alte Formulare werden dadurch nicht korrekt wiedergegeben, aber
        # alle, fr die eine jghdef vorliegt.
        from ebkus.html.jghstatistik import make_option_list
        # Wenn es einen default gibt, brauchen wir kein empty option ' '
        no_empty_option = (self.vorgabe_id is not None)
        value = self.jgh.get(self.item.ebkus_name)
        if value is None: # kein vorhandener Wert
            if self.vorgabe_id:
                selected = self.vorgabe_id
            else:
                selected = ' '
        else:
            selected = value
        # fr make_option_list sind codes eine Liste von dicts mit 'id' und 'name'
        # id wurde in self.generate() hinzugefgt
        codes = [dict(id=o.id, name=o.text) for o in self.item.optionen]
        res = make_option_list(codes, selected, no_empty_option)
        return ''.join(res)
    def zahl(self):
        name = self.item.ebkus_name
        if name == 'jahr':
            return self.berichtsjahr()
        value = self.jgh.get(name)
        value = '' if value is None else value
        # print 'INT ', name, value
        readonly = self.item.readonly
        maxlength=len(str(self.item.maximum))
        if maxlength <= 2:
            textboxclass = 'textboxsmall'
        elif maxlength <= 5:
            textboxclass = 'textboxmid'
        else:
            textboxclass = self.textboxclass
        # if self.parent.readonly:
        #     textboxclass = textboxclass + ' grey'
        return h.TextItem(label=self.text,
                          label_class=self.labelclass,
                          label_align=self.label_align,
                          label_width=self.label_width,
                          name=name,
                          value=str(value),
                          readonly=readonly,
                          maxlength=str(maxlength),
                          class_=textboxclass)

    def string(self):
        # textboxclass = 'textbox310',
        name = self.item.ebkus_name
        value = self.jgh[name]
        value = '' if value is None else value
        readonly = self.item.readonly
        maxlength = self.item.feldlaenge
        return h.TextItem(label=self.text,
                          label_class=self.labelclass,
                          label_align=self.label_align,
                          label_width=self.label_width,
                          name=name,
                          value=value,
                          readonly=readonly,
                          maxlength=str(maxlength),
                          class_=self.textboxclass,
                          n_col=self.n_col)
    def checkbox(self):
        name = self.item.ebkus_name
        value = self.item.checked_wert
        return h.CheckItem(label=self.text,
                           label_class=self.labelclass,
                           label_align=self.label_align,
                           label_width=self.label_width,
                           name=name,
                           value=value,
                           checked=(self.jgh.get(name)==value))


# Die folgenden Teile der Bundesstatistikdefinition wird in EBKuS intern
# gehalten, um keine Verwirrung zu stiften. Die Implentierung ist nicht
# strikt deklarativ.

# Werden bei der Initialisierung des jghdef-Objekts einbezogen (s.o.).

_interne_items = [
    dict(
        art='Item',
        name='Jahr',
        langname='Berichtsjahr',
        ebkus_name='jahr',
        typ='Ganzzahl',
        feldlaenge=4,
        minimum=1990,
        maximum=2070,
    ),
    dict(
        art='Item',
        name='Fallnummer',
        langname='Fallnummer',
        ebkus_name='fall_fn',
        typ='Zeichenkette',
        feldlaenge=10,
        readonly=True,
    ),
    dict(
        art='Item',
        name='Mitarbeiter',
        langname='Mitarbeiter',
        ebkus_name='mit_id__name',
        typ='Zeichenkette',
        feldlaenge=50,
        readonly=True,
    ),
    dict(
        art='Item',
        name='Geschwisterfall',
        langname='Geschwisterfall',
        ebkus_name='gfall',
        typ='Option',
        optionsgruppe='gfall',
        feldlaenge=1,
        vorgabe_wert='1',
        optionen=[
            dict(
                text='Nein',
                code='1',
            ),
            dict(
                text='Ja',
                code='2',
            ),
        ],
    ),
    dict(
        art='Item',
        name='Wohntausserhalb',
        langname='Wohnt ausserhalb',
        ebkus_name='wohnt_ausserhalb',
        typ='Zeichenkette',
        feldlaenge=50,
        readonly=True,
    ),
    dict(
        art='Item',
        name='Fallabschluss',
        langname='Fallabschluss',
        ebkus_name='fallabschluss',
        typ='Zeichenkette',
        feldlaenge=50,
        readonly=True,
    ),
    dict(
        art='Item',
        name='Beginn',
        langname='Fallbeginn',
        ebkus_name='beginn',
        typ='Zeichenkette',
        feldlaenge=7,
        readonly=True,
    ),
]

_abschnitt_falldaten = dict(
    art='AbschnittFalldaten',
    gliederung='Falldaten',
    titel='Falldaten',
    layout=[['Jahr', 'Fallnummer'], ['Beginn', 'Fallabschluss'], ['Mitarbeiter', 'BerichtseinheitID'],
            ['Geschwisterfall', 'LaufendeNummer'], ['Wohntausserhalb']],
    teile=[
        dict(
            art='Eingabe',
            item='Jahr',
            text='Jahr',
        ),
        dict(
            art='Eingabe',
            item='Fallnummer',
            text='Fallnummer',
        ),
        dict(
            art='Eingabe',
            item='Mitarbeiter',
            text='Mitarbeiter',
        ),
        dict(
            art='Eingabe',
            item='BerichtseinheitID',
            text='BerichtseinheitID',
        ),
        dict(
            art='Eingabe',
            item='Geschwisterfall',
            text='Geschwisterfall',
        ),
        dict(
            art='Eingabe',
            item='LaufendeNummer',
            text='laufende Nummer',
        ),
        dict(
            art='Eingabe',
            item='Wohntausserhalb',
            text='Wohnt auerhalb',
        ),
        dict(
            art='Eingabe',
            item='Beginn',
            text='Beginn',
        ),
        dict(
            art='Eingabe',
            item='Fallabschluss',
            text='Fallabschluss',
        ),
    ],
)

