# coding: latin-1

import os
import re
import logging
from glob import glob
from ebkus.config import config
from ebkus.app.ebapi import FallList, JugendhilfestatistikList, Jugendhilfestatistik2007List, \
    today, EE, Code, Jugendhilfestatistik2007
from ebkus.app.ebupd import jgh_laufende_nummer_setzen
from ebkus.html.jghstatistik import get_ags_plz_ort_if_wohnort_ausserhalb
from ebkus.app.ebapi import cc, str2date, get_berichtseinheitid
from ebkus.app.jghdef import get_jghdef

export_formats = dict(datei='Dateimeldung', idev='IDEV-CSV', core='CORE-CSV')
EXPORT_DIR = os.path.join(config.DOCUMENT_ROOT, 'daten', 'export')

def jghexport2(jahr, format, export_dir=EXPORT_DIR):
    """
    Den Export der Bundesstatistik durchfhren und die resultierenden
    Dateien im Exportverzeichnis ablegen.
    Fr jeden Export werden sechs Dateien erzeugt. Je eine fr die andauernden 
    Flle, die abgeschlossenen Flle und alle Flle gemeinsam. Fr jede
    dieser Dateien wird zustzlich eine Logdatei zu Kontrollzwecken geschrieben.
    :return: anzahl_andauernd, anzahl_abgeschlossen, anzahl_gesamt
    """
    if jahr < 2022:
        raise EE("Export der Jugendhilfestatistik nur ab 2022 mglich")
    jghexp = JGHExport(jahr, format, export_dir=export_dir)
    return jghexp.export()
def jghexport2_all_formats_and_test(jahr, export_dir, ppdata=False):
    """
    Den Export der Bundesstatistik durchfhren und die resultierenden
    Dateien im einem Testverzeichnis ablegen und die erzeugten Daten prfen..
    Vor dem Export werden alle vorhandene Dateien in dem Testverzeichnis gelscht.
    Ich vergleiche die drei Log-Dateien in den jeweiligen Formaten.
    
    """
    if jahr < 2022:
        raise EE("Export der Jugendhilfestatistik nur ab 2022 mglich")

    def get_alt_lines():
        from ebkus.app.jghexport import jghexport as jgh_export_alt
        assert jahr == 2022
        andauernd_alt, andauernd_log_alt = jgh_export_alt(jahr, andauernd=True)
        abgeschlossen_alt, abgeschlossen_log_alt = jgh_export_alt(jahr, andauernd=False)
        # print andauernd_log_alt
        # print
        # print abgeschlossen_log_alt
        lines = andauernd_log_alt.splitlines()[8:]
        lines += abgeschlossen_log_alt.splitlines()[8:]
        return lines
    # evt. vorhandene Dateien lschen
    filelist = os.listdir(export_dir)
    for f in filelist:
        os.remove(os.path.join(export_dir, f))
    # exportieren in allen drei Formaten
    for f in export_formats.keys():
        try:
            jghexport2(jahr, f, export_dir=export_dir)
        except EE:
            raise
            #raise EE("Exporttest nicht mglich, da Format '{}' nicht definiert ist.".format(f))
    # die drei dateien ffnen, readlines
    def readlines(format):
        path = generate_export_datei_name(jahr, format,
                                          welche=None, log=True,
                                          export_dir=export_dir,
                                          fullpath=True)
        with open(path, 'rb') as f:
            lines = f.readlines()
        return lines
    alt_lines = get_alt_lines()
    idev_lines = readlines('idev')
    core_lines = readlines('core')
    datei_lines = readlines('datei')
    # log_header abtrennen, ist fr alle Dateien gleich
    # fr idev/core csv_header abtrennen
    # die verbliebenen Zeilen sind die Log-Datensaetze fr jeweils dieselben flle.
    # 'Hilfe' kommt bestimmt in den Headerattibuten vor :-)
    idev_data_begin = 11 if 'Hilfe' in idev_lines[10] else 10
    for i, (datei, idev, core, alt) in enumerate(
            zip(datei_lines[10:], idev_lines[idev_data_begin:], core_lines[11:], alt_lines)):
        compare_and_print_datensaetze(jahr, datei, idev, core, alt, ppdata)
    logging.info("jghexport2_all_formats_and_test() war erfolgreich")
def compare_and_print_datensaetze(
        jahr, datei_all_datensatz, idev_datensatz, core_datensatz, alt_all_datensatz, ppdata):
    if ppdata:
        print 'DATEI:', datei_all_datensatz
        print 'IDEV:', idev_datensatz
        print 'CORE:', core_datensatz
        print 'ALTES FORMAT:', alt_all_datensatz
    alt_datensatz = alt_all_datensatz[24:] # ohne fallnummer, mitarbeiter (ist schon ohne \r\n)
    datei_datensatz = datei_all_datensatz[24:-2] # ohne fallnummer, mitarbeiter, \r\n
    idev_all_values = idev_datensatz.strip().split(';') # ohne \r\n
    core_all_values = core_datensatz.strip().split(';') # ohne \r\n
    idev_values = idev_all_values[2:] # ohne fallnummer, mitarbeiter
    core_values = core_all_values[3:] # ohne fallnummer, mitarbeiter, berichtseinheitid
    fallnummer = datei_all_datensatz[:14].strip()
    alt_fallnummer = alt_all_datensatz[:14].strip()
    #print 'FALLNUMMER', fallnummer, alt_fallnummer
    #print datei_datensatz
    #print alt_datensatz
    berichtseinheitid = core_all_values[2]
    assert fallnummer == core_all_values[0] == idev_all_values[0]
    assert berichtseinheitid == datei_datensatz[1:15] # alt_datensatz ist anders, ok
    jgh = Jugendhilfestatistik2007(fall_fn=fallnummer)
    jghdef = get_jghdef(jahr, strict=True, raise_exception=True)
    assert len(alt_datensatz) == jghdef.datei_satz_laenge
    assert len(datei_datensatz) == jghdef.datei_satz_laenge
    assert len(idev_values) == jghdef.idev_csv_positionen_laenge
    assert len(core_values) == jghdef.core_csv_positionen_laenge
    for item, dpos, ipos, cpos, idev_csv_name, core_csv_name in jghdef.export_item_info:
        # if dpos and not item.berechnet:
        #     print "{:<40} {:<10} {:<10} {}".format(
        #         item.ebkus_name,
        #         datei_datensatz[dpos[0]-1:dpos[1]].strip(),
        #         alt_datensatz[dpos[0]-1:dpos[1]].strip(),
        #         item.get_export_value(jgh))
        if item.ebkus_name in ('berichtseinheitid', 'zustw'):
            continue
        values = []
        if dpos:
            values.append(datei_datensatz[dpos[0]-1:dpos[1]].strip())
            values.append(alt_datensatz[dpos[0]-1:dpos[1]].strip())
        if ipos:
            values.append(idev_values[ipos-1])
        if cpos:
            values.append(core_values[cpos-1])
        if values:
            if item.typ == 'Ganzzahl':
                # frher Ganzzahlen mit fhrenden Nullen formatiert, heute nicht mehr
                values = [v.lstrip('0') for v in values]
            value = values[0]
            # if item.optional:
            #     print item.ebkus_name, values
            for v in values[1:]:
                assert v == value, "'{}' ist nicht '{}'".format(v, value)
            if not item.berechnet:
                jgh_value = item.get_export_value(jgh)
                if jgh_value is None:
                    assert value == ''
                else:
                    assert value == str(jgh_value), "'{}' ist nicht '{}'".format(value, str(jgh_value))
def list_export_datei_groupdicts(jahr, format, welche, export_dir=EXPORT_DIR):
    """
    Liefert fr jede matchende Exportdatei eine dict mit:
    ['name', 'name_log', 'format', 'jahr', 'datum', 'ext', 'welche', 'count']
    
    Es werden immer nur genau die spezifierten Dateinamen geliefert, z.B.
    bei welche=None nur die, die nicht andauernd oder abgeschlossen sind, also
    nur die, die alle Flle enthalten.

    Der Name der zugehrigen Logdatei wird unter name_log abgelegt.
    """
    regex_count = re.compile(r"^Anzahl der Datens[^\d]+tze: (?P<count>\d+)")
    filenames = os.listdir(export_dir)
    groupdicts = [match_export_datei_name(name, jahr, format, welche, False)
                  for name in filenames]
    groupdicts = [d for d in groupdicts if d]
    for d in groupdicts:
        base, ext = os.path.splitext(d['name'])
        d['name_log'] = "{}_log{}".format(base, ext)
        del d['log']
        # Anzahl der Datenstzte (count) der Logdatei entnehmen, oder leer lassen
        d['count'] = ''
        if d['name_log']:
            for line in open(os.path.join(export_dir, d['name_log'])).readlines():
                match = regex_count.match(line)
                if match:
                    d['count'] = match.groupdict()['count']
                    break
    groupdicts.sort(key=lambda x: x['datum'])
    return groupdicts
     
    
def generate_export_datei_name(jahr, format, datum=None,
                               welche=None, log=False,
                               export_dir=EXPORT_DIR,
                               fullpath=False):
    """
    Liefert Namen fr Exportdatei, abhngig von den Kriterien:
    Immer:
    jgh_<jahr>_am_<datum.jahr>-<format>
    Dann:
    _andauernd oder _abgeschlossen, falls nicht vollstndig.
    Dann:
    _log, falls Log datei.
    Dann:
    .txt falls welche == datei, sonst .csv

    Grundstzlich werden bei einem Export immer 6 Dateien geschrieben:
    andauernd, abgeschlossen, alle und fr jede eine Logdatei.

    Fr klassischen Datensatzexport gilt:
    - Suffix immer .txt
    - Log-Datei wie bisher auch
    Fr IDEV- bzw. CORE-Export gilt:
    - Suffix immer .csv
    - Log-Datei mit Attributnamen in der ersten Zeile,
      Fallnummer und Benutzerkrzel in den ersten beiden Spalten,
      so dass man das gut in die Tabellenkalkulation laden kann

    Beispiele:
    jgh_2022_2022-03-15_datei_andauernd_log.txt
    jgh_2022_2022-03-15_idev_abgeschlossen.csv
    jgh_2022_2022-03-15_core.csv
    jgh_2022_2022-03-15_idev_log.csv
    jgh_2022_2022-03-15_datei.txt
    """
    date = datum if datum else today()
    name_list = ['jgh', str(jahr), 'am', "{}-{:0>2}-{:0>2}".format(
        date.year,
        date.month, date.day), format]
    if welche in ('andauernd', 'abgeschlossen'):
        name_list.append(welche)
    if log:
        name_list.append('log')
    ext = '.txt' if format == 'datei' else '.csv'
    name = '_'.join(name_list) + ext
    if fullpath:
        path = os.path.join(export_dir, name)
        return path
    return name
def match_export_datei_name(name, jahr, format, welche, log):
    """
    Liefert groupdict mit:
    ['name', 'format', 'jahr', 'datum', 'ext', 'welche', 'log']

    name, jahr und format mssen immer eindeutig angegeben werden.
    welche muss None sein oder 'andauernd' oder 'abgeschlossen'
    log wird nur als bool(log) verwendet.
    """
    regex_export_datei_name_alt = re.compile(
        r"(?P<name>(jgh_(?P<jahr>\d{4})_am_(?P<datum>\d\d\d\d-\d\d-\d\d)"
        "(_(?P<welche>andauernd|abgeschlossen)))(?P<log>_log)?\.txt)")

    regex_export_datei_name = re.compile(
        r"(?P<name>(jgh_(?P<jahr>\d{4})_am_(?P<datum>\d\d\d\d-\d\d-\d\d)"
        "_(?P<format>datei|idev|core)"
        "(_(?P<welche>andauernd|abgeschlossen))?)(?P<log>_log)?\.(?P<ext>txt|csv))")
    jahr = str(jahr)
    try:
        resd = regex_export_datei_name.match(name).groupdict()
    except AttributeError:
        # None hat keine groupdict
        try:
            resd =  regex_export_datei_name_alt.match(name).groupdict()
            resd['format'] = 'datei'
            resd['ext'] = 'txt'
        except AttributeError:
            # beide haben nicht gematcht
            return None
    ext = 'txt' if format == 'datei' else 'csv'
    if resd['jahr'] != jahr:
            return None
    if resd['format'] != format:
            return None
    if resd['welche'] != welche:
            return None
    if bool(resd['log']) != bool(log):
            return None
    if resd['ext'] != ext:
            return None
    resd['datum'] = str2date(resd['datum'])
    return resd


class JGHExport(object):
    """
    Datenexport fr die Bundesstatistik

    Ersetzt das alte jghexport und wird durch die neue eingefhrte Bundesstatistikdefinition
    gesteuert.
    Untersttzt dreierlei Formate: Die traditionelle Dateimeldung und den Export per CSV-Datei,
    jeweils fr das IDEV-Verfahren und fr das EStatistik-Core-Verfahren des statistischen Bundesamts.
    Fr jeder Verfahren gibt es eine Unterklasse.
    """
    def __init__(self, jahr, format, export_dir=EXPORT_DIR):
        assert jahr >= 2022, "Export der Jugendhilfestatistik nur ab 2022 mglich"
        # Fr den Export muss die Bundesstatistikdefinition fr das Berichtsjahr
        # existieren.
        self.jahr = jahr
        self.jghdef = get_jghdef(jahr, strict=True, raise_exception=True)
        if format not in self.jghdef.available_export_formats:
            raise EE("Das Exportformat '{}' ist fr das Jahr {} nicht verfgbar.".format(
                export_formats[format], jahr))
        self.format = format
        self.export_dir = export_dir
        self.stz_id_to_berichtseinheitid = {}
        self.items = None
        if format == 'datei':
            self.__class__ =  JGHExportDatei
        elif format == 'idev':
            self.__class__ =  JGHExportCSVIDEV
        elif format == 'core':
            self.__class__ =  JGHExportCSVCore
    def export(self):
        from ebkus.app.ebupd import jgh_laufende_nummer_setzen
        jgh_laufende_nummer_setzen()
        anzahl = self.get_jgh_listen()
        if anzahl == 0:
            raise EE("Keine Datenstze fr den Export vorhanden fr das Berichtsjahr "
                     "{}.".format(self.jahr))
        self.annotieren_mit_extras()
        self.get_export_items()
        return self.write_export_files()
    def get_jgh_listen(self):
        self.jgh_list_andauernd = Jugendhilfestatistik2007List(
            where = "ey is NULL and jahr = {}".format(self.jahr), order = 'lnr')
        jgh_list_abgeschlossen_mit_fall = Jugendhilfestatistik2007List(
            where="ey = {} and fall.zday > 0".format(self.jahr),
            join = [('fall', 'jghstat07.fall_id = fall.id')])
        jgh_list_abgeschlossen_ohne_fall = Jugendhilfestatistik2007List(
            where="ey = {} and fall_id IS NULL".format(self.jahr))
        self.jgh_list_abgeschlossen = jgh_list_abgeschlossen_mit_fall + jgh_list_abgeschlossen_ohne_fall
        self.jgh_list_andauernd.sort('lnr')
        self.jgh_list_abgeschlossen.sort('lnr')
        return len(self.jgh_list_andauernd) + len(self.jgh_list_abgeschlossen)
    def get_berichtseinheitid(self, stz_id):
        try:
            return self.stz_id_to_berichtseinheitid[stz_id]
        except KeyError:
            stz = Code(stz_id)
            berichtseinheitid = get_berichtseinheitid(stz['code'])
            self.stz_id_to_berichtseinheitid[stz_id] = berichtseinheitid
            return berichtseinheitid
    def annotieren_mit_extras(self):
        """
        Folgende Variablen werden berechnet und zustzlich zu den Datenbankwerten
        an jedem JGH-Objekt unter dem ebkus_name  hinzugefgt.

        berichtseinheitid

        Betrifft Abschnitt A "Inanspruchnahme mehrerer Einzelhilfen sowie
        Beginn und Anlass der Hilfegewhrung" des Statistikfragebogens:

        extra_kennnummer
        extra_mehrere_hilfen
        extra_hilfen_lnr

        Falls der Klient auerhalb lebt:

        ags
        plz
        wohnort

        """
        jgh_list = (self.jgh_list_andauernd +
                    self.jgh_list_abgeschlossen)
        jgh_list.sort('bgy', 'bgm')
        klienten = {} # id --> count
        for jgh in jgh_list:
            try:
                akte_id = jgh['fall__akte_id']
            except:
                # Kann passieren, wenn der zugehrige Fall gelscht wurde
                akte_id = None
            if akte_id:
                count = klienten.get(akte_id, 0)
                count += 1
                klienten[akte_id] = count
                extra_kennnummer = "%04d%s" % (jgh['jahr'], akte_id)
                extra_mehrere_hilfen = '1' if count > 1 else '2'
                extra_hilfen_lnr = ("%02d" % count) if count > 1 else None # wird durch substitution behandelt
            else:
                count = 1
                # kein Fall, evt. schon gelscht.
                # Mehrfache Hilfen knnen in diesem Fall nicht festgestellt werden.
                # Kennnummer bestehend aus jahr, '00', jgh_id
                # Wegen eingeschobener '00' garantierte keine Verwechslung
                # mit akte_id
                extra_kennnummer = "%04d00%s" % (jgh['jahr'], jgh['id'])
                extra_mehrere_hilfen = '2'
                extra_hilfen_lnr = None
            ags = plz = ort = ''
            ags_plz_ort = get_ags_plz_ort_if_wohnort_ausserhalb(jgh)
            if ags_plz_ort:
                ags, plz, ort = ags_plz_ort
                if ags:
                    plz = ort = '' # plz, ort nur wenn Gemeindeschlssel unbekannt ist.
            jgh['plz'] = plz
            jgh['ort'] = ort
            jgh['ags'] = ags
            jgh['berichtseinheitid'] = self.get_berichtseinheitid(jgh['stz'])
            jgh['extra_kennnummer'] = extra_kennnummer
            jgh['extra_mehrere_hilfen'] = extra_mehrere_hilfen
            jgh['extra_hilfen_lnr'] = extra_hilfen_lnr
    def get_datensatz_values(self, jgh):
        """
        Liste mit den zu exportierenden Daten.
        self.items wird in der jeweiligen Formatklasse
        initialisiert.
        """
        values = []
        for item in self.items:
            # get_export_value() ist formatabhngig definiert
            value = self.get_export_value(item, jgh)
            values.append(value)
        berichtseinheitid = jgh['berichtseinheitid']
        fallnummer = jgh['fall_fn']
        mitarbeiterkennung = jgh['mit_id__ben']
        return berichtseinheitid, fallnummer, mitarbeiterkennung, values
    def write_export_files(self):
        """
        Alle 6 Exportdateien fr ein Format schreiben.
        Fr die Log-Dateien den Log-Header hinzufgen.
        Fr die CSV-Dateien die erste Zeile mit Attributnamen hinzufgen.
        Log-Message ausgeben im Erfolgsfall.
        :return: anzahl_andauernd, anzahl_abgeschlossen, anzahl_gesamt
        CSV-Dateien in utf8, das alte Format 'datei' bleibt bei latin1
        """
        def get_name(welche, log):
            name = generate_export_datei_name(self.jahr, self.format,
                                              welche=welche, log=log,
                                              fullpath=False)
            return name
        def get_path(welche, log):
            path = generate_export_datei_name(self.jahr, self.format,
                                              welche=welche, log=log,
                                              export_dir=self.export_dir,
                                              fullpath=True)
            return path
        def ltu(s):
            if self.format == 'datei':
                return s
            else:
                return s.decode('latin1').encode('utf8')
        anzahl_andauernd = len(self.jgh_list_andauernd)
        anzahl_abgeschlossen = len(self.jgh_list_abgeschlossen)
        anzahl_gesamt = anzahl_andauernd + anzahl_abgeschlossen
        fan = get_path('andauernd', False)
        fanl = get_path('andauernd', True)
        fab = get_path('abgeschlossen', False)
        fabl = get_path('abgeschlossen', True)
        fal = get_path(None, False)
        fall = get_path(None, True)
        if anzahl_gesamt > 0:
            log_message = "Bundesstatistik erfolgreich exportiert fuer Jahr: {}, Format: {}, Anzahl: {}, welche Faelle: {}."
            with open(fal, 'wb') as fal, open(fall, 'wb') as fall:
                log_header = self.get_log_header(get_name(None, False), None, anzahl_gesamt)
                csv_headers, log_csv_headers = self.get_csv_headers()
                fal.write(ltu(csv_headers))
                fall.write(ltu(log_header))
                fall.write(ltu(log_csv_headers))
                if anzahl_andauernd > 0:
                    with open(fan, 'wb') as fan, open(fanl, 'wb') as fanl:
                        log_header_andauernd = self.get_log_header(get_name('andauernd', False), 'andauernd', anzahl_andauernd)
                        fan.write(ltu(csv_headers))
                        fanl.write(ltu(log_header_andauernd))
                        fanl.write(ltu(log_csv_headers))
                        for jgh in self.jgh_list_andauernd:
                            res = self.get_datensatz_values(jgh)
                            datensatz, log_datensatz = self.datensatz_join(*res)
                            fan.write(ltu(datensatz))
                            fanl.write(ltu(log_datensatz))
                            fal.write(ltu(datensatz))
                            fall.write(ltu(log_datensatz))
                    logging.info(log_message.format(self.jahr, export_formats[self.format], anzahl_andauernd, 'andauernd'))
                if anzahl_abgeschlossen > 0:
                    with open(fab, 'wb') as fab, open(fabl, 'wb') as fabl:
                        log_header_abgeschlossen = self.get_log_header(get_name('abgeschlossen', False), 'abgeschlossen', anzahl_abgeschlossen)
                        fab.write(ltu(csv_headers))
                        fabl.write(ltu(log_header_abgeschlossen))
                        fabl.write(ltu(log_csv_headers))
                        for jgh in self.jgh_list_abgeschlossen:
                            res = self.get_datensatz_values(jgh)
                            datensatz, log_datensatz = self.datensatz_join(*res)
                            fab.write(ltu(datensatz))
                            fabl.write(ltu(log_datensatz))
                            fal.write(ltu(datensatz))
                            fall.write(ltu(log_datensatz))
                    logging.info(log_message.format(self.jahr, export_formats[self.format], anzahl_abgeschlossen, 'abgeschlossen'))
            logging.info(log_message.format(self.jahr, export_formats[self.format], anzahl_gesamt, 'alle'))
        return anzahl_andauernd, anzahl_abgeschlossen, anzahl_gesamt
    
    def get_log_header(self, dateiname, welche, anzahl):
        log_header = """Erzeugen der Datei mit den Datenstzen fr die Jugendhilfestatistik fr {jahr}

Welche: {welche}
Format: {format}
Erstellt am: {datum}
Datei: {dateiname}
Anzahl der Datenstze: {anzahl}

Datenstze (vorangestellt zustzlich Fallnummer und Mitarbeiterkennung):

"""
        if welche is None:
            welche_faelle = 'alle Flle'
        elif welche == 'andauernd':
            welche_faelle = 'nur andauernde Flle'
        elif welche == 'abgeschlossen':
            welche_faelle = 'nur abgeschlossene Flle'
        else:
            welche_faelle = ''
        format = export_formats[self.format]
        return log_header.format(jahr=self.jahr, welche=welche_faelle, format=format,
                                 datum=today(), dateiname=dateiname, anzahl=anzahl)

class JGHExportDatei(JGHExport):
    "Die klassische Art, mit Datenstzen"
    def get_export_items(self):
        if self.items is None:
            self.items = self.jghdef.get_datei_export_items()
    def get_export_value(self, item, jgh):
        """
        Den Wert fr das Item dem Fall (jgh) entnehmen und in das korrekte
        Exportformat bringen.
        """
        return item.get_datei_export_value(jgh)
    def datensatz_join(self, berichtseinheitid, fallnummer, mitarbeiterkennung, values):
        datensatz = ''.join(values)
        assert len(datensatz) == self.jghdef.datei_satz_laenge
        log_datensatz = "{:<14}{:<10}{}".format(fallnummer, mitarbeiterkennung, datensatz)
        datensatz += '\r\n'
        log_datensatz += '\r\n'
        return datensatz, log_datensatz
    def get_csv_headers(self):
        return '', ''
class JGHExportCSV(JGHExport):
    "Klasse fr alle CSV-Export-Formate"
    def get_export_value(self, item, jgh):
        """
        Den Wert fr das Item dem Fall (jgh) entnehmen und in das korrekte
        Exportformat bringen.
        """
        return item.get_csv_export_value(jgh)
    def get_csv_headers(self):
        if self.csv_header_schreiben():
            namen, log_namen = self.get_namen_fuer_erste_csv_zeile()
            headers = self.csv_trenner.join(namen)
            log_headers = self.csv_trenner.join(log_namen)
            headers += '\r\n'
            log_headers += '\r\n'
            return headers, log_headers
        else:
            return '',''
class JGHExportCSVIDEV(JGHExportCSV):
    "Klasse fr IDEV-Export-Format"
    csv_trenner = ';'
    def csv_header_schreiben(self):
        return self.jghdef.idev_csv_header_schreiben
    def get_namen_fuer_erste_csv_zeile(self):
        """
        Liefert eine Liste von Namen fr die erste Zeile
        der CSV-Datei
        """
        namen = []
        log_namen = ['Fallnummer', 'Mitarbeiter']
        for (_,_,name) in self.item_data:
            namen.append(name)
            log_namen.append(name)
        return namen, log_namen
    def get_export_items(self):
        self.item_data = self.jghdef.get_idev_export_item_data()
        self.items = [t[0] for t in self.item_data]
    def datensatz_join(self, berichtseinheitid, fallnummer, mitarbeiterkennung, values):
        # print 'VALUES:', berichtseinheitid, fallnummer, mitarbeiterkennung, values
        assert len(values) == self.jghdef.idev_csv_positionen_laenge
        datensatz = self.csv_trenner.join(values)
        log_datensatz = self.csv_trenner.join([fallnummer, mitarbeiterkennung] + values)
        datensatz += '\r\n'
        log_datensatz += '\r\n'
        return datensatz, log_datensatz
class JGHExportCSVCore(JGHExportCSV):
    "Klasse fr Core-Export-Format"
    csv_trenner = ';'
    def get_export_items(self):
        self.item_data = self.jghdef.get_core_export_item_data()
        self.items = [t[0] for t in self.item_data]
    def datensatz_join(self, berichtseinheitid, fallnummer, mitarbeiterkennung, values):
        assert len(values) == self.jghdef.core_csv_positionen_laenge
        values.insert(0, berichtseinheitid)
        datensatz = self.csv_trenner.join(values)
        log_datensatz = self.csv_trenner.join([fallnummer, mitarbeiterkennung] + values)
        datensatz += '\r\n'
        log_datensatz += '\r\n'
        return datensatz, log_datensatz
    def csv_header_schreiben(self):
        return self.jghdef.core_csv_header_schreiben
    def get_namen_fuer_erste_csv_zeile(self):
        """
        Liefert eine Liste von Namen fr die erste Zeile
        der CSV-Datei
        """
        namen = ['BerichtseinheitID']
        log_namen = ['Fallnummer', 'Mitarbeiter', 'BerichtseinheitID']
        for (_,_,name) in self.item_data:
            namen.append(name)
            log_namen.append(name)
        return namen, log_namen

