#!/usr/bin/env python
# -*- coding: latin-1 -*-

"""
virtualenv play2 mit cherryp cheroot bottle
Bottle-Server (mit cherrypy oae) mit mehreren gemounteten WSGI-Apps

"""
import os

msg = "{name} ist nicht installiert. Installieren mit 'pip install {package}'"
try:
    from bottle import Bottle, debug, static_file, abort, request, response, redirect
except:
    msg.format(name='Bottle', package='bottle')

SERVER = 'bottle'
if SERVER == 'cherrypy':

    try:
        import cheroot
    except:
        msg.format(name='Cheroot', package='cheroot')
    try:
        import cherrypy
    except:
        msg.format(name='Cherrypy', package='cherrypy')


# fake bobo REQUEST and RESPONSE objects
class REQUEST(object):
    def __init__(self, bottle_request):
        """Initialisiert mit dem request-Objekt von Bottle.
        Gilt nur fr die Dauer des Requests."""
        self.bottle_request = bottle_request
    def get(self, name):
        return self.bottle_request.get_cookie(name)
    def __getitem__(self, key):
        return self.environ[key]
    @property
    def environ(self):
        return self.bottle_request.environ
    @property
    def form(self):
        f = self.bottle_request.params
        # print type(f), f.dict
        rf = {}
        for k,v in f.dict.items():
            rf[k] = v[0] if len(v) == 1 else v
        # print rf
        return rf
        #return self.bottle_request.params
class RESPONSE(object):
    def __init__(self, bottle_response, bottle_redirect):
        """Initialisiert mit dem response-Objekt von Bottle.
        Gilt nur fr die Dauer des Requests."""
        self.bottle_response = bottle_response
        self.bottle_redirect = bottle_redirect
    def setCookie(self, name, value, max_age):
        #maxage ging nicht
        #self.bottle_response.set_cookie(name, value, maxage=max_age)
        #print name, value
        self.bottle_response.set_cookie(name, value, max_age=max_age)
    def expireCookie(self, name):
        self.bottle_response.delete_cookie(name)
    def setStatus(self, name):
        assert name == 'InternalError'
        self.bottle_response.status = '500 Interner EBKuS-Fehler'
    def setHeader(self, name, value):
        self.bottle_response.set_header(name, value)
    def redirect(self, url, status=303):
        self.bottle_response.status = status
        self.bottle_response.set_header('location', url)

def bottle_entry_point(ebkus_app, klass, request, response):
    response.set_header('content-type', 'text/html; charset=ISO-8859-1')
    return ebkus_app.dispatch(klass, REQUEST(request), RESPONSE(response, redirect))

def set_bottle_routes(bottle_app, config):
    #print 'INSTANZ', instance_name
    @bottle_app.route('/ebkus/' + config.INSTANCE_NAME + '/cgi/do/<klass>', ['GET', 'POST'])
    def ebkus_cgi(klass):
        return bottle_entry_point(request.app.ebkus, klass, request, response)
    @bottle_app.route('/ebkus/' + config.INSTANCE_NAME + '/<filepath:path>')
    def static_ebkus_instance(filepath):
        #print request.path, request.app, request.app.ebkus
        # DOCUMENT_ROOT
        return static_file(filepath, root=config.DOCUMENT_ROOT)
    @bottle_app.route('/ebkus/doc/<filepath:path>')
    def static_ebkus_doc(filepath):
        #print request.path, request.app, request.app.ebkus
        # EBKUS_HOME
        return static_file(filepath, root=os.path.join(config.EBKUS_HOME, 'doc'))
    @bottle_app.route('/ebkus/<filepath:path>')
    def static_ebkus(filepath):
        #print request.path, request.app, request.app.ebkus
        # EBKUS_DOCUMENT_ROOT
        return static_file(filepath, root=config.EBKUS_DOCUMENT_ROOT)
    

def getapp():
    import init
    from ebkus.app import EBKuS
    from ebkus.config import config
    ebkus_app = EBKuS.EBKuS()
    bottle_app = Bottle()
    bottle_app.ebkus = ebkus_app
    set_bottle_routes(bottle_app, config)
    from ebkus.db.sql import closedb
    closedb()
    return bottle_app                          



if __name__ == '__main__':
    debug(True)
    # patch_bottle bottle 12 kann offenbar nicht richtig aus cherrypy den wsgiserver laden
    # habe daher bottle direkt gepatcht
    #main_app.run(server="cherrypy", reloader=True, host="192.168.178.31", port=10431)
    app = getapp()
    if SERVER == 'cherrypy':
        app.run(server="cherrypy", reloader=True, host="0.0.0.0", port=7777)
    elif SERVER == 'bottle':
        app.run(host="0.0.0.0", port=7777)
    #app.run(port=7777, reloader=True)
    #run(server="paste", reloader=True, port=10431)
    #main_app.run(port=10431)

