# coding: latin-1
"""
ZIP-Datei erzeugen, in dem die Datenbank 1:1 im Json-Format (utf-8) gespeichert
wird.
Nur die Datenbank, aber die ganze Datenbank.
Nur die Daten. Kein Schema. Indizes, Tabellendefinitionen, etc.

Fr jede Tabelle eine Datei. Zip-Archiv besteht aus exakt sovielen Dateien wie es 
Tabellen gibt.

Datentypen: Alle Daten in EBKuS sind entweder Integer oder String. Ausnahme:
dokument.dok, gruppendokument.dok und register.value
Das sind Blobs. Die werden aber nicht gebraucht. 
Der Code spricht diese Felder also gar nicht an.

Einfach weglassen? Ja. Wird wirklich nicht gebraucht.
(oder oder, oder mit Verlust encoden. Oder base64.

Encodings:
Sollten kein Problem sein. Alle in EBKuS erzeugten SQL-Queries sind Latin-1. 
Unicode kommt gar nicht vor. In dbadapter.py werden diese Queries alle
nach Unicode umgewandelt (siehe _mysqldb_needs_unicode). 
[Nicht mehr!!]
Im Server
werden alle Daten wg. der Tabellendefinition als Latin-1 abgelegt.
In den queries kommt keine Unicode zurck, sondern Strings, die Latin-1 
kodiert sind.

Es msste also gewhrleistet sein, dass

json.dumps(query_result_rows, encoding='latin1')

immer fehlerfrei funktioniert.

Format einer Tabelle:
{
   tablename: name,
   fields: [<feldnamen>],
   data: [
   [<daten einer zeile>]
   ]
}

"""

import os
import json
from zipfile import ZipFile, ZIP_DEFLATED
import logging
from ebkus.db.sql import getDBHandle, opendb



def dump(path=None):
    DumpRestoreZippedJson(path=path).dump()
def restore(path=None):
    DumpRestoreZippedJson(path=path).restore()

class DumpRestoreZippedJson(object):
    def __init__(self, path=None, directory=None, filename=None):
        """
        Create a ZIP-Archive in directory with a json dump file
        for each table in the database.
        Use path for the archive if given. 
        If directory not given, use working directory.
        If filename not given, use: database_dump_json.zip
        """
        if path:
            self.path_zip = path
        else:
            if not directory:
                directory = os.getcwd()
            assert os.path.isdir(directory), "Directory not found: {}".format(directory)
            if filename is None:
                filename = 'database_dump_json.zip'
            self.path_zip = os.path.join(directory, filename)
        opendb()
        self.db = getDBHandle()
    def dump(self):
        logging.info("Dump database as json to {}".format(self.path_zip))
        zipfile = ZipFile(self.path_zip, 'w', ZIP_DEFLATED)
        tables = self.db.listtables()
        for table in tables:
            dt = DumpRestoreTable(self.db, table)
            json_string = dt.dump()
            zipfile.writestr("{}.json".format(table), json_string)
        zipfile.close()
    def restore(self):
        logging.info("Restore database from json from {}".format(self.path_zip))
        zipfile = ZipFile(self.path_zip, 'r')
        json_filenames = [name for name in zipfile.namelist() 
                          if name.endswith('.json')]
        for filename in json_filenames:
            json_string = zipfile.read(filename)
            table = filename[:-5]
            dt = DumpRestoreTable(self.db, table)
            dt.restore(json_string)
        
class DumpRestoreTable(object):
    def __init__(self, db, table):
        self.db = db
        self.table = table
    def dump(self):
        fields = [field[0] for field in self.db.listfields(self.table)]
        rows = getDBHandle().query("SELECT * FROM {}".format(self.table))
        result_dict = dict(table=self.table, fields=fields, data=rows)
        json_string = json.dumps(result_dict, encoding='latin1')
        logging.info("Dumped {} rows of table {}".format(len(rows), self.table))
        return json_string
    def restore(self, json_string):
        table_dict = json.loads(json_string)
        table = table_dict['table']
        fields = table_dict['fields']
        data = table_dict['data']
        # values ist nun unicode, das sollte aber kein Problem sein
        # die query sowieso umgewandelt wird.
        sql = u"INSERT INTO {} ({}) VALUES ({})".format(
            self.table,
            u','.join(fields),
            u','.join([u'%s' for _ in range(0, len(fields))]))
        cursor = self.db.dbhandle.cursor()
        res1 = cursor.execute(u'TRUNCATE TABLE {}'.format(self.table))
        res2 = cursor.executemany(sql, data)
        logging.info("Restored {} rows of table {}".format(len(data), table))

        
