# coding: latin-1
"""Module fr die Abfragen."""

import sys
from ebkus.config import config
from ebkus.db.sql import escape
from ebkus.app import Request
from ebkus.app.ebapi import JugendhilfestatistikList, Jugendhilfestatistik2007List, \
     ZustaendigkeitList, AkteList, BezugspersonList, Fall, FallList, GruppeList, \
     Tabelle, Code, Feld, Mitarbeiter, MitarbeiterList, MitarbeiterGruppeList, \
     AbfrageList, \
     today, cc, check_int_not_empty, check_list, \
     check_str_not_empty, EBUpdateDataError, EE, getQuartal, get_rm_datum, Date
from ebkus.app.neumeldung_statistik import \
    get_neumelde_abschluss_daten, get_neumelde_nach_region_daten, get_mitarbeiter_ergebnisse
from ebkus.html.abfragedef import Query
from ebkus.app.statistik import BereichsKategorieAuszaehlung        
import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share

try:
    a = set()
except:
    from sets import Set as set

class fall_abfrage(Request.Request, akte_share):
    

    def fall_auswahl_form(self):
        default_sort = ('bgy', 'fn_count', 'akte__na',
                        'zuletzt_zustaendig__mit__na',
                        'has_fachstatistik', 'has_jghstatistik')
        default_sort_code = '012345' # Indizes in default_sort
        tmpl = '<option value="%s"%s>%s</option>'
        # erster Wert ist Index in default_sort
        sort_options_data = (
            (0, 'Jahr des Fallbeginns'),
            (1, 'Fallnummer'),
            (2, 'Name des Klienten'),
            (3, 'Name des Mitarbeiters'),
            (4, 'Fachstatistik vorhanden'),
            (5, 'Bundesstatistik vorhanden'),
            )
        welche = check_str_not_empty(self.form, 'w', "Keine Fallart", 'laufend')
        jahr = check_int_not_empty(self.form, 'jahr', "Fehler in ab Jahr", '')
        fn_count = check_int_not_empty(self.form, 'fnc', "Fehler in laufender Nummer", '')
        bis_jahr = check_int_not_empty(self.form, 'bis_jahr', "Fehler in bis Jahr", '')
        bis_fn_count = check_int_not_empty(self.form, 'bis_fnc', 
                                           "Fehler in bis laufender Nummer", '')
        ab_jahr_zda = check_int_not_empty(self.form, 'ab_jahr_zda', 
                                          "Fehler in ab Jahr Abschluss", '')
        bis_jahr_zda = check_int_not_empty(self.form, 'bis_jahr_zda', 
                                          "Fehler in ab Jahr Abschluss", '')

        if fn_count and not jahr:
            raise EE("Ab laufende Fallnummer nur zusammen mit ab Jahr")
        if bis_fn_count and not bis_jahr:
            raise EE("Bis laufende Fallnummer nur zusammen mit bis Jahr")
        if ab_jahr_zda or bis_jahr_zda:
            # in diesem Fall nur abgeschlossene Flle
            welche = 'abgeschlossen'
        mit_id = check_str_not_empty(self.form, 'mitid', "Kein Mitarbeiter", '')
        seq = check_str_not_empty(self.form, 'seq', "Keine Sortierung", default_sort_code)
        seq = [int(x) for x in seq]
        seq_new = check_str_not_empty(self.form, 'seqn', "Keine Sortierung", '1')
        if seq_new:
            seq_new = int(seq_new)
            assert seq_new in seq, 'Fehler beim Sortieren'
            if seq_new == 1:
                seq.remove(1)
                seq.remove(0)
                seq = [0,1] + seq
            else:
                seq.remove(seq_new)
                seq = [seq_new] + seq
        sort = tuple([default_sort[i] for i in seq])

        mitarbeiter = None
        if self.mitarbeiter['benr__code'] == 'verw':
            if mit_id:
                mitarbeiter = Mitarbeiter(mit_id)
        elif self.mitarbeiter['benr__code'] == 'bearb':
            mitarbeiter = self.mitarbeiter
        legend='Anzuzeigende Beratungen',
        return dict(welche=welche,
                    ab_jahr=jahr,
                    ab_fallnummer=fn_count,
                    bis_jahr=bis_jahr,
                    bis_fallnummer=bis_fn_count,
                    ab_jahr_zda=ab_jahr_zda,
                    bis_jahr_zda=bis_jahr_zda,
                    mitarbeiter=mitarbeiter,
                    sort_options_data=sort_options_data,
                    sort_sel=seq_new, # index der auswahl
                    sort_seq=seq,     # liste von indices
                    sort=sort,        # tuple der sortier-felder
                    )
        

    def get_fall_auswahl(self, legend, welche, ab_jahr, ab_fn_count,
                         bis_jahr, bis_fn_count,
                         ab_jahr_zda, bis_jahr_zda, 
                         mitarbeiter, sort_options_data, sort_sel,
                         submitop='op',
                         ):
        tmpl = '<option value="%s"%s>%s</option>'
        selected = ' selected="selected"'
        welche_options = '\n'.join([tmpl % (v,
                                            v==welche and selected or '',
                                            v.capitalize())
                                    for v in ('laufend', 'abgeschlossen', 'alle')])
        sel = mitarbeiter and mitarbeiter['id'] or None
        alle_sel = (not mitarbeiter) and selected or ''
        mitarbeiter_options = (tmpl % ('', alle_sel, 'Alle')
                               + self.for_mitarbeiter(sel=sel))
        sort_options = '\n'.join([tmpl % (c, c==sort_sel and selected or '', n)
                                  for c, n in sort_options_data])
        anzeige = h.FieldsetInputTable(
            legend=legend,
            daten=[[h.SelectItem(label='Welche',
                                 name='w',
                                 options=welche_options,
    tip='Nur laufende, nur abgeschlossene, oder alle Flle zeigen',
                                 ),
                    h.SelectItem(label='Fallbeginn ab Jahr',
                                 name='jahr',
                                 class_='listbox45',
                                 tip='Nur Flle ab dem gewhlten Jahr zeigen',
                                 options=self.for_jahre(sel=ab_jahr,
                                                        erster_eintrag='Alle'),
                                 ),
                    h.TextItem(label='ab laufender Nummer',
                               name='fnc',
                               class_='textboxmid',
                               value=ab_fn_count,
                               tip='z.B. 9, 23, etc.',
                               ),
                    ],
                    [self.mitarbeiter['benr__code'] == 'verw' and
                     h.SelectItem(label='Mitarbeiter',
                                  name='mitid',
                                  tip='Nur Flle des gewhlten Mitarbeiters zeigen',
                                  options=mitarbeiter_options,
                                  ) or
                    h.TextItem(label='Mitarbeiter',
                               name='xxx',
                               value=mitarbeiter['na'],
                               readonly=True,
                               ),
                    h.SelectItem(label='Fallbeginn bis Jahr',
                                 name='bis_jahr',
                                 class_='listbox45',
                                 tip='Nur Flle bis zu dem gewhlten Jahr zeigen',
                                 options=self.for_jahre(sel=bis_jahr,
                                                        erster_eintrag='Alle'),
                                 ),
                    h.TextItem(label='bis laufender Nummer',
                               name='bis_fnc',
                               class_='textboxmid',
                               value=bis_fn_count,
                               tip='z.B. 9, 23, etc.',
                               ),
                    ],
                    [h.SelectItem(label='Sortieren nach',
                                  name='seqn',
                                  tip='Wonach die Flle sortiert sein sollen',
                                  options=sort_options,
                                  ),
                    h.SelectItem(label='Fallabschluss ab Jahr',
                                 name='ab_jahr_zda',
                                 class_='listbox45',
                                 tip='Nur Flle zeigen, die ab dem gewhlten Jahr abgeschlossen wurden',
                                 options=self.for_jahre(sel=ab_jahr_zda,
                                                        erster_eintrag='Alle'),
                                 ),
                    h.SelectItem(label='Fallabschluss bis Jahr',
                                 name='bis_jahr_zda',
                                 class_='listbox45',
                                 tip='Nur Flle zeigen, die bis zu dem gewhlten Jahr abgeschlossen wurden',
                                 options=self.for_jahre(sel=bis_jahr_zda,
                                                        erster_eintrag='Alle'),
                                 ),
                    ],
                   [h.Dummy(n_col=8)],
                   [h.Button(value="Anzeigen",
                             name=submitop,
                             tip="Beratungen anzeigen",
                             type='submit',
                             n_col=8,
                             ),
                    ],
                   ],
            )
        return anzeige

#class _abfr(Request.Request, akte_share):
class _abfr(fall_abfrage):
    "Superklasse fr Abfragen"
    def get_table_daten(self, elems, fields):
        "Element von fields kann ein Tupel sein: string, url"
        "Dann wird ein verlinkter Text angezeigt."
        daten = []
        for e in elems:
            zeile = []
            for f in fields:
                if isinstance(f, tuple):
                    acc, tmpl = f
                    zeile.append(h.Link(string=e[acc], url=tmpl % e))
                else:
                    v = e[f]
                    if v == None:
                        zeile.append(h.String(string=''))
                    elif isinstance(v, Date):
                        zeile.append(h.Datum(date=e[f]))
                    else:
                        zeile.append(h.String(string=v))
            daten.append(zeile)
        return daten

    def beratungen_gruppe(self,
                          welche=None,
                          mitarbeiter=None,
                          stelle=None,
                          ab_jahr=None,
                          grname=None):
        join=[('mitarbeitergruppe', 'gruppe.id=mitarbeitergruppe.gruppe_id'),
              ('mitarbeiter', 'mitarbeitergruppe.mit_id=mitarbeiter.id')]
        # nur die eigene Stelle
        where = []
        if stelle:
            where.append("gruppe.stz=%s" % stelle['id'])
        if ab_jahr:
            where.append("gruppe.bgy>=%s" % ab_jahr)
        if mitarbeiter:
            where.append("mitarbeiter.id=%s" % mitarbeiter['id'])
        if grname:
            expr = escape('%' + grname + '%')
            where.append("(gruppe.name like %s or gruppe.thema like %s or gruppe.gn like %s)" % \
                     (expr, expr, expr))
        if welche=='laufend':
            where.append('gruppe.ey=0')
        elif welche=='abgeschlossen':
            where.append('gruppe.ey>0')
        gruppe_list = GruppeList(
            where=' and '.join(where),
            join=join,
            )
        sort = ('bgy', 'bgm', 'bgd', 'name')
        gruppe_list.sort(*sort)
        return gruppe_list


class abfr1(_abfr):
    """Ergebnis der Abfrage aller Klienten, Beratungen
    (Tabellen: Fall, Akte, Zustndigkeit)."""
    permissions = Request.ABFR_PERM

## Jetzt in akte_share
##     def beratungen(...):
##         pass
    
    def gef8a_nicht_beantwortet(self, fall):
        # Diese Prfung ist nur sinnvoll fr Bundesstatistiken, die vor dem EBKuS Update
        # erstellt wurden, das gef8a einfhrt (2012-03)
        jgh = fall['jgh']
        # fr sptere Jahre wird das nicht mehr bentigt
        if jgh and jgh.get('jahr') == 2012 and not jgh.get('gef8a'):
            return True
        return False

    def processForm(self, REQUEST, RESPONSE):
        # from ebkus.db import sql
        params = self.fall_auswahl_form()

        welche = params['welche']
        ab_jahr =  params['ab_jahr']
        ab_fallnummer =  params['ab_fallnummer']
        mitarbeiter = params['mitarbeiter']
        sort_options_data = params['sort_options_data']
        sort_sel = params['sort_sel']
        sort_seq = params['sort_seq']
        sort = params['sort']
        bis_jahr = params['bis_jahr']
        bis_fallnummer = params['bis_fallnummer']
        ab_jahr_zda =  params['ab_jahr_zda']
        bis_jahr_zda =  params['bis_jahr_zda']

        title = (welche=='laufend' and 'Laufende' or
                 welche=='abgeschlossen' and 'Abgeschlossene' or
                 welche=='alle' and 'Alle' or '') + ' Beratungen'
        if ab_fallnummer:
            legend_app = ' ab Fallnummer %s-%s%s' % (ab_fallnummer, ab_jahr, self.stelle['code'])
        elif ab_jahr:
            legend_app = ' ab Jahr %s' % (ab_jahr,)
        else:
            legend_app = ''

        #print 'PARAMS: ', params    

        anzeige = self.get_fall_auswahl('Anzuzeigende Beratungen', welche, 
                                        ab_jahr, ab_fallnummer, 
                                        bis_jahr, bis_fallnummer, 
                                        ab_jahr_zda, bis_jahr_zda,
                                        mitarbeiter, 
                                        sort_options_data, sort_sel)
        beratungen = self.beratungen(welche=welche,
        #beratungen = beratungen_sql(welche=welche,
                                     stelle=self.stelle,
                                     mitarbeiter=mitarbeiter,
                                     ab_jahr=ab_jahr,
                                     ab_fallnummer=ab_fallnummer,
                                     bis_jahr=bis_jahr,
                                     bis_fallnummer=bis_fallnummer,
                                     ab_jahr_zda=ab_jahr_zda,
                                     bis_jahr_zda=bis_jahr_zda,
                                     sort=sort)

        #faelle = ('fn', 'vn', 'na', 'gb', 'fallbeginn', 'fallende', 'mitarbeiter', 'fs', 'jgh')
        if config.KEINE_BUNDESSTATISTIK:
            report = h.FieldsetDataTable(
                legend=title + legend_app,
                headers=('Fallnr.', 'Vorname', 'Name', 'Geb.', 'Beginn', 'z.d.A', 'Zustndig', 'FS',),
                daten=[[h.Link(string=fall['fn'],
                               url='klkarte?akid=%(akte_id)s' % fall),
                        h.String(string=fall['akte__vn_xxx']),
                        h.String(string=fall['akte__na_xxx']),
                        h.String(string=fall['akte__gb']),
                        h.Datum(date=fall.getDate('bg')),
                        h.Datum(date=fall.getDate('zda')),
                        # h.String(string=fall['bg']),
                        # h.String(string=fall['zda']),
                        h.String(string=fall['zuletzt_zustaendig__mit__na']),
                        h.String(string=fall['has_fachstatistik']),
                ] for fall in beratungen],
                )
        else:
            report = h.FieldsetDataTable(
                legend=title + legend_app + " ({})".format(len(beratungen)),
                headers=('Fallnr.', 'Vorname', 'Name', 'Geb.', 'Beginn', 'z.d.A', 'Zustndig', 'FS', 'BS'),
                # Falls javascript zum Sortieren der Spalten eingebunden ist ...
                header_class='sortable',
                daten=[[h.Link(string=fall['fn'],
                               url='klkarte?akid=%(akte_id)s' % fall),
                        h.String(string=fall['akte__vn_xxx']),
                        h.String(string=fall['akte__na_xxx']),
                        h.String(string=fall['akte__gb']),
                        h.Datum(date=fall.getDate('bg')),
                        h.Datum(date=fall.getDate('zda')),
                        # h.String(string=fall['bg']),
                        # h.String(string=fall['zda']),
                        h.String(string=fall['zuletzt_zustaendig__mit__na']),
                        h.String(string=fall['has_fachstatistik']),
                        h.String(string=fall['has_jghstatistik']),
#                         h.String(string=fall['has_jghstatistik']+(self.gef8a_nicht_beantwortet(fall) and
# ('<a href="updjgh07?jghid=%d" title="Bundesstatistik unvollstndig: Gefhrdungseinschtzung '
# 'gem  8a" style="color:red;font-weight:bold">(!)</a>' % fall['jgh']['id']) or '')),
#                         h.String(string=fall['has_jghstatistik']+(self.gef8a_nicht_beantwortet(fall) and
# '<span title="Bundesstatistik unvollstndig: Beantwortung der neuen Frage '
# '\"Einleitung der Hilfe auf Grund einer vorangegangenen Gefhrdungseinschtzung '
# 'gem  8a Absatz 1 SGB VIII\" fehlt." style="color:red;">(!)</span>' or '')),
                ] for fall in beratungen],
                )
        # print 'nach report', sql.count_sql
        res = h.FormPage(
            title=title,
            name='beratungen',action="abfr1",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden=(("seq", ''.join([('%s' % i) for i in sort_seq])),
                    ),
            rows=(self.get_auswertungs_menu(),
                  anzeige,
                  report,
                  ),
            )
        # print 'nach FormPage', sql.count_sql
        res1 = res.display()
        # print 'nach display', sql.count_sql
        return res1
        

class abfr2(_abfr):
    """Flle mit mehr als x Wochen ohne Kontakt"""
    permissions = Request.ABFR_PERM

    def get_fall_auswahl(self, legend, erledigt, mitarbeiter, 
                         sort_options_data, sort_sel,
                         submitop='op',
                         ):
        tmpl = '<option value="%s"%s>%s</option>'
        selected = ' selected="selected"'
        erledigt_options = '\n'.join([tmpl % (v,
                                            v==erledigt and selected or '',
                                            v.capitalize())
                                    for v in ('nicht erledigt', 'erledigt', 'alle')])
        sel = mitarbeiter and mitarbeiter['id'] or None
        alle_sel = (not mitarbeiter) and selected or ''
        mitarbeiter_options = (tmpl % ('', alle_sel, 'Alle')
                               + self.for_mitarbeiter(sel=sel))
        sort_options = '\n'.join([tmpl % (c, c==sort_sel and selected or '', n)
                                  for c, n in sort_options_data])
        anzeige = h.FieldsetInputTable(
            legend=legend,
            daten=[[h.SelectItem(label='Welche',
                                 name='w',
                                 options=erledigt_options,
    tip='Nur nicht erledigte, erledigte oder alle Flle zeigen',
                                 ),
                    self.mitarbeiter['benr__code'] == 'verw' and
                    h.SelectItem(label='Mitarbeiter',
                                 name='mitid',
                                 tip='Nur Flle des gewhlten Mitarbeiters zeigen',
                                 options=mitarbeiter_options,
                                 ) or
                    h.TextItem(label='Mitarbeiter',
                               name='xxx',
                               value=mitarbeiter['na'],
                               readonly=True,
                               ),
                    h.SelectItem(label='Sortieren nach',
                                 name='seqn',
                                 tip='Wonach die Flle sortiert sein sollen',
                                 options=sort_options,
                                  ),
                    ],
                   [h.Dummy(n_col=8)],
                   [h.Button(value="Anzeigen / Speichern",
                             name=submitop,
                             tip="Beratungen anzeigen und Vernderungen (erledigt/nicht erledigt) speichern",
                             type='submit',
                             n_col=8,
                             class_='buttonbig',
                             ),
                    ],
                   ],
            )
        return anzeige
    def fall_auswahl_form(self):
        default_sort = ('bgy', 'fn_count', 'akte__na',
                        'zuletzt_zustaendig__mit__na',
                        )
        default_sort_code = '0123' # Indizes in default_sort
        tmpl = '<option value="%s"%s>%s</option>'
        # erster Wert ist Index in default_sort
        sort_options_data = (
            (0, 'Jahr des Fallbeginns'),
            (1, 'Fallnummer'),
            (2, 'Name des Klienten'),
            (3, 'Name des Mitarbeiters'),
            )
        erledigt = check_str_not_empty(self.form, 'w', "Keine Fallart", 'nicht erledigt')
        mit_id = check_str_not_empty(self.form, 'mitid', "Kein Mitarbeiter", '')
        seq = check_str_not_empty(self.form, 'seq', "Keine Sortierung", default_sort_code)
        seq = [int(x) for x in seq]
        seq_new = check_str_not_empty(self.form, 'seqn', "Keine Sortierung", '1')
        if seq_new:
            seq_new = int(seq_new)
            assert seq_new in seq, 'Fehler beim Sortieren'
            if seq_new == 1:
                seq.remove(1)
                seq.remove(0)
                seq = [0,1] + seq
            else:
                seq.remove(seq_new)
                seq = [seq_new] + seq
        sort = tuple([default_sort[i] for i in seq])

        mitarbeiter = None
        if self.mitarbeiter['benr__code'] == 'verw':
            if mit_id:
                mitarbeiter = Mitarbeiter(mit_id)
        elif self.mitarbeiter['benr__code'] == 'bearb':
            mitarbeiter = self.mitarbeiter
        return dict(erledigt=erledigt,
                    mitarbeiter=mitarbeiter,
                    sort_options_data=sort_options_data,
                    sort_sel=seq_new, # index der auswahl
                    sort_seq=seq,     # liste von indices
                    sort=sort,        # tuple der sortier-felder
                    )

    def processForm(self, REQUEST, RESPONSE):
        params = self.fall_auswahl_form()

        # Das war nur zum testen:
        # from ebkus.db.dbapp import _cache
        # n_cache = len(_cache.data)
        # if n_cache > 20:
        #     raise Exception("Cache size was %s" % n_cache)
        # Ende testen

        erledigt = params['erledigt']
        mitarbeiter = params['mitarbeiter']
        sort_options_data = params['sort_options_data']
        sort_sel = params['sort_sel']
        sort_seq = params['sort_seq']
        sort = params['sort']
        #print 'PARAMS: ', params    

        submitop = self.form.get('submitop')

        anzeige = self.get_fall_auswahl('Anzuzeigende Beratungen', erledigt, 
                                        mitarbeiter, 
                                        sort_options_data, sort_sel,
                                        submitop='submitop')

        # Beim ersten Aufruf ist das immer leer, da keine Form abgeschickt wurde und keine defaults definiert wurde.
        erledigt_markiert = [int(id) for id in check_list(self.form, 'erledigt', 'Keine Auswahl', [])]
        erledigt_markiert_set = set(erledigt_markiert)
        # Das ist leer, weil es aus hidden input kommt, und beim Aufruf kein erledigten aufgefhrt werden.
        vorher_erledigt = [int(id) for id in check_list(self.form, 'nerledigt', 'Keine Auswahl', [])]
        vorher_erledigt_set = set(vorher_erledigt)
        #print 'ERLEDIGT_MARKIERT', erledigt_markiert
        #print 'VORHER_ERLEDIGT', vorher_erledigt
        # Wir mssen was tun, wenn 
        # - welche erledigt markiert sind und nicht schon vorher erledigt waren:
        if erledigt_markiert:
            for id in erledigt_markiert:
                if id not in vorher_erledigt_set:
                    fall = Fall(id)
                    fall.update({'erledigt': cc('ja_nein', '1')})
        # - welche schon vorher erledigt waren und jetzt nicht mehr erledigt sind:
        if vorher_erledigt:
            for id in vorher_erledigt:
                if id not in erledigt_markiert_set:
                    fall = Fall(id)
                    fall.update({'erledigt': cc('ja_nein', '2')})
        beratungen = self.laufende_beratungen_lange_ohne_kontakt(
            mitarbeiter=mitarbeiter,
            stelle=self.stelle, 
            erledigt=erledigt,
            sort=sort)
        mehr_als_x = "Mehr als %s Wochen ohne Kontakt" % config.ERINNERN_NACH_WOCHEN_OHNE_KONTAKT
        title = "%s, %s (%s)" % (mehr_als_x, erledigt, len(beratungen))
        report = h.FieldsetDataTable(
            legend=title,
            header_class='sortable',
            headers=('Fallnr.', 'Vorname', 'Name', 'Beginn', 'Zustndig', 'Letzter Kontakt', '', 'Erledigt'),
            daten=[[h.Link(string=fall['fn'],
                           url='klkarte?akid=%(akte_id)s' % fall),
                    h.String(string=fall['akte__vn']),
                    h.String(string=fall['akte__na']),
                    h.Datum(date=fall.getDate('bg')),
                    h.String(string=fall['zuletzt_zustaendig__mit__na']),
                    #h.Datum(date=fall['beratungskontakte'][-1].getDate('k')),
                    h.String(string=fall['letzter_beratungskontakt_datum']),
                    h.CheckItem(
                        name='erledigt',
                        value=fall['id'],
                        #checked=True,
                        checked=fall['erledigt']==cc('ja_nein', '1'),
                        ),
                    ] for fall in beratungen],
            )
        hidden=[("seq", ''.join([('%s' % i) for i in sort_seq])),
                ]
        # Durch Vergleich mit markierten Fllen kann ich festellen, wo erledigt rckgngig gemacht wurde.
        hidden += [('nerledigt', fall['id']) for fall in beratungen if fall['erledigt']==cc('ja_nein', '1')]
        res = h.FormPage(
            title=title,
            name='ohnekontakt',action="abfr2",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden=hidden,
            rows=(self.get_auswertungs_menu(),
                  anzeige,
                  report,
                  ),
            )
        return res.display()
        

class abfr3(_abfr):
    """Ergebnis der Suche in der Klienten- oder Gruppenkartei."""
    permissions = Request.ABFR_PERM


    def _mitarbeiter_stelle(self, mitarbeiter):
        join=[('zustaendigkeit', 'zustaendigkeit.fall_id=fall.id'),
              ('mitarbeiter', 'zustaendigkeit.mit_id=mitarbeiter.id'),]
        # nur die eigene Stelle
        where = "akte.stzbg=%s" % self.stelle['id']
        # nur die letzte Zustndigkeit deren Endedatum gleich dem ZDA-Datum ist
        where += """ and fall.zday = zustaendigkeit.ey and 
        fall.zdam = zustaendigkeit.em and 
        fall.zdad = zustaendigkeit.ed"""
        if mitarbeiter:
            where += " and mitarbeiter.id = %s" % mitarbeiter['id']
        # keine gesperrten akten
        where += " and (akte.sperre is null or akte.sperre != %s)" % cc('ja_nein', '1')
        return join, where

    def search_name_akte(self, mitarbeiter, expr):
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        join += [('fall', 'fall.akte_id=akte.id'),]
        name_expr = escape('%' + expr + '%')
        table = 'akte'
        resKlass = AkteList
        where += " and (%s.vn like %s or %s.na like %s)" % (table, name_expr, table, name_expr) 
        res_list = resKlass(where=where, join=join)
        return res_list
    def search_name_bezugsperson(self, mitarbeiter, expr):
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        join += [('fall', 'fall.akte_id=akte.id'),]
        join += [('akte', 'bezugsperson.akte_id=akte.id'),]
        name_expr = escape('%' + expr + '%')
        table = 'bezugsperson'
        resKlass = BezugspersonList
        where += " and (%s.vn like %s or %s.na like %s)" % (table, name_expr, table, name_expr) 
        res_list = resKlass(where=where, join=join)
        return res_list
    def _get_tel_expr(self, expr):
        """33 4 --> '334', '%3%3%4%'
           33-4 --> '334', '%3%3%4%'
           33/4 --> '334', '%3%3%4%'
           33a4 --> None, None
        """
        tel_expr = tel_expr_like = None
        ziffern = []
        for e in expr:
            if e.isdigit():
                ziffern.append(e)
            elif e in (' ', '-', '/', '.', '+'):
                pass
            else:
                # kein Telefonmatch
                return None, None
        # ziffern enthlt nur Ziffern, expr enthlt nur Ziffern und Leerzeichen
        tel_expr = ''.join(ziffern)
        tel_expr_like = escape('%%%s%%' % '%'.join(ziffern))
        return tel_expr, tel_expr_like
    def _ziffern(self, expr):
        return ''.join([e for e in expr if e.isdigit()])
    def _filter_telefon_list(self, telefon_list, tel_expr):
        def match_telefon(data):
            if (tel_expr in self._ziffern(data['tl1']) or 
                tel_expr in self._ziffern(data['tl2']) or 
                tel_expr in self._ziffern(data['no'])):
                return True
            return False
        res_list = telefon_list.filter(match_telefon)
        return res_list
    def search_telefon_akte(self, mitarbeiter, tel_expr, tel_expr_like):
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        table = 'akte'
        resKlass = AkteList
        join += [('fall', 'fall.akte_id=akte.id'),]
        where += " and (%s.tl1 like %s or %s.tl2 like %s or %s.no like %s)" % \
            (table, tel_expr_like, table, tel_expr_like, table, tel_expr_like)
        res_list = resKlass(where=where, join=join)
        res_list = self._filter_telefon_list(res_list, tel_expr)
        return res_list
    def search_telefon_bezugsperson(self, mitarbeiter, tel_expr, tel_expr_like):
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        table = 'bezugsperson'
        resKlass = BezugspersonList
        join += [('fall', 'fall.akte_id=akte.id'),]
        join += [('akte', 'bezugsperson.akte_id=akte.id'),]
        where += " and (%s.tl1 like %s or %s.tl2 like %s or %s.no like %s)" % \
            (table, tel_expr_like, table, tel_expr_like, table, tel_expr_like)
        res_list = resKlass(where=where, join=join)
        res_list = self._filter_telefon_list(res_list, tel_expr)
        return res_list
    def search_telefon_notiz_akte(self, mitarbeiter, expr, tel_expr, tel_expr_like):
        notiz_expr = escape('%' + expr + '%')
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        table = 'akte'
        resKlass = AkteList
        join += [('fall', 'fall.akte_id=akte.id'),]
        if tel_expr:
            where += " and (%s.tl1 like %s or %s.tl2 like %s or %s.no like %s)" % \
                (table, tel_expr_like, table, tel_expr_like, table, tel_expr_like)
        else:
            where += " and %s.no like %s" % (table, notiz_expr)
        res_list = resKlass(where=where, join=join)
        if tel_expr:
            res_list = self._filter_telefon_list(res_list, tel_expr)
        return res_list
    def search_telefon_notiz_bezugsperson(self, mitarbeiter, expr, tel_expr, tel_expr_like):
        notiz_expr = escape('%' + expr + '%')
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        table = 'bezugsperson'
        resKlass = BezugspersonList
        join += [('fall', 'fall.akte_id=akte.id'),]
        join += [('akte', 'bezugsperson.akte_id=akte.id'),]
        if tel_expr:
            where += " and (%s.tl1 like %s or %s.tl2 like %s or %s.no like %s)" % \
                (table, tel_expr_like, table, tel_expr_like, table, tel_expr_like)
        else:
            where += " and %s.no like %s" % (table, notiz_expr)
        res_list = resKlass(where=where, join=join)
        if tel_expr:
            res_list = self._filter_telefon_list(res_list, tel_expr)
        return res_list
    def search_fallnummer_fall(self, mitarbeiter, expr):
        join, where = self._mitarbeiter_stelle(mitarbeiter)
        table = 'fall'
        resKlass = FallList
        join += [('akte', 'fall.akte_id=akte.id'),]
        name_expr = escape('%' + expr + '%')
        where += " and fall.fn like %s" % name_expr
        res_list = resKlass(where=where, join=join)
        return res_list
    def treffer_in_fallnummer_fall(self, mitarbeiter, expr):
        fall_list = self.search_fallnummer_fall(mitarbeiter, expr)
        if fall_list:
            fall_list.sort('zday', 'fn_count')
            return self.get_klienten_tabelle(fall_list, 'Fallnummer', expr)
    def treffer_in_klienten_name(self, mitarbeiter, expr):
        akte_list = self.search_name_akte(mitarbeiter, expr)
        if akte_list:
            fall_list = FallList([akte['letzter_fall__id'] for akte in akte_list])
            fall_list.sort('zday', 'fn_count')
            return self.get_klienten_tabelle(fall_list, 'Klientenname', expr)
    def treffer_in_bezugspersonen_name(self, mitarbeiter, expr):
        bezugsperson_list = self.search_name_bezugsperson(mitarbeiter, expr)
        if bezugsperson_list:
            bezugsperson_list.sort('akte__letzter_fall__zday', 'akte__letzter_fall__fn_count')
            return self.get_bezugspersonen_tabelle(bezugsperson_list, 'Bezugspersonenname', expr)
    def treffer_in_klienten_telefon(self, mitarbeiter, expr):
        tel_expr, tel_expr_like = self._get_tel_expr(expr)
        akte_list = self.search_telefon_notiz_akte(mitarbeiter, expr, tel_expr, tel_expr_like)
        if akte_list:
            fall_list = FallList([akte['letzter_fall__id'] for akte in akte_list])
            fall_list.sort('zday', 'fn_count')
            if tel_expr is None:
                return self.get_klienten_tabelle(fall_list, 'Klientennotiz', expr)
            else:
                return self.get_klienten_tabelle(fall_list, 'Kliententelefon', tel_expr)
    def treffer_in_bezugspersonen_telefon(self, mitarbeiter, expr):
        tel_expr, tel_expr_like = self._get_tel_expr(expr)
        bezugsperson_list = self.search_telefon_notiz_bezugsperson(mitarbeiter, expr, tel_expr, tel_expr_like)
        if bezugsperson_list:
            bezugsperson_list.sort('akte__letzter_fall__zday', 'akte__letzter_fall__fn_count')
            if tel_expr is None:
                return self.get_bezugspersonen_tabelle(bezugsperson_list, 'Bezugspersonnotiz', expr)
            else:
                return self.get_bezugspersonen_tabelle(bezugsperson_list, 'Bezugspersonentelefon', tel_expr)
    def treffer_in_gruppen(self, mitarbeiter, expr):
        gruppe_list = self.beratungen_gruppe(mitarbeiter=mitarbeiter, grname=expr)
        if gruppe_list:
            return self.get_gruppen_tabelle(gruppe_list, expr)
    def get_klienten_tabelle(self, faelle, treffer_in, expr):
            headers=('Fallnr.', 'Klientenname', 'Telefon', 'Beginn', 'z.d.A', 'Zustndig', 'Notiz')
            daten = self.get_table_daten(faelle,
                                         (('fn', 'klkarte?akid=%(akte_id)s'),
                                          'akte__name', 'akte__telefon',
                                          'bg', 'zda', 'zuletzt_zustaendig__mit__na',
                                          'akte__no',
                                          ),
                                         )
            legend="Suchergebnisse fr: %s enthlt '%s' (%s)" % (treffer_in, expr, len(faelle))
            tabelle = h.FieldsetDataTable(
                legend=legend,
                headers=headers,
                daten=daten,
                )
            return tabelle
    def get_bezugspersonen_tabelle(self, bezugspersonen, treffer_in, expr):
            headers=('Fallnr.', 'Bezugspersonname', 'Bezug', 'Telefon', 'Beginn', 'z.d.A', 'Zustndig', 'Notiz')
            daten = self.get_table_daten(bezugspersonen,
                                         (('akte__letzter_fall__fn', 'klkarte?akid=%(akte_id)s'),
                                          'name', 'verw__name', 'telefon',
                                          'akte__letzter_fall__bg', 'akte__letzter_fall__zda', 'akte__letzter_fall__zuletzt_zustaendig__mit__na',
                                          'no',
                                          ),
                                         )
            legend="Suchergebnisse fr: %s enthlt '%s' (%s)" % (treffer_in, expr, len(bezugspersonen))
            tabelle = h.FieldsetDataTable(
                legend=legend,
                headers=headers,
                daten=daten,
                )
            return tabelle
    def get_gruppen_tabelle(self, gruppen, expr):
        headers=('Gruppennr.', 'Mitarbeiter', 'Name',
                 'Thema', 'Art', 'Teilnehmer', '-zahl', 'Beginn', 'Ende')
        daten = self.get_table_daten(gruppen,
                                     (('gn', 'grkarte?gruppeid=%(id)s'),
                                      'mitarbeiternamen', 'name', 'thema',
                                      'grtyp__name', 'teiln__name', 'tzahl',
                                      'bg', 'e',
                                      ),
                                     )
        legend = "Suchergebnisse fr: Gruppenname oder -thema enthlt '%s' (%s)" % (expr, len(gruppen))
        tabelle = h.FieldsetDataTable(
            legend=legend,
            headers=headers,
            daten=daten,
            )
        return tabelle
    def processForm(self, REQUEST, RESPONSE):
        expr = check_str_not_empty(self.form, 'expr', "Kein Suchausdruck")
        table = check_str_not_empty(self.form, 'table', "Keine Suchklasse")
        if not expr:
            raise EE('Kein Suchausdruck')
        if not table:
            raise EE('Keine Suchklasse')
        if self.mitarbeiter['benr__code'] == 'bearb':
            # eingeschrnkt auf angemeldeten Mitarbeiter
            mitarbeiter = self.mitarbeiter
        elif self.mitarbeiter['benr__code'] == 'verw':
            mitarbeiter = None
        tabelle_klienten_name = None
        tabelle_klienten_telefon = None
        tabelle_klienten_fallnummer = None
        tabelle_bezugspersonen_name = None
        tabelle_bezugspersonen_telefon = None
        tabelle_gruppen = None
        if table == 'alle':
            tabelle_klienten_name = self.treffer_in_klienten_name(mitarbeiter, expr)
            tabelle_klienten_telefon = self.treffer_in_klienten_telefon(mitarbeiter, expr)
            tabelle_klienten_fallnummer = self.treffer_in_fallnummer_fall(mitarbeiter, expr)
            tabelle_bezugspersonen_name = self.treffer_in_bezugspersonen_name(mitarbeiter, expr)
            tabelle_bezugspersonen_telefon = self.treffer_in_bezugspersonen_telefon(mitarbeiter, expr)
            tabelle_gruppen = self.treffer_in_gruppen(mitarbeiter, expr)
        elif table == "akte":
            tabelle_klienten_name = self.treffer_in_klienten_name(mitarbeiter, expr)
        elif table == "fall":
            tabelle_klienten_fallnummer = self.treffer_in_fallnummer_fall(mitarbeiter, expr)
        elif table == "bezugsperson":
            tabelle_bezugspersonen_name = self.treffer_in_bezugspersonen_name(mitarbeiter, expr)
        elif table == "gruppe":
            tabelle_gruppen = self.treffer_in_gruppen(mitarbeiter, expr)
        elif table == "telefon":
            tabelle_klienten_telefon = self.treffer_in_klienten_telefon(mitarbeiter, expr)
            tabelle_bezugspersonen_telefon = self.treffer_in_bezugspersonen_telefon(mitarbeiter, expr)
        # for el in (tabelle_klienten_name,tabelle_klienten_telefon
        #        ,tabelle_bezugspersonen_name,tabelle_bezugspersonen_telefon
        #        ,tabelle_klienten_fallnummer,tabelle_gruppen):
        #     print 'SUCHE', el
        if not (tabelle_klienten_name or tabelle_klienten_telefon
        #if not (tabelle_klienten_telefon or tabelle_klienten_telefon
                or tabelle_bezugspersonen_name or tabelle_bezugspersonen_telefon
                or tabelle_klienten_fallnummer or tabelle_gruppen
            ):
            keine_treffer = h.FieldsetDataTable(legend="Keine Treffer fr '%s'" % expr)
        else:
            keine_treffer = None
        res = h.Page(
            title='Suchergebnisse',
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            rows=(self.get_zurueck(),
                  keine_treffer,
                  tabelle_klienten_name,
                  tabelle_klienten_telefon,
                  tabelle_klienten_fallnummer,
                  tabelle_bezugspersonen_name,
                  tabelle_bezugspersonen_telefon,
                  tabelle_gruppen,
                  ),
            )
        return res.display()
                                         
        
class abfr4(_abfr):
    """Anzahl der Neumeldungen u. Abschlsse pro Jahr und Quartal."""
    permissions = Request.ABFR_PERM
        
    def processForm(self, REQUEST, RESPONSE):
        jahr = check_int_not_empty(self.form, 'jahr', "Fehler im Jahr", '')
        if not jahr:
            jahr = today().year
        fallbeginn = self.form.get(
            'fallbeginn', 
            config.FALLBEGINN_BEI_ERSTER_LEISTUNG and 'fb_lbg' or 'fb_anm')
        anzeige = h.FieldsetInputTable(
            legend='Jahr whlen',
            daten=[[h.SelectItem(label='Jahr',
                                 name='jahr',
                                 class_='listbox45',
                                 tip='Neumeldungen und Abschlsse fr das gewhlte Jahr',
                                 options=self.for_jahre(sel=jahr),
                                 ),
                    h.Button(value="Anzeigen",
                             name='op',
                             tip="Neumeldungen und Abschlsse anzeigen",
                             type='submit',
                             n_col=2,
                             ),
                    config.FALLBEGINN_WAEHLBAR and h.RadioItem(label='Fallbeginn bei Anmeldung',
                                name='fallbeginn',
                                checked=(fallbeginn=='fb_anm'),
                                value='fb_anm',
                                 ) or None,
                    config.FALLBEGINN_WAEHLBAR and h.RadioItem(label='Fallbeginn bei erster Leistung',
                                name='fallbeginn',
                                checked=(fallbeginn=='fb_lbg'),
                                value='fb_lbg',
                                 ) or None,
                    ]],
            )
        report = self.get_report(jahr, fallbeginn, 
                                 bundesstatistik=(not config.KEINE_BUNDESSTATISTIK))
        res = h.FormPage(
            title='Neumelde- und Abschlusszahlen',
            name='neumelde',action="abfr4",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden=(),
            rows=(self.get_auswertungs_menu(),
                  anzeige,
                  report,
                  self.get_neumelde_nach_region_tabelle(jahr, fallbeginn=fallbeginn),
                  ),
            )
        return res.display()
        #faelle = ('fn', 'vn', 'na', 'gb', 'fallbeginn', 'fallende', 'mitarbeiter', 'fs', 'jgh')

    def get_report(self, jahr, fallbeginn, bundesstatistik):
        data = get_neumelde_abschluss_daten(jahr, self.stelle, fallbeginn, bundesstatistik)
        if bundesstatistik:
            return self.get_report_mit_bundesstatistik(jahr, data, fallbeginn)
        else:
            return self.get_report_ohne_bundesstatistik(jahr, data, fallbeginn)

    def get_report_mit_bundesstatistik(self, jahr, data, fallbeginn):
        monats_ergebnisse, quartals_ergebnisse, gesamt_ergebnisse = data
        # print monats_ergebnisse
        # print quartals_ergebnisse
        # print gesamt_ergebnisse
        fallbeginn = 'Anmeldung' if fallbeginn == 'fb_anm' else 'erste Leistung'
        report = h.FieldsetDataTable(
            legend='Neumeldungen und Abschlsse %s' % jahr,
            headers=('Monat', 
                     ('Laufende am Monatsende', 
                      "Fallbeginn ({}) vor dem Monatsende und "
                      "zdA danach bzw. Fall noch offen".format(fallbeginn)),
                     ('Neu', "Fallbeginn ({}) im Monat".format(fallbeginn)),
                     ('davon ASD', "Zugang ber ASD"),
                     ('Hauptfall', "Geschwisterfall 'nein' in der Bundesstatistik angegeben"),
                     ('Geschwisterfall', "Geschwisterfall 'ja' in der Bundesstatistik angegeben"),
                     ('abgeschl. Bundesstatistik', 
                      "Ende der Hilfe/Beratung (Jahr und Monat) fr diesen Monat in die "
                      "Bundesstatistik eingetragen"), 
                     ('davon noch ohne z.d.A', 
                      "Ende der Hilfe/Beratung fr diesen Monat in die Bundesstatistik eingetragen, "
                      "aber noch kein zdA in diesem Monat erfolgt. "
                      "(In Klammern: bis heute noch kein zdA erfolgt. "
                      "Achtung: Flle ohne zdA werden trotz Eintrag des "
                      "Endes in die Bundesstatistik nicht in die Statistik "
                      "fr die Landesmter bernommen!)")),
            daten=[[h.String(string=m[0]),
                    h.String(string=m[1]),
                    h.String(string=m[2]),
                    h.String(string=m[3]),
                    h.String(string=m[4]),
                    h.String(string=m[5]),
                    h.String(string=m[6]),
                    #h.String(string=m[7]),
                    h.String(string=('' if m[7]=='' else ("{} ({})".format(m[7], m[8])))),
            ] for m in monats_ergebnisse] +
            [[h.String(string='Quartal&nbsp;%s' % m[0],
                      class_='tabledatabold'),
             h.String(string=''),
             h.String(string=m[1],
                      class_='tabledatabold'),
             h.String(string=m[2],
                      class_='tabledatabold'),
             h.String(string=m[3],
                      class_='tabledatabold'),
             h.String(string=m[4],
                      class_='tabledatabold'),
             h.String(string=m[5],
                      class_='tabledatabold'),
             h.String(string=''),
             ] for m in quartals_ergebnisse] +
            [[h.String(string='Gesamt',
                      class_='tabledatabold'),
             h.String(string=''),
             h.String(string=gesamt_ergebnisse[0],
                      class_='tabledatabold'),
             h.String(string=gesamt_ergebnisse[1],
                      class_='tabledatabold'),
             h.String(string=gesamt_ergebnisse[2],
                      class_='tabledatabold'),
             h.String(string=gesamt_ergebnisse[3],
                      class_='tabledatabold'),
             h.String(string=gesamt_ergebnisse[4],
                      class_='tabledatabold'),
             h.String(string=''),
             ]],
            )
        return report

    def get_report_ohne_bundesstatistik(self, jahr, data, fallbeginn):
        monats_ergebnisse, quartals_ergebnisse, gesamt_ergebnisse = data
        report = h.FieldsetDataTable(
            legend='Neumeldungen und Abschlsse %s' % jahr,
            headers=('Monat', 'Laufende am Monatsende', 'Neu', 'davon ASD', 
                     'Abgeschlossen', #'abgeschl. Bundesstatistik', 
                     ),
            daten=[[h.String(string=m[0]),
                    h.String(string=m[1]),
                    h.String(string=m[2]),
                    h.String(string=m[3]),
                    h.String(string=m[4]),
            ] for m in monats_ergebnisse] +
            [[h.String(string='Quartal&nbsp;%s' % m[0],
                      class_='tabledatabold'),
             h.String(string=''),
             h.String(string=m[1],
                      class_='tabledatabold'),
             h.String(string=m[2],
                      class_='tabledatabold'),
             h.String(string=m[3],
                      class_='tabledatabold'),
             h.String(string=''),
             ] for m in quartals_ergebnisse] +
            [[h.String(string='Gesamt',
                      class_='tabledatabold'),
             h.String(string=''),
             h.String(string=gesamt_ergebnisse[0],
                      class_='tabledatabold'),
             h.String(string=gesamt_ergebnisse[1],
                      class_='tabledatabold'),
             h.String(string=gesamt_ergebnisse[2],
                      class_='tabledatabold'),
             h.String(string=''),
             ]],
            )
        return report

    def get_neumelde_nach_region_tabelle(self, jahr, fallbeginn):
        """Neumeldungen aufgeteilt nach Region.
        Gibts nur, wenn die Konfigurationsvariable 'neumeldungen_nach_region' 
        eingerichtet ist.
        neumeldungen_nach_region: name1;name2;name3
        name1, ... sind Namen von Teilmengendefinitionen, die in der Tabelle abfrage
        gespeichert sind.
        """
        regionen, monats_ergebnisse, quartals_ergebnisse, gesamt_ergebnisse = \
            get_neumelde_nach_region_daten(jahr, self.stelle, fallbeginn)
        if not regionen:
            return None
        report_nach_region = h.FieldsetDataTable(
            legend='Neumeldungen nach Region %s' % jahr,
            headers=('Monat', 'Neu gesamt',) + tuple([("davon aus %s" % r) for r in regionen]),
            daten=
            [[h.String(string=m[i]) for i in range(len(m))]
              for m in monats_ergebnisse] +

            [[h.String(string='Quartal&nbsp;%s' % q[0],
                       class_='tabledatabold'),] +
             [h.String(string=q[i],
                       class_='tabledatabold')
              for i in range(1, len(q))]
             for q in quartals_ergebnisse] +
        
            [[h.String(string='Gesamt',
                       class_='tabledatabold'),] +
             [h.String(string=g,
                       class_='tabledatabold')
              for g in gesamt_ergebnisse]
             ],
        )
        return report_nach_region
        
        

class abfr5(_abfr):
    """Klientenzahl pro Mitarbeiter u. Jahr."""
    permissions = Request.ABFR_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = check_int_not_empty(self.form, 'jahr', "Fehler im Jahr", '')
        if not jahr:
            jahr = today().year
        fallbeginn = self.form.get(
            'fallbeginn', 
            config.FALLBEGINN_BEI_ERSTER_LEISTUNG and 'fb_lbg' or 'fb_anm')
        anzeige = h.FieldsetInputTable(
            legend='Jahr whlen',
            daten=[[h.SelectItem(label='Jahr',
                                 name='jahr',
                                 class_='listbox45',
                                 tip='Klientenzahl pro Mitarbeiter fr das gewhlte Jahr',
                                 options=self.for_jahre(sel=jahr),
                                 ),
                    h.Button(value="Anzeigen",
                             name='op',
                             tip="Klientenanzahl pro Mitarbeiter anzeigen",
                             type='submit',
                             n_col=2,
                             ),
                    config.FALLBEGINN_WAEHLBAR and h.RadioItem(label='Fallbeginn bei Anmeldung',
                                name='fallbeginn',
                                checked=(fallbeginn=='fb_anm'),
                                value='fb_anm',
                                 ) or None,
                    config.FALLBEGINN_WAEHLBAR and h.RadioItem(label='Fallbeginn bei erster Leistung',
                                name='fallbeginn',
                                checked=(fallbeginn=='fb_lbg'),
                                value='fb_lbg',
                                 ) or None,
                    ],
                   ],
            )
        mitarbeiter_ergebnisse, gesamt = get_mitarbeiter_ergebnisse(
            jahr, self.getMitarbeiterliste(status=('i', 'a')), fallbeginn)
        report = h.FieldsetDataTable(
            legend='Neumeldungen und Abschlsse pro Mitarbeiter %s' % jahr,
            headers=('Mitarbeiter', 'bernommen', 'Neu', 'davon Fallbeginn', 'Laufend', 'Beendet','davon z.d.A.'),
            daten=[[h.String(string=m[0]),
                    h.String(string=m[6],
                             tip='Zustndigkeit aus dem Vorjahr fortgesetzt'),
                    h.String(string=m[1],
                             tip='Zustndigkeit neu bernommen'),
                    h.String(string=m[2],
                             tip='Zustndigkeit fr neuen Fall bernommen'),
                    h.String(string=m[3],
                             tip='Laufende Flle am Ende des Jahres'),
                    h.String(string=m[4],
                             tip='Zustndigkeit beendet'),
                    h.String(string=m[5],
                             tip='Zustndigkeitsende durch Fallabschluss'),
            ] for m in mitarbeiter_ergebnisse 
                   # nur Mitarbeiter, die berhaupt Zustndigkeiten hatten
                   if (m[1] or m[2] or m[3] or m[4] or m[5] or m[6])] +
            [[h.String(string='Gesamt',
                      class_='tabledatabold'),
             h.String(string=gesamt[5],
                      class_='tabledatabold',
                      tip='Zustndigkeit aus dem Vorjahr fortgesetzt'),
             h.String(string=gesamt[0],
                      class_='tabledatabold',
                      tip='Zustndigkeit neu bernommen'),
             h.String(string=gesamt[1],
                      class_='tabledatabold',
                      tip='Zustndigkeit fr neuen Fall bernommen'),
             h.String(string=gesamt[2],
                      class_='tabledatabold',
                      tip='Laufende Flle am Ende des Jahres'),
             h.String(string=gesamt[3],
                      class_='tabledatabold',
                      tip='Zustndigkeit beendet'),
             h.String(string=gesamt[4],
                      class_='tabledatabold',
                      tip='Zustndigkeitsende durch Fallabschluss'),
             ]],
            )
        res = h.FormPage(
            title='Neumeldungen und Abschlsse pro Mitarbeiter',
            name='neumelde',action="abfr5",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden=(),
            rows=(self.get_auswertungs_menu(),
                  anzeige,
                  report,
                  ),
            )
        return res.display()


class abfr8(_abfr):
    "Gruppenbersicht"
    permissions = Request.ABFR_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = check_int_not_empty(self.form, 'jahr', "Fehler im Jahr", '')
        anzeige = h.FieldsetInputTable(
            legend='Jahr whlen',
            daten=[[h.SelectItem(label='Ab Jahr',
                                 name='jahr',
                                 class_='listbox45',
                                 tip='Gruppenbeginn ab dem gewhlten Jahr',
                                 options=self.for_jahre(sel=jahr,
                                                        erster_eintrag='Alle'),
                                 ),
                    h.Button(value="Anzeigen",
                             name='op',
                             tip="Gruppen anzeigen",
                             type='submit',
                             n_col=2,
                             ),
                    ],
                   ],
            )
        mitarbeiter = welche = stelle = None
        gruppe_list = self.beratungen_gruppe(ab_jahr=jahr,
                                             mitarbeiter=mitarbeiter,
                                             welche=welche,
                                             stelle=stelle)
        headers=('Gruppennr.', 'Mitarbeiter', 'Name',
                 'Thema', 'Art', 'Teilnehmer', '-zahl', 'Beginn', 'Ende')
        daten = self.get_table_daten(gruppe_list,
                                     (('gn', 'grkarte?gruppeid=%(id)s'),
                                      'mitarbeiternamen', 'name', 'thema',
                                      'grtyp__name', 'teiln__name', 'tzahl',
                                      'bg', 'e',
                                      ),
                                     )
        report = h.FieldsetDataTable(
            legend='Gruppenberblick',
            headers=headers,
            daten=daten,
            )
        res = h.FormPage(
            title='Gruppenberblick',
            name='gruppenue',action="abfr8",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden=(),
            rows=(self.get_auswertungs_menu(),
                  anzeige,
                  report,
                  ),
            )
        return res.display()
        
class wartezeiten(_abfr):
    """Tabelle mit Wartezeiten zwischen Anmeldung und Erstleistung."""
    permissions = Request.ABFR_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = check_int_not_empty(self.form, 'jahr', "Fehler im Jahr", '')
        if not jahr:
            jahr = today().year
        anzeige = h.FieldsetInputTable(
            legend='Jahr whlen',
            daten=[[h.SelectItem(label='Jahr',
                                 name='jahr',
                                 class_='listbox45',
                                 tip='Wartezeiten fr das gewhlte Jahr',
                                 options=self.for_jahre(sel=jahr),
                                 ),
                    h.Button(value="Anzeigen",
                             name='op',
                             tip="Tabelle mit Wartezeiten anzeigen",
                             type='submit',
                             n_col=2,
                             ),
                    ],
                   ],
            )
        # Flle ermitteln, wo der Leistungsbeginn in dem gewhlten
        # Jahr liegt
        faelle = self.beratungen(welche='alle',
                                 stelle=self.stelle,
                                 ab_jahr=jahr-1, # Vorjahr mit einbeziehen
                                 bis_jahr=jahr,
                                 sort=('bg',))
        faelle = faelle.filter(lambda x: x['leistungsbeginn'].year==jahr)

        auszaehlung = BereichsKategorieAuszaehlung(faelle,
                                                   'wartezeit', 
                                                   'wartez',
                                                   title='Verteilung der Wartezeiten',
                                                   )
        ausz_res = auszaehlung.get_result()
        summe_haeufigkeiten = sum([m[1] for m in ausz_res])
        summe_prozent = sum([m[2] for m in ausz_res])
        haeufigkeiten = h.FieldsetDataTable(
            legend='Wartezeiten fr Flle mit erster Leistung im Jahr %s' % jahr,
            headers=('', 'Hufigkeit', 'Prozent',),
            daten=[[h.String(string=m[0]),
                    h.String(string=m[1],
                             class_="tabledataright",),
                    h.String(string="%.2f" % m[2],
                             class_="tabledataright",),
            ] for m in ausz_res] + 
            [[h.String(string='Summe',
                       class_='tabledatabold',
                       ),
              h.String(string=summe_haeufigkeiten,
                       class_='tabledataboldright',
                       align="right",
                       ),
              h.String(string="%.2f" % summe_prozent,
                       align="right",
                       class_='tabledataboldright',
                       ),
              ]
             ]
            )

        faelle.sort('wartezeit')
#         for f in faelle:
#             print f['fn'], f['bg'], f['leistungsbeginn'], f['wartezeit']
        if len(faelle) > 0:
            median = faelle[(len(faelle)/2)]['wartezeit']
            mittelwert = sum([f['wartezeit'] for f in
                              faelle])/float(len(faelle))
        else:
            median = 'NA'
            mittelwert = 'NA'
        
        mittel = h.FieldsetDataTable(
            legend='Mittelwert und Median der Wartezeiten in '
            'Kalendertagen fr Flle mit erster Leistung im Jahr %s' % jahr,
            daten=[[h.String(string='Mittelwert (n=%s)' % len(faelle),
                             class_='tabledatabold',
                             ),
                    h.String(string="%s" % (isinstance(mittelwert,
                                                       (int,long,float))
                                            and ("%.2f" % mittelwert)
                                            or mittelwert,),
                             align="right",
                             class_='tabledataboldright',
                             ),
                    ],
                   [h.String(string='Median (n=%s)' % len(faelle),
                             class_='tabledatabold',
                             ),
                     h.String(string="%s" % median,
                              align="right",
                              class_='tabledataboldright',
                              ),
                    ],
                   ]
            )
        res = h.FormPage(
            title='Wartezeiten',
            name='wartezeiten',action="wartezeiten",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ),
            hidden=(),
            rows=(self.get_auswertungs_menu(),
                  anzeige,
                  haeufigkeiten,
                  mittel,
                  ),
            )
        return res.display()

