# coding: latin-1

"""Module fr Akte und Fall."""

import string,time,sys
import logging

from ebkus.db.sql import SQL
from ebkus.db.dbapp import DBAppError
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, Gruppe, FallList, GruppeList, Altdaten, Anmeldung, \
     Zustaendigkeit, today, cc, check_list, check_code, check_int_not_empty, \
     check_str_not_empty, check_date, EE

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share

class _akte(Request.Request, akte_share):
    """Gemeinsame Methode zur Anzeige der Aktenformulare.
    """
    def _process(self,
                 title,
                 file,
                 akte,
                 anmeldung,
                 formname,
                 hidden,
                 force_strkat=None,
                 ):
        notiz = h.FieldsetInputTable(
            legend='Notiz',
            daten=[[h.TextareaItem(label='Notiz',
                                   label_width='14%',
                                   name='no',
                                   value=akte['no'],
                                   class_='textareanormal100prz',
                                   tip="Freies Feld fr Notizen",
                                   )],
                   ]
            )
        klientendaten = self.get_klientendaten(akte)
        anschrift = self.get_anschrift(akte, force_strkat)
        falldaten = leistung = altdaten = None
        if file in ('akteeinf', 'waufneinf'):
            falldaten = h.FieldsetInputTable(
                legend='Falldaten',
                daten=[[h.SelectItem(label='Bearbeiter',
                                     name='zumitid',
                                     tip='Fallbearbeiter auswhlen',
                                     options=self.for_mitarbeiter(sel=self.mitarbeiter['id'])),
                        h.DatumItem(label='Anmeldedatum',
                                    name='zubg',
                                    tip='Anmeldedatum',
                                    year=today().year,
                                    month=today().month),
                        ]],
                )
            leistung = h.FieldsetInputTable(
                legend='Leistung',
                daten=[[h.SelectItem(label='Mitarbeiter',
                                     name='lemitid',
                                     tip='Mitarbeiter auswhlen, der die Leistung erbracht hat',
                                     options=self.for_mitarbeiter(sel=self.mitarbeiter['id'])),
                        h.SelectItem(label='Leistung',
                                     name='le',
                                     tip='Art der erbrachten Leistung',
                                     class_='listbox250',
                                     options=self.for_kat('fsle', akte['fs'])),
                        ],[h.DummyItem(),
                           h.DatumItem(label='Am',
                                       name='lebg',
                                       tip='Datum der Leistung oder des Beginns der Leistung',
                                       date=today()),
                           ]],
                )

        if file=='akteeinf' and \
               SQL("select count(*) from altdaten").execute()[0][0] > 0:
            # Altdaten vorhanden
            altdaten = h.FieldsetInputTable(
                legend='Nach Altdaten suchen',
                tip="Bitte zuerst nach Altdaten suchen, dann erst weitere Daten eingeben",
                button=h.Button(value='Altdaten',
                                name='xxx',
                                onClick="go_to_url('altlist')",
                                tip="Suche in den Altdaten aus frheren EDV-Systemen",
                                ),
                )
        if config.ANMELDUNGSDATEN_OBLIGATORISCH and file in ('akteeinf', 'waufneinf'):
            anmeldungf = self.get_anmeldekontakt(anmeldung)
        else:
            anmeldungf = None
        res = h.FormPage(
            title=title,
            name=formname,action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                          (not file=='akteeinf') and
                           ('Klientenkarte', 'klkarte?akid=%s' % akte['id'])
                           or None,
                           ),
            hidden=(("akid", akte['id']),
                    ("file", file),
                    ("strid", ""), # wird nur von strkat mit javascript gesetzt
                    ) + hidden,
            rows=(h.Pair(left=klientendaten,
                         right=(altdaten, anschrift),
                         ),
                  notiz,
                  anmeldungf,
                  falldaten,
                  leistung,
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class akteneu(_akte):
    """Neue Fallakte anlegen (Tabellen: Akte, Fall, Zustndigkeit, Leistung)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        alt_ids = check_list(self.form, 'uebern', 'Fehler in Altdaten', [])
        if len(alt_ids) > 1:
            raise EE("Bitte nur einen Eintrag zur bernahme markieren!")
        if alt_ids:
            alt = Altdaten(alt_ids[0])
        else:
            alt = {}
        gs_alt = alt.get('geschlecht')
        if gs_alt:
            gs = (gs_alt=='w' and cc('gs', '2')) or (gs_alt=='m' and cc('gs', '1'))
        else:
            gs = ' '
        if alt:
            no = "Alte Fallnummer: %(fallnummer)s, Jahr: %(jahr)s, Frherer Mitarbeiter: %(mitarbeiter)s" % alt
        else:
            no = ''
        str = alt.get('strasse', '')
        if str:
            for end in ('trasse', 'trae'):
                if str.endswith(end):
                    i = str.index(end)
                    str = str[:i] + 'tr.'
        akte = Akte()
        akte.init(
            id=Akte().getNewId(),
            fs=cc('fsfs', '999'),
            vn=alt.get('vorname', ''),
            na=alt.get('name', ''),
            gb=alt.get('geburtsdatum', ''),
            ort=alt.get('ort', ''),
            plz=alt.get('plz', ''),
            str=str,
            hsnr=alt.get('hausnummer', '').upper(),
            gs=gs,
            tl1=alt.get('telefon1', ''),
            tl2=alt.get('telefon2', ''),
            no=no,
            aufbew=cc('aufbew', '1'),
            stzbg=self.stelle['id'],
            lage=(config.STRASSENKATALOG and  cc('lage', '0') or
                  cc('lage', '1')),
            )
        hidden = (('stzbg', akte['stzbg']),
                  )
        anm = None
        if config.ANMELDUNGSDATEN_OBLIGATORISCH:
            anm = Anmeldung()
            anm.init(
                id=Anmeldung().getNewId(),
                zm=cc('fszm', '999'),
                )
            # Nachname und Telefon des Klienten
            # im Formular anbieten.
            anm['von'] = akte['na']
            anm['mtl'] = akte['tl1']
            if config.ANMELDEPROBLEME_UND_BESCHAEFTIGUNG_BEI_ANMELDUNG_ERFASSEN:
                anm['anmprobleme'] = None
                anm['qualij'] = ' '
                anm['bkm'] = ' '
                anm['bkv'] = ' '
            hidden += (('anmid', anm['id']),)
        
        return self._process(
            title='Neue Akte anlegen',
            file='akteeinf',
            akte=akte,
            anmeldung=anm,
            formname='akteform',
            hidden=hidden,
            )

class waufnneu(_akte):
    """Wiederaufnahme einer vorhandener  Fallakte
    (Tabellen: Akte, Fall, Zustndigkeit, Leistung)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        akid = self.form.get('akid')
        if not akid:
            raise EE("Keine ID fuer die Akte erhalten")
        akte = Akte(akid)
        hidden=(('fallid', Fall().getNewId()),
                ('status', cc('stand', 'l')),
                )
        anm = None
        if config.ANMELDUNGSDATEN_OBLIGATORISCH:
            anm = Anmeldung()
            anm.init(
                id=Anmeldung().getNewId(),
                zm=cc('fszm', '999'),
                )
            # Nachname und Telefon des Klienten
            # im Formular anbieten.
            anm['von'] = akte['na']
            anm['mtl'] = akte['tl1']
            hidden += (('anmid', anm['id']),)
        return self._process(
                 title="Wiederaufnahme des Klienten",
                 file='waufneinf',
                 akte=akte,
                 anmeldung=anm,
                 formname='akteform',
                 hidden=hidden,
                 )
        
class updakte(_akte):
    """Akte ndern (Tabelle Akte)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        strkat = self.form.get('strkat')
        if strkat == '0':
            force_strkat = False
        elif strkat == '1':
            force_strkat = True
        else:
            force_strkat = None
        if self.form.has_key('akid'):
            akid = self.form.get('akid')
            akte = Akte(akid)
        else:
            self.last_error_message = "Keine ID fuer die Akte erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        return self._process(
            title='Akte aktualisieren',
            file='updakte',
            akte=akte,
            anmeldung=None,
            formname='akteform',
            hidden=(),
            force_strkat=force_strkat,
            )

class updfall(_akte):
    """Fall updaten (Tabelle: Fall)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fuer den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        zustaendigkeiten_list = fall['zustaendigkeiten']
        beginndatum = h.FieldsetInputTable(
            legend='Beginndatum',
            daten=[[h.DatumItem(label='Fallbeginn',
                                name='bg',
                                tip='Fallbeginn',
                                date=fall.getDate('bg')),
            ]],
            )
        res = h.FormPage(
            title='Beginndatum ndern',
            name='akteform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=(("fallid", fallid),
                    ("file", 'updfall'),
                    ),
            rows=(beginndatum,
                  self.get_zustaendigkeiten(zustaendigkeiten_list),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()
        
class zda(_akte):
    """Fallakte abschliessen (Tabellen: Fall und Zustndigkeit)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fuer den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        zustaendigkeiten_list = fall['zustaendigkeiten']
        zustaendigkeiten_list.sort('bgd', 'bgm', 'bgy')
        aktuell_zustaendig = fall['zustaendig']
        falldaten = h.FieldsetInputTable(
            legend='Falldaten',
            daten=[[h.DatumItem(label='Beginndatum',
                                name='bg',
                                tip='Fallbeginn',
                                date=fall.getDate('bg')),
                    h.DatumItem(label='Abschlussdatum',
                                name='zda',
                                tip='Fallabschlussdatum',
                                date=today()),
            ]],
            )
        res = h.FormPage(
            title='Abschlussdatum eintragen',
            name='akteform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=(("fallid", fallid),
                    ("file", 'zdaeinf'),
                    ("aktuellzustid", aktuell_zustaendig['id']),
                    ("aktuellmitid", aktuell_zustaendig['mit_id__id']),
                    ),
            rows=(falldaten,
                  self.get_bisherige_zustaendigkeit(
            aktuell_zustaendig,
            legend='Bisherige Zustndigkeit wird ausgetragen'),
                  self.get_zustaendigkeiten(zustaendigkeiten_list),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()
        
class zdar(_akte):
    """Fallabschluss rckgngig machen und letzte Zustaendigkeit wiederherstellen
    (Tabellen: Fall und Zustndigkeit)."""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        fallid = self.form.get('fallid')
        if not fallid:
            raise EE("Keine ID fuer den Fall erhalten")
        fall = Fall(fallid)
        zuletzt_zustaendig = fall['zuletzt_zustaendig']
        res = h.FormPage(
            title='Abschlussdatum rckgngig machen',
            name='zdarform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=(("fallid", fallid),
                    ("file", 'zdareinf'),
                    ),
            rows=(self.get_bisherige_zustaendigkeit(
            zuletzt_zustaendig,
            legend='Letzte Zustndigkeit wird wiederhergestellt',
            ),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class _zust(_akte):
    def _process(self,
                 title,
                 file,
                 fall,
                 zustaendigkeit, # Form schon fertig, da unterschiedlich upd/neu
                 hidden,
                 ):
        aktuell_zustaendig = fall['zustaendig']
        zustaendigkeiten_list = fall['zustaendigkeiten']
        zustaendigkeiten_list.sort('bgy', 'bgm', 'bgd')
        if file == 'zusteinf':
            bisherige_zustaendigkeit = self.get_bisherige_zustaendigkeit(
                aktuell_zustaendig,
                legend='Bisherige Zustndigkeit wird ausgetragen',
                )
        elif file == 'updzust':
            bisherige_zustaendigkeit = None
        res = h.FormPage(
            title=title,
            name='zustform',action="klkarte",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientenkarte', 'klkarte?akid=%(akte_id)s' % fall),
                           ),
            hidden=hidden,
            rows=(self.get_klientendaten_kurz(fall),
                  zustaendigkeit,
                  bisherige_zustaendigkeit,
                  self.get_zustaendigkeiten(zustaendigkeiten_list),
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class zustneu(_zust):
    """Neue Zustndigkeit eintragen. (Tabelle: Zustndigkeit.)"""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fr den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(fallid)
        aktuell_zustaendig = fall['zustaendig']
        neue_zust = Zustaendigkeit()
        neue_zust.init(
            id=Zustaendigkeit().getNewId(),
            mit_id=self.mitarbeiter['id'],
            fall_id=fall['id'],
            )
        neue_zust.setDate('bg', today())
        zustaendigkeit = h.FieldsetInputTable(
            legend="Neue Zustndigkeit eintragen",
            daten=[[h.SelectItem(label='Bearbeiter',
                                 name='mitid',
                                 tip='Fallbearbeiter auswhlen',
                                 options=self.for_mitarbeiter(neue_zust['mit_id'])),
                    h.DatumItem(label='Beginn',
                                name='bg',
                                tip='Datum des Zustndigkeitbeginns',
                                date=neue_zust.getDate('bg')),
                    # keine Endedataum
                    ]],
            )
        return self._process(
            title="Neue Zust&auml;ndigkeit eintragen",
            file='zusteinf',
            fall=fall,
            zustaendigkeit=zustaendigkeit,
            hidden=(('zustid', neue_zust['id']),
                    ("aktuellzustid", aktuell_zustaendig['id']),
                    ('file', 'zusteinf'),
                    ('fallid', fall['id']),
                    ),
            )
        
class updzust(_zust):
    """Neue Zustndigkeit eintragen. (Tabelle: Zustndigkeit.)"""
    permissions = Request.UPDATE_PERM
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('zustid'):
            id  = self.form.get('zustid')
        else:
            self.last_error_message = "Keine ID fr die Zustndigkeit erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        zust = Zustaendigkeit(id)
        fall = Fall(zust['fall_id'])
        zustaendigkeit = h.FieldsetInputTable(
            legend="Zustndigkeit bearbeiten",
            daten=[[h.SelectItem(label='Bearbeiter',
                                 name='mitid',
                                 tip='Fallbearbeiter auswhlen',
                                 options=self.for_mitarbeiter(zust['mit_id'])),
                    h.DatumItem(label='Beginn',
                                name='bg',
                                tip='Datum des Zustndigkeitbeginns',
                                date=zust.getDate('bg')),
                    h.DatumItem(label='Ende',
                                name='e',
                                tip='Datum des Zustndigkeitendes',
                                date=zust.getDate('e')),
                    ]],
            )

        return self._process(
            title="Zust&auml;ndigkeit bearbeiten",
            file='updzust',
            fall=fall,
            zustaendigkeit=zustaendigkeit,
            hidden=(('zustid', zust['id']),
                    ('file', 'updzust'),
                    ),
            )

        
