# coding: latin-1

"""Module fr Akte und Fall."""


import logging

from ebkus.db.sql import SQL
from ebkus.db.dbapp import DBAppError
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, Gruppe, FallList, GruppeList, Altdaten, Anmeldung, \
     Zustaendigkeit, today, cc, cn, check_list, check_code, check_int_not_empty, \
     check_str_not_empty, check_date, EE, str2date
import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share


def get_akten_loeschen_menu():
    """Dropdownmen zum Aktenlschen fr die Administratorhauptseite."""
    url_name_pairs = (
        ("", ("wg. Fehler (Statistik wird ebenfalls gelscht)", None)),
        ("rmaktefa", "- Einzlene Akte"),
        ("rmakteff", "- Einzelner Fall"),
        ("", "wg. Datenschutz (Statistik bleibt erhalten)"),
        ("rmakted", "- Einzelne Akte"),
        ("", ("wg. Alter jahrgangsweise (Statistik bleibt erhalten)",
              "Jeweils der lteste Jahrgang wird zum Lschen angeboten, wenn der "
              "der Abschluss des letzten Falls einer Akte lnger zurckliegt als die jeweils eingestellte "
              "Lschfrist (Aufbewahrungskategorie oder explizit abweichend)")),
        ("rmaktenb", ("- Beratung", "Lschfrist: {} Monate".format(get_loeschfrist('Beratung')))),
        ("rmaktent", ("- Therapie", "Lschfrist: {} Monate".format(get_loeschfrist('Therapie')))),
        ("rmaktenm", ("- Missbrauch", "Lschfrist: {} Monate".format(get_loeschfrist('Missbrauch')))),
        ("rmaktenabw", ("- abweichende Lschfrist", "Lschfrist explizit angegeben")),
        )
    dropdown = h.DropdownMenu(name='Lschen',
                              tip='',
                              url_name_pairs=url_name_pairs,
                              )
    return dropdown

def get_loeschfrist(aufbewahrungskategorie):
    # config.LOESCHFRIST, config.LOESCHFRIST_THERAPIE, config.LOESCHFRIST_MISSBRAUCH, 
    loeschfrist = getattr(config, 
                          'LOESCHFRIST' if aufbewahrungskategorie == 'Beratung'
                          else 'LOESCHFRIST_' + aufbewahrungskategorie.upper())
    return loeschfrist

# def get_loeschfrist(akte):
#     abwlf = akte['abwlf']
#     if abwlf:
#         return abwlf
    

def get_loeschfrist_stichtag(aufbewahrungskategorie):
    """
    Das Datum Heute - (Lschfrist+1) Monate, wobei die
    Lschfrist fr die jeweilige Aufbewahrungskategorie konfiguriert ist.
    """
    loeschfrist = get_loeschfrist(aufbewahrungskategorie)
    stichtag = today().add_month(-(loeschfrist + 1))
    return stichtag

class _rmakte(Request.Request, akte_share):
    """
    Einzelnen Fall bzw. einzelne Akte lschen, sowohl wg. Fehler
    als auch wg. Datenschutz.
    Details in den Subklassen.
    """
    permissions = Request.ADMIN_PERM

    def _get(self,
             title,
             legend,
             action,
             ganze_akte,
             statistik_auch,
             aktuell_auch,
             loeschfrist_ignorieren,
             fn,
             anmeldedatum,
             tip_fn,
             tip_anm,
             tip_button,
             ):
        # fn = check_str_not_empty(self.form, 'fn', 'Keine Fallnummer', '')
        # anmeldedatum = check_date(self.form, 'bg', "Fehler im Anmeldedatum", (0,0,0))
        # ganze_akte = self.form.get('ganze_akte')
        auswahl_akte = h.FieldsetFormInputTable(
            name="rmfallanzeigen",
            action=action,
            method="post",
            hidden=(
                ('ganze_akte', '1' if ganze_akte else None),
                ('statistik_auch', '1' if statistik_auch else None),
                ('aktuell_auch', '1' if aktuell_auch else None),
                ('action', action),
                ('loeschfrist_ignorieren', '1' if loeschfrist_ignorieren else None),
                ),
            legend=legend,
            daten=[[h.TextItem(label='Fallnummer',
                               name='fn',
                               value=fn,
                               tip=tip_fn,
                               ),
                    h.DatumItem(label='Anmeldedatum',
                                date=anmeldedatum,
                                name='bg',
                                tip_anm=tip_anm,
                                ),
                    h.CheckItem(label="Lschfrist ignorieren",
                                name="loeschfrist_ignorieren",
                                value="1",
                                checked=False,
                                tip="Ankreuzen, um die Akte aus Grnden des Datenschutzes "
                                "vor Ablauf der Lschfrist zu lschen.") 
                    if not loeschfrist_ignorieren else
                    h.DummyItem(name='loeschfrist_ignorieren', value='1'),
                    ],
                   [h.Dummy(n_col=4)],
                   ],
            button=h.Button(value="Lschen",
                            name='op',
                            tip=tip_button,
                            type='submit',
                            n_col=4,
                            )
            )
        res = h.Page(
            title=title,
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  auswahl_akte,
                  ),
            )
        return res.display()

    def post(self, REQUEST, RESPONSE):
        """
        2 Flle
        1. Fallauswahl mit Fallnummer und Anmeldedatum
        --> zurck Angebot mit Lschen der Akte oder des Falles
        2. fall_id und ok: Die Lschung des Falls durchfhren.
           akte_id und ok: Die Lschung der Akte durchfhren.
        --> Besttigung der Lschung mit Fallnummer bzw. ganze Akte mit Fallnummern. 
        """
        ok = self.form.get('ok')                    # ok zum endgltigen Lschen
        action = self.form.get('action')
        if ok:
            return self._einzelfall_oder_akte_loeschen(action)
        else:
            return self._einzelfall_oder_akte_loeschangebot(action)
    def _einzelfall_oder_akte_loeschangebot(self, action):
        fn = check_str_not_empty(self.form, 'fn', 'Keine Fallnummer')
        anmeldedatum = check_date(self.form, 'bg', "Kein Anmeldedatum")
        statistik_auch = self.form.get('statistik_auch') == '1'
        aktuell_auch = self.form.get('aktuell_auch') == '1'
        loeschfrist_ignorieren = self.form.get('loeschfrist_ignorieren') == '1'
        if anmeldedatum.year == 0:
            EE('Kein Anmeldedatum')
        try:
            fall = Fall(fn=fn, bgy=anmeldedatum.year, 
                        bgm=anmeldedatum.month, bgd=anmeldedatum.day)
        except DBAppError:
            raise EE("Kein Fall gefunden mit Fallnumer {fn} "
                     "und Anmeldedatum {anmeldedatum}".format(**locals()))
        ganze_akte = self.form.get('ganze_akte') == '1'
        akte = fall['akte']
        if not loeschfrist_ignorieren:
            self._check_loeschfrist(akte)
        if not aktuell_auch and akte['aktueller_fall']:
            if ganze_akte:
                raise EE("Akte mit offenem Fall kann nicht gelscht werden")
            else:
                raise EE("Offener Fall kann nicht gelscht werden")
        faelle = akte['faelle']
        einzelfall = len(faelle) == 1
        faelle.sort('bgy', 'bgm', 'bgd')
        fn_str = ', '.join(f['fn'] for f in faelle)
        assert fall in faelle, "Fall nicht in Fllen der Akte :-("
        if not ganze_akte and len(faelle) > 1 and fall != akte['letzter_fall']:
            raise EE(
                "Der zu lschende Fall muss der neueste Fall aus der Akte sein. "
                "Die Akte besteht aus den Fllen {}. ".format(fn_str))
        if statistik_auch:
            msg_statistik = ("Evt. vorhandene Fach- bzw. Bundesjugendhilfestatistik "
            "wird ebenfalls gelscht.")
        else:
            msg_statistik = "Fach- bzw. Bundesjugendhilfestatistik bleibt erhalten."
        if ganze_akte or einzelfall:
            zeilen = []
            legend = 'Ganze Akte lschen'
            if einzelfall and not ganze_akte:
                zeilen.append("Da die Akte nur den zu lschenden Fall enthlt, "
                "wird die gesamte Akte gelscht. ")
            zeilen.append(msg_statistik)
            if einzelfall:
                msg = "Ganze Akte bestehend aus dem Fall {}".format(fn_str)
            else:
                msg = "Ganze Akte bestehend aus den Fllen {}".format(fn_str)
            zeilen.append(msg + " endgltig lschen?")
            hidden = [('akid', akte['id']),
                      ('ok', '1'),
                      ('msg', msg),
                      ('statistik_auch', statistik_auch),
                      ('action', action),
                      ]
        else:
            # Einen Fall lschen
            msg = 'Fall {}'.format(fn)
            legend = msg + ' lschen'
            zeilen = [msg_statistik, msg + " endgltig lschen?"]
            hidden = [('fallid', fall['id']),
                      ('ok', '1'),
                      ('msg', msg),
                      ('statistik_auch', statistik_auch),
                      ('action', action),
                      ]
        if loeschfrist_ignorieren:
            hidden.append(('loeschfrist_ignorieren', '1'))
        return h.SubmitOrBack(legend=legend,
                              action=action,
                              method='post',
                              hidden=hidden,
                              zeilen=zeilen,
                              ).display()
    def _einzelfall_oder_akte_loeschen(self, action):
        from ebkus.app.ebupd import remove_faelle, remove_akten
        fall_id = self.form.get('fallid')           # Diesen Fall endgltig lschen
        akte_id = self.form.get('akid')             # Diese Akte endgltig lschen
        msg = self.form.get('msg')
        statistik_auch = self.form.get('statistik_auch') == '1'
        if not (fall_id or akte_id):
            raise EE('Nichts zu lschen')
        if fall_id:
            remove_faelle([int(fall_id)], 
                          statistik_auch=statistik_auch, aktuell_auch=True)
        elif akte_id:
            remove_akten([int(akte_id)], 
                         statistik_auch=statistik_auch, aktuell_auch=True)
        res = h.Meldung(legend=msg,
                         zeilen=[msg + " gelscht"],
                         weiter=action).display()
        return res
    def _check_loeschfrist(self, akte):
        aufbewahrungskategorie = akte['aufbew__name']
        loeschfrist = get_loeschfrist(aufbewahrungskategorie)
        stichtag = get_loeschfrist_stichtag(aufbewahrungskategorie)
        letzter_abschluss = akte['letzter_fall'].getDate('zda')
        if letzter_abschluss > stichtag:
            raise EE(
                "Lschfrist ist noch nicht erreicht. <br>"
                "Lschfrist fr Aufbewahrungskategorie '{}' betrgt {} Monate. <br>"
                "Letzter Fallabschluss war am {}. <br>".format(
                    aufbewahrungskategorie, loeschfrist, letzter_abschluss))

class rmaktefa(_rmakte):
    def get(self, REQUEST, RESPONSE):
        return  self._get(
            title='Einzelne Akte lschen wg. Fehler',
            legend='Fallnummer und Anmeldedatum eines Falles '
            'in der zu lschenden Akte (wg. Fehler)',
            action="rmaktefa",
            ganze_akte=True,
            statistik_auch=True,
            aktuell_auch=True,
            loeschfrist_ignorieren=True,
            fn='',
            anmeldedatum='',
            tip_fn='Fallnummer eines Falles in der zu lschenden Akte (wg. Fehler)',
            tip_anm="Anmeldedatum des zu lschenden Falls muss sicherheitshalber "
            "angegeben um irrtmliche Lschungen zu vermeiden",
            tip_button="Akte mit dem angegebenen Fall endgltig lschen",
            )
class rmakteff(_rmakte):
    def get(self, REQUEST, RESPONSE):
        return  self._get(
            title='Einzelnen Fall lschen wg. Fehler',
            legend='Fallnummer und Anmeldedatum des zu lschenden Falls (wg. Fehler)',
            action="rmakteff",
            ganze_akte=False,
            statistik_auch=True,
            aktuell_auch=True,
            loeschfrist_ignorieren=True,
            fn='',
            anmeldedatum='',
            tip_fn='Fallnummer des zu lschenden Falls',
            tip_anm="Anmeldedatum des zu lschenden Falls muss sicherheitshalber "
            "angegeben um irrtmliche Lschungen zu vermeiden",
            tip_button="Angegebenen Fall lschen",
            )
class rmakted(_rmakte):
    def get(self, REQUEST, RESPONSE):
        return  self._get(
            title='Einzelne Akte lschen wg. Datenschutz',
            legend='Fallnummer und Anmeldedatum des zu lschenden Falls (wg. Datenschutz)',
            action="rmakted",
            ganze_akte=True,
            statistik_auch=False,
            aktuell_auch=False,
            loeschfrist_ignorieren=False, # kann im Formular mit einem Hkchen berschrieben werden
            fn='',
            anmeldedatum='',
            tip_fn='Fallnummer des zu lschenden Falls',
            tip_anm="Anmeldedatum des zu lschenden Falls muss sicherheitshalber "
            "angegeben werden, um irrtmliche Lschungen zu vermeiden",
            tip_button="Angegebenen Fall lschen",
            )


class _rmakten(Request.Request, akte_share):
    """
    Akten nach Ablauf der Lschfrist jahrgangsweise und gruppiert
    nach Aufbewahrungskategorie lschen.
    Details in den Subklassen.
    """
    permissions = Request.ADMIN_PERM

    def _get(self, action, aufbewahrungskategorie):
        """
        """
        try:
            del self.session.data['akten_loeschkandidaten']
        except KeyError:
            pass # Kein Problem, wenn nicht vorhanden
        jahr, loeschkandidaten = self._get_loeschkandidaten(aufbewahrungskategorie)
        if jahr and loeschkandidaten:
            anzahl = len(loeschkandidaten)
            loeschfrist = get_loeschfrist(aufbewahrungskategorie)
            self.session.data['akten_loeschkandidaten'] = (jahr, loeschkandidaten)
            hidden = (
                ('session_key', 'akten_loeschkandidaten'),
                ('action', action),
                ('stichtag', str(get_loeschfrist_stichtag(aufbewahrungskategorie))),
                )
            legend = "Akten aus dem Jahr {jahr} ({anzahl}) lschen".format(**locals())
            zeilen = ["Es gibt {anzahl} Akten aus dem Jahr {jahr} ".format(**locals()),
                      "aus der Aufbewahrungskategorie {aufbewahrungskategorie}, ".
                      format(**locals()),
                      "fr die die Lschfrist ({loeschfrist} Monate) abgelaufen ist.".format(**locals()),
                      "Sollen diese Akten endgltig gelscht werden?"]
            return h.SubmitOrBack(legend=legend,
                                  title=legend,
                                  action=action,
                                  method='post',
                                  hidden=hidden,
                                  zeilen=zeilen,
                                  ).display()
        else:
            legend = "Keine Akten zum Lschen gefunden"
            res = h.Meldung(legend=legend,
                            title=legend,
                            zeilen=["Keine Akten zum Lschen vorhanden"]).display()
            return res

    def post(self, REQUEST, RESPONSE):
        session_key = self.form.get('session_key')
        action =  self.form.get('action')
        stichtag =  self.form.get('stichtag')
        if stichtag is not None:
            stichtag =  str2date(stichtag)
        jahr, akte_ids = self.session.data.get(session_key)
        if not akte_ids:
            raise EE("Keine Daten zum Lschen erhalten")
        from ebkus.app.ebupd import remove_akten
        anzahl = remove_akten(akte_ids, statistik_auch=False, 
                              aktuell_auch=False, zda_vor=stichtag)
        legend="Akten gelscht ({})".format(anzahl)
        res = h.Meldung(legend=legend,
                        title=legend,
                        zeilen=["{} Akten mit Abschluss im Jahr {} gelscht".format(anzahl, jahr)],
                        weiter='menu').display()
        return res

        
    def _get_loeschkandidaten(self, aufbewahrungskategorie):
        """
        Gibt die Ids aller Akten zurck, fr die die Lschfrist ihrer Aufbewahrungskategorie
        abgelaufen ist, aber von diesen nur die aus dem ersten Jahr. zB wenn 
        Lschkandidaten aus 2012, 2013 und 2015 vorhanden sind, nur die aus 2012.
        Vielleicht doch am einfachsten ohne SQL."""
        # Wenn letztes zda vor dem Stichtag liegt, ist die Lschfrist abgelaufen
        stichtag = get_loeschfrist_stichtag(aufbewahrungskategorie)
        # Das kleinste Jahr, fr das ein zda existiert
        res = SQL("SELECT MIN(zday) FROM fall WHERE zday > 0").execute()
        minjahr = res[0][0]
        if minjahr is None:
            return None, None
        for jahr in range(minjahr, stichtag.year + 1):
            # die mit abweichener Lschfrist ausschlieen
            fall_list = FallList(where='zday = {} and akte.abwlf is null'.format(jahr),
                                 join=[('akte', 'fall.akte_id=akte.id'),], cache_related=True)
            loeschkandidaten = [f['akte_id'] for f in fall_list
                                if f['akte__letzter_fall'] == f and
                                f['akte__aufbew__name'] == aufbewahrungskategorie and
                                f.getDate('zda') <= stichtag]
            if loeschkandidaten:
                return jahr, loeschkandidaten
        return None, None

class rmaktenb(_rmakten):
    def get(self, REQUEST, RESPONSE):
        return  self._get('rmaktenb', aufbewahrungskategorie='Beratung')

class rmaktent(_rmakten):
    def get(self, REQUEST, RESPONSE):
        return  self._get('rmaktent', aufbewahrungskategorie='Therapie')
class rmaktenm(_rmakten):
    def get(self, REQUEST, RESPONSE):
        return  self._get('rmaktenm', aufbewahrungskategorie='Missbrauch')
class rmaktenabw(_rmakten):
    def get(self, REQUEST, RESPONSE):
        try:
            del self.session.data['akten_loeschkandidaten']
        except KeyError:
            pass # Kein Problem, wenn nicht vorhanden
        # Das kleinste Jahr, fr das ein zda existiert
        res = SQL("SELECT MIN(zday) FROM fall WHERE zday > 0").execute()
        minjahr = res[0][0]
        if minjahr:
            for jahr in range(minjahr, today().year + 1):
                loeschkandidaten, min_abwlf, max_abwlf = _get_loeschkandidaten_abweichend(today(), jahr)
                if loeschkandidaten:
                    anzahl = len(loeschkandidaten)
                    self.session.data['akten_loeschkandidaten'] = (jahr, loeschkandidaten)
                    hidden = (
                        ('session_key', 'akten_loeschkandidaten'),
                        ('action', 'rmaktenabw'),
                    )
                    legend = "Akten aus dem Jahr {jahr} ({anzahl}) lschen".format(**locals())
                    zeilen = ["Es gibt {anzahl} Akten aus dem Jahr {jahr} ".format(**locals()),
                              "mit explizit angegebener abweichender Lschfrist,  ",
                              "fr die die Lschfrist (zwischen {min_abwlf} und {max_abwlf} Monaten) abgelaufen ist.".format(**locals()),
                              "Sollen diese Akten endgltig gelscht werden?"]
                    return h.SubmitOrBack(legend=legend,
                                          title=legend,
                                          action='rmaktenabw',
                                          method='post',
                                          hidden=hidden,
                                          zeilen=zeilen,
                                          ).display()
        legend = "Keine Akten zum Lschen gefunden"
        return h.Meldung(legend=legend,
                         title=legend,
                         zeilen=["Keine Akten zum Lschen vorhanden"]).display()

def _get_loeschkandidaten_abweichend(heute, jahr):
    min_abwlf = 999
    max_abwlf = 0
    akte_ids = []
    # abgeschlossene Flle, deren Akte eine abweichende Lschfrist hat
    fall_list = FallList(where='zday = {} and akte.abwlf is not null'.format(jahr),
                         join=[('akte', 'fall.akte_id=akte.id'),], cache_related=True)
    for fall in fall_list:
        akte = fall['akte']
        if akte['letzter_fall'] != fall:
            continue
        abwlf = akte['abwlf']
        if heute.year*12 + heute.month <= fall['zday']*12 + fall['zdam'] + abwlf:
            continue
        min_abwlf = min(min_abwlf, abwlf)
        max_abwlf = max(max_abwlf, abwlf)
        akte_ids.append(akte['id'])
    return akte_ids, min_abwlf, max_abwlf
        
    
