# coding: latin-1

"""Module fr das Aktenvorblatt."""

import string

from ebkus.app import Request
from ebkus.app.ebapi import Akte, Fall, FallGruppeList, BezugspersonGruppeList, today, cc, getDate
from ebkus.app_surface.aktenvorblatt_templates import *
from ebkus.app.pdfgen import CustomPDF

class vorblattpdf(Request.Request):
    """Aktenvorblatt in PDF ausgeben"""
    permissions = Request.KLKARTE_PERM
    def processForm(self, REQUEST, RESPONSE):
        mitarbeiterliste = self.getMitarbeiterliste()
        user = self.user
        fallid = self.form.get('fallid')
        akte_id = self.form.get('akid')
        if fallid:
            fall = Fall(int(fallid))
            akte = fall['akte']
        elif akte_id: 
            akte = Akte(akte_id)
        else:
            self.last_error_message = "Keine ID fuer die Akte bzw. den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        doc = AktenvorblattPDF(akte)
        datei = doc.create_pdf()
        RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
        RESPONSE.setHeader('filename', 'aktenvorblatt_%d.pdf' % (akte['id'],))
        RESPONSE.setBody(datei)    

class vorblatt(Request.Request):
    """Aktenvorblatt in HTML ausgeben"""
    
    permissions = Request.KLKARTE_PERM
    
    def processForm(self, REQUEST, RESPONSE):
        mitarbeiterliste = self.getMitarbeiterliste()
        user = self.user
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
        else:
            self.last_error_message = "Keine ID fuer den Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        fall = Fall(int(fallid))
        akte = fall['akte']
        bezugspersonen = akte['bezugspersonen']
        einrichtungen = akte['einrichtungen']
        faelle = akte['faelle']
        faelle.sort('bgy', 'bgm', 'bgd')
        bezugspersonen.sort('verw__sort')
        einrichtungen.sort('status', 'na')
        
        # Aktueller bzw. letzter Fall, Wiederaufnehmbarkeit
        
        letzter_fall = akte['letzter_fall']
        aktueller_fall = akte['aktueller_fall']
        wiederaufnehmbar =  akte['wiederaufnehmbar']
        res = []
        res.append(head_normal_ohne_help_t % "Aktenvorblatt")
        res.append(vkopf_t % today())
        res.append(vakten1_t % akte)
        if akte['no'] !='':
            akte['nobedakte'] = 'Notiz'
        else:
            akte['nobedakte'] = ''
        if akte['ber'] != '' or akte['fs'] != cc('fsfs', '999') or akte['no'] !='':
            res.append(vakten2_t % akte)
        del akte['nobedakte']
        
        for b in bezugspersonen:
            res.append(vbezugspersonen1_t % b)
            if b['ber'] != '' or b['fs'] != cc('fsfs', '999') or b['no'] != '':
                res.append(vbezugspersonen2_t % b)
        i = 0
        res.append(vbezugspersonenzeile_t)
        
        res.append(veinrichtungs_kopf1_t)
        res.append(veinrichtungs_kopf2_t)
        for e in einrichtungen:
            res.append(veinrichtung_t % e)
        i = 0
        res.append(veinrichtungszeile_t)
        
        res.append(vanmeldung_kopf_t)
        for f in faelle:
            for a in f['anmeldung']:
                if a['no'] !='':
                    a['nobedanm'] = 'Notiz'
                else:
                    a['nobedanm'] = ''
                res.append(vanmeldung_t % a)
                del a['nobedanm']
                
        res.append(vleistungs_kopf_t)
        for f in faelle:
            for l in f['leistungen']:
                res.append(vleistungs_t % l)
                if l['ey'] == 0:
                    res.append(vleistungsendeleer_t)
                else:
                    res.append(vleistungsendedatum_t % l)
        i = 0
        res.append(vleistungszeile_t)
        
        res.append(vbearbeiter_kopf_t)
        for f in faelle:
            for z in f['zustaendigkeiten']:
                res.append(vbearbeiter_t % z)
                if z['ey'] == 0:
                    res.append(vbearbeiterendeoffen_t)
                else:
                    res.append(vbearbeiterendedatum_t % z)
        i = 0
        res.append(vbearbeiterzeile_t)
        
        res.append(vfall_kopf_t)
        for f in faelle:
            res.append(vfall_t % f)
            if f['aktuell'] == 1:
                res.append(vfalloffen_t)
            else:
                res.append(vfallendedatum_t % f)
        i = 0
        res.append(vfallzeile_t)
        res.append(vtabende_t)
        
        res.append(vnotiz_t)
        if akte['no']:
            res.append(vnotizakte_t % akte)
        for b in bezugspersonen:
            if b['no']:
                res.append(vnotizbperson_t % b)
        for e in einrichtungen:
            if e['no']:
                res.append(vnotizeinr_t % e)
        for f in faelle:
            for a in f['anmeldung']:
                if a['no']:
                    res.append(vnotizanm_t % a)
                    
        for f in faelle:
            fallgruppen = FallGruppeList(where = 'fall_id = %s' % f['id'])
            for g in fallgruppen:
                res.append(klkartegruppef_t % f)
                res.append(klkartegruppe_t % g)
                
        for b in bezugspersonen:
            bezugspersongruppen = BezugspersonGruppeList(where = 'bezugsp_id = %s' % b["id"])
            for e in bezugspersongruppen:
                res.append(klkartegruppeb_t % b)
                res.append(klkartegruppe_t % e)
        res.append(vnotizende_t)
        return string.join(res, '')


from reportlab.lib.units import inch, cm, mm
from reportlab.lib import colors
from reportlab.platypus import Paragraph, PageBreak, Spacer, Flowable
from reportlab.platypus.tables import Table, TableStyle

class AktenvorblattPDF(CustomPDF):
    # Margins
    lm = 10*mm
    rm = 10*mm
    tm = 10*mm
    bm = 10*mm
    # Show boundary around frame
    sb = False
    topline = False # explizite Linie auf dem topMargin
    bottomline = False # explizite Linie auf dem bottomMargin
    header_on_first_page = False
    header_left = ''
    header_center = ''
    header_right = ''
    def __init__(self, akte, noheader=False):
        CustomPDF.__init__(self)
        self.akte = akte
        self.noheader = noheader
    def add_bezugsperson(self, bezugsperson, daten, table_style, offset):
        """
        offset: row index of first row added here
        """
        templates = [
            ('{verw__name}',  '{na},<br/>{vn}','{gs__name[0]}', '{gb}',  '{plz} {ort}<br/>{str} {hsnr}', '{tl1}<br/>{tl2}'),
            ('',  'Ausbildung: {ber}', '', '', 'Wohnt bei: {fs__name}', 'Email: {mail}'),
            ('',  '{nobed__name}: {no}', '', '', '', ''),
            ]
        daten_bp = [self.formatzelle(t, bezugsperson, formatlabels=True) for t in templates]
        daten_bp = self.do_for_elements([(self.para(self.HeaderCell), (0,0), (0, 2))], daten_bp)
        daten_bp = self.do_for_elements([(self.para(self.DataCell), (1,0), (-1,-1))], daten_bp)
        o = offset
        table_style.add('SPAN', (0,0+o), (0,2+o))
        table_style.add('VALIGN', (0,0+o), (0,2+o), 'MIDDLE')
        table_style.add('SPAN', (1,1+o), (3,1+o))
        table_style.add('SPAN', (1,2+o), (5,2+o))
        daten += daten_bp
    def add_personen(self):
        bezugspersonen = self.bezugspersonen
        akte = self.akte
        templates = [
            ('', 'Name', 'G', 'Geb. am', 'Anschrift', 'Telefon'),
            ('Klient',  '{na},<br/>{vn}','{gs__name[0]}', '{gb}',  '{plz} {ort}<br/>{str} {hsnr}', '{tl1}<br/>{tl2}'),
            ('',  'Geschwister: {geschw}', '', '', 'Wohnt bei: {fs__name}', 'Email: {mail}'),
            ('',  'Ausbildung: {ber}', '', '', 'Aufbewahrungskategorie:<br/>{aufbew__name}', 'Abweichende<br/>Lschfrist: {abwlf}'),
            ('',  'Notiz: {no}', '', '', '', ''),
            ]
        daten = [self.formatzelle(t, akte, formatlabels=True) for t in templates]
        daten = self.do_for_elements([(self.para(self.HeaderCell), (0,0), (-1, 0))], daten)
        daten = self.do_for_elements([(self.para(self.HeaderCell), (0,1), (0, 4))], daten)
        daten = self.do_for_elements([(self.para(self.DataCell), (1,1), (-1,-1))], daten)
        table_style = self.get_table_style('StandardTable')
        table_style.add('SPAN', (0,1), (0,4))
        table_style.add('VALIGN', (0,0), (0,4), 'MIDDLE')
        table_style.add('SPAN', (1,2), (3,2))
        table_style.add('SPAN', (1,3), (3,3))
        #table_style.add('SPAN', (4,3), (5,3)) # nicht mehr SPAN, weil jetzt die abweischende Lschfrist eine eigenen Zelle braucht.
        table_style.add('SPAN', (1,4), (5,4))
        for i, bezugsperson in enumerate(bezugspersonen):
            self.add_bezugsperson(bezugsperson, daten, table_style, 5+(3*i))
        table = Table(daten, colWidths=(None,None,5*mm,2*cm,4*cm,3*cm))
        table.setStyle(table_style)
        self.addf(table)

    def add_einrichtung(self):
        einrichtungen = self.einrichtungen
        daten = [('Art','Kontakt','Telefon','Email','Akt.')]
        styles = []
        for i, einr in enumerate(einrichtungen):
            templates = [('{insta__name}', '{na}', '{tl1}<br/>{tl2}', '{mail}', '{status__code}'),
                         ('{nobed__name}: {no}', '',  '',  '',  '')]
            daten_einr = [self.formatzelle(t, einr, formatlabels=True) for t in templates]
            offset = 1 +(2*i) # Index der jeweils ersten sich wiederholenden Zeile
            styles.append(('SPAN', (0,1+offset), (4,1+offset)))
            daten += daten_einr
        self.addt(daten, colWidths=(None,None,3*cm,None,1*cm),
                  extratablestyles=styles)

    def add_anmeldung(self):
        faelle = self.faelle
        daten = [('Datum', 'Gemeldet von','Telefon','Zugangsmodus','durch', 'Anmeldegrund')]
        styles = []
        i = 0
        for  f in faelle:
            for anm in f['anmeldung']:
                templates = ('{fall__bg}', '{von}', '{mtl}', '{zm__name}', '{me}', '{mg}')
                daten += [self.formatzelle(templates, anm, formatlabels=False)]
                templates = ('Notiz: {no}', '',  '',  '',  '', '')
                daten += [self.formatzelle(templates, anm, formatlabels=True)]
                offset = 1 +(2*i) # Index der jeweils ersten sich wiederholenden Zeile
                styles.append(('SPAN', (0,1+offset), (5,1+offset)))
                i += 1
        self.addt(daten, colWidths=(2*cm,None,3*cm,None,None,None), extratablestyles=styles)

    def add_leistung(self):
        faelle = self.faelle
        daten = [('Mitarbeiter', 'Leistung','Am','Bis')]
        for  f in faelle:
            leistungen = f['leistungen']
            leistungen.sort('bgy', 'bgm', 'bgd')
            for lei in leistungen:
                e = lei['e']
                lei['eorempty'] = '' if e.year == 0 else str(e)
                templates = ('{mit__na}', '{le__name}', '{bg}', '{eorempty}')
                daten += [self.formatzelle(templates, lei)]
        self.addt(daten, colWidths=(None,None,2*cm,2*cm))

    def add_zustaendigkeit(self):
        faelle = self.faelle
        daten = [('Mitarbeiter', 'Beginn','Ende')]
        for  f in faelle:
            zustaendigkeiten = f['zustaendigkeiten']
            zustaendigkeiten.sort('bgy', 'bgm', 'bgd')
            for zust in zustaendigkeiten:
                e = zust['e']
                zust['eorempty'] = '' if e.year == 0 else str(e)
                daten += [self.formatzelle(
                        ('{mit__na},{mit__vn}', '{bg}', '{eorempty}'), 
                        zust)]
        self.addt(daten, colWidths=(None, 2*cm,2*cm))

    def add_stand(self):
        faelle = self.faelle
        daten = [('Fallnummer', 'Beginn','z.d.A.')]
        for fall in faelle:
            zda = fall['zda']
            fall['ppzda'] = zda if zda.year > 0 else ''
            daten += [self.formatzelle(('{fn}', '{bg}', '{ppzda}'), fall)]
        self.addt(daten)
                                                     
    def add_notizen(self):
        faelle = self.faelle
        akte = self.akte
        daten = [('Bezug', 'Notiz'),]
        if akte['no']:
            #print [self.formatzelle(("<i>Klient:</i> {na}, {vn}", "{no}"), akte),]
            daten += [self.formatzelle(("<i>Klient:</i> {na}, {vn}", "{no}"), akte),]
        for bezugsperson in self.bezugspersonen:
            if bezugsperson['no']:
                if bezugsperson['nobed__code'] == 't':
                    templates = ("<i>Bezugsperson:</i> {na}, {vn}", "(Wichtig) {no}")
                else:
                    templates = ("<i>Bezugsperson:</i> {na}, {vn}", "{no}")
                daten += [self.formatzelle(templates, bezugsperson),]
        for einr in self.einrichtungen:
            templates = []
            if einr['no']:
                if einr['nobed__code'] == 't':
                    templates = ("<i>Einrichtung:</i> {insta__name}, {na}", "(Wichtig) {no}")
                else:
                    templates = ("<i>Einrichtung:</i> {insta__name}, {na}", "{no}")
                daten += [self.formatzelle(templates, einr)]
        for  f in faelle:
            for anm in f['anmeldung']:
                if anm['no']:
                    daten += [self.formatzelle(("<i>Anmeldung:</i> vom {fall__bg}", "{no}"), anm)]
        if len(daten) == 1:
            daten += [(u'',u'')]
        #print 'ADD NOTIZEN', daten
        self.addt(daten)

    def add_gruppen(self):
        faelle = self.faelle
        bezugspersonen = self.bezugspersonen
        daten = [('Name', 'Gruppe', 'Gruppennr.', 'von', 'bis')]
        for f in faelle:
            fallgruppen = FallGruppeList(where = 'fall_id = %s' % f['id'])
            for g in fallgruppen:
                e = g['e']
                g['eorempty'] = '' if e.year == 0 else str(e)
                templates = ("{fall__akte__na},{fall__akte__vn} (Klient)",
                             "{gruppe__name}","{gruppe__gn}", "{bg}","{eorempty}")
                daten += [self.formatzelle(templates, g)]
        for b in bezugspersonen:
            bezugspersongruppen = BezugspersonGruppeList(where = 'bezugsp_id = %s' % b["id"])
            for g in bezugspersongruppen:
                e = g['e']
                g['eorempty'] = '' if e.year == 0 else str(e)
                templates = ("{bezugsp__na},{bezugsp__vn} ({bezugsp__verw__name})",
                             "{gruppe__name}","{gruppe__gn}", "{bg}","{eorempty}")
                daten += [self.formatzelle(templates, g)]
        if len(daten) > 1:
            self.addp("Gruppenteilnahme", 'MyHeading4')
            self.addt(daten, colWidths=(None,None,None,2*cm,2*cm))

    def add_content(self):
        akte = self.akte
        self.bezugspersonen = akte['bezugspersonen']
        self.einrichtungen = akte['einrichtungen']
        self.faelle = akte['faelle']
        self.faelle.sort('bgy', 'bgm', 'bgd')
        self.bezugspersonen.sort('verw__sort')
        self.einrichtungen.sort('status', 'na')
        
        # Aktueller bzw. letzter Fall, Wiederaufnehmbarkeit
        
        letzter_fall = akte['letzter_fall']
        aktueller_fall = akte['aktueller_fall']
        wiederaufnehmbar =  akte['wiederaufnehmbar']
        if not self.noheader:
            self.addp("<b>Aktenvorblatt vom %s</b>" % today(), 'Normal')
        #self.P("<b>Klient:</b> %(na)s, %(vn)s geb. am %(gb)s" % akte, 'Normal')
        self.addf(Spacer(1, 10))
        
        self.addp("Klient und Bezugspersonen", 'MyHeading4')
        self.add_personen()
        self.addp("Einrichtung", 'MyHeading4')
        self.add_einrichtung()
        self.addp("Anmeldung", 'MyHeading4')
        self.add_anmeldung()

        self.addp("Leistung", 'MyHeading4')
        self.add_leistung()
        self.addp(u"Zustndigkeit", 'MyHeading4')
        self.add_zustaendigkeit()
        self.addp("Stand", 'MyHeading4')
        self.add_stand()
        self.addp("Notizen", 'MyHeading4')
        self.add_notizen()
        self.add_gruppen() # berschrift nur wenn Gruppen da sind.
        self.addf(Spacer(1, 20))
        


