# coding: latin-1

"""Module fr den Im- und Export aus der Datenbank."""

import time, os
import re
from operator import itemgetter
from ebkus.utils import Bunch
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Code, JugendhilfestatistikList, today, cc, EE, ER, EW, Fall, \
    check_int_not_empty, check_str_not_empty, FallList, register_get, register_set, \
    get_berichtseinheitid
from ebkus.html.akte_share import akte_share
import ebkus.html.htmlgen as h
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.datenaustausch_templates import *
from ebkus.app.jghexport import jghexport as get_jghexport_datensaetze, \
    get_daten_fuer_uebersicht, get_daten_fuer_uebersicht_sql
from ebkus.app.jghexport2 import list_export_datei_groupdicts, export_formats, \
    jghexport2, jghexport2_all_formats_and_test
from ebkus.db.sql import SQL, getDBHandle

EXPORT_DIR = os.path.join('daten', 'export')

def align_right(daten):
    "Man msste htmlgen erweiteren ..."
    if daten:
        for zeile in daten:
            if zeile:
                for zelle in zeile[1:]:
                    if zelle:
                        zelle.class_ = 'tabledata_right'

class jghexportform(Request.Request, akte_share):
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = self.form.get('jahr')
        if jahr and jahr.isdigit():
            jahr = int(jahr)
        else:
            # default: Berichtsjahr ist das Vorjahr
            jahr = today().year - 1
        format = self.form.get('format')
        if format:
            if self.form.get('standard_export_format'):
                register_set('standard_export_format', format)
        else:
            # standard format evt. gespeichert
            format = register_get('standard_export_format')
            if format is None:
                format = 'datei'
        assert format in export_formats.keys()
        res = h.Page(
            title='Bundesjugendhilfestatistik exportieren fr das Berichtsjahr {}'.format(jahr),
            breadcrumbs = (('Administratorhauptmen', 'menu'),),
            rows=(self.get_hauptmenu(),
                  #self.get_berichtsjahr_auswahl(jahr),
                  self.get_auswahl(jahr, format),
                  self.export_durchfuehren_button(jahr, format),
                  self.get_uebersicht(jahr),
                  self.get_export_liste(jahr, format, None),
                  self.get_export_liste(jahr, format, 'abgeschlossen'),
                  self.get_export_liste(jahr, format, 'andauernd'),
                  # self.get_export_liste_mit_button(jahr, 'andauernd'),
                  # self.get_export_liste_mit_button(jahr, 'abgeschlossen'),
                  # self.get_export_liste_mit_button(jahr, 'andauernd'),
                  # self.get_addressen_ausserhalb_pruefen(),
                  ),
            )
        return res.display()

    def get_auswahl(self, berichtsjahr, format):
        minjahr = max(today().year - 2, 2022)
        maxjahr = today().year
        url_name_pairs_jahr = [('jghexportform?jahr={}&format={}'.format(jahr, format), "{}".format(jahr)) 
                               for jahr in range(maxjahr, minjahr-1, -1)
                               if jahr != berichtsjahr]
        url_name_pairs_format = [('jghexportform?jahr={}&format={}'.format(berichtsjahr, f), "{}".format(fname)) 
                                 for f, fname in export_formats.items()
                                 if f != format]
        checked = (register_get('standard_export_format')==format)
        on_click_url = "jghexportform?jahr={}&format={}&standard_export_format=1".format(jahr, format)
        on_click = "go_to_url('{}')".format(on_click_url) if not checked else None
        stz_berichtseinheitid = get_berichtseinheitid()
        if len(set(stz_berichtseinheitid.values())) == 1:
            # eine berichtseinheitid fr alle
            berichtseinheitid_zeilen = [
                [h.Dummy(n_col=4),
                 h.String(string='BerichtseinheitID: ', class_='labelclass', align='right',
                          tip="Zur Identifikation der die Statistik abgebenden Stelle. Manchmal auch als Identnummer bekannt."),
                 h.String(string="<code>{}</code".format(stz_berichtseinheitid.values()[0]), class_='labelclass'),
                 #h.String(string="Stelle A: 12345678123456; Stelle B: 12345678234567; Stelle C: 12345678345678;", class_='labelclass'),
                 ]
            ]
        else:
            berichtseinheitid_zeilen = [
                [h.Dummy(n_col=4),
                 h.String(string='BerichtseinheitID: ', class_='labelclass', align='right',
                          tip="Zur Identifikation der die Statistik abgebenden Stelle. Manchmal auch als Identnummer bekannt."),
                 h.String(string="Stelle {}: <code>{}</code".format(stz_code, berichtseinheitid), class_='labelclass'),
                 #h.String(string="Stelle A: 12345678123456; Stelle B: 12345678234567; Stelle C: 12345678345678;", class_='labelclass'),
                 ]
                for (stz_code, berichtseinheitid) in stz_berichtseinheitid.items()
            ]
        anzeige = h.FieldsetInputTable(
            legend='Berichtsjahr und Format auswhlen',
            daten=berichtseinheitid_zeilen + [
                # [h.Dummy(n_col=4),
                #  h.String(string='BerichtseinheitID: ', class_='labelclass', align='right',
                #           tip="Zur Identifikation der die Statistik abgebenden Stelle. Manchmal auch als Identnummer bekannt."),
                #  h.StringPre(string=config.BERICHTSEINHEITID, class_='labelclass'),
                #  #h.String(string="Stelle A: 12345678123456; Stelle B: 12345678234567; Stelle C: 12345678345678;", class_='labelclass'),
                #  ],
                [h.DummyItem(label_width="80%"),
                 h.DropdownMenu(name="Berichtsjahr", url_name_pairs=url_name_pairs_jahr,
                                tip="Sinnvollerweise macht man den Datenexport fr die "
                                "Bundesjugendhilfestatistik am Beginn des Folgejahres, "
                                "wenn die andauernden Flle zum Ende des Berichtsjahrs noch nicht "
                                "abgeschlossen sind. Fr abgeschlossene Flle kann man aber auch "
                                "fr frhere Jahre den Export wiederholen, da EBKuS die Statistik "
                                "fr abgeschlossene Flle nicht berschreibt."), 
                 h.String(string=berichtsjahr, class_='largetextbold'),
                 h.DummyItem(label_width="80%"),
                 ],
                [h.DummyItem(label_width="80%"),
                 h.DropdownMenu(name="Format", url_name_pairs=url_name_pairs_format,
                                tip="Neben dem bisherigen Format 'Dateimeldung', das voraussichtlich "
                                "ab dem 01.01.2025 nicht mehr zur Verfgung steht, "
                                "bietet EBKuS auch das Format Datenimport in das IDEV-Formular 'IDEV-CSV' "
                                "(IDEV-spezifische CSV-Datei) "
                                "und Dateibermittlung fr eSTATISTIK.core 'CORE-CSV' "
                                "(eSTATISTIK.core-spezifische CSV-Datei) an."
                                ), 
                 h.String(string=export_formats[format], class_='largetextbold'),
                 h.CheckItem(label="Als Standardformat festlegen",
                             name='standard_export_format',
                             value='1',
                             onClick=on_click,
                             checked=checked,
                             ),
                 ],
            ],
        )
        return anzeige
    def export_durchfuehren_button(self, jahr, format):
        if format == 'Dateimeldung':
            format_spezifisch = "Die Logdateien knnen mit einem Texteditor gelesen werden."
        else:
            format_spezifisch = ("Die Logdateien im CSV-Format knnen mit einer "
            "Tabellenkalkulation (z.B. Excel oder Calc) gelesen werden.")
            
        button_exportieren = h.Button(value="Exportieren",
                                      name='submitop',
                                      tip="Es wird je eine Datei mit den zu exportierenden Daten "
                                      "erstellt fr andauernde Flle, fr abgeschlossene Flle "
                                      "sowie fr alle Flle kombiniert. Fr jede dieser drei "
                                      "Dateien zustzlich eine Logdatei zur Kontrolle. {}".format(
                                          format_spezifisch),
                                      type='submit',
                                      )
        export_durchfuehren = h.FieldsetForm(
            legend='Export durchfhren fr das Berichtsjahr {} im {}-Format'.format(
                jahr, export_formats[format]),
            action='jghexport',
            method='post',
            name='jghexport',
            hidden=(
                ('file', 'export'),
                ('format', format),
                ('jahr', jahr),
                ),
            rows=(
                button_exportieren,
                ),
            )
        return export_durchfuehren
        
    def get_uebersicht(self, berichtsjahr):
        """
        Im Berichtsjahr 2018 gab es 2772 Flle, davon 1150 bernahmen aus 2017 und 1650 
        Neuaufnahmen.

                                       bernahmen  Neuaufnahmen  Insgesamt ohne Fall
        Flle im Berichtsjahr:

                                               mit zdA  ohne zdA  exportieren  zu klren  insgesamt ohne Fall (nur falls vorhanden)
                                                                   
        Bundesstatistik mit Eintrag 
        Ende der Hilfe/Beratung 2018              234     123         236         123         432      2        

        Bundesstatistik mit Eintrag
        Hilfe/Beratung dauert an fr 2018           0     344         345           0         345      1
        
        Keine Bundesstatistik vorhanden             0      23           0          23          23

        Sonstige                                  123      23           0          23         146

        Summe                                    


                           Button        Problematische Flle anzeigen

                                                                                        
                           Fallnummer    Beginn (Anmeldung) letzte Zustndigkeit    zdA   Bundesstatistik  fortdauernd
                                                                                          Berichtsjahr
                           123-2016HAU     02.08.2016              Tester          offen         2017           ja


        Abgeschlossene Flle im Berichtsjahr:

                                                      
        Fordauernde Flle am Ende des Berichtsjahrs: 

        """
        jahr = berichtsjahr
        #d = get_daten_fuer_uebersicht(berichtsjahr)
        d = get_daten_fuer_uebersicht_sql(berichtsjahr)
        tip_ohne_fall = "Statistik vorhanden, Falldaten aber gelscht, z.B. auf Verlangen des Klienten oder nach Ablauf der Lschfrist"
        tip_uebernahme = "siehe 'Neumeldungen und Abschlsse', Zeile 1, Spalte 2"
        tip_neuaufnahme = "siehe 'Neumeldungen und Abschlsse', letzte Zeile, Spalte 3"
        tip_summe = "Anzahl der Flle, die irgenwann im Berichtsjahr mal aktiv waren"
        headers = (
            ('bernahmen aus dem Vorjahr', tip_uebernahme),
            ('Neuaufnahmen', tip_neuaufnahme),
            ('ohne Fall', tip_ohne_fall),
            ('Summe', tip_summe)
            )
        #if d.sum_BS_ohne_fall > 0:
        # if True:
        #     headers.append(('ohne Fall', "verbliebene Statistik bei gelschten Falldaten"))

        daten = [[h.String(string='Flle im Berichtsjahr:',
                           class_='normaltext'),
                  h.String(string=d.sum_uebernahmen, tip=tip_uebernahme),
                  h.String(string=d.sum_neuaufnahmen, tip=tip_neuaufnahme),
                  h.String(string=d.sum_BS_ohne_fall, tip=tip_ohne_fall),
                  h.String(string=d.sum_faelle + d.sum_BS_ohne_fall, tip=tip_summe),
                  ],
               ]
        align_right(daten)
        faelle = h.TableDataTable(
            headers=headers,
            noheaders=1,
            daten=daten,
            )
        headers = (
            ('mit zdA', "abgeschlossene Flle, zdA erfolgt"),
            ('ohne zdA', "offene Flle, zdA noch nicht erfolgt"),
            ('zu exportieren', "Flle, fr die zum jetzigen Zeitpunkt Datenstze "
             "fr die Bundesjugendhilfstatistik erstellt werden knnen"),
            ('zu klren', "Flle, fr die potentiell bei entsprechender Aktualisierung "
             "Datenstze fr die Bundesjugendhilfstatistik erstellt werden knnten"),
            ('ohne Fall', tip_ohne_fall),
            ('Summe', "Zeilensumme"),
            )
        daten=[[h.String(string='Bundesstatistik mit Eintrag\n'
                            'Ende der Hilfe/Beratung {}:'.format(jahr),
                            class_='normaltext'),
                h.String(string=d.sum_BS_Berichtsjahr_beendet_mit_zda),
                h.String(string=d.sum_BS_Berichtsjahr_beendet -
                         d.sum_BS_Berichtsjahr_beendet_mit_zda,
                         tip="Diese Flle knnen erst nach erfolgtem Fallabschluss (zdA) "
                         "fr die Bundesjugendhilfstatistik bernommen werden. "),
                h.String(string=d.sum_BS_Berichtsjahr_beendet_mit_zda +
                         d.sum_BS_ohne_fall_beendet),
                h.String(string=d.sum_BS_Berichtsjahr_beendet -
                         d.sum_BS_Berichtsjahr_beendet_mit_zda,
                         tip="Diese Flle knnen erst nach erfolgtem Fallabschluss (zdA) "
                         "fr die Bundesjugendhilfstatistik bernommen werden. "),
                h.String(string=d.sum_BS_ohne_fall_beendet),
                h.String(string=d.sum_BS_Berichtsjahr_beendet + 
                         d.sum_BS_ohne_fall_beendet),
                ],
               [h.String(string='Bundesstatistik mit Eintrag\n'
                            'Hilfe/Beratung dauert an fr {}:'.format(jahr),
                            class_='normaltext'),
                h.String(string='0'),
                h.String(string=d.sum_BS_Berichtsjahr_andauernd),
                h.String(string=d.sum_BS_Berichtsjahr_andauernd +
                         d.sum_BS_ohne_fall_andauernd),
                h.String(string='0'),
                h.String(string=d.sum_BS_ohne_fall_andauernd), 
                h.String(string=d.sum_BS_Berichtsjahr_andauernd + 
                         d.sum_BS_ohne_fall_andauernd),
                ],
               [h.String(string='Keine Bundesstatistik vorhanden:',
                         class_='normaltext',
                         tip="Logisch mglich sind hier nur Flle ohne zdA, "
                         "da EBKuS das Vorhandensein der Statistik bei Fallabschluss prft"),
                h.String(string='0', 
                         tip="Logisch unmglich, da EBKuS den Fallabschluss ohne Statistik nicht zulsst"),
                h.String(string=d.sum_BS_fehlt),
                h.String(string='0'),
                h.String(string=d.sum_BS_fehlt,
                         tip="Offene Flle, fr die die Bundesstatistik noch nachgetragen werden kann."),
                h.String(string='0'),
                h.String(string=d.sum_BS_fehlt),
                ],
               [h.String(string='Sonstige:',
                         class_='normaltext',
                         tip="Fortdauernde Flle, die inzwischen aber abgeschlossen wurden, "
                         "und offene Flle, fr die die Bundesstatistik noch aktualisiert werden kann."),
                h.String(string=d.sum_BS_sonstige_zda,
                         tip="Diese Flle waren am Ende des Berichtsjahr fortdauernd, inzwischen ist "
                         "aber zdA erfolgt und die Bundesstatistik damit berschrieben. "
                         "Datenstze fr fortdauernde Flle mssen rechtzeitig exportiert werden!"
                         ""),
                h.String(string=d.sum_BS_sonstige_ohne_zda,
                         tip="Fr diese offenen Flle kann die Bundesstatistik noch aktualisiert werden"),
                h.String(string='0'),
                h.String(string=d.sum_BS_sonstige_ohne_zda,
                         tip="Fr diese offenen Flle kann die Bundesstatistik noch aktualisiert werden"),
                h.String(string='0'),
                h.String(string=d.sum_BS_sonstige),
                ],
               ]
        summen_zeile = [h.String(string='Summe:',
                                 tip='Spaltensumme',
                                 class_='normaltext')]
        for spalten_index in range(1,7):
            summe = 0
            for zeile in daten:
                summe += int(zeile[spalten_index].string)
            summen_zeile.append(h.String(string=str(summe)))
        daten.append(summen_zeile)
        button_zu_klaeren = h.Button(value='Zu klrende Flle anzeigen',
                                     class_='buttonhuge',
                                     onClick="go_to_url('zuerfaelle?jahr={}')".format(jahr),
                                     tip="Die Flle aus der Spalte 'zu klren' anzeigen. "
                                     "Sinnvoll ist dies hauptschlich am Beginn des neuen "
                                     "Jahres, um den Datenexport fr das vergangene Berichtsjahr "
                                     "vorzubereiten.",
                                     )
        align_right(daten)
        statistik_uebersicht = h.TableDataTable(
            headers=headers,
            noheaders=1,
            daten=daten,
            )
        uebersicht = h.Fieldset(
            legend='bersicht fr das Berichtsjahr {}'.format(jahr),
            rows=(faelle,
                  statistik_uebersicht,
                  button_zu_klaeren,
            ),
            )
        return uebersicht

    def get_export_liste(self, jahr, format, welche):
        """
        Datentabelle mit Exportdateien zum runterladen

        Dateiname Log-Datei Exportdatum Anzahl der Datenstze

        Immer nur die Dateien fr das Berichtsjahr.
        """
        dateien_dicts = list_export_datei_groupdicts(jahr, format, welche)
        headers=('Dateiname', 'Log-Datei', 'Exportdatum', 'Anzahl Datenstze')
        daten=[[h.Link(string=d['name'], url="jghexport?file=download&filename={}".format(d['name'])),
                h.Link(string=d['name_log'], url="jghexport?file=download&filename={}".format(d['name_log'])),
                h.String(string=d['datum']),
                h.String(string=d['count'], align='right'),
                ] for d in dateien_dicts
               ]
        export_dateien = h.TableDataTable(
            headers=headers,
            daten=daten,
            )
        welche_to_label = {
            'andauernd': 'Andauernde',
            'abgeschlossen': 'Abgeschlossene',
            None: 'Alle'}
        export_liste = h.Fieldset(
            legend='Exportdateien fr das Berichtsjahr {}: {} Flle'.format(jahr, welche_to_label[welche]),
            rows=(
                export_dateien,
                ),
            )
        return export_liste

            
class zuerfaelle(Request.Request, akte_share):
    """Flle anzeigen, die potentiell fr den Datenexport in Frage kommen nach
    entsprechender Aktualisierung.
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        jahr = check_int_not_empty(self.form, 'jahr', "Jahr fehlt")
        # welche sind zu klren? nur offene Flle
        offene_faelle = FallList(where = 'bgy <= {jahr} and zday = 0'.format(jahr=jahr), 
                                 cache_related=True)
        # ganz ohne statistik -> knnten fortdauernd sein
        faelle_ohne_statistik = [f for f in offene_faelle if f['jgh'] is None]
        # mit statistik ende eingetragen, aber kein zda
        faelle_ohne_zda = [f for f in offene_faelle 
                           if f['jgh'] and f['jgh__ey']==jahr]
        # kein zda, statistik vorhanden aber falsches jahr
        faelle_mit_statistik_sonstige = (
            [f for f in offene_faelle if f['jgh'] and 
             ((f['jgh__ey']!=None and f['jgh__ey']!=jahr) or 
              (f['jgh__ey']==None and f['jgh__jahr']!=jahr))])
        

        inhalt = h.FieldsetDataTable(
            legend='Inhalt',
            daten=[[h.String(string="Zu klrende Flle fr den Bundesstatistikexport fr " 
                             "das Berichtsjahr {}".format(jahr),
                             class_='largetextbold')],
                   [h.Link(string='Offene Flle ohne Bundesstatistik ({})'.
                           format(len(faelle_ohne_statistik)),
                           url='#ohne_statistik')],
                   [h.Link(string='Ende eingetragen, aber kein zdA ({})'.
                           format(len(faelle_ohne_zda)),
                           url='#ohne_zda')],
                   [h.Link(string='Berichtsjahr oder Endejahr falsch ({})'.
                           format(len(faelle_mit_statistik_sonstige)),
                           url='#sonstige')],
                   ],
            )

        # Felder:
        # fallnummer, beginn, letzte zustndigkeit
        
        headers=('Fallnummer', 'Beginn', 'zdA', 
                 ('Berichtsjahr', "Das in der Bundesstatistik eingetragene Jahr."),
                 ('Ende Monat/Jahr bzw. fortdauernd', "'Ende der Hilfe/Beratung' ist in "
                  "die Bundesstatistik eingetragen oder 'Hilfe/Beratung dauert an'."),
                 # ('Fortdauernd', "'Hilfe/Beratung dauert an' ist in "
                 #  "die Bundesstatistik eingetragen."),
                 ('Mitarbeiter', "Zuletzt zustndige(r) Mitarbeiter(in)"),
                 )
        def zeile(f):
            return [h.String(string=f['fn']),
                    h.Datum(date=f.getDate('bg')),
                    h.Datum(date=f.getDate('zda')),
                    h.String(string=f['jgh__jahr']),
                    h.String(string="{}/{}".
                             format(f['jgh__em'],f['jgh__ey']) 
                             if f['jgh__ey'] else 
                             ('fortdauernd' if f['jgh__hda__code']=='1'
                             else '')),
                    #h.String(string=f['jgh__hda__name']),
                    h.String(string=f['zuletzt_zustaendig__mit__na']),
                    ]
        ohne_statistik = h.FieldsetDataTable(
            legend='Offene Flle ohne Bundesstatistik ({})'.format(len(faelle_ohne_statistik)),
            anchor='ohne_statistik',
            headers=headers,
            header_class='sortable',
            daten=[zeile(f) for f in faelle_ohne_statistik],
            )
        kein_zda = h.FieldsetDataTable(
            legend='Ende eingetragen, aber kein zdA ({})'.format(len(faelle_ohne_zda)),
            anchor='ohne_zda',
            headers = headers,
            header_class='sortable',
            daten=[zeile(f) for f in faelle_ohne_zda],
            )
        sonstige = h.FieldsetDataTable(
            legend='Berichtsjahr oder Endejahr falsch ({})'.format(len(faelle_mit_statistik_sonstige)),
            anchor='sonstige',
            headers = headers,
            header_class='sortable',
            daten=[zeile(f) for f in faelle_mit_statistik_sonstige],
            )
        res = h.Page(
            title='Zu klrende Flle fr den Bundesstatistikexport {}'.format(jahr),
            breadcrumbs = (('Administratorhauptmen', 'menu', ),
                           ('Bundesjugendhilfstatistik exportieren', 
                            'jghexportform?jahr={}'.format(jahr)),
                           ),
            rows=(self.get_zurueck(),
                  inhalt,
                  ohne_statistik,
                  kein_zda,
                  sonstige,
                  self.get_zurueck(),
                ),
            )
        return res.display()

class jghexport(Request.Request):
    """Die action von post request
    file=download
    evt. file=delete
    file=export
    am Ende immer ein redirect auf jghexportform
    """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        #print 'JGHEXPORT:', self.form
        file = self.form.get('file')
        if not file or file not in ('download', 'export'):
            raise EE('Parameter file fehlt oder falsch')
        if file == 'download':
            filename = check_str_not_empty(self.form, 'filename', 'Kein Dateiname')
            self.download(filename)
            return ''
        elif file == 'export':
            jahr = check_int_not_empty(self.form, 'jahr', 'Kein Berichtsjahr')
            format = check_str_not_empty(self.form, 'format', 'Kein format')
            #test = True
            test = False
            if test:
                # Alternativ zum Testen
                jghexport2_all_formats_and_test(
                    jahr,
                    os.path.join(config.DOCUMENT_ROOT, 'daten', 'test'),
                    ppdata=False)
                RESPONSE.redirect('jghexportform?jahr={}&format={}'.format(jahr, format), status=303)
                return None
            else:
                anzahl_andauernd, anzahl_abgeschlossen, anzahl_gesamt = jghexport2(jahr, format)
                legend = "Export erfolgreich"
                res = h.Meldung(
                    legend=legend,
                    title=legend,
                    zeilen=["{} Datenstze erfolgreich exportiert fr das Berichtsjahr {} im Format {}".format(anzahl_gesamt, jahr, export_formats.get(format)),
                            "(davon {} fr andauernde und {} fr abgeschlossene Flle)".format(anzahl_andauernd, anzahl_abgeschlossen),
                            ],
                    weiter='jghexportform?jahr={}&format={}'.format(jahr, format),
                ).display()
                return res

    def download(self, filename):
        #print 'DOWNLOAD:'
        path = os.path.join(config.DOCUMENT_ROOT, EXPORT_DIR, filename)
        f = open(path, 'rb')
        content = f.read()
        f.close()
        if filename.endswith('.csv'):
            self.RESPONSE.setHeader('content-type', 'text/csv')
        else:
            self.RESPONSE.setHeader('content-type', 'text/plain')
        # zum Test besser, wenns im Browser angezeigt wird.
        self.RESPONSE.setHeader('content-disposition', 'attachment; filename=%s' % filename)
        self.RESPONSE.setBody(content)

