# coding: latin-1

"""Module fr die Dokumente."""

import os

from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Fall, Akte, Dokument, Gruppendokument, DokumentList, \
     GruppendokumentList, Gruppe, today, cc, get_akte_path, get_gruppe_path, is_binary
from ebkus.app.ebapih import get_codes, mksel
from ebkus.app_surface.dokument_templates import *
from ebkus.app_surface.standard_templates import *

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share

def get_permissions():
    """Zugriff auf Beratungsdoku erlauben fr die Rolle 'verw',
    wenn BERATUNGSKONTAKT_DOKUMENTATION konfiguriert ist.
    """
    if config.BERATUNGSKONTAKT_DOKUMENTATION:
        return 'bearb', 'verw'
    else:
        return 'bearb',

class _dokument(Request.Request, akte_share):
    def _edit_dokument(self,
                       title,
                       file,
                       dok,
                       ):
        #print '_EDIT_DOKUMENT', file, dok
        fall_id = dok.get('fall_id')
        if fall_id:
            fall = Fall(fall_id)
            hidden_val = ('fallid', fall_id)
        else:
            fall = None
            gruppe = Gruppe(dok['gruppe_id'])
            hidden_val = ('gruppeid', gruppe['id'])
        new = file in ('dokeinf', 'dokgreinf', 'uploadeinf', 'uploadgreinf')
        upload = file in ('uploadeinf', 'uploadgreinf')
        meta = h.FieldsetInputTable(
            legend='Betreff, Art, Datum',
            daten=[[h.SelectItem(label='Art des Dokuments',
                                 name='art',
                                 options=self.for_kat('dokart', sel=dok['art']),
                                 ),
                    h.DatumItem(label='Datum',
                                name='v',
                                date=dok.getDate('v'),
                                ),
                    ],
                   [h.TextItem(label='Betreff',
                               name='betr',
                               value='%(betr)s' % dok,
                               class_='textboxverylarge',
                               n_col=4,
                               ),
                    ],
                   ],
            )
        if upload:
            inp = h.FieldsetInputTable(
                legend='Datei importieren',
                daten=[[h.UploadItem(label='Lokaler Dateiname',
                                     name='datei',
                                     tip='Lokale Datei hochladen',
                                     class_='textboxlarge',
                                     ),
                        ]],
                )
        elif not is_binary(dok):
            inp = h.FieldsetInputTable(
                legend='Inhalt',
                daten=[[h.TextareaItem(label_width='122pt',
                                       name='text',
                                       value=dok['text'],
                                       rows='10',
                                       cols='70',
                                       class_='textareaverylarge',
                                       tip='Text schreiben oder einfgen',
                                     ),
                ]],
                )
        else:
            inp = None
        if fall:
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Klientendokumente', 'kldok?fallid=%(id)s' % fall),
                           )
        else:
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Gruppenmen', 'menugruppe'),
                           ('Gruppendokumente', 'grdok?gruppeid=%(id)s' % gruppe),
                           )
        res = h.FormPage(
            title=title,
            # TODO hier mssen neue Kapitel  rein, oder nachsehen
            breadcrumbs=breadcrumbs,
            #help=help,
            name='dokform',
            action=fall and 'kldok' or 'grdok',
            method='post',
            hidden=(('file', file),
                    ('dokid', dok['id']),
                    ('mitid', dok['mit_id']),
                    hidden_val,
                    ),
            rows=(fall and self.get_klientendaten_kurz(fall) or
                  self.get_gruppendaten_kurz(gruppe),
                  meta,
                  inp,
                  h.SpeichernZuruecksetzenAbbrechen(),
                  ),
            )
        return res.display()

class vermneu(_dokument):
    """Neuen Text  eintragen (Tabelle: Dokument, Gruppendokument)."""
    permissions = get_permissions()
    def processForm(self, REQUEST, RESPONSE):
        # Das Formular ist 1 Fall oder 1 Gruppe zugeordnet.
        gruppeid = self.form.get('gruppeid')
        fallid = self.form.get('fallid')
        if gruppeid:
            gruppe = Gruppe(gruppeid)
            dok = Gruppendokument()
            dok.init(
                id=Gruppendokument().getNewId(),
                gruppe_id=gruppe['id'],
                )
            file = 'dokgreinf'
        elif fallid:
            fall = Fall(fallid)
            dok = Dokument()
            dok.init(
                id=Dokument().getNewId(),
                fall_id=fall['id'],
                )
            file = 'dokeinf'
            dokid = Dokument().getNewId()
        else:
            self.last_error_message = "Keine ID fuer Gruppe oder Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)

        dok.setDate('v', today())
        dok.init(
            mit_id=self.mitarbeiter['id'],
            art=cc('dokart', 'bnotiz'),
            mtyp=cc('mimetyp', 'txt'),
            )
        dok['text'] = '' # kein db-Feld!
        return self._edit_dokument(title='Neues Textdokument erstellen',
                                   file=file,
                                   dok=dok,
                                   )
    
        
class updverm(_dokument):
    """Text ndern (Tabelle: Dokument, Gruppendokument)."""
    permissions = get_permissions()
    def processForm(self, REQUEST, RESPONSE):
        # Das Formular ist 1 Fall oder 1 Gruppe zugeordnet.
        gruppeid = self.form.get('gruppeid')
        fallid = self.form.get('fallid')
        dokid = self.form.get('dokid')
        if not dokid:
            self.last_error_message = "Keine ID fuer Dokument erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        if gruppeid:
            dok = Gruppendokument(dokid)
            file='updgrvermeinf'
            gruppe_path = get_gruppe_path(int(gruppeid))
            fname = os.path.join(gruppe_path, dok['fname'])
        elif fallid:
            file='updvermeinf'
            dok = Dokument(dokid)
            fall = Fall(fallid)
            fname = os.path.join(get_akte_path(fall['akte__id']),
                                 dok['fname'])
        else:
            self.last_error_message = "Keine ID fuer Gruppe oder Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        if not is_binary(dok):
            dok['text'] = open(fname, 'r').read()
        return self._edit_dokument(title='Dokument bearbeiten',
                                   file=file,
                                   dok=dok,
                                   )

# dieselber Prozedur fr Gruppendolumente
updgrverm = updverm

class upload(_dokument):
    """Dokument uploaden (Tabellen: Dokument, Gruppendokument)."""
    permissions = get_permissions()
    def processForm(self, REQUEST, RESPONSE):
        gruppeid = self.form.get('gruppeid')
        fallid = self.form.get('fallid')
        if gruppeid:
            gruppe = Gruppe(gruppeid)
            dok = Gruppendokument()
            dok.init(
                id=Gruppendokument().getNewId(),
                gruppe_id=gruppe['id'],
                )
            file = 'uploadgreinf'
        elif fallid:
            fall = Fall(fallid)
            dok = Dokument()
            dok.init(
                id=Dokument().getNewId(),
                fall_id=fall['id'],
                )
            file = 'uploadeinf'
            dokid = Dokument().getNewId()
        else:
            self.last_error_message = "Keine ID fuer Gruppe oder Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        dok.setDate('v', today())
        dok.init(
            mit_id=self.mitarbeiter['id'],
            art=cc('dokart', 'Bericht'),
            )
        return self._edit_dokument(title='Dokument importieren',
                                   file=file,
                                   dok=dok,
                                   )

        
class rmdok(_dokument):
    """Lsche Dokument (Tabellen: Dokument, Gruppendokument)."""
    permissions = get_permissions()
    def processForm(self, REQUEST, RESPONSE):
        dokid = self.form.get('dokid')
        fallid = self.form.get('fallid')
        gruppeid = self.form.get('gruppeid')
        if not dokid:
            self.last_error_message = "Keine ID fr Dokument erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        if fallid:
            dok = Dokument(dokid)
            file = 'removedoks'
            action = 'kldok'
            hidden_val = ('fallid', fallid)
        elif gruppeid:
            dok = Gruppendokument(dokid)
            file = 'removegrdoks'
            action = 'grdok'
            hidden_val = ('gruppeid', gruppeid)
        else:
            self.last_error_message = "Keine ID fuer Fall oder Gruppe erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        return h.SubmitOrBack(
            legend='Dokument lschen',
            action=action,
            method='post',
            hidden=(('file', file),
                    ('dokids', dokid),
                    hidden_val,
                    ),
            zeilen=('Soll ','',
                    "%(art__name)s: %(betr)s (Typ: %(mtyp__code)s, Datum: %(vd)s.%(vm)s.%(vy)s, Mitarbeiter: %(mit_id__na)s)" % dok, '',
                    ' gelscht werden?',
            )
            ).display()
        
        
        
