# coding: latin-1

"""Module fr die Mitarbeiter-Daten."""

import string

from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Mitarbeiter, MitarbeiterList, Code, today, cc, EE
from ebkus.app.ebapih import get_codes, get_all_codes, mksel,mksel_benr
from ebkus.app_surface.mitarbeiter_templates import *
from ebkus.app_surface.standard_templates import *

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share

class _mitarbeiter(Request.Request, akte_share):
    def get_mitarbeiter(self, 
                        edit_button=False, # falls False, kein edit button
                        hinzufuegen_button=False, # falls False, kein hinzufgen button
                        welche='im_dienst',
                        ):
        im_dienst = cc('status', 'i')
        nicht_im_dienst = cc('status', 'a')
        status = dict(im_dienst=im_dienst,
                      nicht_im_dienst=nicht_im_dienst,
                      alle="{},{}".format(im_dienst,nicht_im_dienst))
        mitarbeiter_list = MitarbeiterList(
            where='stat in ({})'.format(status[welche]))
        mitarbeiter_list.sort('stz__code', 'na')
        mitarbeiter_table = h.FieldsetFormDataTable(
            name="mitausw",action="mitausw",method="get",
            legend='Mitarbeiter',
            daten_before_headers = [
                [h.Dummy(n_col=1),
                 h.RadioItem(label='im Dienst',
                             name='w',
                             value='im_dienst',
                             checked=(welche=='im_dienst'),
                             onClick="this.form.submit()",
                             tip='Nur Mitarbeiter im Dienst anzeigen',
                ),
                 h.RadioItem(label='nicht im Dienst',
                             name='w',
                             value='nicht_im_dienst',
                             checked=(welche=='nicht_im_dienst'),
                             onClick="this.form.submit()",
                             tip='Nur Mitarbeiter nicht im Dienst anzeigen',
                 ),
                 h.RadioItem(label='alle',
                             name='w',
                             value='alle',
                             checked=(welche=='alle'),
                             onClick="this.form.submit()",
                             tip='Alle Mitarbeiter anzeigen',
                 ),
                ],
            [h.Dummy(n_col=7),],
            [h.Dummy(n_col=7),],
            ],
            headers=('Benutzer', 'Name', 'Vorname', 'Status', 'Rechte', 'Stelle'),
            # header_class='sortable', # funktioniert nicht, weil Hinzufgen etc. mitsortiert wird
            daten=[[edit_button and h.Icon(href= 'updmit?mitid=%(id)d' % mit,
                                            icon= "/ebkus/ebkus_icons/edit_button.gif",
                                            tip= 'Mitarbeiter bearbeiten')
                                      or None,
                    h.String(string=mit['ben']),
                    h.String(string=mit['na']),
                    h.String(string=mit['vn']),
                    h.String(string=mit['stat__name']),
                    h.String(string=mit['benr__name']),
                    h.String(string=mit['stz__name']),
                    ]
                    for mit in mitarbeiter_list],
            button=(hinzufuegen_button and
                    h.Button(value="Hinzufgen",
                             tip="Mitarbeiter hinzufgen",
                             onClick=
                             "go_to_url('mitneu')"
                             ) or None),
            )
        return mitarbeiter_table

    def _process(self, 
                 title,
                 file,
                 mit,
                 ):
        edit = h.FieldsetInputTable(
            legend=title,
            daten=[[h.TextItem(label='Vorname',
                               name='vn',
                               value=mit['vn'],
                               tip='Vorname des Mitarbeiters'
                               ),
                    h.TextItem(label='Nachname',
                               name='na',
                               value=mit['na'],
                               tip='Nachname des Mitarbeiters'
                               ),
                    h.TextItem(label='Benutzerkennung',
                               name='ben',
                               value=mit['ben'],
                               tip='Benutzerkennung des Mitarbeiters'
                               ),
                    ],
                   [h.SelectItem(label='Status',
                                 name='stat',
                                 options=self.for_kat('status',
                                                      sel=mit['stat'], all=True),
                                 ),
                    h.SelectItem(label='Rechte',
                                 name='benr',
                                 options=self.for_kat('benr',
                                                      sel=mit['benr'], all=True),
                                 ),
                    h.SelectItem(label='Stelle',
                                 name='stz',
                                 options=self.for_kat('stzei',
                                                      sel=mit['stz'], all=True),
                                 ),
                    ],
                   file in ('updmit',) and
                   [h.CheckItem(label='Passwort zurcksetzen',
                                name='changepassword',
                                value='1',
                                checked=False,
                                tip='Neues Passwort ist identisch mit der Benutzerkennung'
                               ),
                    h.DummyItem(),
                    ] or None,
                   ],
            )
        res = h.FormPage(
            title=title,
            name='mitedit',action="mitausw",method="post",
            breadcrumbs = (('Hauptmen', 'menu'),
                           ('Mitarbeiter', 'mitausw'),
                           ),
            hidden=(("mitid", mit['id']),
                    ("file", file),
                    ),
            rows=(edit,
                  h.SpeichernZuruecksetzenAbbrechen(),
                  #self.get_mitarbeiter(),
                  ),
            )
        return res.display()

class mitausw(_mitarbeiter):
    """Auswahlformular zum ndern der Mitarbeiterdaten. """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        file = self.form.get('file')
        if file:
            from ebkus.app import ebupd
            function = getattr(ebupd, file)
            function(self.form)
        welche = self.form.get('w')
        if not welche:
            welche = 'alle'
        res = h.Page(
            title='Mitarbeiter',
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  self.get_mitarbeiter(edit_button=True,
                                       hinzufuegen_button=True,
                                       welche=welche),
                  ),
            )
        return res.display()

class mitneu(_mitarbeiter):
    """Mitarbeiterstammdatenformular."""
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        mit = Mitarbeiter()
        mit.init(
            id=Mitarbeiter().getNewId(),
            vn='',
            na='',
            ben='',
            benr=cc('benr', 'bearb'),
            stat=cc('status', 'i'),
            stz=get_codes('stzei')[0],
            )
        return self._process('Neuer Mitarbeiter',
                             'miteinf',
                             mit)
    
class updmit(_mitarbeiter):
    """Updateformular fr die Stammdaten der Mitarbeiter. """
    permissions = Request.ADMIN_PERM
    def processForm(self, REQUEST, RESPONSE):
        mit_id = self.form.get('mitid')
        if not mit_id:
            raise EE('Keine Mitarbeiter-Id')
        mit = Mitarbeiter(mit_id)
        return self._process('Mitarbeiter bearbeiten',
                             'updmit',
                             mit)

