# coding: latin-1

#

from ebkus.app.ebapih import get_codes, get_all_codes, make_option_list
from ebkus.app.ebapi import KategorieList, ZustaendigkeitList, today, AbfrageList, \
    BezugspersonList, MitarbeiterGruppeList, MitarbeiterberatungskontaktList, \
    Fall, ZusaetzlicheBerechtigungList
from ebkus.db.sql import SQL
from ebkus.config import config

def get_land_kr_einrnr(kat, stelle):
    """Das dok-Feld der Merkmale von kr, land, einrnr, kann mit
    Stelle <stellencode> beginnen. Dann wird das Merkmal in
    Option-List an erste Stelle gesetzt. Relevant fr die
    Bundesstatistik, damit verschiedene Stellen unterschiedliche
    Merkmale haben knnen und diese nicht fehleranfllig auswhlen
    mssen.
    """
    assert kat in ('kr', 'land', 'einrnr')
    codes = get_codes(kat)
    if len(codes) == 1:
        return codes[0]
    stz_code = stelle['code'].lower()
    found = None
    for c in codes:
        dok = c['dok']
        if dok:
            dok = dok.strip().lower()
            if ('stelle ' + stz_code + ';') in dok:
                found = c
                break
    return found

class options(object):
    """Klasse zur Generierung von Option-Listen
    """
    option_tmpl = '<option value="%s"%s>%s</option>'


    def for_land_kr_einrnr(self, kat):
        assert kat in ('kr', 'land', 'einrnr')
        codes = get_codes(kat)
        found = self.get_land_kr_einrnr(kat)
        if found:
            codes.remove(found)
            codes.insert(0, found)
        return  make_option_list(codes, 'id', 'name')
    def get_land_kr_einrnr(self, kat, stelle=None):
        """Das dok-Feld der Merkmale von kr, land, einrnr, kann mit
        Stelle <stellencode> beginnen. Dann wird das Merkmal in
        Option-List an erste Stelle gesetzt. Relevant fr die
        Bundesstatistik, damit verschiedene Stellen unterschiedliche
        Merkmale haben knnen und diese nicht fehleranfllig auswhlen
        mssen.
        """
        if stelle is None:
            stelle = self.stelle
        return get_land_kr_einrnr(kat, stelle)
    def for_plz(self):
        "Alle Postleitzahlen im Straenkatalog"
        plzs = SQL("select distinct plz from strkatalog").execute()
        return '\n'.join([(self.option_tmpl % (p[0],'',p[0])) for p in plzs])
    def url_name_pairs_for_auswertungen(self):
        url_name_pairs = [
                      ("", "Beratungen"),
                      ("abfr1?w=alle&jahr={}".format(today().year-4), "- alle Beratungen"),
                      ("abfr1?w=laufend", "- laufende Beratungen"),
                      ("abfr1?w=abgeschlossen&jahr={}".format(today().year-4), "- abgeschlossene Beratungen"),
                      #("", ""),
                      ("", "Klientenzahl"),
                      ("abfr4", "- Neumeldungen u. Abschl&uuml;sse"),
                      ("abfr5", "- Klienten pro Mitarbeiter"),
                      ("wartezeiten", "- Wartezeiten"),
                      #("", ""),
                      ("", "Gruppen"),
                      ("abfr8", "- Gruppen&uuml;berblick"),
                      #("", ""),
                      ("", "Adressen exportieren"),
                      ("adressen?wel=fall", "- Flle"),
                      ("adressen?wel=gruppe", "- Gruppen"),
                      ]
        #url_name_pairs.append(("", "")),
        if config.KEINE_BUNDESSTATISTIK:
            url_name_pairs.append(("", "Fach- und Regionalstatistik"))
        else:
            url_name_pairs.append(("", "Fach- und Bundesstatistik"))
        url_name_pairs.append(("statabfr", "- Statistik"))
        if config.BERATUNGSKONTAKTE_BS or config.FALLUNABHAENGIGE_AKTIVITAETEN_BS:
            #url_name_pairs.append(('',''))
            url_name_pairs.append(('','Zeiten'))
            if config.BERATUNGSKONTAKTE_BS:
                url_name_pairs.append(("bkontbsabfr", "- Beratungskontaktzeiten"))
            if config.FALLUNABHAENGIGE_AKTIVITAETEN_BS:
                url_name_pairs.append(("fuabsabfr", "- Fallunabhngige Aktivitten"))
        return url_name_pairs

    def for_mitarbeiter(self, sel=None, empty_option=False, status=('i',), stellen=None):
        if sel and isinstance(sel, basestring):
            sel = [int(x) for x in sel.split()]
        return make_option_list(self.getMitarbeiterliste(status=status, stellen=stellen),
                                'id', 'na',
                                selected=sel, empty_option=empty_option)

    def for_teilmengen(self, sel=None):
        "Definierte Teilmengen, ermglichen Abfragen ber definierte Gruppen "
        "in der Fach- und Bundesstatistik."
        abfrage_list = AbfrageList(where='')
        return (self.option_tmpl % ('', '', 'Alle')) + make_option_list(abfrage_list,
                                                               'id', 'name',
                                                               selected=sel)

    def for_jahre(self, sel=None, first=None, last=None, erster_eintrag=None):
        "Optionen fr Jahre, entweder explizit oder anhand aller Fallbeginne"
        def jeq(x, y):
            try:
                return int(x) == int(y)
            except (TypeError, ValueError):
                return x == y
        if not last:
            last = today()['year']
        if not first:
            jahre = SQL("select distinct bgy from fall order by bgy").execute()
            if jahre:
                first = jahre[0][0]
            else:
                first = last
        jahre = [j for j in range(first, last+1)]
        options = []
        select_attr = ' selected="selected" '
        if erster_eintrag:
            options.append(self.option_tmpl % ('', '', erster_eintrag))
        options += [self.option_tmpl % (j, jeq(j, sel) and select_attr or '', j)
                    for j in jahre]
        return '\n'.join(options)

    def for_quartal(self, sel=None, erster_eintrag=None):
        "Optionen fr Quartale 1,2,3,4"
        options = []
        select_attr = ' selected="selected" '
        if erster_eintrag:
            options.append(self.option_tmpl % ('', '', erster_eintrag))
        options += [self.option_tmpl % (j, j==sel and select_attr or '', j)
                    for j in (1,2,3,4)]
        return '\n'.join(options)

    def for_monat(self, sel=None, erster_eintrag=None):
        "Optionen fr Monate 1,2,3,4,5,6,7,8,9,10,11,12"
        options = []
        select_attr = ' selected="selected" '
        if erster_eintrag:
            options.append(self.option_tmpl % ('', '', erster_eintrag))
        options += [self.option_tmpl % (j, j==sel and select_attr or '', j)
                    for j in (1,2,3,4,5,6,7,8,9,10,11,12)]
        return '\n'.join(options)

    def for_kat(self, kat, sel=None, add_empty=False, all=False):
        if sel in ('', ' ',) or add_empty:
            empty_option = True
        else:
            empty_option = False
        # das ist ein hack. Eigentlich msste ein multi-kat Feld eine List von Integern liefern.
        # Ist aber ein String von Zahlen, zB "233 44 444"
        if isinstance(sel, basestring):
            sel = [int(x) for x in sel.split()]
        if all:
            codes = get_all_codes(kat)
        else:
            codes = get_codes(kat)
        return  make_option_list(codes,
                                 'id', 'name',
                                 selected=sel,
                                 empty_option=empty_option)

    
    def for_fs_kategorie(self, sel=None):
        from ebkus.html.fskonfig import fs_customize
        kat_ids = ', '.join([str(f['kat_id']) for f in fs_customize.fd.values()
                                 if f['kat_id']])
        fs_kategorien = KategorieList(where="id in (%s)" % kat_ids)
        fs_kategorien.sort('name')
        return make_option_list(fs_kategorien,
                                'id', 'name',
                                selected=sel)

    def for_code_sort(self, kat, sel=None):
        elems = [c['sort'] for c in kat['codes']]
        elems.sort()
        if sel and sel > max(elems):
            elems.append(sel)
        elems = [{'sort': s} for s in elems]
        return make_option_list(elems, 'sort', 'sort', selected=sel)


    def get_zusaetzliche_berechtigungen(self):
        """
        Gibt zurck eine Liste aller zustzlichen Zustndigkeiten, die fr den angemeldeten
        Mitarbeiter in der Tabelle zusaetzliche_berechtigung eingetragen sind.
        Eigene Zustndigkeiten sollten nicht dabei sein (wre redundant).
        Wird momentan nur gemacht, wenn beratungskontakt_dokumentation: true
        konfiguriert ist.
        """
        if (not config.ZUSAETZLICHE_BERECHTIGUNGEN or
            not self.mitarbeiter['benr__code'] == 'bearb'):
            return []
        def get_zustaendig(zb):
            fall = Fall(zb['fall_id'])
            if fall['aktuell']:
                return fall['zustaendig']
        zusaetzliche_berechtigungen = ZustaendigkeitList(
            [x for x in [get_zustaendig(zb) for zb in ZusaetzlicheBerechtigungList(
                where='mit_id = {}'.format(self.mitarbeiter['id']))]
             if x])
        zusaetzliche_berechtigungen.sort('mit_id__na', 'fall_id__akte_id__na',
                                         'fall_id__akte_id__vn')
        return zusaetzliche_berechtigungen
                    
        
    def get_aktuelle_zustaendigkeiten(self, order=None):
        """Generiert alle Zustndigkeiten fr den angemeldeten Mitarbeiter"""
        wherestr = 'ed = 0'
        benr = self.mitarbeiter['benr__code']
        if benr in ('bearb', 'verw'):
            if benr == 'bearb':
                wherestr += ' and mit_id = %s' % self.mitarbeiter['id']
            # Hack. Die joins deshalb, weil es Fehler gab wegen
            # referentieller Integritt. Kein Fall fr existierende
            # Zustndigkeit. Solche fehlerhaften Zustndigkeiten
            # einfach weglassen.
            zustaendigkeiten = ZustaendigkeitList(where=wherestr,
                                                  join=(
                                                      ('fall', 'fall.id = fall_id'),
                                                      ('akte', 'fall.akte_id = akte.id'),
                                                  ),
                                                  cache_related=True)
            if order == 'akte_na':
                zustaendigkeiten.sort('fall_id__akte_id__na',
                                      'fall_id__akte_id__vn')
            else:
                zustaendigkeiten.sort('mit_id__na', 'fall_id__akte_id__na',
                                      'fall_id__akte_id__vn')
            for z in zustaendigkeiten:
                if z['fall_id__akte_id__stzbg'] == self.stelle['id']:
                    yield z
            for z in self.get_zusaetzliche_berechtigungen():
                yield z

    def get_aktuelle_bezugspersonen(self, order=None):
        res = []
        for z in self.get_aktuelle_zustaendigkeiten():
            for b in z['fall__akte__bezugspersonen']:
                b['fn'] = z['fall__fn'] # drangepappt fr das Template unten,
                                        # was Fallnummer braucht
                res.append(b)
        if not res:
            return
        bezugsperson_list = BezugspersonList(data=res)
        if order == 'bp_na':
            bezugsperson_list.sort('na', 'vn')
        for b in bezugsperson_list:
            yield b

    # TBD Verzockt besser mit make_option_list
    def for_klienten(self, sel=None, kurz=False, empty_option=False, order=None):
        """Optionen fr Klientenauswahl"""
        selected = ' selected="selected" '
        if not sel:
            sel = ''
        # sel kann so "123 432 543" (str) oder so 133 (int) aussehen, je nach
        # single oder multiple kat item
        if isinstance(sel, basestring):
            sel = [int(s) for s in str(sel).split()]
        elif isinstance(sel, (int, long)):
            sel = [sel]
#         tmpl = """<option value="%(fall_id)s"%(xxsel)s>%(mit__na)s | %(fall__akte__vn)s %(fall__akte__na)s, %(fall__akte__gb)s | %(fall__fn)s </option>"""
#         tmpl_kurz = """<option value="%(fall_id)s"%(xxsel)s>%(fall__akte__vn)s %(fall__akte__na)s | %(fall__fn)s</option>"""
        # Probehalber Name, Vorname
        #tmpl = """<option value="%(fall_id)s"%(xxsel)s>%(mit__na)s | %(fall__akte__na)s, %(fall__akte__vn)s, %(fall__akte__gb)s | %(fall__fn)s </option>"""
        # Vollstndige Klientangaben zustzlich im Tooltip, fall Kasten zu klein ist:
        tmpl = """<option title="%(mit__na)s | %(fall__akte__na)s, %(fall__akte__vn)s, %(fall__akte__gb)s | %(fall__fn)s"
    value="%(fall_id)s"%(xxsel)s>%(mit__na)s | %(fall__akte__na)s, %(fall__akte__vn)s, %(fall__akte__gb)s | %(fall__fn)s </option>"""
        tmpl_kurz = """<option value="%(fall_id)s"%(xxsel)s>%(fall__akte__na)s, %(fall__akte__vn)s | %(fall__fn)s</option>"""
        if kurz:
            tmpl = tmpl_kurz
        if empty_option:
            options = "<option value="" >&nbsp;</option>"
        else:
            options = ''
        for z in self.get_aktuelle_zustaendigkeiten(order):
            if z['fall_id'] in sel:
                z['xxsel'] = selected
            else:
                z['xxsel'] = ''
            options += tmpl % z
            if z.has_key('xxsel'):
                del z['xxsel']
        return options

    def for_bezugspersonen(self, order=None):
        """Optionen fr Bezugspersonenauswahl"""
        tmpl = """<option value="%(id)s">%(na)s, %(vn)s | %(fn)s</option>"""
        options = ''
        for b in self.get_aktuelle_bezugspersonen(order):
                options += tmpl % b
        return options

    def for_gruppen(self, alle=True, sel=None):
        """Optionen fr Gruppenauswahl erstellen"""
        option_t = '<option value="%(gruppe_id)s"%(xxsel)s>%(mit_id__na)s | %(gruppe_id__name)s | %(gruppe_id__gn)s</option>\n'
        options = ''
        where = "gruppe.stz=%s" % self.stelle['id']
        if not alle:
            # kein Ende angegeben, oder Gruppe luft noch, dh Ende ist angegeben, liegt aber in der Zukunft
            now = today()
            y = now.year; m = now.month; d = now.day
            where += (' and (gruppe.ey = 0 or (gruppe.ey > %s ) or (gruppe.ey = %s and gruppe.em > %s) or (gruppe.ey = %s and gruppe.em = %s and gruppe.ed > %s))' 
                      % (y, y, m, y, m, d))
        if self.mitarbeiter['benr__code'] == 'bearb':
            where += ' and mit_id = %s' % self.mitarbeiter['id']
        elif self.mitarbeiter['benr__code'] == 'verw':
            pass
        else:
            raise EE('Keine Berechtigung')
        mitarbeitergruppenl = MitarbeiterGruppeList(
                where=where,
                join=[('gruppe', 'mitarbeitergruppe.gruppe_id=gruppe.id')])
        if sel:
            sel = [int(s) for s in sel]
        else:
            sel = []
        mitarbeitergruppenl.sort('mit_id__na', 'gruppe_id__name')
        for m in mitarbeitergruppenl:
            m['xxsel'] = (m['gruppe_id'] in sel) and ' selected ' or ''
            options += option_t % m
        return options
        
