# coding: latin-1

"""Modul zum Sperren/Entsperrn."""

import string,time,sys
import logging

from ebkus.db.sql import SQL, escape
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Akte, Fall, Gruppe, FallList, GruppeList, Altdaten, Anmeldung, \
     Zustaendigkeit, today, cc, check_list, check_code, check_multi_code, \
     check_int_not_empty, check_str_not_empty, EE
from ebkus.app.ebapih import get_codes
from ebkus.app_surface.standard_templates import *
from ebkus.app_surface.akte_templates import *

import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share
from ebkus.html.abfragen import fall_abfrage


class Data(dict):
    def __getattr__(self, a):
        "Don't raise an exception if an attribute doesn't exist."
        return self[a]
    def __setattr__(self, a, v):
        "Don't raise an exception if an attribute doesn't exist."
        self[a] = v

class sperren_reset(Request.Request, akte_share):
    """Fr alle Akten das Feld 'sperre' auf NULL setzen.
    Nur fr Tests. Sollte in der Produktion nicht zur Verfgung stehen.
    """
    permissions = Request.ADMIN_PERM

    def processForm(self, REQUEST, RESPONSE):
        from ebkus.app.ebupd import sperrakten_reset
        n = sperrakten_reset()
        legend = "Sperren zurckgesetzt"
        spf = config.SPERRFRIST
        if spf <= 0:
            msg = "Sperren wurden zurckgesetzt. Keine automatisches Sperren konfiguriert."
        else:
            msg = """Sperren wurden zurckgesetzt. Bei der nchsten Anmeldung eine Benutzers werden 
wieder Akten automatisch gesperrt. Sperrfrist ist %s Monate.""" % spf
            
        sperren_reset = h.FieldsetDataTable(
            legend=legend,
            daten=[[h.String(string=msg),
                    ],
                   [h.String(string="Das Zurcksetzen aller Sperren ist in der Produktionsversion nicht vorgesehen. Nur in der Testdemo um die Funktion des automatischen Sperrens nachzuvollziehen."),
                    ]],
            )
        res = h.Page(
            title=legend,
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           #('Sperren reset', None),
                           ),
            rows=(self.get_hauptmenu(),
                  sperren_reset
                  ),
            )
        return res.display()


class spakten(fall_abfrage):
    """Abfrageformular zum Sperren von Akten.
    Es knnen alle Akten ohne aktuellen Fall gesperrt werden.
    Das hat zur Folge, dass
    - die Klientakte und ihre Bestandteile nicht mehr aufgerufen werden kann
    - die Namen des Klienten (und dessen Bezugspersonen) in allen Listen, die auf die Akte 
      oder Teile davon verweisen, durch xxx ersetzt werden.
    Die gesperrte Akte wird in Statistiken und bersichten auch weiterhin
    ausgewertet.


    Wie soll das Formular aussehen?
    Anzeigen: Abschluss von Jahr, bis Jahr
              2 Hkchen: nur gesperrt, nur nicht gesperrt
              Stelle, falls es mehr als eine Stelle gibt
              
    Sortieren: Fallnummer, Namen, Abschlussdatum, Abschlussmonat
    Wie Beratungen, aber sortieren ohne Statistiken, keine offenen Flle

    submitops: Anzeigen, Speichern

    """
    permissions = Request.ADMIN_PERM

    def get_default_params(self,
                   # Liste alle vorkommende Parameter mit defaults. 
                   # Defaults werden verwendet,
                   # wenn die Seite als GET aufgerufen wird.
                   # Liefert immer eine frische Kopie
                           ):

        # Default: Nur das erste Jahr anzeigen, fr das es abgeschlossene Flle gibt.
        minjahr = SQL("SELECT MIN(zday) FROM fall WHERE zday > 0").execute()[0][0]
        maxjahr = (minjahr + 1) if minjahr else None
        params = Data(
            ab_jahr_zda=minjahr,
            bis_jahr_zda=maxjahr,
            akten_ids=[],           # Alle Akten im Formular
            akten_ids_gesperrt=[],  # Falls Hkchen gesetzt
            suche='',
            mit_id=None,
            nur_gesperrt=None,
            nur_nicht_gesperrt=None,
            sort_options_map = {
                '1': (('bgy', 'fn_count', 'akte__stzbg__code'), 'Fallnummer'),
                '2': (('akte__na', 'akte__vn',), 'Name des Klienten'),
                '3': (('zuletzt_zustaendig__mit__na',), 'Name des Mitarbeiters'),
                '4': (('zday',), 'Jahr des Fallabschlusses'),
                '5': (('zday', 'zdam'), 'Monat des Fallabschlusses'),
                '6': (('zday', 'zdam', 'zdad'), 'Tag des Fallabschlusses'),
                },
            sort_sequence="432156",  # In dieser Reihenfolge wird sortiert. Immer als hidden mit im Formular.
            sort_choice='4',         # ndert die Sortierfolge, indem das gewhlte Element an die erste Stelle gesetzt wird
            stz=' '.join([str(x['id']) for x in get_codes('stzei')]),
            submitop=None,
            )
        #print 'DEFAULT_PARAMS', params
        return params
    def get_params(self, form=None, defaults=None):
                   # Holte Daten von form
                   # Wirft Exceptions
                   # Liefert Data: param->value(s)
                   # Liefert Werte aus defaults, falls kein Parameter vorhanden
        form = form if form else self.form
        params = defaults if defaults else self.get_default_params()
        # params.ab_jahr_zda = check_int_not_empty(form, 'ab_jahr_zda', 
        #                                          "Fehler in ab Jahr Abschluss", params.ab_jahr_zda)
        # params.bis_jahr_zda = check_int_not_empty(form, 'bis_jahr_zda', 
        #                                           "Fehler in ab Jahr Abschluss", params.bis_jahr_zda)
        #print 'PARAMS before get', params
        #print 'FORM', form
        p = params
        p.ab_jahr_zda = form.get('ab_jahr_zda', p.ab_jahr_zda)
        p.bis_jahr_zda = form.get('bis_jahr_zda', p.bis_jahr_zda)
        p.akten_ids = check_list(form, 'akten_ids', 'Keine Auswahl', p.akten_ids)
        p.akten_ids_gesperrt = check_list(form, 'akten_ids_gesperrt', 'Keine Auswahl', p.akten_ids_gesperrt)
        p.suche = check_str_not_empty(form, 'suche', 'Kein Suchstring', '')
        mit_id = form.get('mit_id')
        p.mit_id = None if mit_id in (None, '', ' ') else mit_id
        p.nur_gesperrt = form.get('nur_gesperrt')
        p.nur_nicht_gesperrt = form.get('nur_nicht_gesperrt')
        p.sort_sequence = form.get('sort_sequence', p.sort_sequence)
        #print 'PARAMS.SORT_SEQUENCE', p.sort_sequence
        sort_choice = form.get('sort_choice', p.sort_choice)
        if sort_choice is not None:
            seq = p.sort_sequence
            #print 'SEQ', seq
            if sort_choice != seq[0]:
                # Die Sortierreihenfolge soll verndert werden, da das gewhlte Kriterium
                # nicht schon am Anfang steht.
                head, tail = seq.split(sort_choice)
                #print 'HEAD, TAIL', head, tail
                p.sort_sequence = sort_choice + head + tail
        p.stz = check_multi_code(form, 'stz', 'stzei', "Keine Stelle", p.stz)
        p.submitop = form.get('submitop', p.submitop)
        #print 'PARAMS.SORT_SEQUENCE', p.sort_sequence
        #print 'PARAMS after get', p
        return p
    def auswahl_form(self,
                     # alle Werte des Formulars mssen belegt sein.
                     # Bei ersten Mal sind die defaults von get_params()
                     params,
                     legend="Zu sperrende/entsperrende Akte auswhlen",
                     ):
        """FieldSet mit Controls zur Auswahl von Akten"""
        p = params
        selected = ' selected="selected"'
        tmpl = '<option value="%s"%s>%s</option>\n'
        def make_sort_options():
            omap = params.sort_options_map
            first = params.sort_sequence[0]
            others = params.sort_sequence[1:]
            # erstes Element immer selected
            sort_options = tmpl % (first, selected, omap[first][1])
            for x in others:
                sort_options += tmpl % (x, '', omap[x][1])
            return sort_options
        def make_mitarbeiter_options():
            mitarbeiter_options = (tmpl % ('', 
                                           selected if params.mit_id in (None, '', ' ', 'Alle') else '',
                                           'Alle') +
                                   self.for_mitarbeiter(sel=params.mit_id, status=('i', 'a'), stellen='alle'))
            return mitarbeiter_options
        #print 'AUSWAHL_FORM params', params
        anzeige = h.FieldsetInputTable(
            legend=legend,
            daten=[
                [h.SelectItem(label='Fallabschluss ab Jahr',
                                 name='ab_jahr_zda',
                                 class_='listbox45',
                                 tip='Nur Flle zeigen, die ab dem gewhlten Jahr abgeschlossen wurden',
                                 options=self.for_jahre(sel=p.ab_jahr_zda,
                                                        erster_eintrag='Alle'),
                                 ),
                    h.SelectItem(label='Fallabschluss bis Jahr',
                                 name='bis_jahr_zda',
                                 class_='listbox45',
                                 tip='Nur Flle zeigen, die bis zu dem gewhlten Jahr abgeschlossen wurden',
                                 options=self.for_jahre(sel=p.bis_jahr_zda,
                                                        erster_eintrag='Alle'),
                                 ),
                 ],[ 
                    h.CheckItem(label='Nur gesperrt',
                                name='nur_gesperrt',
                                value=1,
                                checked=p.nur_gesperrt,
                                tip='Nur gesperrte Akten zeigen',
                                ),
                    h.CheckItem(label='Nur nicht gesperrt',
                                name='nur_nicht_gesperrt',
                                value=1,
                                checked=p.nur_nicht_gesperrt,
                                tip='Nur nicht gesperrte Akten zeigen',
                                ),
                    ],
                [h.SelectItem(label='Mitarbeiter',
                              name='mit_id',
                              tip='Nur Flle zeigen, wo Mitarbeiter zustndig war (auch vorbergehend)',
                              options=make_mitarbeiter_options(),
                              ),
                 h.TextItem(label='Suchen',
                            name='suche',
                            class_='textbox',
                            value=p.suche,
                            tip='Nur Flle zeigen, wo der Suchtext im Name, im Vornamen oder in der Fallnummer vorkommt',
                            ),
                 ],
                [h.SelectItem(label='Sortieren nach',
                              name='sort_choice',
                              tip='Sortierfolge. Durch wiederholtes nach vorne bringen eines Kriteriums kann man beliebige Sortierfolgen bewirken.',
                              class_='listbox170',
                              options=make_sort_options(),
                              ),
                    len(get_codes('stzei')) > 1 and h.SelectItem(label='Stelle',
                                 name='stz',
                                 multiple=True,
                                 rowspan=2,
                                 tip='Nur Flle aus den gewhlten Stellen zeigen',
                                 size=3,
                                 options=self.for_kat('stzei', sel=p.stz),
                                 ) or h.DummyItem(),
                 ],
                [h.Dummy(n_col=4)],
                [h.Button(value="Anzeigen",
                             name='submitop',
                             tip="Beratungen anzeigen",
                             type='submit',
                             n_col=4,
                             ),
                 ],
                ],
        )
        return anzeige

    def akten_sperr_form(self, faelle):
        """FieldSet mit Liste von Akten mit Checkbox zum Sperren und Button zum Speichern.

        Jede Akte hat ein hidden input akte__<id>, so dass man am Ende wei, welche Akten 
        in der Form waren und welche davon als gesperrt bzw. nicht gesperrt markiert waren.
        """
        def fruehere_faelle(fall):
            faelle = fall['akte__faelle']
            faelle.sort('bgy', 'bgm', 'bgd')
            return ', '.join([f['fn'] for f in faelle[:-1]])
        akten = h.FieldsetDataTable(
            name="sperrl",
            action="spakten",
            method="post",
            header_class='sortable',
            hidden = (),
            legend='Sperren oder Entsperren ({})'.format(len(faelle)),
            headers=('Letzte Fallnummer', 'Frhere Fallnummern', 'Name', 'Vorname', 'Geburtsdatum', 'Letzter Fallabschluss', 'Letzte Zustndigkeit', '', 'Sperre'),
            daten=[[h.String(string=fall['fn']),
                    h.String(string=fruehere_faelle(fall)),
                    h.String(string=fall['akte__na']),
                    h.String(string=fall['akte__vn']),
                    h.String(string=fall['akte__gb']),
                    h.Datum(date=fall.getDate('zda')),
                    h.String(string=fall['zuletzt_zustaendig__mit__na']),
                    h.CheckItem(label='',
                                name='akten_ids_gesperrt',
                                value=fall['akte_id'],
                                checked=fall['akte__gesperrt'],
                                tip='Hier markieren, um die Akte zu sperren',
                                ),
                    ] for fall in faelle] + [
                [h.Dummy(n_col=6)]
                ],

            no_button_if_empty=True,
            button=h.Button(value="Speichern",
                       name='submitop',
                       tip="Markierte Akten sperren, nicht markierte entsperren",
                       type='submit',
                       class_='buttonbig',
                       #n_col=5,
                 ),
            empty_msg="Keine Akten gefunden.",
            )
        return akten
                
        
    def get_akten(self, params):
                  # von_zday, von_zdam, bis_zday, bis_zdam,
                  # von_zday, von_zdam, bis_zday, bis_zdam,
                  # nur_gesperrt=False, nur_nicht_gesperrt=False,
                  # name=None,         # String kommt im Namen vor (like %name%)
                  # mitarbeiter=None,  # zuletzt zustndig
                  # aufbew=None,       # nur diesen Typ
                  # fn=None,           # String kommt in Fallnummer vor (like %name%)
        """ """
        p = params
        join=[('zustaendigkeit', 'zustaendigkeit.fall_id=fall.id'),
              ('akte', 'fall.akte_id=akte.id'),
              #('akte a2', 'fall.akte_id=a2.id'),
              ('mitarbeiter', 'zustaendigkeit.mit_id=mitarbeiter.id'),]
        where = ["zday > 0"] # nur abgeschlossene Flle
        if p.ab_jahr_zda:
            where.append("zday >= %s" % p.ab_jahr_zda)
        if p.bis_jahr_zda:
            where.append("zday <= %s" % p.bis_jahr_zda)
        if p.suche:
            s = escape('%' + p.suche + '%') # schon single-gequoted
            where.append("(akte.vn like %s or akte.na like %s or fall.fn like %s)" % (s,s,s))
        if p.mit_id:
            where.append("mitarbeiter.id = %s" % p.mit_id)
        if p.nur_gesperrt:
            where.append("akte.sperre = %s" % cc('ja_nein', '1'))
        if p.nur_nicht_gesperrt:
            where.append("(akte.sperre is null or akte.sperre != %s)" % cc('ja_nein', '1'))
        if p.stz:
            where.append("akte.stzbg in (%s)" % ','.join(p.stz.split()))

        # join=[('zustaendigkeit', 'zustaendigkeit.fall_id=fall.id'),
        #       ('akte', 'fall.akte_id=akte.id'),
        #       ('mitarbeiter', 'zustaendigkeit.mit_id=mitarbeiter.id'),]
        # where = ["zday > 0"] # nur abgeschlossene Flle
        # if p.ab_jahr_zda:
        #     where.append("zday >= %s" % p.ab_jahr_zda)
        # if p.bis_jahr_zda:
        #     where.append("zday <= %s" % p.bis_jahr_zda)
        # if p.suche:
        #     s = escape('%' + p.suche + '%') # schon single-gequoted
        #     where.append("(akte.vn like %s or akte.na like %s or fall.fn like %s)" % (s,s,s))
        # if p.mit_id:
        #     where.append("mitarbeiter.id = %s" % p.mit_id)
        # if p.nur_gesperrt:
        #     where.append("akte.sperre = %s" % cc('ja_nein', '1'))
        # if p.nur_nicht_gesperrt:
        #     where.append("akte.sperre != %s" % cc('ja_nein', '1'))
            

        alle_faelle = FallList(
            where=' and '.join(where),
            join = join, 
            order='zday, zdam, zdad',
            cache_related=True,
            )
        #faelle = []
        faelle = FallList()
        for f in alle_faelle:
            try:
                if f == f['akte__letzter_fall']:
                    faelle.append(f)
            except Exception, e:
                t = sys.exc_info()[0]
                logging.exception("Interner Fehler: %s: %s", t, e)
                logging.exception("  Falle ohne Akte? fall_id=%s" % f['id'])
        # Sortieren
        keys = ()
        for x in p.sort_sequence:
            keys += p.sort_options_map[x][0]
        faelle.sort(*keys)
        return faelle
    def processForm(self, REQUEST, RESPONSE):
        # get_params
        params = self.get_params(self.form)
        #print 'SPERRAKTEN params', params
        if params.submitop == 'Anzeigen':
            pass
        elif params.submitop == 'Speichern':
            from ebkus.app.ebupd import sperrakten
            n_gesperrt, n_entsperrt = sperrakten(params)
            if n_gesperrt or n_entsperrt:
                return h.Meldung(
                    legend='Akten gesperrt/entsperrt',
                    weiter='menu',
                    zeilen=(
                        'Gesperrt: %s' % (n_gesperrt if n_gesperrt else 'keine'),
                        'Entsperrt: %s' % (n_entsperrt if n_entsperrt else 'keine'),
                        'Weiter zum  Hauptmen&uuml ...',
                        ),
                    ).display()
        anzeige = self.auswahl_form(params)
        faelle = self.get_akten(params)
        akten = self.akten_sperr_form(faelle)
        hidden = [('akten_ids', fall['akte_id']) for fall in faelle]
        hidden += [('sort_sequence', params.sort_sequence)]
        # Alles in eine einzige Form
        res = h.FormPage(
            title='Akten sperren/entsperren',
            name="sperr",action="spakten",method="post",
            breadcrumbs = (('Aministratorhauptmen', 'menu'),
                           ),
            hidden=hidden,
            rows=(self.get_hauptmenu(),
                  anzeige,
                  akten,
                  ),
            )
        return res.display()
        
