# coding: latin-1
import re
import os
import logging
from glob import glob
from datetime import datetime
from ebkus.app import Request
from ebkus.config import config
import ebkus.html.htmlgen as h
from ebkus.html.akte_share import akte_share 

class sql_sicherung(Request.Request, akte_share):
    permissions = Request.ADMIN_PERM
    separator = h.Tr(cells=[h.Dummy(height=12)])
    def processForm(self, REQUEST, RESPONSE):
        sichern = self.form.get('sichern')
        if sichern == 'SQL-Datenbankdump erzeugen':
            self.sicherung_durchfuehren()
            legend="SQL-Datenbankdump erfolgreich"
            res = h.Meldung(legend=legend,
                            title=legend,
                            zeilen=["SQL-Datenbankdump erfolgreich"]).display()
            return res

        res = h.Page(
            title='SQL Sicherung',
            breadcrumbs = (('Administratorhauptmen', 'menu'),
                           ),
            rows=(self.get_hauptmenu(),
                  self.separator,
                  self.sicherung_durchfuehren_button(),
                  self.separator,
                  self.vorhandene_sql_sicherungen_anzeigen(),
                  ),
            )
        return res.display()
    def sicherung_durchfuehren(self):
        from ebkus.Install import InstallFromInstance
        installer = InstallFromInstance(config, quiet=True)
        installer.init()
        installer._log = None
        installer.components[config.INSTANCE_NAME].dump_database(dir=config.INSTANCE_HOME)
        logging.info("SQL-Datenbankdump erfolgreich erstellt in '{}'".format(config.INSTANCE_HOME))

    def sicherung_durchfuehren_button(self):
        button_sichern = h.Button(value='SQL-Datenbankdump erzeugen',
                                  name='sichern',
                                  type='submit',
                                  class_='buttonhuge',
                                  tip="Auf dem Server wird in dem Instanzverzeichnis ein SQL-Dump der Datenbank erzeugt. "
                                  "Das entspricht dem Befehl 'datenbank_sichern.py --sql'.",
                                  )
        sicherung_durchfuehren = h.FieldsetForm(
            legend='SQL-Datenbankdump erzeugen',
            action='sql_sicherung',
            method='post',
            name='sql_sicherung',
            rows=(
                button_sichern,
                ),
            )
        return sicherung_durchfuehren
    def vorhandene_sql_sicherungen_anzeigen(self):
        paths = sorted(glob(os.path.join(
            config.INSTANCE_HOME,
            "{}_dump*.sql".format(config.INSTANCE_NAME))),
                       reverse=True)
        sql_sicherungen_anzeigen = h.FieldsetDataTable(
            headers=('Datum', 'Name', 'Gre'),
            header_class="tabledatabold",
            legend='Frhere Sicherungen',
            daten=[
                [h.String(string=datetime.strftime(
                    datetime.fromtimestamp(os.path.getmtime(path)),
                    "%Y-%m-%d %H:%M:%S"
                ),
                          ),
                 h.String(string=os.path.basename(path),
                          ),
                 h.String(string=os.path.getsize(path),
                          class_='tabledata_right'),
                ] for path in paths
            ],
            )
        return sql_sicherungen_anzeigen
