# coding: latin-1

"""Module fr das Anzeigen der Dokumente (im PDF-Format)."""

import string
import os
from ebkus.app import Request
from ebkus.config import config
from ebkus.app.ebapi import Fall, Akte, Dokument, Gruppendokument, Code, GruppendokumentList, DokumentList, Gruppe, today, cc, get_akte_path, get_gruppe_path
from ebkus.app.ebapih import get_codes, mksel, mk_columns, mk_text
from ebkus.app_surface.viewdokument_templates import *
from ebkus.app_surface.standard_templates import *
# Package Reportlab
from reportlab.pdfgen import canvas
from reportlab.lib.units import inch, cm
from reportlab.lib.pagesizes import A4


def get_permissions():
    if config.BERATUNGSKONTAKT_DOKUMENTATION:
        return 'bearb', 'verw'
    else:
        return 'bearb',

utf8_umlaute = ('\xc3\x84', '\xc3\x96', '\xc3\x9c',  # AE OE UE
                '\xc3\xa4', '\xc3\xb6', '\xc3\xbc',  # ae oe ue
                '\xc3\x9f',                          # sz
                #        '\xe2\x82\xac',             # euro
                )
unicode_umlaute = [x.decode('utf-8') for x in utf8_umlaute]
latin1_umlaute = [x.decode('utf-8').encode('latin-1') for x in utf8_umlaute]
ersatz_umlaute = ('Ae', 'Oe', 'Ue', 'ae', 'oe', 'ue', 'ss')
def contains_utf8_umlaute(text):
    return sum([text.count(x) for x in utf8_umlaute]) > 0
def contains_latin1_umlaute(text):
    return sum([text.count(x) for x in latin1_umlaute]) > 0

def mydecode(txt):
    """Krcke um Unicode fr txt liefern. Entscheiden, ob txt latin1 oder utf8
    enthlt nur anhand der deutschen Umlaute.
    """
    #print 'MYDECODE IN:', txt
    if contains_latin1_umlaute(txt):
        try:
            res = txt.decode('latin1', 'strict')
            #print 'MYDECODE', 'latin1', res
            return res
        except:
            pass
    if contains_utf8_umlaute(txt):
        try:
            res = txt.decode('utf8')
            #print 'MYDECODE', 'utf8', res
            return res
        except:
            pass
    res = txt.decode('utf8', 'replace')
    #print 'MYDECODE', 'replace', res
    return res

def asciify(txt):
    uni = mydecode(txt)
    for i in range(7):
        uni = uni.replace(unicode_umlaute[i], ersatz_umlaute[i])
    uni = uni.encode('ascii', 'replace')
    uni = uni.replace('?', '_')
    return uni
    



class dokview(Request.Request):
    """1 Dokument senden (Tabellen Dokument, Gruppendokument"""
    
    permissions = get_permissions()
    
    def processForm(self, REQUEST, RESPONSE):
        mitarbeiterliste = self.getMitarbeiterliste()
        user = self.user
        
        if self.form.has_key('gruppeid'):
            dokid = self.form.get('dokid')
            gruppeid = self.form.get('gruppeid')
            gruppe = Gruppe(gruppeid)
            dok = Gruppendokument(dokid)
            mimetyp = Code(dok['mtyp'])
            try:
                gruppe_path = get_gruppe_path(gruppe['id'])
                f = open('%s/%s'
                         % (gruppe_path, dok['fname']), 'rb')
                datei = f.read()
                f.close()
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
        else:
            fallid = self.form.get('fallid')
            fall = Fall(int(fallid))
            akte = Akte(fall['akte_id'])
            dokid = self.form.get('dokid')
            dok = Dokument(int(dokid))
            mimetyp = Code(dok['mtyp'])
            
            try:
                akte_path = get_akte_path(akte['id'])
                f = open('%s/%s'
                         % (akte_path, dok['fname']), 'rb')
                datei = f.read()
                f.close()
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
                
        if dok['mtyp'] == cc('mimetyp','txt'):
            kopfzeile = '%(vd)d.%(vm)d.%(vy)d: %(art__name)s' % dok
            fuss = '%(mit_id__vn)s %(mit_id__na)s' % dok
            betreff = mk_columns(dok['betr'],70)
            zeilen = string.split(betreff,'\n')
            betr = ''
            for z in zeilen:
                betr = betr + '%s\n' % z
            datei_out = '\n%s.\nBetr.: %s\n%s\n\n%s\n'\
                        % (kopfzeile, betr, datei, fuss)
        else:
            datei_out = datei
            
        if RESPONSE:
            RESPONSE.setHeader('content-type', '%(name)s' % mimetyp)
            RESPONSE.setHeader('filename', dok['fname'])
            RESPONSE.setBody(datei_out)
        else:
            return "Fehler beim Anzeigen des Dokumentes"
            
            
class dokview2(Request.Request):
    """Mehrere Ascii-Texte der Akte zusammenfassen und senden (Tabellen: Dokument, Gruppendokument)"""
    
    permissions = get_permissions()
    
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('gruppeid'):
            gruppeid = self.form.get('gruppeid')
            gruppe = Gruppe(gruppeid)
            titel = 'Auszug vom %(day)d.%(month)d.%(year)d.' % today() + \
                      ' - %s -' % gruppe['gn'] + '\n\n\n'
        else:
            fallid = self.form.get('fallid')
            fall = Fall(int(fallid))
            akte = Akte(fall['akte_id'])
            faelle = akte['faelle']
            faelle.sort('bgy','bgm','bgd')
            titel = 'Auszug vom %(day)d.%(month)d.%(year)d.' % today() + ' - %s -' % fall['fn'] + '\n\n\n'
            
        art = self.form.get('art')
        
        out = ''
        
        # Aktendokumente
        
        if self.form.has_key('fallid'):
            for f in faelle:
                if art == 'bnotiz':
                    dokl = DokumentList(where = 'fall_id = %s and art = %s and mtyp = %s and mit_id = %s' % (f['id'], cc('dokart', 'bnotiz'), cc('mimetyp', 'txt'), self.mitarbeiter['id']), order = 'vy,vm,vd')
                    
                elif art=='anotiz':
                    adokl = DokumentList(where = 'fall_id = %s and mtyp = %s'
                                           % (f['id'], cc('mimetyp', 'txt')), order = 'vy,vm,vd')
                    dokl = []
                    for d in adokl:
                        if d['art'] != cc('dokart', 'bnotiz'):
                            dokl.append(d)
                            
                out = out + mk_text(dokl, get_akte_path(akte['id']))
                
                # Gruppendokumente
                
        elif gruppeid and art == 'bnotiz':
            dokl = GruppendokumentList(where = 'gruppe_id = %s and art = %s and mtyp = %s and mit_id = %s'% (gruppe['id'], cc('dokart', 'bnotiz'), cc('mimetyp', 'txt'), self.mitarbeiter['id']), order = 'vy,vm,vd')
            out = mk_text(dokl, get_gruppe_path(gruppe['id']))
            
        elif gruppeid and art=='anotiz':
            adokl = GruppendokumentList(where = 'gruppe_id = %s and mtyp = %s'
                                   % (gruppe['id'], cc('mimetyp', 'txt')), order = 'vy,vm,vd')
            dokl = []
            for d in adokl:
                if d['art'] != cc('dokart', 'bnotiz'):
                    dokl.append(d)
            out = mk_text(dokl, get_gruppe_path(gruppe['id']))
            
            # Plain-Text-Ausgabe fuer den Browser. Alternativ: print_pdf.py
            
        if out is None or out == '':
            out = 'Kein Ascii-Text vorhanden'
        else:
            out = titel + out
            
            # Datei uebertragen
            
        if RESPONSE:
            RESPONSE.setHeader('content-type', '%s' % 'text/plain')
            RESPONSE.setHeader('filename', 'zusammenfassung.txt')
            RESPONSE.setBody(out)
        else:
            return "Fehler beim Anzeigen des Dokumentes"
            
            #
            # return "Fehler ..." durch Fehlerbehandlung ersetzen.
            #
            
            
            # Layout fuer pdf
top_margin = A4[1] - inch
bottom_margin = inch
left_margin = inch
right_margin = A4[0] - inch
frame_width = right_margin - left_margin


class xprint_pdf(Request.Request):
    """PDF-Text aus Ascii-Texten erstellen und senden (Tabelle: Dokument.)"""
    
    permissions = get_permissions()
    
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('gruppeid'):
            self.__class__ = printgr_pdf
            # damit print_pdf auch fr Gruppen funktioniert
            return printgr_pdf.processForm(self, REQUEST, RESPONSE)
        art = self.form.get('art')
        if self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
            fall = Fall(int(fallid))
            akte = Akte(fall['akte_id'])
            faelle = akte['faelle']
            faelle.sort('bgy','bgm','bgd')
            
            try:
                akte_path = get_akte_path(akte['id'])
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
                
        else:
            meldung = {'titel':'Fehler',
                      'legende':'Fehlerbeschreibung',
                      'zeile1': 'Kein Fall',
                      'zeile2':'Versuchen Sie es bitte erneut.'}
            return (meldung_t %meldung)
            
        if art == 'bnotiz':
            head_l = 'Beraternotizen'
        elif art == 'anotiz':
            head_l = 'Aktenauszug'
        head_m = fall['fn']
        head_r = '%(day)d.%(month)d.%(year)d' % today()
        
        # Seitenlayout
        
        def drawPageFrame(canv, head_links, head_mitte, head_rechts):
            canv.line(left_margin, top_margin, right_margin, top_margin)
            canv.setFont('Helvetica-Oblique',11)
            canv.drawString(left_margin, top_margin + 2, "%s" % head_links + "                                  - %s - " % head_mitte)
            canv.drawString(right_margin - 0.85*inch, top_margin + 2, '%s'
                            % head_rechts)
            canv.line(left_margin, top_margin, right_margin, top_margin)
            canv.line(left_margin, bottom_margin, right_margin, bottom_margin)
            canv.drawCentredString(0.5*A4[0], 0.5 * inch,
                         "- %d -" % canv.getPageNumber())
            
        canv = canvas.Canvas('%s/%s_%s.pdf' % (akte_path, art, akte['id']))
        canv.setPageCompression(0)
        drawPageFrame(canv, head_l, head_m, head_r)
        canv.setAuthor('%(vn) %(na)s' % self.mitarbeiter)
        canv.setTitle('%s - %s -' % (art, fall['fn']))
        
        # Inhaltsverzeichnis
        
        canv.setFont("Helvetica-Bold", 14)
        canv.drawString(left_margin, top_margin - 0.5 * inch, "Inhaltsverzeichnis")
        
        canv.setFont("Helvetica", 11)
        tx = canv.beginText(left_margin, top_margin - 0.9 * inch)
        
        for f in faelle:
            if art == 'bnotiz':
                dokl = DokumentList(where = 'fall_id = %s and art = %s \
                and mit_id = %s' % (f['id'], cc('dokart', 'bnotiz'),
                self.mitarbeiter['id']), order = 'vy,vm,vd')
                
                for d in dokl:
                    zeilen = string.split(mk_columns(d['betr'],55),'\n')
                    tx.textLine(mydecode("%(vd)s.%(vm)s.%(vy)s:" % d + " %s" % zeilen[0]))
                    if len(zeilen) > 1:
                        for z in zeilen[1:]:
                            tx.textLine("                    %s" % z)
                            
            elif art == 'anotiz':
                aktendokl = DokumentList(where = 'fall_id = %s  '
                                       % (f['id']), order = 'vy,vm,vd')
                dokl = []
                #print aktendokl
                for d in aktendokl:
                    if d['art'] != cc('dokart', 'bnotiz'):
                        zeilen = string.split(mk_columns(d['betr'],60),'\n')
                        tx.textLine("%(vd)s.%(vm)s.%(vy)s: %(art__name)s" % d )
                        for z in zeilen:
                            tx.textLine("       %s" % z)
        canv.drawText(tx)
        canv.showPage()
        
        # Die einzelnen Textdateien plus Items aus der DB
        
        drawPageFrame(canv, head_l, head_m, head_r)
        canv.setFont("Helvetica", 11)
        tx = canv.beginText(left_margin, top_margin - 0.5*inch)
        
        for f in faelle:
            if art == 'bnotiz':
                dokl = DokumentList(where = 'fall_id = %s and art = %s and mtyp = %s \
                and mit_id = %s'% (f['id'], cc('dokart', 'bnotiz'),
                cc('mimetyp', 'txt'), self.mitarbeiter['id']), order = 'vy,vm,vd')
                
            elif art == 'anotiz':
                aktendokl = DokumentList(where = 'fall_id = %s and mtyp = %s'
                                       % (f['id'], cc('mimetyp', 'txt')),
                                         order = 'vy,vm,vd')
                dokl = []
                for d in aktendokl:
                    if d['art'] != cc('dokart', 'bnotiz'):
                        dokl.append(d)
                        
            for b in dokl:
                try:
                    data = open('%s/%s' % (akte_path, b['fname']), 'r').readlines()
                except Exception, e:
                    meldung = {'titel':'Fehler',
                           'legende':'Fehlerbeschreibung',
                           'zeile1': str(e),
                           'zeile2':'Versuchen Sie es bitte erneut.'}
                    return (meldung_t %meldung)
                zeilen = string.split(mk_columns(d['betr'],70),'\n')
                tx.textLine("%(vd)s.%(vm)s.%(vy)s: %(art__name)s." %b )
                tx.textLine(mydecode('Betr.: %s' % zeilen[0]))
                for z in zeilen[1:]:
                    tx.textLine("      %s" % z)
                tx.textLine('')
                
                lines = data + ['%(mit_id__vn)s %(mit_id__na)s' % b ]
                for line in lines:
                    tx.textLine(mydecode(line))
                    y = tx.getY()   #get y coordinate
                    if y < bottom_margin + 0.6*inch:
                        canv.drawText(tx)
                        canv.showPage()
                        drawPageFrame(canv,head_l, head_m, head_r)
                        canv.setFont('Helvetica', 11)
                        tx = canv.beginText(left_margin, top_margin - 0.5*inch)
                        
                        #page
                        pg = canv.getPageNumber()
                        if pg % 10 == 0:
                            #print 'formatted page %d' % canv.getPageNumber()
                            pass
                tx.textLine('')
        if tx:
            canv.drawText(tx)
            canv.showPage()
            
        canv.save()
        
        f = open('%s/%s_%s.pdf' % (akte_path, art, akte['id']), 'r')
        datei = f.read()
        f.close()
        try:
            os.remove('%s/%s_%s.pdf' % (akte_path, art, akte['i']))
        except: pass
        
        # Datei uebertragen
        
        if RESPONSE:
            RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
            RESPONSE.setHeader('filename', '%s_%d.pdf' % (art, akte['id']))
            RESPONSE.setBody(datei)
        else:
            meldung = {'titel':'Fehler',
                       'legende':'Fehlerbeschreibung',
                       'zeile1': 'Fehler beim Anzeigen des Dokumentes',
                       'zeile2':'Versuchen Sie es bitte erneut.'}
            return (meldung_t %meldung)
            
class xprintgr_pdf(Request.Request):
    """PDF-Text aus Ascii-Texten erstellen und senden
    (Tabelle: GruppenDokument)."""
    
    permissions = get_permissions()
    
    def processForm(self, REQUEST, RESPONSE):
        art = self.form.get('art')
        if self.form.has_key('gruppeid'):
            gruppeid = self.form.get('gruppeid')
            gruppe = Gruppe(int(gruppeid))
            dokid = self.form.get('dokid')
            dok = Gruppendokument(dokid)
            
            try:
                gruppe_path = get_gruppe_path(gruppe['id'])
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
        if art == 'bnotiz':
            head_l = 'Beraternotizen'
        elif art == 'anotiz':
            head_l = 'Gruppenakte'
        head_m = gruppe['gn']
        head_r = '%(day)d.%(month)d.%(year)d' % today()
        
        # Seitenlayout
        
        def drawPageFrame(canv, head_links, head_mitte, head_rechts):
            canv.line(left_margin, top_margin, right_margin, top_margin)
            canv.setFont('Helvetica-Oblique',11)
            canv.drawString(left_margin, top_margin + 2, "%s" % head_links + "                                  - %s - " % head_mitte)
            canv.drawString(right_margin - 0.85*inch, top_margin + 2, '%s'
                            % head_rechts)
            canv.line(left_margin, top_margin, right_margin, top_margin)
            canv.line(left_margin, bottom_margin, right_margin, bottom_margin)
            canv.drawCentredString(0.5*A4[0], 0.5 * inch,
                         "- %d -" % canv.getPageNumber())
            
        canv = canvas.Canvas('%s/%s_%s.pdf' % (gruppe_path, art, gruppe['id']))
        canv.setPageCompression(0)
        drawPageFrame(canv, head_l, head_m, head_r)
        canv.setAuthor('%(vn) %(na)s' % self.mitarbeiter)
        canv.setTitle('%s - %s -' % (art, gruppe['gn']))
        
        # Inhaltsverzeichnis
        
        canv.setFont("Helvetica-Bold", 14)
        canv.drawString(left_margin, top_margin - 0.5 * inch, "Inhaltsverzeichnis")
        
        canv.setFont("Helvetica", 11)
        tx = canv.beginText(left_margin, top_margin - 0.9 * inch)
        
        if art == 'bnotiz':
            dokl = GruppendokumentList(where = 'gruppe_id = %s and art = %s \
            and mit_id = %s' % (gruppe['id'], cc('dokart', 'bnotiz'),
            self.mitarbeiter['id']), order = 'vy,vm,vd')
            
            for d in dokl:
                zeilen = string.split(mk_columns(d['betr'],55),'\n')
                tx.textLine(mydecode("%(vd)s.%(vm)s.%(vy)s:" % d + " %s" % zeilen[0]))
                if len(zeilen) > 1:
                    for z in zeilen[1:]:
                        tx.textLine("                    %s" % z)
                        
        elif art == 'anotiz':
            aktendokl = GruppendokumentList(where = 'gruppe_id = %s  '
                                   % (gruppe['id']), order = 'vy,vm,vd')
            dokl = []
            #print aktendokl
            for d in aktendokl:
                if d['art'] != cc('dokart', 'bnotiz'):
                    zeilen = string.split(mk_columns(d['betr'],60),'\n')
                    tx.textLine("%(vd)s.%(vm)s.%(vy)s: %(art__name)s" % d )
                    for z in zeilen:
                        tx.textLine(mydecode("       %s" % z))
        canv.drawText(tx)
        canv.showPage()
        
        
        # Die einzelnen Textdateien plus Items aus der DB
        
        
        drawPageFrame(canv, head_l, head_m, head_r)
        canv.setFont("Helvetica", 11)
        tx = canv.beginText(left_margin, top_margin - 0.5*inch)
        
        if art == 'bnotiz':
            dokl = GruppendokumentList(where = 'gruppe_id = %s and art = %s and mtyp = %s \
            and mit_id = %s'% (gruppe['id'], cc('dokart', 'bnotiz'),
            cc('mimetyp', 'txt'), self.mitarbeiter['id']), order = 'vy,vm,vd')
            
        elif art == 'anotiz':
            aktendokl = GruppendokumentList(where = 'gruppe_id = %s and mtyp = %s'
                                   % (gruppe['id'], cc('mimetyp', 'txt')),
                                     order = 'vy,vm,vd')
            dokl = []
            for d in aktendokl:
                if d['art'] != cc('dokart', 'bnotiz'):
                    dokl.append(d)
                    
        for b in dokl:
            try:
                data = open('%s/%s' % (gruppe_path, b['fname']), 'r').readlines()
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
                
            zeilen = string.split(mk_columns(d['betr'],70),'\n')
            tx.textLine("%(vd)s.%(vm)s.%(vy)s: %(art__name)s." % b )
            tx.textLine('Betr.: %s' % zeilen[0])
            for z in zeilen[1:]:
                tx.textLine("      %s" % z)
            tx.textLine('')
            
            lines = data + ['%(mit_id__vn)s %(mit_id__na)s' % b ]
            for line in lines:
                tx.textLine(mydecode(line))
                y = tx.getY()   #get y coordinate
                if y < bottom_margin + 0.6*inch:
                    canv.drawText(tx)
                    canv.showPage()
                    drawPageFrame(canv,head_l, head_m, head_r)
                    canv.setFont('Helvetica', 11)
                    tx = canv.beginText(left_margin, top_margin - 0.5*inch)
                    
                    #page
                    pg = canv.getPageNumber()
                    if pg % 10 == 0:
                        #print 'formatted page %d' % canv.getPageNumber()
                        pass
            tx.textLine('')
        if tx:
            canv.drawText(tx)
            canv.showPage()
            
        canv.save()
        
        f = open('%s/%s_%s.pdf' % (gruppe_path, art, gruppe['id']), 'r')
        datei = f.read()
        f.close()
        try:
            os.remove('%s/%s_%s.pdf' % (gruppe_path, art, gruppe['i']))
        except: pass
        
        # Datei uebertragen
        
        if RESPONSE:
            RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
            RESPONSE.setHeader('filename', '%s_%d.pdf' % (art, gruppe['id']))
            RESPONSE.setBody(datei)
        else:
            meldung = {'titel':'Fehler',
                       'legende':'Fehlerbeschreibung',
                       'zeile1': 'Fehler beim Anzeigen des Dokumentes',
                       'zeile2':'Versuchen Sie es bitte erneut.'}
            return (meldung_t %meldung)
            
class suchetxt(Request.Request):
    """Suche Text (Expression) im Ascii-Text
    (Tabelle: Dokument, Gruppendokument)."""
    
    permissions = get_permissions()
    def processForm(self, REQUEST, RESPONSE):
        import os
        popen = os.popen
        
        akte = None
        #print "***************************"
        #print self.form.has_key('fallid')
        
        mitarbeiterliste = self.getMitarbeiterliste()
        user = self.user
        if self.form.has_key('expr'):
            expr = self.form.get('expr')
        else:
            self.last_error_message = "Kein Suchaudruck erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
            
        string.strip(expr)
        if len(expr) > 2:
            pass
        else:
            meldung = {'titel':'Fehler',
                       'legende':'Fehlerbeschreibung',
                       'zeile1': 'Der Suchausdruck sollte mind. 3 Buchstaben haben',
                       'zeile2':'Versuchen Sie es bitte erneut.'}
            return (meldung_t %meldung)
            
        if self.form.has_key('gruppeid'):
            gruppeid = self.form.get('gruppeid')
            gruppe = Gruppe(int(gruppeid))
            
            try:
                gruppe_path = get_gruppe_path(gruppe['id'])
                cwd = os.getcwd()
                os.chdir('%s' % gruppe_path)
                fd = popen('agrep -n -i %s *.txt ' % expr)
                ergebnis = fd.readlines()
                fd.close()
                os.chdir(cwd)
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
                
            menue = menuegruppe_t % gruppe
            header = ("Suchergebnis in den Texten der Gruppe &quot;%(name)s&quot; " % gruppe)
            formhiddenv = formhiddennamevalues_t % ({'name' : 'gruppeid' ,
                                                     'value' : gruppeid})
            
        elif self.form.has_key('fallid'):
            fallid = self.form.get('fallid')
            fall = Fall(fallid)
            akte = Akte(fall['akte_id'])
            letzter_fall = akte['letzter_fall']
            aktueller_fall = akte['aktueller_fall']
            
            try:
                akte_path = get_akte_path(akte['id'])
                cwd = os.getcwd()
                os.chdir('%s' % akte_path)
                #print "************* VOR POPEN *******************"
                fd = popen('agrep -n -i %s *.txt /dev/null' % expr)
                #print "************* NACH POPEN ******************"
                #print fd.readlines()
                ergebnis = fd.readlines()
                fd.close()
                os.chdir(cwd)
            except Exception, e:
                meldung = {'titel':'Fehler',
                         'legende':'Fehlerbeschreibung',
                         'zeile1': str(e),
                         'zeile2':'Versuchen Sie es bitte erneut.'}
                return (meldung_t %meldung)
                
            if aktueller_fall:
                formhiddenv = formhiddennamevalues_t % ({'name' : 'fallid' ,
                                                    'value' : aktueller_fall['id']})
            else:
                formhiddenv = formhiddennamevalues_t % ({'name' : 'fallid' ,
                                                    'value' : letzter_fall['id']})
                
        res = []
        #res.append(head_normal_t %("Suchergebnis in der Akte &quot;%(vn)s %(na)s&quot; " % akte))
        res.append(formkopfv_t % 'suchetxt')
        res.append(formhiddenv)
        res.append(dokausgabe7_t % ('Suche in den Texten', expr))
        res.append(dokausgabe1_t % 'Suchergebnis')
        
        if not ergebnis:
            res.append('Kein Ergebnis.')
        else:
            n = 0
            datei = []
            for line in ergebnis:
                i = string.index(line,'.txt')
                datei.append(line[:i])
                if self.form.has_key('gruppeid'):
                    if n == 0:
                        dok = Gruppendokument(line[:i])
                        res.append(dokausgabe2b_t % dok)
                    if datei[n] != datei[n-1]:
                        dok = Gruppendokument(line[:i])
                        res.append(dokausgabe2b_t % dok)
                    res.append(dokausgabe8_t % line[i+5:])
                    n = n + 1
                    
                elif self.form.has_key('fallid'):
                    if n == 0:
                        dok = Dokument(line[:i])
                        res.append(dokausgabe2_t % dok)
                    if datei[n] != datei[n-1]:
                        dok = Dokument(line[:i])
                        res.append(dokausgabe2_t % dok)
                    res.append(dokausgabe8_t % line[i+5:])
                    n = n + 1
        res.append(dokausgabe3_t)
        return string.join(res, '')
        
        
        
        
class print_pdf(Request.Request):
    """PDF-Text aus Ascii-Texten erstellen und senden (Tabelle: Dokument.)"""
    
    permissions = get_permissions()
    
    def processForm(self, REQUEST, RESPONSE):
        if self.form.has_key('gruppeid'):
            self.__class__ = printgr_pdf
            # damit print_pdf auch fr Gruppen funktioniert
            return printgr_pdf.processForm(self, REQUEST, RESPONSE)
        art = self.form.get('art')
        fallid = self.form.get('fallid')
        fall = Fall(fallid)
        akte = fall['akte']
        if not (fallid):
            self.last_error_message = "Kein Fall erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        dok_pdf = AkteDokumentePDF(akte, self.mitarbeiter, beraternotizen=(art=='bnotiz'))
        datei = dok_pdf.create_pdf() 
        RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
        RESPONSE.setHeader('filename', 'klient_%s_%d.pdf' % (art, akte['id']))
        RESPONSE.setBody(datei)
        
class printgr_pdf(Request.Request):
    """PDF-Text aus Ascii-Texten erstellen und senden (Tabelle: Dokument.)"""
    
    permissions = get_permissions()
    
    def processForm(self, REQUEST, RESPONSE):
        art = self.form.get('art')
        gruppe_id = self.form.get('gruppeid')
        if not (gruppe_id):
            self.last_error_message = "Keine Gruppe erhalten"
            return self.EBKuSError(REQUEST, RESPONSE)
        gruppe = Gruppe(gruppe_id)
        gruppe_path = get_gruppe_path(gruppe_id)
        dok_pdf = GruppenDokumentePDF(gruppe, self.mitarbeiter, beraternotizen=(art=='bnotiz'))
        datei = dok_pdf.create_pdf() 
        RESPONSE.setHeader('content-type', '%s' % 'application/pdf')
        RESPONSE.setHeader('filename', 'gruppe_%s_%d.pdf' % (art, gruppe['id']))
        RESPONSE.setBody(datei)

from reportlab.lib.units import inch, cm, mm
from reportlab.lib import colors
from reportlab.platypus import Paragraph, PageBreak, Spacer, Flowable
from reportlab.platypus.tables import Table, TableStyle
from ebkus.app.pdfgen import CustomPDF



class DokumentePDF(CustomPDF):
    # Margins
    lm = 20*mm
    rm = 15*mm
    tm = 20*mm
    bm = 20*mm
    topline = True # explizite Linie auf dem topMargin
    bottomline = True # explizite Linie auf dem bottomMargin
    header_on_first_page = True
    header_left = ''
    header_center = ''
    header_right = ''
    header_left = 'Klientenakte vom Datum'
    header_center = 'Name und Geburtsdatum'
    header_right = 'Stamm- und Falldaten'

    def __init__(self, mitarbeiter, beraternotizen=False):
        """Falls beraternotizen True ist, die nehmen statt Dokumente."""
        super(DokumentePDF, self).__init__()
        self.mitarbeiter = mitarbeiter
        self.beraternotizen = beraternotizen
        if beraternotizen:
            self.init_beraternotizen()
        else:
            self.init_dokumente()
    def init_beraternotizen(self):
        # Beraternotizen nur fr den aufrufenden Mitarbeiter!
        self.where_art = "art = {} and mit_id = {}".format(cc('dokart', 'bnotiz'), self.mitarbeiter['id'])
        # Titel von __init__() berschreiben
        self.titel = 'Beraternotizen'
        self.eintrag_t = "{v}: {betr}{datei}"
    def init_dokumente(self):
        self.where_art = " art != {}".format(cc('dokart', 'bnotiz'))
        # Titel schon gesetzt in __init__
        self.eintrag_t = "{v} <i>{art__name}</i>: {betr}{datei}"
    def get_dok_ueberschrift(self, dok):
        prio = {}
        if dok['mtyp'] != cc('mimetyp', 'txt'):
            datei = self.formatzelle((" (Datei: {fname})",), 
                                     ebkus_objekt=dok)[0]
            prio = dict(datei=datei)
        eintrag = self.formatzelle(
            (self.eintrag_t,), 
            prio=prio, ebkus_objekt=dok)[0]
        return eintrag

    def add_dok_inhaltsverzeichnis(self):
        self.addp(self.titel, 'Heading2')
        self.addp('Inhaltsverzeichnis', 'Heading4')
        for dok in self.dokumente:
            self.addp(self.get_dok_ueberschrift(dok), 'Normal')
        self.addf(Spacer(1, 30))
    def get_dok_inhalt(self, dok, path):
        # Inhalt aus der Textdatei holen und newlines durch <br/> ersetzen
        lines = open(os.path.join(path, dok['fname']), 'r').readlines()
        lines = [l.rstrip()+'<br/>' for l in lines]
        return ''.join([mydecode(l) for l in lines])
    def add_dok_inhalte(self):
        # Inhalte nur fr Textdokumente eintragen
        # TODO Leerzeichen nach <br/> irgendwie erhalten,
        # So dass man durch Leerzeichen einrcken kann.
        for dok in self.dokumente:
            if dok['mtyp'] == cc('mimetyp', 'txt'):
                self.addp(self.get_dok_ueberschrift(dok), 'HeaderCell')
                self.addf(Spacer(1, 5))
                inhalt = self.get_dok_inhalt(dok, self.path)
                inhalt_mit_name = self.formatzelle(
                    ("{inhalt}<br/><i>{mit__name}</i>",), 
                    prio=dict(inhalt=inhalt), ebkus_objekt=dok)[0]
                self.addp(inhalt_mit_name, 'BodyText')
                self.addf(Spacer(1, 30))

    def add_content(self):
        self.set_header()
        self.add_dok_inhaltsverzeichnis()
        #self.addf(PageBreak())
        self.add_dok_inhalte()

class AkteDokumentePDF(DokumentePDF):
    def __init__(self, akte, mitarbeiter, beraternotizen=False):
        self.akte = akte
        self.faelle = akte['faelle']
        self.faelle.sort('bgy', 'bgm', 'bgd')
        self.titel = 'Dokumente'
        self.path = get_akte_path(akte['id'])
        super(AkteDokumentePDF, self).__init__(mitarbeiter, beraternotizen)
        self.dokumente = DokumentList(
            where='fall.akte_id = %s and %s' % (akte['id'], self.where_art), 
            join=[('fall', 'fall.id = fall_id'),],
            order='vy,vm,vd')
    def set_header(self):
        self.header_left = self.formatzelle(
            ('{titel} (Stand: {today})',), 
            prio=dict(titel=self.titel, today=str(today())))[0]
        self.header_right = self.formatzelle(
            ('Klient: {name} geb. {gb}',),
            ebkus_objekt=self.akte)[0]
        self.header_center = u""

class GruppenDokumentePDF(DokumentePDF):
    def __init__(self, gruppe, mitarbeiter, beraternotizen=False):
        self.gruppe = gruppe
        super(GruppenDokumentePDF, self).__init__(mitarbeiter, beraternotizen)
        self.titel = 'Gruppendokumente'
        self.path = get_gruppe_path(gruppe['id'])
        self.dokumente = GruppendokumentList(
            where='gruppe_id = %s and %s' % (gruppe['id'], self.where_art), 
            order='vy,vm,vd')
    def set_header(self):
        self.header_left = self.formatzelle(
            ('{titel} (Stand: {today})',), 
            prio=dict(titel=self.titel, today=str(today())))[0]
        self.header_right = self.formatzelle(
            ('Gruppe: {name} ({gn})',),
            ebkus_objekt=self.gruppe)[0]
        self.header_center = u""
