# coding: latin-1

"""
Akten lschen.
Als Parameter knnen einzelne Akten-Ids oder Intervalle ohne Leerzeichen angegeben werden, zB
akten_loeschen 2-5 100-1276 8 9

"""

import sys, os
import init
from ebkus.app.ebupd import remove_akten
from ebkus.db.sql import opendb

def usage():
    s = """\
Akten mit einer direkt angegebenen Id oder in einem Intervall 
befindlichen Id loeschen. Der Intervall ist inklusiv.

Es werden alle Arten von Akten geloescht, einschliesslich der
zugehoerigen Statistik.

akten_loeschen <id> ... [--nodump]
akten_loeschen <start-id>-<ende-id> ...  [--nodump]

Mit der Option --nodump wird eine Sicherung der Datenbank unterbunden.
Mit der Option --console werden Meldungen ueber geloeschte Akten 
auf der Konsole ausgegeben.

Beispiele:

akten_loeschen 234
   Existierende Datenbank wird gesichert.
   Die Akte mit der Id 234 wird geloescht.
   Keine Meldung auf der Konsole.

akten_loeschen 1-1200 5677 6001-6003 8999 --nodump --console
   Alle Akten mit Ids im Bereich der Intervalle inklusive
   Beginn und Ende des Intervalls und mit den einzeln
   aufgefuehrten Ids werden geloescht.
   Es wird keine Datenbanksicherung durchgefuehrt.
   Meldungen ueber geloeschte Akten werden ausgegeben.

"""
    print s
    sys.exit(1)

def parse_command_line(args):
    if not args:
        usage()
    id_list = []
    dump = True
    try:
        for arg in args:
            if arg == '--nodump':
                dump = False
            elif arg == '--console':
                pass
            elif '-' in arg:
                start = int(arg.split('-')[0])
                end = int(arg.split('-')[1])
                id_list.append((start, end))
            else:
                id_list.append(int(arg))
    except Exception:
        usage()
    return id_list, dump

def loeschen(id_list):
    opendb()
    for el in id_list:
        try:
            start, end = el
            remove_akten(range(start, end+1),
                         statistik_auch=True, 
                         aktuell_auch=True)
        except TypeError:
            remove_akten([el,],
                         statistik_auch=True, 
                         aktuell_auch=True)
                               
def main(args=None):
    if args is None:
        args = sys.argv[1:]
    if not args:
        usage()
    id_list, dump = parse_command_line(args)
    # print id_list, dump
    if dump:
        from ebkus.Install import InstallFromInstance
        from ebkus.config import config
        installer = InstallFromInstance(config)
        installer.init()
        instance = installer.ebkus[config.INSTANCE_NAME]
        res = instance.backup_instance(dir=os.getcwd())
        if not res:
            sys.exit(1)
    loeschen(id_list)
    
if __name__=='__main__':
    main()
