# coding: latin-1

"""
Datenbank zu Testzwecken anonymisieren, ohne den Versuch zu machen, 
es echt aussehen zu lassen.

Namen: werden formalisiert
Mitarbeiter Kennung und Passwort: test bzw. test<lfd-nr>, verw bzw verw<lfd-nr>, admin bzw. admin<lfd-nr>
Dienststelle Name: wird formalisiert Stelle<id>
Geburtsdatum: bleibt
Telefon: wird normalisiert
Anmeldung: Textfelder und Telefon formalisieren
Adressen: werden gelscht
Notizen: werden formalisiert
Strassenkatalog: bleibt
Gemeindeschlssel: bleiben
altdaten: werden gelscht
Beratungskontakt: alle Textfelder und Varchar formalisieren, falls inhalte da waren.
Dokument betr: formalisieren
Gruppe betr: formalisieren

mitarbeiter
+ben, VARCHAR(25), Benutzer, p,
+pass, VARCHAR(50), Passwort, p,

strkatalog
+name, VARCHAR(60), Strassenname, p,
+plz, VARCHAR(5), Postleitzahl, p,
+ort, VARCHAR(60), Ort, p,
+ortsteil, VARCHAR(60), Ortsteil, p,
+samtgemeinde, VARCHAR(60), Samtgemeinde, p,
+bezirk, VARCHAR(60), Bezirk, p,

akte
+str, VARCHAR(35), Strasse, p,
+plz, VARCHAR(9), Postleitzahl, p,
+ort, VARCHAR(60), Ort, p,

bezugsperson
+str, VARCHAR(35), Strasse, p,
+plz, VARCHAR(9), Postleitzahl, p,
+ort, VARCHAR(35), Ort, p,

beratungskontakt
+faktor, VARCHAR(255), offenes Merkmal zur Verrechnung, p

dokument
+fname, VARCHAR(255), Dateiname, p,

gruppendokument
+fname, VARCHAR(254), Dateiname, p,

fachstat
+plz, VARCHAR(9), Postleitzahl, p,
+ort, VARCHAR(60), Ort, p,
+ortsteil, VARCHAR(60), Ortsteil, p,
+samtgemeinde, VARCHAR(60), Samtgemeinde, p,
+bezirk, VARCHAR(60), Bezirk, p,

altdaten

ags
+plz, CHAR(5), Postleitzahl, p,
+ort, VARCHAR(60), Ort, p,

"""

import sys, os
import sha
try:
    import init
    from ebkus.db.sql import getDBHandle, opendb
    from ebkus.app.ebapi import MitarbeiterList, CodeList
except ImportError:
    print "[Funktioniert nur im Instanzverzeichnis der Instanz, "
    print "deren Datenbank anonymisiert werden soll.]"
    print

class Anonymisieren(object):

    # Diese Felder werden standardm befllt nach dem Muster <tabellenname>-<feldname>-<id>
    # Fr die Funktionalitt von EBKuS haben sie keine Bedeutung, lediglich Info fr die Nutzer.
    standard_text_felder = (
        ('mitarbeiter', ('vn', 'na', 'anr', 'tl1', 'fax', 'mail')),
        ('akte', ('vn', 'na', 'tl1', 'tl2', 'mail', 'no', 'geschw')),
        ('anmeldung', ('von', 'mtl', 'me', 'mg', 'no')),
        ('bezugsperson', ('vn', 'na', 'ber', 'tl1', 'tl2', 'mail', 'no')),
        ('einrichtung', ('na', 'tl1', 'tl2', 'mail', 'no')),
        ('beratungskontakt', ('no', 'inhalt', 'vorgehen', 'diagnose', 'kontrakt', 'richtung', 'verlauf', 'vereinbarung')),
        ('fua_bs', ('no',)),
        ('dokument', ('betr',)),
        ('gruppendokument', ('betr',)),
        ('gruppe', ('name', 'thema')),
        ('fachstat', ('no', 'no2', 'no3')),
    )
    # Diese Felder werden geleert. Der Funktionalitt von EBKuS tut das keinen Abbruch, da
    # die Angabe einer Adresse nicht obligatorisch ist.
    adress_felder = (
        ('akte', ('str','hsnr', 'plz','ort')),
        ('bezugsperson', ('str','hsnr','plz','ort')),
        ('fachstat', ('plz','ort','ortsteil','samtgemeinde','bezirk')),
    )
    # In der anonymisierten Datenbank werden Benutzernamen umbenannt nach folgendem
    # Schema abhngig von den Benutzerrechten:
    #   test<id-des-Mitarbeiters> fr Bearbeiter
    #   verw<id-des-Mitarbeiters> fr Verwaltungskraft
    #   admin<id-des-Mitarbeiters> fr Administratoren
    
    mitarbeiter_namen = {
        'verw': 'verw',
        'bearb': 'test',
        'admin': 'admin',
    }

    def __init__(self):
        opendb()
        self.db = getDBHandle()

    def anonymisieren(self):
        self.standard_text_felder_anonymisieren()
        self.adress_felder_leeren()
        self.mitarbeiter_anonymisieren()
        self.dienststellen_anonymisieren()
        self.altdaten_loeschen()
        
    def standard_text_felder_anonymisieren(self):
        """
        Standardtextfelder sind: alle Namen, Telefonnummern, Emailadressen, Notizen sowie alle
        anderen Textfelder, die Informationen fr den Benutzer enthalten.
        Siehe die Liste standard_text_felder.
        Alle diese Felder werden nach folgendem Muster befllt:
               <tabellenname>.<feldname>-<id-des-Datensatzes>
        Muster:: update <tb> set <feld> = concat(<tb>.<feld>, id) where <feld> like '%_%';
        """
        for tabelle, felder in self.standard_text_felder:
            for feld in felder:
                sql = """UPDATE {tabelle} SET {feld} = CONCAT('{tabelle}.{feld}-', id) 
where {feld} like '%_%'""".format(**locals())
                self.db.query(sql)
                print 'Textfeld anonymisiert: {}.{}'.format(tabelle, feld)
    def adress_felder_leeren(self):
        for tabelle, felder in self.adress_felder:
            for feld in felder:
                sql = """UPDATE {tabelle} SET {feld} = '' 
where {feld} like '%_%'""".format(**locals())
                self.db.query(sql)
                print 'Adressfeld geleert: {}.{}'.format(tabelle, feld)
        
    def mitarbeiter_anonymisieren(self):
        """
        Es ists gewhrleistet, dass es immer einen Berater test,
        eine Verwaltungskraft verw und einen Administrator admin
        gibt, jeweils mit Passwort identisch mit dem Namen.

        Der Anmeldename wird fr alle Mitarbeiter wie folgt ersetzt:
        Berater: test bzw. test<laufende-nummer>
        Verwaltungskraft: verw bzw. verw<laufende-nummer>
        Admin: admin bzw. admin<laufende-nummer>
        """
        count = dict(admin=0, verw=0, bearb=0)
        for mitarbeiter in MitarbeiterList(where=''):
            alter_name = mitarbeiter['ben']
            benr = mitarbeiter['benr__code']
            basis = self.mitarbeiter_namen[benr]
            if count[benr] == 0:
                neuer_name = basis
            else:
                neuer_name = basis + str(count[benr])
            count[benr] += 1
            # Passwort identisch mit dem Namen
            neues_passwort = sha.new(neuer_name).hexdigest()
            mitarbeiter.update({'ben': neuer_name,
                                'pass': neues_passwort})
            print ('Mitarbeiter umbenannt: {} --> {} (neues Passwort: {}'.
                   format(alter_name, neuer_name, neuer_name))
            
            
    def altdaten_loeschen(self):
       self.db.query("TRUNCATE TABLE altdaten")
       print 'Altdaten geloescht'
    def dateinamen_anonymisieren(self):
        # TODO
        pass
    
    def dienststellen_anonymisieren(self):
        for code in CodeList(where="off = 0 and kat_code = 'stzei'"):
            alter_name = code['name']
            neuer_name = "Stelle" + str(code['id'])
            code.update(dict(name=neuer_name))
            print 'Dienststelle umbenannt: {} --> {}'.format(alter_name, neuer_name)


def usage():
    s = """\
Akten anonymisieren.

Personenbezogene Daten werden formalisiert umbenannt.
Mitarbeitername werden ebenfalls umbenannt.

Es gibt in jedem Fall die Benutzer test, verw und admin, mit jeweils
demselben Passwort.

anonymisieren.py run

Mit der Option --nodump wird eine Sicherung der Datenbank unterbunden.

Beispiele:
  anonymisieren.py run
  anonymisieren.py --nodump run
"""
    print s
    sys.exit(1)

def main(args=None):
    if args is None:
        args = sys.argv[1:]
    if not 'run' in args or '-h' in args or '--help' in args:
        usage()
    if not '--nodump' in args:
        from ebkus.Install import InstallFromInstance
        from ebkus.config import config
        installer = InstallFromInstance(config)
        installer.init()
        instance = installer.ebkus[config.INSTANCE_NAME]
        res = instance.backup_instance(dir=os.getcwd())
        if not res:
            sys.exit(1)
        print 'Um die vorherige Datensicherung zu vermeiden, bitte "--nodump" angeben' 
    anon = Anonymisieren()
    # anon.standard_text_felder_anonymisieren()
    # anon.adress_felder_leeren()
    anon.anonymisieren()

if __name__=='__main__':
    main()
