#!/usr/bin/python
# coding: latin-1

"""
Alle Datenbankzeilen ausgeben, die eine ungltige
Fremdschlssel aufweisen.
In einer konsistenten Datenbank drfen die nicht 
vorkommen.
"""

import sys
from itertools import groupby
from operator import itemgetter
import init
from ebkus.db.sql import getDBHandle, opendb
from ebkus.app.ebapi import TabelleList
from ebkus.app import ebapi

def usage():
    s = """\
check_referential_integrity.py
  Alle Datenbankzeilen ausgeben, die eine ungltige
  Fremdschlssel aufweisen.
"""    
    sys.exit(1)

class CheckForeignKeys(object):
    # In dieses Feld werden einfach Minuten eingetragen,
    # ist also nicht wirklich ein Fremdschlssel obwohl
    # schemadata.py so markiert (fskd)
    exclude = (('code', 'beratungskontakt', 'dauer'),)
    def __init__(self):
        opendb()
        self.db = getDBHandle()
        self.fk_data = self.get_fk_data()
    def get_fk_data(self):
        """
        Return dict: fk_table -> list of (fk_table, table, fk_key)
        Fr jede refenzierte Tabelle eine Liste mit der referierenden
        Tabelle und das referierende Feld (foreign key).
        """
        classes = [kl for kl in [getattr(ebapi, t['klasse'], None)
                   for t in TabelleList(where='')] if kl]
        data = []
        for klass in classes:
            for fk_name, (fk_klass, inverse) in klass.foreignfieldtypes.items():
                data.append((fk_klass.table, klass.table, fk_name))
        data.sort()
        fk_data = {}
        for fk_table, fk_data_list in groupby(data, itemgetter(0)):
            fk_data[fk_table] = list(fk_data_list)
        for elem in self.exclude:
            data_list = fk_data[elem[0]]
            data_list.remove(elem)
        return fk_data
    def sql_for_one_foreign_key(self, fk_table, table, fk_name):
        sql = """  
  SELECT id, '{table}' AS tabelle, '{fk_name}' AS feld, {fk_name} as {fk_table}_id 
  FROM {table}
  WHERE {fk_name} IS NOT NULL""".format(**locals())
        return sql
    def sql_for_one_referenced_table(self, fk_table):
        # Die Union aller Tabelle mit einem FK nach fk_table
        alias_for_union = fk_table + '_dep'
        # Der Name des Fremdschlssels in der Union-Tabelle
        alias_for_ref_key = fk_table + '_id'
        sql = """
SELECT id, tabelle, feld, {fk_table}_id
FROM (""".format(**locals())
        sql += """
  UNION""".join([self.sql_for_one_foreign_key(*data)
                 for data in self.fk_data[fk_table]])
        sql += """
  ) {alias_for_union}
WHERE NOT EXISTS (SELECT id 
                  FROM {fk_table}
                  WHERE id = {alias_for_union}.{alias_for_ref_key});
""".format(**locals())
        return sql
    def get_sql(self):
        sql = ''
        for fk_table in self.fk_data:
            sql += self.sql_for_one_referenced_table(fk_table)
        return sql
    def get_invalid_references(self):
        rows = []
        for fk_table in self.fk_data:
            sql = self.sql_for_one_referenced_table(fk_table)
            rows += self.db.query(sql)
        return rows
    def sql_for_null_values(self, fk_table, table, field):
        sql = """
SELECT '{fk_table}', '{table}', '{field}', count(id) FROM {table} WHERE {field} IS NULL;""".format(**locals())
        return sql
    def run_sql_for_null(self):
        for data_list in self.fk_data.values():
            for fk_table, table, field in data_list: 
                sql = self.sql_for_null_values(fk_table, table, field)
                res = self.db.query(sql)
                for row in res:
                    print row

def main():
    cri = CheckForeignKeys()
    #print cri.get_sql()
    rows = []
    rows += cri.get_invalid_references()
    if rows:
        for row in rows:
            print row
    else:
        print 'Keine ungueltigen Fremdschluessel gefunden'
if __name__ == '__main__':
    if '--help' in sys.argv or '-h' in sys.argv:
        usage()
    main()


