import re
import os
from os.path import join
import sys
# import chardet

path = '/home/atms/dev/ebkus/anonymisieren'

# Listen von ungeordneten Namen in Latin1
pool_dateien = dict(
    vornamen_weiblich =   join(path, 'vornamen_weiblich_ungeordnet.txt'),
    vornamen_maennlich =   join(path, 'vornamen_maennlich_ungeordnet.txt'),
    vornamen =   join(path, 'vornamen_ungeordnet.txt'),
    nachnamen =   join(path, 'nachnamen_ungeordnet.txt'),
    strassen =   join(path, 'strassennamen_ungeordnet.txt'),
    orte  =   join(path, 'ortsnamen_ungeordnet.txt'),
    domains  =   join(path, 'domains.txt'),
    )
logpath = join(path, 'anonlog.txt')

class DatenbankAnonymisieren(object):
    def __init__(self):
        import ebkus.app.ebapigen as ac
        for name, path in pool_dateien.items():
            setattr(self, name, open(path, 'rb').read().split('\n'))
        self.log = open(logpath, 'wb')
        vornamen_umbenenner = VornamenUmbenenner(self.vornamen_weiblich, self.vornamen_maennlich)
        nachnamen_umbenenner = NamenUmbenenner(self.nachnamen)
        strassen_umbenenner = PoolUmbenenner(self.strassen)
        orte_umbenenner = PoolUmbenenner(self.orte)
        telefon_umbenenner = TelefonUmbenenner()
        plz_umbenenner = PLZUmbenenner()
        email_umbenenner = EmailUmbenenner(self.vornamen, self.domains)
        # klasse, feld, pool
        self.todo = (
            (ac.AkteList, 'vn', vornamen_umbenenner),
            (ac.AkteList, 'na', nachnamen_umbenenner),
            (ac.AkteList, 'ort', orte_umbenenner),
            (ac.AkteList, 'plz', plz_umbenenner),
            (ac.AkteList, 'str', strassen_umbenenner),
            (ac.AkteList, 'tl1', telefon_umbenenner),
            (ac.AkteList, 'tl2', telefon_umbenenner),
            (ac.AkteList, 'mail', email_umbenenner),

            (ac.BezugspersonList, 'vn', vornamen_umbenenner),
            (ac.BezugspersonList, 'na', nachnamen_umbenenner),
            (ac.BezugspersonList, 'ort', orte_umbenenner),
            (ac.BezugspersonList, 'plz', plz_umbenenner),
            (ac.BezugspersonList, 'str', strassen_umbenenner),
            (ac.BezugspersonList, 'tl1', telefon_umbenenner),
            (ac.BezugspersonList, 'tl2', telefon_umbenenner),
            (ac.BezugspersonList, 'mail', email_umbenenner),

            (ac.AnmeldungList, 'mtl', telefon_umbenenner),

            (ac.EinrichtungskontaktList, 'na', nachnamen_umbenenner),
            (ac.EinrichtungskontaktList, 'tl1', telefon_umbenenner),
            (ac.EinrichtungskontaktList, 'tl2', telefon_umbenenner),
            (ac.EinrichtungskontaktList, 'mail', email_umbenenner),

            (ac.StrassenkatalogNeuList, 'ort', orte_umbenenner),
            (ac.StrassenkatalogNeuList, 'plz', plz_umbenenner),
            (ac.StrassenkatalogNeuList, 'name', strassen_umbenenner),

            (ac.FachstatistikList, 'ort', orte_umbenenner),
            (ac.FachstatistikList, 'plz', plz_umbenenner),

            (ac.AltdatenList, 'name', nachnamen_umbenenner),
            (ac.AltdatenList, 'ort', orte_umbenenner),
            (ac.AltdatenList, 'plz', plz_umbenenner),
            (ac.AltdatenList, 'strasse', strassen_umbenenner),
            (ac.AltdatenList, 'telefon1', telefon_umbenenner),
            (ac.AltdatenList, 'telefon2', telefon_umbenenner),
            )
        self.todo_vorname = (
            (ac.AltdatenList, 'vorname', vornamen_umbenenner),
            )
        self.todo_mitarbeiter = (
            (ac.MitarbeiterList, 'vn'),
            (ac.MitarbeiterList, 'na'),
            (ac.MitarbeiterList, 'ben'),
            (ac.MitarbeiterList, 'pass'),
            )
    def do(self):
        self.do_mitarbeiter()
        for klassList, feld, umbenenner in self.todo:
            self.do_one(klassList, feld, umbenenner)
    def do_mitarbeiter(self):
        from ebkus.app.ebapi import MitarbeiterList
        import sha
        specials = {
            'Administrator': 'Administrator',
            'Admin': 'Admin',
            'Detlef': 'Tester',
            'Jansen': 'Tester',
            'jansen': 'test',
            'Test': 'Test',
            'Birger': 'Tomas',
            'Lahnert': 'Main',
            'lahnert': 'main',
            'Vera': 'Eva',
            'Schreiber': 'Leser',
            'schreiber': 'leser',
            'Hanna': 'Berta',
            'Hempfling': 'Lager',
            'hempfling': 'lager',
            'Adelheit': 'Verw',
            'Sekretariat': 'Verw',
            'sekretariat': 'verw',
            'Anne': 'Rita',
            'Gregl': 'Mai',
            'gregl': 'mai',
            }
        vornamen_umbenenner = VornamenUmbenenner([],[], specials=specials)
        nachnamen_umbenenner = NamenUmbenenner([], specials=specials)
        benutzer_umbenenner = BenutzerUmbenenner(specials=specials)
        self.do_one(MitarbeiterList, 'vn', vornamen_umbenenner)
        self.do_one(MitarbeiterList, 'na', nachnamen_umbenenner)
        self.do_one(MitarbeiterList, 'ben', benutzer_umbenenner)
        mlist = MitarbeiterList(where='')
        for m in mlist:
            # set passwort
            ben = m['ben']
            passwort = sha.new(ben).hexdigest()
            print >> self.log, 'Mitarbeiter ben:',ben, passwort
            print 'Mitarbeiter ben:',ben, passwort
            m.update({'pass': passwort})
    def do_one(self, klassList, feld, umbenenner, **kw):
        objekte = klassList(where='')
        for obj in objekte:
            kw = {}
            try:
                if feld == 'vn':
                    kw['geschlecht'] = obj['gs__name'][0]
                elif feld == 'vorname':
                    kw['geschlecht'] = obj['geschlecht'][0]
            except:
                pass
            name = obj[feld]
            new_name = umbenenner.umbenennen(name, **kw)
            print >> self.log, obj.__class__.__name__, feld, ">{}<-->{}<".format(name, new_name)
            print obj.__class__.__name__, feld, ">{}<-->{}<".format(name, new_name)
            obj.update({feld: new_name})

DA = DatenbankAnonymisieren

def replace(name, map, pool, specials):
    #print name, map, pool, specials
    if name in specials:
        return specials[name]
    if name in map:
        return map[name]
    new_name = pool.pop()
    map[name] = new_name
    return new_name

class Umbenenner(object):
    """
    >>> pu = Umbenenner()
    >>> pu.umbenennen('lbr g', tomas='Gevatter')
    'lbr g'
    >>> pu.umbenennen('tomas', tomas='Gevatter')
    'Gevatter'
    """
    def umbenennen(self, name, **kw):
        return self._get_replacement(name, **kw)
    def _get_replacement(self, name, **kw):
        try:
            return replace(name, {}, set(), kw)
        except KeyError:
            return name
class BenutzerUmbenenner(Umbenenner):
    """
    >>> pu = BenutzerUmbenenner(dict(alb='Albrecht', pete='Peter'))
    >>> pu.umbenennen('alb40')
    'Albrecht40'
    >>> pu.umbenennen('pete')
    'Peter'
    """
    _re_suffix = re.compile(r'^([a-zA-Z]+)([0-9]*)?$')
    def __init__(self, specials):
        self.specials = specials
    def _get_replacement(self, name, **kw):
        ben, suffix = self._re_suffix.match(name).groups()
        return self.specials[ben] + suffix
class PoolUmbenenner(Umbenenner):
    """
    >>> pu = PoolUmbenenner({'U', 'v', 'W'}, specials={'VVV': 'XXX'})
    >>> res = set(pu.umbenennen(x) for x in (' ', '1', '22', 'A', 'VVV'))
    >>> res == set(['XXX', 'U', 'v', ' ', 'W'])
    True
    """
    def __init__(self, pool, specials={}):
        self.pool = pool
        self.specials = specials
        self.map = {}
    def _get_replacement(self, name, **kw):
        if not name.strip():
            return name
        if len(self.pool) < 2:
            print 'POOL', self.pool
        return replace(name, self.map, self.pool, self.specials)
class NamenUmbenenner(PoolUmbenenner):
    """
    >>> pu = NamenUmbenenner(['Eva', 'Maria', 'Waltraut'])
    >>> pu.umbenennen('Angela-Magda--4')
    'Waltraut-Maria--4'
    >>> pu.umbenennen('Magda Donna')
    'Maria Eva'
    >>> pu.umbenennen('Donna--345')
    'Eva--345'
    """
    _re_suffix = re.compile(r'^(.+?)(--[0-9]+)*$')
    def _get_replacement(self, name, **kw):
        name = name.strip()
        if not name:
            return name
        #print 'Name:', name
        name, suffix = self._re_suffix.match(name).groups()
        assert '--' not in name
        assert suffix is None or '--' in suffix
        name_list = [name]
        sep = ''
        for x in (' ', '-'):
            if x in name:
                name_list = name.split(x)
                sep = x
                break
        new_name_list = []
        for n in name_list:
            new_name_list.append(replace(n, self.map, self.pool, self.specials))
        new_name = sep.join(new_name_list) + (suffix if suffix else '')
        return new_name
class VornamenUmbenenner(Umbenenner):
    """
    >>> pu = VornamenUmbenenner(['Eva', 'Tina', 'Vera'], ['Rob', 'Thomas', 'Walter'], specials={'Donna': 'Rebecca'})
    >>> pu.umbenennen('Angela-Thomas', geschlecht='w')
    'Vera-Tina'
    >>> pu.umbenennen('Albrecht', geschlecht='m')
    'Walter'
    >>> pu.umbenennen('Donna')
    'Rebecca'
    >>> pu.umbenennen('Frieda', geschlecht='w')
    'Eva'
    """
    def __init__(self, vornamen_weiblich, vornamen_maennlich, specials={}):
        self.vornamen_maennlich = vornamen_maennlich
        self.vornamen_weiblich = vornamen_weiblich
        self.specials = specials
        self.map = {}
    def _get_replacement(self, name, geschlecht=None, **kw):
        name = name.strip()
        name_list = [name]
        sep = ''
        for x in (' ', '-'):
            if x in name:
                name_list = name.split(x)
                sep = x
                break
        new_name_list = []
        if geschlecht == 'm':
            pool = self.vornamen_maennlich
        else:
            pool = self.vornamen_weiblich
        for n in name_list:
            new_name_list.append(replace(n, self.map, pool, self.specials))
        new_name = sep.join(new_name_list)
        return new_name
class TelefonUmbenenner(Umbenenner):
    """
    # >>> pu = TelefonUmbenenner()
    # >>> pu.umbenennen('01568 1234')
    # '09999 1234'
    # >>> pu.umbenennen('0156771234')
    # '0999971234'
    >>> pu = TelefonUmbenenner()
    >>> pu.umbenennen('019 X28')
    '014 X73'
    >>> pu.umbenennen(' 29')
    ' 74'
    >>> pu.umbenennen('0551232')
    '0006787'
    """
    def _get_replacement(self, name):
        # Zu jeder Ziffer 5 dazuzaehlen:
        if name.startswith('01'):
            new_name = '01'
            name = name[2:]
        elif name.startswith('0'):
            new_name = '0'
            name = name[1:]
        else:
            new_name = ''
        # for char in name:
        #     if char.isdigit():
        #         res = int(char) + 5
        #         if res > 10:
        #             res -= 10
        #         new_name += str(res)
        #     else:
        #         new_name += char
        char_list = list(name)
        int_list = [(int(c) if c.isdigit() else c) for c in char_list]
        new_list = [(d+5 if isinstance(d, int) else d) for d in int_list]
        new_list = [(d-10 if isinstance(d, int) and d > 9 else d) for d in new_list]
        new_name += ''.join([str(x) for x in new_list])
        return new_name
class PLZUmbenenner(Umbenenner):
    """
    >>> pu = PLZUmbenenner()
    >>> pu.umbenennen('33455')
    '43354'
    >>> pu.umbenennen('95000')
    '94899'
    >>> pu.umbenennen(' 29')
    '9928'
    >>> pu.umbenennen('a98f')
    'a98f'
    """
    def _get_replacement(self, name):
        try:
            plz_zahl = int(name)
            plz_zahl += 9899
            if plz_zahl > 100000:
                plz_zahl -= 10000
            assert plz_zahl < 100000
            return str(plz_zahl)
        except (ValueError, AssertionError):
            return name
class EmailUmbenenner(Umbenenner):
    """
    >>> pu = EmailUmbenenner(['a l:',"e'v"], ['web.de', 'aol.com'])
    >>> pu.umbenennen('albrecht@in-berlin.de')
    'e.v@web.de'
    >>> pu.umbenennen('albrecht@schmied.org')
    'e.v@aol.com'
    >>> pu.umbenennen('rob@schmied.org')
    'a.l.@aol.com'
    >>> pu.umbenennen('rob@schmied.de')
    'a.l.@web.de'
    >>> pu.umbenennen('robschmied.de')
    'robschmied.de'
    >>> pu.umbenennen('')
    ''
    """
    def __init__(self, userpool, domainpool):
        self.userpool = userpool
        self.domainpool = domainpool
        self.usermap = {}
        self.domainmap = {}
        self.domainindex = 0
    def _get_replacement(self, name):
        try:
            user, domain = name.split('@')
        except ValueError:
            return name
        if user in self.usermap:
            new_user = self.usermap[user]
        else:
            new_user = self.userpool.pop()
            new_user_list = list(new_user)
            new_user_list = [(c if c.isalpha() else '.') for c in new_user_list]
            new_user = ''.join(new_user_list)
            self.usermap[user] = new_user
        if domain in self.domainmap:
            new_domain = self.domainmap[domain]
        else:
            new_domain = self.domainpool[self.domainindex]
            self.domainindex += 1
            if self.domainindex >= len(self.domainpool):
                self.domainindex = 0
            self.domainmap[domain] = new_domain
        return "{}@{}".format(new_user, new_domain)

def tolatin(line):
    try:
        u = line.decode('utf8')
        return u.encode('latin1')
    except:
        pass
    try:
        u = line.decode('latin1')
        return u.encode('latin1')
    except:
        pass
    try:
        return u.encode('latin1')
    except:
        pass
    return None

def convert(filename):
    newfilename = filename+'.latin1'
    try: os.unlink(newfilename)
    except: pass
    errors = 0
    f = open(newfilename,'w', encoding='latin1')
    for line in open(filename, 'r').readlines():
        #print(chardet.detect(line))
        #res = line.encode('latin1')
        line = line.strip()
        line = line.strip('"')
        line = line.strip()
        res = line+'\n'
        try:
            f.write(res)
        except:
            errors += 1
    #print (errors)
        # else:
        #     print(res

def norm(filename):
    newfilename = filename+'.unique'
    try: os.unlink(newfilename)
    except: pass
    names = set()
    for line in open(filename, 'r').readlines():
        line = line.strip()
        line = line.strip('"')
        line = line.strip()
        names.add(line)
    names = list(names)
    names.sort()
    f = open(newfilename,'w', encoding='latin1')
    for line in names:
        f.write(line)


def choose(filename, n):
    from random import choice
    newfilename = filename+'.auswahl-'+str(n)
    try: os.unlink(newfilename)
    except: pass
    chosen = []
    lines = open(filename, 'r', encoding='latin1').readlines()
    chosen = [choice(lines) for x in range(n)]
    chosen.sort()
    f = open(newfilename,'w', encoding='latin1')
    for line in chosen:
        f.write(line)
    f.close()
    
def pr(filename):
    lines = open(filename, 'r', encoding='latin1').readlines()
    for line in lines:
        #print(line[:-1])
        print line[:-1]
        

def shuffle(filename):
    from random import shuffle
    newfilename = filename[:-4]+'_ungeordnet'+'.txt'
    try: os.unlink(newfilename)
    except: pass
    lines = open(filename, 'r', encoding='latin1').readlines()
    lines = list(set(lines))
    shuffle(lines)
    f = open(newfilename,'w', encoding='latin1')
    for line in lines:
        f.write(line)
    f.close()
    
def mixdata(filename1, filename2, newfilename):
    f = open(newfilename, 'wb')
    f1 = open(filename1, 'rb').readlines()
    f2 = open(filename2, 'rb').readlines()
    for x,y in zip(f1,f2):
        f.write(x)
        f.write(y)
    f.close()





if __name__ == "__main__":
    import doctest
    doctest.testmod()
    # convert(sys.argv[1])
    # norm(sys.argv[1])
    # choose(sys.argv[1], int(sys.argv[2]))
    # pr(sys.argv[1])
    # shuffle(sys.argv[1])
    # pu = VornamenUmbenenner(['Eva', 'Tina', 'Vera'], ['Rob', 'Thomas', 'Walter'], specials={'Donna': 'Rebecca'})
    # res = pu.umbenennen('Angela-Thomas', geschlecht='w')
    # print 'RES', res
    # mixdata(sys.argv[1], sys.argv[2], sys.argv[3]) 
