# coding: latin-1

"""Utilities.

Sollten ohne EBKuS-Import auf oberster Ebene auskommen.
"""

import sys
import os
import time
import datetime

def timeit(func, prompt, quiet=False, accu=None):
    def wrap(*args, **kwargs):
        started_at = time.time()
        result = func(*args, **kwargs)
        duration = time.time() - started_at
        if not quiet:
            print prompt, "%.3f" % (duration,)
        if accu:
            accu.add(duration, 
                     #func=func.func_name,
                     )
        return result
    return wrap


class Bunch(dict):
    """
    >>> b = Bunch()
    >>> b.myattr #doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    ...
    AttributeError:
    >>> b['myattr'] #doctest: +IGNORE_EXCEPTION_DETAIL
    Traceback (most recent call last):
    ...
    KeyError:
    >>> b.myattr = 3
    >>> b.myattr
    3
    >>> b['myattr']
    3
    >>> 'myattr' in b
    True
    """
    def __init__(self, *args, **kwds):
        super(Bunch, self).__init__(*args, **kwds)
        self.__dict__ = self

# Das funktioniert nicht
# class BunchNone(Bunch):
#     """
#     >>> b = BunchNone()
#     >>> b.myattr = 3
#     >>> b.myattr
#     3
#     >>> b['myattrpp']
#     >>> b.myattrpp
#     >>> b['myattr']
#     3
#     >>> b.myattr
#     3
#     >>> 
#     """
#     def __getattr__(self, attr):
#         #print 'GETATTR'
#         if attr in self.__dict__:
#             return  self.__dict__[attr]
#         return None
#     def __getitem__(self, key):
#         #print 'GETKEY'
#         if key in self.__dict__:
#             return  self.__dict__[key]
#         return None
# class BunchEmptyString(Bunch):
#     """
#     >>> b = BunchEmptyString()
#     >>> b.myattr
#     ''
#     >>> b['myattrpp']
#     ''
#     >>> b.myattr = 3
#     >>> b.myattr
#     3
#     >>> b['myattr']
#     3
#     """
#     def __getattribute__(self, attr):
#         #print 'GETATTR'
#         if attr in self.__dict__:
#             return  self.__dict__[attr]
#         return ''
#     def __getitem__(self, key):
#         #print 'GETKEY'
#         if key in self.__dict__:
#             return  self.__dict__[key]
#         return ''
    

def break_lines(long_string, maxlen):
    """In long_string umbrechen, so das jede Zeile maximal maxlen lang ist.
    (Ausnahme: ein Wort ist größer als maxlen).
    Ergebnis ist wieder ein langer String, aber mit hinreichen vielen '\r\n'.
    >>> x = 3
    >>> x
    3
    >>> s = "1234567890 1234567890 1234567890 1234567890"
    >>> break_lines(s, 13)
    '1234567890\\r\\n1234567890\\r\\n1234567890\\r\\n1234567890'
    >>> break_lines(s, 8)
    '1234567890\\r\\n1234567890\\r\\n1234567890\\r\\n1234567890'
    >>> break_lines(s, 21)
    '1234567890 1234567890\\r\\n1234567890 1234567890'
    >>> break_lines(s, 32)
    '1234567890 1234567890 1234567890\\r\\n1234567890'
    """
    def adjust_line(line):
        """In Worte spalten und jede Zeile auf 65 auffüllen"""
        words = line.split()
        lines = []
        line = ""
        for word in words:
            if len(word) > (maxlen - 5):
                # Wort zu lang
                if line:
                    lines.append(line)
                lines.append(word)
                line = ""
            elif len(line) + len(word) < maxlen:
                # Passt
                line += ' ' if line else ''
                line += word
            else:
                # Neue Zeile
                lines.append(line)
                line = word
        if line:
            lines.append(line)
        return lines
    adjusted_lines = []
    lines = long_string.split('\r\n')
    for line in lines:
        if len(line) < maxlen:
            adjusted_lines.append(line)
        else:
            adjusted_lines += adjust_line(line)
    return '\r\n'.join(adjusted_lines)


dtstr_format = "%Y%m%d_%H%M%S"

def dtstr_from_dt(dt):
    return datetime.datetime.strftime(dt, dtstr_format)
def getmtime_as_dtstr(path):
    return dtstr_from_dt(datetime.datetime.fromtimestamp(int(os.path.getmtime(path))))
def dt_from_dtstr():
    return datetime.datetime.stpftime(dtstr, dtstr_format)
    
# dtstr_from_dt()
# dt_from_dtstr

def write_dict_to_file(dict_, dir, prompt=''):
    # print 'WRITE_DICT_TO_FILE', dict_,dir, prompt
    now = dtstr_from_dt(datetime.datetime.now())
    name = "{}_{}".format(now, prompt)
    path = os.path.join(dir, name)
    res = "{}\n".format(name)
    for k in sorted(dict_.keys()):
        res += "{:<10} {}\n".format(k, dict_[k])
    with open(path, 'wb') as f:
        f.write(res)
        


if __name__ == '__main__':
    import doctest
    doctest.testmod()

    # >>> b.myattrpp = 3
    # >>> b['myattrpp']
    # 3
