#!/usr/bin/env python
# coding: latin-1

# Die http-Tests funktionier nur, wenn das Pythonpaket requests in der Version 2.2.1 oder 2.3.0 installiert ist.
# Hhere Versionen funktionieren nicht, ich habe nicht verstanden warum nicht.
# Nach dem logout, das ein Redirect mit location: login generiert, antwortet requests > 2.3.0 mit pwchange
# anstatt mit login. Das fhr auf eine Seite 'Zugriff verweigert'.

"""
test   -- Regressionstests fr EBKuS

Zum ersten Mal soll es einen automatisierten Test geben. Durch Aufruf dieser Datei:

python test.py

Evt. viele Parameter. Also gleich ein Skript mit argparse etc.? Ist vielleicht das sauberste.

Anzustreben:
- ein Test fr jedes Formular, also fr jedes Modul in /html/
- berhaupt fr jedes Modul
- Unittests ohne server, zB fr dbapp.
- Tests ber Testinstanz auf http-Ebene.



Http-Tests
==========

Wie soll das Framework aussehen? 
Vielleicht so:
- Die Tests knnen nur dann durchgefhrt werden, wenn die ebkus.conf eine Instanz
  'test' enthlt.
- im Verz. sql gibt es eine test.sql, mit der die Datenbank fr die Tests initialisiert wird.
- Whrend eines Tests kann die Datenbank immer wieder zurckgestellt werden auf ihren Anfangszustand,
  entweder durch initialisieren oder per sql.

"""

import os
import sys
import subprocess
import glob
import time
import traceback
import datetime
assert sys.version_info >= (2,6), "Es wird Python 2.6 oder hher bentigt"
from os.path import join, split, dirname, basename, exists, isdir, isfile, normpath, normcase, abspath, realpath, splitext

import argparse

"""
Ist es mglich, eine Klasse fr jeden Subparser zu machen?
"""


class TestCtl(object):
    def __init__(self, argv, **kw):
        #print '__file__', __file__
        parser = argparse.ArgumentParser(description="EBKuS Tests")
        parser.add_argument("--ebkus_installation", dest='install_dir', 
                            help="EBKuS-Installationsverzeichnis. "
                            "Falls nicht angegeben, muss das Skript im Homeverzeichnis ('ebkus') gestartet werden.")
        parser.add_argument("--verbose", "-v", action='count', 
                            help="Verbosity (error, info, debug)")
        parser.add_argument("--http", action='store_true', 
                            help="Test ueber http (WEB)")
        parser.add_argument("--ebupd", action='store_true', 
                            help="Test ueber ebupd (API)")

        self.ns = parser.parse_args(argv[1:])
        if self.ns.install_dir is None:
            # Skript steht im Homeverzeichnis
            self.install_dir = dirname(dirname(normcase(realpath(abspath(__file__)))))
        else:
            self.install_dir = normcase(realpath(abspath(self.ns.install_dir)))
        #print "Installationsverzeichnis:", self.install_dir
        self.home_dir = join(self.install_dir, 'ebkus')
        self.lib_dir = join(self.home_dir, 'lib')
        #print self.lib_dir
        assert isdir(self.lib_dir), "Kein lib-Verzeichnis: {0}".format(self.lib_dir)
        if self.lib_dir != sys.path[0]:
            sys.path.insert(0, self.lib_dir)
        ebkus_conf_path = join(self.home_dir, 'ebkus.conf')
        assert isfile(ebkus_conf_path), "Keine ebkus.conf"
        # try:
        #     from ebkus.ebkus_control import EBKuSControl
        #     from ebkus.ebkus_control import ServiceException
        # except:
        #     from ebkus_control import EBKuSControl
        #     from ebkus_control import ServiceException
        #print 'Args:', self.ns
        from ebkusctl import EBKuSControl, ServiceException
        self.ectl = EBKuSControl(self.home_dir)
        self.ServiceException = ServiceException
    def run_tests(self):
        from unittest import defaultTestLoader as loader, TestSuite, TextTestRunner
        # Testcases importieren
        from ebkus.test.klientenkarte import EBKuSTestCase
        from ebkus.test.klientenkarte import EBKuSHttpTest
        EBKuSHttpTest.verbose = self.ns.verbose
        from ebkus.test.klientenkarte import EBKuSEbupdTest
        EBKuSEbupdTest.verbose = self.ns.verbose
        EBKuSTestCase.ectl = self.ectl
        tests = ()
        both = not self.ns.ebupd and not self.ns.http
        if self.ns.http or both:
            tests += (loader.loadTestsFromTestCase(EBKuSHttpTest),)
        if self.ns.ebupd or both:
            tests += (loader.loadTestsFromTestCase(EBKuSEbupdTest),)
        suite = TestSuite(tests)
        runner = TextTestRunner(verbosity = self.ns.verbose)
        runner.run(suite)
    def execute(self):
        self.run_tests()

if __name__ == '__main__':
    #print argparse.ArgumentParser
    ctl = TestCtl(sys.argv)
    ctl.execute()
    
